Package: MSstatsConvert
Title: Import Data from Various Mass Spectrometry Signal Processing
        Tools to MSstats Format
Version: 1.22.1
Authors@R: c(
  person("Mateusz", "Staniak", email = "mtst@mstaniak.pl", role = "aut"),
  person("Devon", "Kohler", email = "kohler.d@northeastern.edu", role = "aut"),
  person("Anthony", "Wu", email = "wu.anthon@northeastern.edu", role = c("aut", "cre")),
  person("Meena", "Choi", email = "mnchoi67@gmail.com", role = "aut"),
  person("Ting", "Huang", email = "thuang0703@gmail.com", role = "aut"),
  person("Olga", "Vitek", , email = "o.vitek@northeastern.edu", role = "aut")) 
Description: MSstatsConvert provides tools for importing reports of Mass Spectrometry data processing tools
    into R format suitable for statistical analysis using the MSstats and MSstatsTMT packages.
License: Artistic-2.0
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
biocViews: MassSpectrometry, Proteomics, Software, DataImport,
        QualityControl
Depends: R (>= 4.0)
Imports: data.table, methods, checkmate, utils, stringi, Rcpp, parallel
Suggests: tinytest, covr, knitr, arrow, rmarkdown
LinkingTo: Rcpp
Collate: 'clean_ProteinProspector.R' 'clean_Metamorpheus.R'
        'clean_DIANN.R' 'clean_Philosopher.R' 'clean_Spectronaut.R'
        'clean_SpectroMine.R' 'clean_Skyline.R'
        'clean_ProteomeDiscoverer.R' 'clean_Progenesis.R'
        'clean_OpenSWATH.R' 'clean_OpenMS.R' 'clean_MaxQuant.R'
        'clean_DIAUmpire.R' 'MSstatsConvert_core_functions.R'
        'RcppExports.R' 'converters_DIANNtoMSstatsFormat.R'
        'converters_DIAUmpiretoMSstatsFormat.R'
        'converters_FragPipetoMSstatsFormat.R'
        'converters_MaxQtoMSstatsFormat.R'
        'converters_MaxQtoMSstatsTMTFormat.R'
        'converters_MetamorpheusToMSstatsFormat.R'
        'converters_OpenMStoMSstatsFormat.R'
        'converters_OpenMStoMSstatsTMTFormat.R'
        'converters_OpenSWATHtoMSstatsFormat.R'
        'converters_PDtoMSstatsFormat.R'
        'converters_PDtoMSstatsTMTFormat.R'
        'converters_PhilosophertoMSstatsTMTFormat.R'
        'converters_ProgenesistoMSstatsFormat.R'
        'converters_ProteinProspectortoMSstatsTMTFormat.R'
        'converters_SkylinetoMSstatsFormat.R'
        'converters_SpectroMinetoMSstatsTMTFormat.R'
        'converters_SpectronauttoMSstatsFormat.R'
        'utils_MSstatsConvert.R' 'utils_annotation.R'
        'utils_anomaly_score.R' 'utils_balanced_design.R'
        'utils_checks.R' 'utils_classes.R' 'utils_clean_features.R'
        'utils_data_health.R' 'utils_documentation.R'
        'utils_dt_operations.R' 'utils_filtering.R' 'utils_fractions.R'
        'utils_logging.R' 'utils_shared_peptides.R'
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/MSstatsConvert
git_branch: RELEASE_3_23
git_last_commit: fe449ec
git_last_commit_date: 2026-07-16
Repository: Bioconductor 3.23
Date/Publication: 2026-07-19
NeedsCompilation: yes
Packaged: 2026-07-19 23:12:23 UTC; biocbuild
Author: Mateusz Staniak [aut],
  Devon Kohler [aut],
  Anthony Wu [aut, cre],
  Meena Choi [aut],
  Ting Huang [aut],
  Olga Vitek [aut]
Maintainer: Anthony Wu <wu.anthon@northeastern.edu>

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,MSstatsValidated)
S3method(as.data.table,MSstatsValidated)
export(CheckDataHealth)
export(DIANNtoMSstatsFormat)
export(DIAUmpiretoMSstatsFormat)
export(FragPipetoMSstatsFormat)
export(MSstatsAnomalyScores)
export(MSstatsBalancedDesign)
export(MSstatsClean)
export(MSstatsImport)
export(MSstatsLogsSettings)
export(MSstatsMakeAnnotation)
export(MSstatsPreprocess)
export(MSstatsSaveSessionInfo)
export(MaxQtoMSstatsFormat)
export(MaxQtoMSstatsTMTFormat)
export(MetamorpheusToMSstatsFormat)
export(OpenMStoMSstatsFormat)
export(OpenMStoMSstatsTMTFormat)
export(OpenSWATHtoMSstatsFormat)
export(PDtoMSstatsFormat)
export(PDtoMSstatsTMTFormat)
export(PhilosophertoMSstatsTMTFormat)
export(ProgenesistoMSstatsFormat)
export(ProteinProspectortoMSstatsTMTFormat)
export(SkylinetoMSstatsFormat)
export(SpectroMinetoMSstatsTMTFormat)
export(SpectronauttoMSstatsFormat)
export(getDataType)
export(getInputFile)
exportMethods(getDataType)
exportMethods(getInputFile)
import(Rcpp)
import(data.table)
import(parallel)
importFrom(data.table,as.data.table)
importFrom(data.table,fread)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setnames)
importFrom(data.table,uniqueN)
importFrom(methods,new)
importFrom(stats,na.omit)
importFrom(utils,sessionInfo)
useDynLib(MSstatsConvert, .registration = TRUE)

# MSstats development version

<!-- badges: start -->
[![Travis build status](https://travis-ci.org/Vitek-Lab/MSstatsConvert.svg?branch=master)](https://travis-ci.org/Vitek-Lab/MSstatsConvert)
[![Codecov test coverage](https://codecov.io/gh/Vitek-Lab/MSstatsConvert/branch/master/graph/badge.svg)](https://codecov.io/gh/Vitek-Lab/MSstatsConvert?branch=master)
<!-- badges: end -->

[MSstats](https://msstats.org) is an R-based/Bioconductor package for statistical relative quantification of peptides and proteins in mass spectrometry-based proteomic experiments. 
It is applicable to multiple types of sample preparation, including label-free workflows, workflows that use stable isotope labeled reference proteins and peptides, and workflows that use fractionation. 
It is applicable to targeted Selected Reactin Monitoring(SRM), Data-Dependent Acquisiton(DDA or shotgun), and Data-Independent Acquisition(DIA or SWATH-MS). 
This package implements converter functions that are used by MSstats to import data from various signal processing tools.

## Installation 

This development version is only available on Github:

```
devtools::install_github("Vitek-Lab/MSstatsConvert", build_vignettes = TRUE)
```

## Documentation and examples

An extensive documentation can be found on the [official website of the package](https://msstats.org).


## Contributing

We welcome contributions from the community. For details on how to contribute to the
development of MSsstas, please refer to the [CONTRIBUTING](https://github.com/Vitek-Lab/MSstatsConvert/blob/master/.github/CONTRIBUTING.md) file.

## License

[Artistic-2.0](https://opensource.org/licenses/Artistic-2.0)

setOldClass("data.frame")

setClass("MSstatsValidated", contains = "data.frame")
setOldClass("MSstatsValidated", S4Class = "MSstatsValidated")

#' Output format for further analysis by MSstats
#' @param input data.table
#' @param anomaly_metrics character vector of quality metric column names to be used as features in an anomaly detection model
#' @importFrom methods new
#' @return object of class MSstatsValidated that inherits from data.frame
#' @keywords internal
.MSstatsFormat = function(input, anomaly_metrics = c()) {
    input = .selectMSstatsColumns(input, anomaly_metrics)
    new("MSstatsValidated", as.data.frame(input))
}


#' Convert output of converters to data.frame
#' @param x object of class MSstatsValidated
#' @param ... Additional arguments to be passed to or from other methods.
#' @return data.frame
#' @export
as.data.frame.MSstatsValidated = function(x, ...) {
  as.data.frame(unclass(x))
}

#' Convert output of converters to data.table
#' @param x object of class MSstatsValidated
#' @param ... Additional arguments to be passed to or from other methods.
#' @return data.tables
#' @export
as.data.table.MSstatsValidated = function(x, ...) {
  data.table::as.data.table(unclass(x))
}

data_invalid = data.table::data.table(ProteinName = letters[1:10],
                                      PeptideSequence = LETTERS[1:10])
data_no_channel = data.table::data.table(ProteinName = letters[1:10],
                                         PeptideSequence = LETTERS[1:10],
                                         PrecursorCharge = 1,
                                         FragmentIon = NA,
                                         ProductCharge = NA,
                                         Run = 1,
                                         Condition = 1,
                                         BioReplicate = 1,
                                         Intensity = 0.1,
                                         IsotopeLabelType = "L")
data_no_channel_LH = data.table::data.table(ProteinName = letters[1:10],
                                            PeptideSequence = LETTERS[1:10],
                                            IsotopeLabelType = rep(c("L", "H"),
                                                                   each = 5))
data_channel = data.table::data.table(ProteinName = letters[1:10],
                                      PeptideSequence = LETTERS[1:10],
                                      PrecursorCharge = 1,
                                      PSM = "1",
                                      Mixture = 1,
                                      TechRepMixture = 1,
                                      Run = 1,
                                      BioReplicate = 1,
                                      Condition = 1,
                                      Channel = paste0("A", 1:10))
tmt_object = MSstatsConvert:::.MSstatsFormat(data_channel)
labelfree_object = MSstatsConvert:::.MSstatsFormat(data_no_channel)
labelled_object = MSstatsConvert:::.MSstatsFormat(data_no_channel_LH)

# Has the right format for MSstats data
expect_true(is(labelled_object, "MSstatsValidated"))
expect_true(is(labelfree_object, "MSstatsValidated"))
expect_true(is(tmt_object, "MSstatsValidated"))
# Inherits after data.frame
expect_true(is(labelled_object, "data.frame"))
expect_true(is(labelfree_object, "data.frame"))
expect_true(is(tmt_object, "data.frame"))

#' Clean raw data from Skyline
#' @param msstats_object an object of class `MSstatsSpectroMineFiles`.
#' @return data.table
#' @keywords internal
.cleanRawSkyline = function(msstats_object) {
    Truncated = isZero = DetectionQValue = Intensity = NULL
    
    sl_input = getInputFile(msstats_object, "input")
    data.table::setnames(sl_input, c("FileName", "Area"),
                         c("Run", "Intensity"), skip_absent = TRUE)
    
    sl_input = sl_input[, !(colnames(sl_input) == "PeptideSequence"), 
                        with = FALSE]
    data.table::setnames(sl_input, "PeptideModifiedSequence",
                         "PeptideSequence", skip_absent = TRUE)
    suppressWarnings({
        sl_input[, Intensity := as.numeric(as.character(Intensity))]
    })
    if (is.element("DetectionQValue", colnames(sl_input))) {
        sl_input[, DetectionQValue := as.numeric(as.character(DetectionQValue))]
    }
    if (is.character(sl_input$Truncated) | is.factor(sl_input$Truncated)) {
        sl_input[, Truncated := as.character(Truncated) == "True"]
    }
    sl_input[, Truncated := as.character(Truncated)]
    
    sl_cols = c("ProteinName", "PeptideSequence", "PrecursorCharge", 
                "FragmentIon", "ProductCharge", "IsotopeLabelType", "Condition",
                "BioReplicate", "Run", "Intensity", "StandardType")
    sl_cols = c(sl_cols, "Fraction", "DetectionQValue", "Truncated")
    sl_input = sl_input[, intersect(sl_cols, colnames(sl_input)), with = FALSE]
    sl_input[, isZero := ((Intensity == 0) | (Intensity < 1)) & !is.na(Intensity)]
    .logSuccess("Skyline", "clean")
    sl_input
}


#' Handle isotopic peaks
#' @param input data.table preprocessed by one of the `cleanRaw*` functions.
#' @param aggregate if TRUE, isotopic peaks will be summed.
#' @return data.table
#' @keywords internal
.handleIsotopicPeaks = function(input, aggregate = FALSE) {
    FragmentIon = ProductCharge = NULL
    
    if (aggregate) {
        input$isZero = ifelse(input$Intensity == 0, TRUE, input$isZero)
        if (.checkDDA(input)) {
            feature_cols = c("ProteinName", "PeptideSequence", "PrecursorCharge", 
                             "Run")
            feature_cols = c(feature_cols, c("BioReplicate", "Condition", "StandardType", 
                                             "IsotopeLabelType", "DetectionQValue"))
            feature_cols = intersect(feature_cols, colnames(input))
            input = .summarizeMultipleMeasurements(input, sum, feature_cols)
            input[, FragmentIon := NA]
            input[, ProductCharge := NA]
            getOption("MSstatsLog")("INFO", "** Three isotopic preaks per feature and run are summed")
            getOption("MSstatsMsg")("INFO", "** Three isotopic preaks per feature and run are summed")
        }
    }
    input
}


#' Check validity of DDA data
#' @param input data.table preprocessed by one of the `cleanRaw*` functions.
#' @return logical
#' @return logical, TRUE means that the `input` dataset comes from a DDA experiment
#' @keywords internal
.checkDDA = function(input) {
    fragment_ions = as.character(unique(input[["FragmentIon"]]))
    check_DDA = setdiff(c("precursor", "precursor [M+1]", "precursor [M+2]"), 
                        fragment_ions)
    frags = setdiff(fragment_ions, 
                    c('precursor', 'precursor [M+1]', 'precursor [M+2]'))
    precursors = intersect(fragment_ions, 
                           c("precursor", "precursor [M+1]", "precursor [M+2]"))
    if (length(frags) > 0 & length(precursors) > 0) {
        msg = paste("** Please check precursors information in the FragmentIon column.",
                    "If your experiment is DIA or SRM, please remove the precursors.",
                    "If your experiments is DDA, please check the precursor information.",
                    "MSstats only performs analysis on either MS1 or MS2, but not both simultaneously.")
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
    length(check_DDA) < 3
}

# DIAUmpire
dia_frag = data.table::fread("./raw_data/DIAUmpire/dia_frag.csv")
dia_pept = data.table::fread("./raw_data/DIAUmpire/dia_pept.csv")
dia_prot = data.table::fread("./raw_data/DIAUmpire/dia_prot.csv")
dia_import = MSstatsConvert::MSstatsImport(list(Fragments = dia_frag,
                                                Peptides = dia_pept,
                                                Proteins = dia_prot),
                                           "MSstats", "DIAUmpire")
dia_import2 = MSstatsConvert::MSstatsImport(
    list(Fragments = dia_frag,
         Peptides = dia_pept[, colnames(dia_pept) != "Selected_fragments", 
                             with = FALSE],
         Proteins = dia_prot),
    "MSstats", "DIAUmpire")
dia_import3 = MSstatsConvert::MSstatsImport(
    list(Fragments = dia_frag,
         Peptides = dia_pept,
         Proteins = dia_prot[, colnames(dia_prot) != "Selected_peptides", 
                             with = FALSE]),
    "MSstats", "DIAUmpire")
dia_frag2 = data.table::copy(dia_frag)
dia_frag2$Fragment = gsub("\\+[0-9]+", "", dia_frag2$Fragment)
dia_import4 = MSstatsConvert::MSstatsImport(
    list(Fragments = dia_frag2,
         Peptides = dia_pept,
         Proteins = dia_prot),
    "MSstats", "DIAUmpire")
dia_cleaned = MSstatsConvert::MSstatsClean(dia_import, TRUE, TRUE)
dia_cleaned2 = MSstatsConvert::MSstatsClean(dia_import4, TRUE, TRUE)
dia_cleaned3 = MSstatsConvert::MSstatsClean(dia_import, TRUE, FALSE)
expect_error(MSstatsConvert::MSstatsClean(dia_import, FALSE, TRUE))
expect_equal(ncol(dia_cleaned), 5)
expect_equal(ncol(dia_cleaned3), 5)
expect_true(nrow(dia_cleaned) > 0)
expect_true(nrow(dia_cleaned3) > 0)
expect_error(MSstatsConvert::MSstatsClean(dia_import, FALSE, FALSE))
expect_error(MSstatsConvert::MSstatsClean(dia_import2, TRUE, TRUE))
expect_error(MSstatsConvert::MSstatsClean(dia_import3, TRUE, TRUE))
expect_equal(dia_cleaned, dia_cleaned2)
# MaxQuant
mq_ev = data.table::fread("./raw_data/MaxQuant/mq_ev.csv")
mq_pg = data.table::fread("./raw_data/MaxQuant/mq_pg.csv")
mq_import = MSstatsConvert::MSstatsImport(list(evidence = mq_ev, 
                                               protein_groups = mq_pg),
                                          "MSstats", "MaxQuant")
mq_cleaned = MSstatsConvert::MSstatsClean(mq_import, protein_id_col = "Proteins")
mq_cleaned_site = MSstatsConvert::MSstatsClean(mq_import, 
                                               protein_id_col = "Proteins",
                                               remove_by_site = TRUE)
expect_true(nrow(mq_cleaned_site) < nrow(mq_cleaned))
expect_true(nrow(mq_cleaned_site) > 0)
expect_equal(
    ncol(MSstatsConvert::MSstatsClean(mq_import, protein_id_col = "Proteins")),
    7
)
expect_true(nrow(mq_cleaned) > 0)
# MaxQuantTMT
mqtmt_ev = data.table::fread("./raw_data/MaxQuantTMT/mq_ev.csv")
mqtmt_pg = data.table::fread("./raw_data/MaxQuantTMT/mq_pg.csv")
mqtmt_import = MSstatsConvert::MSstatsImport(list(evidence = mqtmt_ev, 
                                                  protein_groups = mqtmt_pg),
                                             "MSstatsTMT", "MaxQuant")
mqtmt_cleaned = MSstatsConvert::MSstatsClean(mqtmt_import, 
                                             protein_id_col = "Proteins")
expect_equal(
    ncol(mqtmt_cleaned),
    8
)
expect_true(nrow(mq_cleaned) > 0)
# OpenMS
openms_input = data.table::fread("./raw_data/OpenMS/openms_input.csv")
openms_import = MSstatsConvert::MSstatsImport(list(input = openms_input), 
                                              "MSstats", "OpenMS")
openms_import2 = MSstatsConvert::MSstatsImport(list(input = openms_input[, -6]), 
                                               "MSstats", "OpenMS")
om_cleaned = MSstatsConvert::MSstatsClean(openms_import)
expect_equal(ncol(om_cleaned), 10)
expect_equal(ncol(MSstatsConvert::MSstatsClean(openms_import2)), 10)
expect_true(nrow(om_cleaned) > 0)
# OpenMSTMT
openmstmt_input = data.table::fread("./raw_data/OpenMSTMT/openmstmt_input.csv")
openmstmt_import = MSstatsConvert::MSstatsImport(list(input = openmstmt_input), 
                                                 "MSstatsTMT", "OpenMS")
omtmt_cleaned = MSstatsConvert::MSstatsClean(openmstmt_import)
expect_equal(ncol(omtmt_cleaned), 12)
expect_true(nrow(omtmt_cleaned) > 0)
# OpenSWATH
openswath_input = data.table::fread("./raw_data/OpenSWATH/openswath_input.csv")
openswath_import = MSstatsConvert::MSstatsImport(list(input = openswath_input), 
                                                 "MSstats", "OpenSWATH")
os_cleaned = MSstatsConvert::MSstatsClean(openswath_import)
expect_equal(ncol(os_cleaned), 8)
expect_true(nrow(os_cleaned) > 0)
# PD
pd_input = data.table::fread("./raw_data/PD/pd_input.csv")
pd_input_frac = data.table::copy(pd_input)
pd_input_frac$Fraction = 1
pd_import = MSstatsConvert::MSstatsImport(list(input = pd_input), 
                                          "MSstats", "ProteomeDiscoverer")
pd_import_frac = MSstatsConvert::MSstatsImport(list(input = pd_input_frac), 
                                               "MSstats", "ProteomeDiscoverer")
pd_cleaned = MSstatsConvert::MSstatsClean(
    pd_import, protein_id_column = "ProteinGroupAccessions",
    sequence_column = "Sequence", quantification_column = "Intensity",
    remove_shared = TRUE)
pd_cleaned_frac = MSstatsConvert::MSstatsClean(
    pd_import_frac, protein_id_column = "ProteinGroupAccessions",
    sequence_column = "Sequence", quantification_column = "Intensity",
    remove_shared = TRUE)
expect_equal(ncol(pd_cleaned), 5)
expect_true(nrow(pd_cleaned) > 0)
expect_equal(pd_cleaned, 
             pd_cleaned_frac[, 
                             colnames(pd_cleaned_frac) != "Fraction", 
                             with = FALSE])
# PD-TMT
pdtmt_input = data.table::fread("./raw_data/PDTMT/pdtmt_input.csv")
pdtmt_input2 = data.table::copy(pdtmt_input)
pdtmt_input2$ProteinMasterAccessions = pdtmt_input2$Protein.Accessions
pdtmt_input2$X..Protein.Groups = pdtmt_input2$X..Proteins
pdtmt_input3 = data.table::copy(pdtmt_input)
pdtmt_input3$Quan.Info = as.character(pdtmt_input3$Quan.Info)
pdtmt_input3$Quan.Info = "UNIQUE"
pdtmt_import = MSstatsConvert::MSstatsImport(list(input = pdtmt_input),
                                             "MSstatsTMT", "ProteomeDiscoverer")
pdtmt_import2 = MSstatsConvert::MSstatsImport(list(input = pdtmt_input2),
                                              "MSstatsTMT", "ProteomeDiscoverer")
pdtmt_import3 = MSstatsConvert::MSstatsImport(list(input = pdtmt_input3),
                                              "MSstatsTMT", "ProteomeDiscoverer")
pdtmt_cleaned = MSstatsConvert::MSstatsClean(pdtmt_import, 
                                             protein_id_column = "ProteinAccessions",
                                             remove_shared = TRUE)
pdtmt_cleaned2 = MSstatsConvert::MSstatsClean(pdtmt_import2, 
                                              protein_id_column = "ProteinMasterAccessions",
                                              remove_shared = TRUE)
pdtmt_cleaned3 = MSstatsConvert::MSstatsClean(pdtmt_import3, 
                                              protein_id_column = "ProteinMasterAccessions",
                                              remove_shared = TRUE)
expect_equal(pdtmt_cleaned, pdtmt_cleaned2)
expect_equal(pdtmt_cleaned[, colnames(pdtmt_cleaned) != "QuanInfo", 
                           with = FALSE], 
             pdtmt_cleaned3[, colnames(pdtmt_cleaned) != "QuanInfo", 
                            with = FALSE])
expect_error(MSstatsConvert::MSstatsClean(pdtmt_import, 
                                          protein_id_column = "ProteinAccessions",
                                          remove_shared = TRUE,
                                          intensity_columns_regexp = "Nothing"))
expect_equal(
    ncol(pdtmt_cleaned), 10
)
expect_true(nrow(pdtmt_cleaned) > 0)
# Progenesis
progenesis_input = data.table::fread("./raw_data/Progenesis/progenesis_input.csv")
progenesis_import = MSstatsConvert::MSstatsImport(list(input = progenesis_input),
                                                  "MSstats", "Progenesis")
progenesis_import2 = MSstatsConvert::MSstatsImport(
    list(input = rbind(progenesis_input[1, ], progenesis_input)),
    "MSstats", "Progenesis")
runs = unique(data.table::fread("./raw_data/Progenesis/progenesis_annot.csv")$Run)
pg_cleaned = MSstatsConvert::MSstatsClean(progenesis_import, runs)
pg_cleaned2 = MSstatsConvert::MSstatsClean(progenesis_import2, runs) 
expect_equal(ncol(pg_cleaned), 5)
expect_true(nrow(pg_cleaned) > 0)
expect_equal(pg_cleaned, pg_cleaned2)
# Skyline
skyline_input = data.table::fread("./raw_data/Skyline/skyline_input.csv")
skyline_input2 = data.table::copy(skyline_input)
skyline_input2$DetectionQValue = 0.01
skyline_input2$Truncated = ifelse(skyline_input2$Truncated, "True", "False")
skyline_input3 = data.table::copy(skyline_input)
skyline_input3$Fragment.Ion = rep(c("precursor", "a", "b"), times = nrow(skyline_input) / 3)
skyline_import = MSstatsConvert::MSstatsImport(list(input = skyline_input), 
                                               "MSstats", "Skyline")
skyline_import2 = MSstatsConvert::MSstatsImport(list(input = skyline_input2), 
                                                "MSstats", "Skyline")
skyline_import3 = MSstatsConvert::MSstatsImport(list(input = skyline_input3), 
                                                "MSstats", "Skyline")
sl_cleaned = MSstatsConvert::MSstatsClean(skyline_import)
sl_cleaned2 = MSstatsConvert::MSstatsClean(skyline_import2)
expect_equal(ncol(sl_cleaned), 13)
expect_true(nrow(sl_cleaned) > 0)
expect_equal(sl_cleaned, sl_cleaned2[, colnames(sl_cleaned), with = FALSE])
expect_error(MSstatsConvert:::.checkDDA(MSstatsConvert::MSstatsClean(skyline_import3)))
# SpectroMine
spectromine_input = data.table::fread("./raw_data/SpectroMine/spectromine_input.csv")
spectromine_import = MSstatsConvert::MSstatsImport(list(input = spectromine_input), 
                                                   "MSstatsTMT", "SpectroMine")
spectromine_import_error = MSstatsConvert::MSstatsImport(list(input = spectromine_input[, 1:19]),
                                                         "MSstatsTMT", "SpectroMine")
sm_cleaned = MSstatsConvert::MSstatsClean(spectromine_import)
expect_equal(ncol(sm_cleaned), 9)
expect_true(nrow(sm_cleaned) > 0)
expect_error(MSstatsConvert::MSstatsClean(spectromine_import_error))
# Spectronaut
spectronaut_input = data.table::fread("./raw_data/Spectronaut/spectronaut_input.csv")
spectronaut_import = MSstatsConvert::MSstatsImport(list(input = spectronaut_input), 
                                                   "MSstats", "Spectronaut")
sn_cleaned = MSstatsConvert::MSstatsClean(spectronaut_import,
                                          intensity = "PeakArea", 
                                          calculateAnomalyScores = FALSE, 
                                          anomalyModelFeatures = c())
expect_equal(ncol(sn_cleaned), 12)
expect_true(nrow(sn_cleaned) > 0)

# Test new peak quality columns
spectronaut_input_2 = spectronaut_input
spectronaut_input_2$`FG.ShapeQualityScore (MS2)` = 1
spectronaut_input_2$`FG.ShapeQualityScore (MS1)` = 1
spectronaut_input_2$`EG.ApexRT` = 1
spectronaut_input_2$`F.PossibleInterference` = TRUE
spectronaut_import_2 = MSstatsConvert::MSstatsImport(list(input = spectronaut_input_2), 
                                                   "MSstats", "Spectronaut")
sn_cleaned_2 = MSstatsConvert::MSstatsClean(spectronaut_import_2,
                                          intensity = "PeakArea", 
                                          calculateAnomalyScores = TRUE, 
                                          anomalyModelFeatures = c("FGShapeQualityScore(MS2)",
                                                                   "FGShapeQualityScore(MS1)",
                                                                   "EGApexRT"))
expect_equal(ncol(sn_cleaned_2), 16)
expect_true(nrow(sn_cleaned_2) > 0)

# Metamorpheus
metamorpheus_table = data.table::fread("./raw_data/Metamorpheus/QuantifiedPeaks.tsv")
input = MSstatsConvert::MSstatsImport(list(input = metamorpheus_table), "MSstats", "Metamorpheus")
expect_identical(is(input), c("MSstatsMetamorpheusFiles", "MSstatsInputFiles"))

#' Import FragPipe files
#' 
#' @param input name of FragPipe msstats.csv export. ProteinName, PeptideSequence, PrecursorCharge, FragmentIon, ProductCharge, IsotopeLabelType, Condition, BioReplicate, Run, Intensity are required.
#' @param ... additional parameters to `data.table::fread`.
#' @inheritParams .sharedParametersAmongConverters
#' 
#' @return data.frame in the MSstats required format.
#' 
#' @author Devon Kohler
#' 
#' @export
#' 
#' @examples 
#' fragpipe_raw = system.file("tinytest/raw_data/FragPipe/fragpipe_input.csv",
#'                               package = "MSstatsConvert")
#' fragpipe_raw = data.table::fread(fragpipe_raw)
#' fragpipe_imported = FragPipetoMSstatsFormat(fragpipe_raw, use_log_file = FALSE)
#' head(fragpipe_imported)
#' 
FragPipetoMSstatsFormat = function(
        input, useUniquePeptide = TRUE, removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE, summaryforMultipleRows = max,
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "FragPipe", ...)
    input = MSstatsConvert::getInputFile(input, "input")
    
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, NULL)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge", "FragmentIon", "ProductCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        feature_cleaning = list(remove_features_with_few_measurements = removeFewMeasurements,
                                summarize_multiple_psms = summaryforMultipleRows),
        columns_to_fill = list("IsotopeLabelType" = "L"))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  remove_few = removeFewMeasurements)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

#' Check validity of parameters to the `MSstatsImport` function.
#' @inheritParams MSstatsImport
#' @return none, throws an error if any of the assertions fail
#' @keywords internal
.checkMSstatsParams = function(input, annotation, 
                               feature_columns,
                               remove_shared_peptides,
                               remove_single_feature_proteins,
                               feature_cleaning) {
    checkmate::assertDataTable(input)
    checkmate::assertTRUE(is.character(annotation) | 
                              inherits(annotation, "data.frame") | 
                              is.null(annotation))
    checkmate::assertCharacter(feature_columns, min.len = 1)
    checkmate::assertLogical(remove_shared_peptides)
    checkmate::assertLogical(
        feature_cleaning[["remove_features_with_few_measurements"]], 
        )
    checkmate::assertLogical(remove_single_feature_proteins)
    checkmate::assertLogical(feature_cleaning[["remove_psms_with_all_missing"]],
                             null.ok = TRUE)
    checkmate::assertFunction(feature_cleaning[["summarize_multiple_psms"]])
}


#' Select an available options from a set of possibilities
#' @param possibilities possible legal values of a variable
#' @param option_set set of values that includes one of the `possibilities`
#' @param fall_back if there is none of the `possibilities` in `option_set`,
#' or there are multiple hits, default to `fall_back`
#' @return same as option_set, usually character
#' @keywords internal
.findAvailable = function(possibilities, option_set, fall_back = NULL) {
    chosen = option_set[option_set %in% possibilities]
    if (length(chosen) != 1L) {
        if (is.null(fall_back)) {
            NULL
        } else {
            if (fall_back %in% possibilities) {
                fall_back
            } else {
                NULL
            }
        }
    } else {
        chosen
    }
}


#' Generic parameter validation for all MSstats converters using configuration object
#' @param config A list containing all converter parameters. See details for required structure.
#' @details
#' The config list should contain the input and optionally other parameters:
#' - input: input data (required)
#' - annotation: annotation data (optional)
#' - intensity: intensity type (optional)
#' - filter_with_Qvalue: Q-value filter setting (default: FALSE)
#' - qvalue_cutoff: Q-value cutoff (default: 0.01)
#' - useUniquePeptide: unique peptide setting (default: TRUE)
#' - removeFewMeasurements: remove few measurements setting (default: TRUE)
#' - removeProtein_with1Feature: remove single feature proteins setting (default: FALSE)
#' - summaryforMultipleRows: aggregation function (default: max)
#' - calculateAnomalyScores: anomaly detection setting (default: FALSE)
#' - anomalyModelFeatures: anomaly model features (default: c())
#' - anomalyModelFeatureTemporal: temporal features (default: c())
#' - removeMissingFeatures: missing feature threshold (default: 0.5)
#' - anomalyModelFeatureCount: feature count for anomaly model (default: 100)
#' - runOrder: run order data (default: NULL)
#' - n_trees: number of trees (default: 100)
#' - max_depth: max tree depth (default: "auto")
#' - numberOfCores: number of cores (default: 1)
#' - use_log_file: logging setting (default: TRUE)
#' - append: append setting (default: FALSE)
#' - verbose: verbose setting (default: TRUE)
#' - log_file_path: log file path (default: NULL)
#' - excludedFromQuantificationFilter: filter setting (default: NULL)
#' @return NULL (throws error if validation fails)
#' @keywords internal
.validateMSstatsConverterParameters = function(config) {
    
    # Ensure config is a list
    if (!is.list(config)) {
        stop("Config must be a list")
    }
    
    # Define all defaults in one place
    defaults = list(
        input = NULL,
        annotation = NULL,
        intensity = NULL,
        excludedFromQuantificationFilter = NULL,
        filter_with_Qvalue = FALSE,
        qvalue_cutoff = 0.01,
        useUniquePeptide = TRUE,
        removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE,
        summaryforMultipleRows = max,
        calculateAnomalyScores = FALSE,
        anomalyModelFeatures = c(),
        anomalyModelFeatureTemporal = c(),
        removeMissingFeatures = 0.5,
        anomalyModelFeatureCount = 100,
        runOrder = NULL,
        n_trees = 100,
        max_depth = "auto",
        numberOfCores = 1,
        use_log_file = TRUE,
        append = FALSE,
        verbose = TRUE,
        log_file_path = NULL
    )
    
    # Merge config with defaults (config values override defaults)
    config = modifyList(defaults, config)
    
    # Input data validation (fail immediately if data is invalid)
    if (is.null(config$input)) {
        stop("Input data cannot be NULL")
    }
    
    if (is.character(config$input)) {
        if (!file.exists(config$input)) {
            stop("Input file does not exist: ", config$input)
        }
        # Quick file size check for very large files
        file_size_mb = file.size(config$input) / (1024^2)
        if (file_size_mb > 1000) {  # Warn for files > 1GB
            warning("Large input file detected (", round(file_size_mb, 1), " MB). ",
                    "Consider validating parameters on a subset first.")
        }
    } else if (is.data.frame(config$input) || data.table::is.data.table(config$input)) {
        # Quick structural validation
        if (nrow(config$input) == 0) {
            stop("Input data is empty (0 rows)")
        }
        if (ncol(config$input) == 0) {
            stop("Input data has no columns")
        }
    } else {
        stop("Input must be a file path, data.frame, or data.table")
    }
    
    # Annotation validation
    if (!is.null(config$annotation)) {
        if (is.character(config$annotation)) {
            if (!file.exists(config$annotation)) {
                stop("Annotation file does not exist: ", config$annotation)
            }
        } else if (!is.data.frame(config$annotation) && !data.table::is.data.table(config$annotation)) {
            stop("Annotation must be NULL, a file path, data.frame, or data.table")
        }
    }
    
    # Q-value filtering parameters
    checkmate::assertLogical(config$filter_with_Qvalue, len = 1)
    checkmate::assertNumber(config$qvalue_cutoff, lower = 0, upper = 1)
    
    # Common processing parameters
    checkmate::assertLogical(config$useUniquePeptide, len = 1)
    checkmate::assertLogical(config$removeFewMeasurements, len = 1)
    checkmate::assertLogical(config$removeProtein_with1Feature, len = 1)
    
    # Aggregation function validation
    if (!is.null(config$summaryforMultipleRows)) {
        checkmate::assertFunction(config$summaryforMultipleRows)
    }
    
    # Core system parameters
    checkmate::assertInt(config$numberOfCores, lower = 1)
    checkmate::assertLogical(config$use_log_file, len = 1)
    checkmate::assertLogical(config$append, len = 1)
    checkmate::assertLogical(config$verbose, len = 1)
    
    # Converter-specific boolean parameters (if provided)
    if (!is.null(config$excludedFromQuantificationFilter)) {
        checkmate::assertLogical(config$excludedFromQuantificationFilter, len = 1)
    }
    
    checkmate::assertLogical(config$calculateAnomalyScores, len = 1)
    
    # Anomaly detection parameter validation (converter-specific)
    if (config$calculateAnomalyScores) {
        # These validations only matter if anomaly detection is enabled
        if (length(config$anomalyModelFeatures) == 0) {
            stop("anomalyModelFeatures cannot be empty when calculateAnomalyScores=TRUE")
        }
        checkmate::assertCharacter(config$anomalyModelFeatures, min.len = 1)
        
        if (length(config$anomalyModelFeatureTemporal) > 0) {
            if (length(config$anomalyModelFeatureTemporal) != length(config$anomalyModelFeatures)) {
                stop("anomalyModelFeatureTemporal must have same length as anomalyModelFeatures or be empty")
            }
            valid_temporal = c("mean_decrease", "mean_increase", "dispersion_increase")
            invalid_temporal = config$anomalyModelFeatureTemporal[
                !is.null(config$anomalyModelFeatureTemporal) & 
                    !config$anomalyModelFeatureTemporal %in% valid_temporal
            ]
            if (length(invalid_temporal) > 0) {
                stop("Invalid temporal directions: ", paste(invalid_temporal, collapse = ", "),
                     ". Must be one of: ", paste(valid_temporal, collapse = ", "), " or NULL")
            }
        }
        
        checkmate::assertInt(config$n_trees, lower = 1)
        if (is.character(config$max_depth)) {
            checkmate::assertChoice(config$max_depth, choices = "auto")
        } else {
            checkmate::assertInt(config$max_depth, lower = 1)
        }
        checkmate::assertInt(config$anomalyModelFeatureCount, lower = 1)
        checkmate::assertNumber(config$removeMissingFeatures, lower = 0, upper = 1)
        
        if (!is.null(config$runOrder)) {
            if (!is.data.frame(config$runOrder) && !data.table::is.data.table(config$runOrder)) {
                stop("runOrder must be a data.frame or data.table")
            }
            required_cols = c("Run", "Order")
            missing_cols = setdiff(required_cols, colnames(config$runOrder))
            if (length(missing_cols) > 0) {
                stop("runOrder is missing required columns: ", paste(missing_cols, collapse = ", "))
            }
            if (!is.numeric(config$runOrder$Order)) {
                stop("runOrder$Order must be numeric")
            }
        }
    } else {
        # When anomaly detection is disabled, these parameters are ignored but warn user
        if (length(config$anomalyModelFeatures) > 0) {
            warning("anomalyModelFeatures provided but calculateAnomalyScores=FALSE, ignoring")
        }
        if (!is.null(config$runOrder)) {
            warning("runOrder provided but calculateAnomalyScores=FALSE, ignoring")
        }
    }
    
    # Log file validation
    if (!is.null(config$log_file_path)) {
        checkmate::assertString(config$log_file_path)
        log_dir = dirname(config$log_file_path)
        if (!dir.exists(log_dir)) {
            stop("Log file directory does not exist: ", log_dir)
        }
        if (!file.access(log_dir, mode = 2) == 0) {
            stop("No write permission for log file directory: ", log_dir)
        }
    }
}

#' Clean raw OpenSWATH files
#' @param msstats_object an object of class `MSstatsSpectroMineFiles`.
#' @return data.table
#' @keywords internal
.cleanRawOpenSWATH = function(msstats_object) {
  PeptideSequence = FragmentIon = Intensity = NULL
  
  os_input = getInputFile(msstats_object, "input")
  os_input = os_input[, c("ProteinName", "FullPeptideName", "Charge", 
                          "filename", "aggr_Fragment_Annotation", 
                          "aggr_Peak_Area", "m_score", "decoy"), with = FALSE]
  data.table::setnames(
    os_input,
    c("FullPeptideName", "Charge", "filename", 
      "aggr_Fragment_Annotation", "aggr_Peak_Area"),
    c("PeptideSequence", "PrecursorCharge", "Run", 
      "FragmentIon", "Intensity"),
    skip_absent = TRUE)
  os_input = os_input[, c("ProteinName", "PeptideSequence", "PrecursorCharge", 
                          "Run", "FragmentIon", "Intensity",
                          "m_score", "decoy"), with = FALSE]
  os_input$Intensity = as.character(os_input$Intensity)
  os_input = os_input[
    , lapply(.SD, 
             function(x) unlist(tstrsplit(x, split = ";", fixed = TRUE))),
    by = c("ProteinName", "PeptideSequence", "PrecursorCharge", 
           "Run", "m_score", "decoy"),
    .SDcols = c("FragmentIon", "Intensity")]
  os_input[, c("PeptideSequence", "FragmentIon") := lapply(
    list(PeptideSequence, FragmentIon), function(x) gsub(":", "_", x)
  )]
  os_input[, Intensity := as.numeric(as.character(Intensity))]
  os_input[, Intensity := ifelse(Intensity < 1, NA, Intensity)]
  .logSuccess("OpenSWATH", "clean")
  os_input
}

#' Handle shared peptides.
#' @inheritParams .removeSharedPeptides
#' @param remove_shared lgl, if TRUE, shared peptides will be removed
#' @return data.table
#' @keywords internal
.handleSharedPeptides = function(input, remove_shared = TRUE,
                                 protein_column = "ProteinName",
                                 peptide_column = "PeptideSequence") {
    if (remove_shared) {
        input = .removeSharedPeptides(input, protein_column, peptide_column)
        getOption("MSstatsLog")("INFO", "** Shared peptides are removed.")
        getOption("MSstatsMsg")("INFO", "** Shared peptides are removed.")
    }
    input
}


#' Remove peptides assigned to more than one protein.
#' @param input data.table pre-processed by one of the .cleanRaw* functions.
#' @param protein_column chr, name of the column with names of proteins.
#' @param peptide_column chr, name of the column with peptide sequences.
#' @return data.table
#' @keywords internal
.removeSharedPeptides = function(input, protein_column, peptide_column) {
    count = NULL

    unique_pairs = unique(input[, c(protein_column, peptide_column),
                                with = FALSE])
    protein_counts = unique_pairs[, list(count = .N), by = peptide_column]
    protein_counts = unique(protein_counts[count == 1L, peptide_column,
                                           with = FALSE])
    merge(input, protein_counts, sort = FALSE, by = peptide_column)[, colnames(input), with = FALSE]
}

data_with_shared = data.table::data.table(
    ProteinName = c("A", "A", "B", "B", "B", 
                    "C", "D", "D", "D", "D"),
    PeptideSequence = c("P1", "P2", "P1", "P3", "P4",
                        "P5", "P6", "P3", "P7", "P8"))
data_no_shared = data.table::data.table(ProteinName = letters[1:10],
                                        PeptideSequence = letters[1:10])

remove_shared = MSstatsConvert:::.handleSharedPeptides(data_with_shared, TRUE)
dont_remove_shared = MSstatsConvert:::.handleSharedPeptides(data_with_shared, FALSE)

no_shared = MSstatsConvert:::.handleSharedPeptides(data_no_shared)

# Correct output class
expect_equal(class(remove_shared)[1], "data.table")
# Nothing changes if remove = FALSE
expect_identical(data_with_shared, dont_remove_shared)
# Nothing changes if there are no shared peptides
expect_identical(no_shared, data_no_shared)
# Shared peptides are removed
expect_equal(nrow(remove_shared), 6L)
expect_true(all(!(c("P1", "P3") %in% remove_shared$PeptideSequence)))

#' MSstatsConvert: An R Package to Convert Data from Mass Spectrometry Signal Processing Tools to MSstats Format
#'
#' MSstatsConvert helps convert data from different types of mass spectrometry experiments and
#' signal processing tools to a format suitable for statistical analysis with the MSstats and MSstatsTMT packages.
#'
#' @section Main functions:
#' \code{\link{MSstatsLogsSettings}} for logs management,
#' \code{\link{MSstatsImport}} for importing files created by signal processing tools,
#' \code{\link{MSstatsClean}} for re-formatting imported files into a consistent format,
#' \code{\link{MSstatsPreprocess}} for preprocessing cleaned files,
#' \code{\link{MSstatsBalancedDesign}} for handling fractions and creating balanced data.
#'
#' @import data.table
#' @name MSstatsConvert
#' @keywords internal
"_PACKAGE"

#' Check if there are overlapping features and remove if needed
#' @param input data.table preprocessed by one of the .cleanRaw* functions and
#' merged with annotation.
#' @return `data.table`
#' @keywords internal
.handleFractions = function(input) {
    if (is.element("Channel", colnames(input))) {
        fractions = unique(input$Fraction)
        if (length(fractions) > 1) {
            input = .handleFractionsTMT(input)
            msg = "** Fractions belonging to same mixture have been combined."
            getOption("MSstatsLog")("INFO", msg)
            getOption("MSstatsMsg")("INFO", msg)
        } else {
            input$Fraction = 1
        }
    } else {
        input = .handleFractionsLF(input)
    }
    input
}

#' Remove peptide ions overlapped among multiple fractions of the same biological mixture
#' @inheritParams .handleFractions
#' @return `data.table`
#' @keywords internal
.handleFractionsTMT = function(input) {
    techrun = feature = id = Mixture = TechRepMixture = Run = Intensity = NULL

    input[, techrun := paste(Mixture, TechRepMixture, sep = "_")]
    input[, id := paste(feature, Run, sep = "_")]
    tmt_cols = c("ProteinName", "PeptideSequence", "PrecursorCharge",
                 "Charge", "PSM", "feature", "Mixture",
                 "TechRepMixture", "Channel", "Condition", "BioReplicate")
    tmt_cols = intersect(tmt_cols, colnames(input))
    
    unoverlapped_list = vector("list", length(unique(input$techrun)))
    names(unoverlapped_list) = unique(input$techrun)
    for (technical_run in unique(input$techrun)) {
        single_run = input[techrun == technical_run & !is.na(Intensity), ]
        features_to_remove = .getOverlappingFeatures(single_run)
        if (length(features_to_remove) > 0) {
            single_run = .filterOverlapped(single_run, mean, 
                                           features_to_remove)
            features_to_remove = .getOverlappingFeatures(single_run)
            msg = paste("** For peptides overlapped between fractions of",
                        technical_run, 
                        "use the fraction with maximal average abundance.")
            getOption("MSstatsLog")("INFO", msg)
            getOption("MSstatsMsg")("INFO", msg)
            if (length(features_to_remove) > 0) {
                single_run = .filterOverlapped(single_run, sum, 
                                               features_to_remove)
                features_to_remove = .getOverlappingFeatures(single_run)
                msg = paste("** For peptides overlapped between fractions of",
                            technical_run,
                            "use the fraction with maximal summation abundance.")
                getOption("MSstatsLog")("INFO", msg)
                getOption("MSstatsMsg")("INFO", msg)
                if (length(features_to_remove) > 0) {
                    single_run = .filterOverlapped(single_run, max, 
                                                   features_to_remove)
                    msg = paste("** For peptides overlapped between fractions of",
                                technical_run,
                                "use the fraction with maximal abundance.")
                    getOption("MSstatsLog")("INFO", msg)
                    getOption("MSstatsMsg")("INFO", msg)
                    features_to_remove = .getOverlappingFeatures(single_run)
                    if (length(features_to_remove) > 0) {
                        single_run = single_run[
                            , list(Intensity = mean(Intensity, na.rm = TRUE)),
                            by = tmt_cols
                        ]
                    }
                }
            }
        }
        unoverlapped_list[[technical_run]] = single_run[, 
                                                        c(tmt_cols, "Intensity"), 
                                                        with = FALSE]
    }
    input = rbindlist(unoverlapped_list, use.names = TRUE)
    input[, Run := paste(Mixture, TechRepMixture, sep = "_")]
    input
}


#' Get features that are overlapped among multiple runs
#' @inheritParams .handleFractions
#' @return `data.table`
#' @keywords internal
.getOverlappingFeatures = function(input) {
    Run = feature = n_runs = NULL
    
    count_fractions = input[, list(n_runs = uniqueN(Run)),
                            by = "feature"]
    count_fractions[n_runs > 1, feature]
}


#' Remove overlapped features
#' @inheritParams .handleFractions
#' @param summary_function summary function (mean, sum, max) that will be used
#' to pick one feature from multiple overlapping features
#' @param overlapped_features features that overlap.
#' @return `data.table`
#' @keywords internal
.filterOverlapped = function(input, summary_function, overlapped_features) {
    Intensity = id = agg_intensity = max_intensity = feature =  NULL
    
    overlapped = input[feature %in% overlapped_features]
    overlapped[, agg_intensity := summary_function(Intensity, na.rm = TRUE),
               by = c("feature", "id")]
    overlapped[, max_intensity := max(agg_intensity),
               by = "feature"]
    overlapped = overlapped[agg_intensity != max_intensity]
    input[!(id %in% overlapped$id), 
          !(colnames(input) %in% c("agg_intensity", "max_intensity")), 
          with = FALSE]
}

#' Get common values from two vectors of features
#' @param features_1 vector of feature names
#' @param features_2 vector of feature_names
#' @return character vector of common values of `features_1` and `features_2`
#' @keywords internal
.countCommonFeatures = function(features_1, features_2) {
    data.table::uniqueN(intersect(as.character(features_1), 
                                  as.character(features_2)))
}


#' Handle overlapping features
#' @param input output of `MSstatsPreprocess`
#' @return data.table
#' @keywords internal
.handleFractionsLF = function(input) {
    check_multi_run = .checkMultiRun(input)
    
    if (check_multi_run$is_risky) {
        msg = paste("** MSstats suspects that there are fractionations and",
                    "potentially technical replicates too.",
                    "Please add Fraction column to the input.")
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    } else if (check_multi_run$has_fractions) {
        if (!is.element("Fraction", colnames(input))) {
            input = .addFractions(input)
            has_missing_fractions = any(is.na(input$Fraction))
            if (has_missing_fractions) {
                msg = paste("** It is hard to find the same fractionation across sample,",
                            "due to lots of overlapped features between fractionations.",
                            "Please add Fraction column in input.")
                getOption("MSstatsLog")("ERROR", msg)
                stop(msg)
            }
        }
        n_fractions = data.table::uniqueN(input$Fraction)
        if (n_fractions > 1) {
            msg = paste("** Multiple fractionations exist:",
                        n_fractions, "fractionations per MS replicate.")
            getOption("MSstatsLog")("INFO", msg)
            getOption("MSstatsMsg")("INFO", msg)
        }
        
        input = .removeOverlappingFeatures(input)
        input = .checkOverlappedFeatures(input)
    } else {
        input$Fraction = 1L
    }
    input
}


#' Check if fractionation exists
#' @param input output of `MSstatsPreprocess`
#' @return list of two elements: `has_fractions` (logical) indicates if 
#' fractions was detected in the `input` dataset, `is_risky` (logical)
#' indicates if there was a problem with detecting fractionation.
#' @keywords internal
.checkMultiRun = function(input) {
    Run = Condition = BioReplicate = Intensity = NULL
    feature = condition = n_obs = NULL
    
    if (is.element("Fraction", colnames(input))) {
        return(list(has_fractions = TRUE, is_risky = FALSE))
    } else {
        count_techreps = input[, list(n_techreps = data.table::uniqueN(Run)),
                               by = c("Condition", "BioReplicate")]
        if (all(count_techreps$n_techreps == 1)) {
            has_fractions = FALSE
            is_risky = FALSE
        } else {
            ## no fraction information, but multiple ms runs per sample
            ## need to distinguish they are tech replicates or we miss fraction information
            info = unique(input[, list(Condition, BioReplicate, Run)])
            info[, condition := paste(Condition, BioReplicate, sep = "_")]
            
            input[, condition := paste(Condition, BioReplicate, sep = "_")]
            measurement_count = input[, n_obs := sum(!is.na(Intensity) & 
                                                         Intensity > 1, 
                                                     na.rm = TRUE),
                                      by = condition]
            max_measurements = which.max(measurement_count$n_obs)[1]
            ref = measurement_count$condition[max_measurements]
            
            single_sample = unique(info[condition == ref,
                                        list(Condition, BioReplicate)])
            single_sample_data = input[!is.na(Intensity) & ## including Intensity < 1 ??
                                           Condition == single_sample$Condition &
                                           BioReplicate == single_sample$BioReplicate]
            single_run_features = unique(single_sample_data[Run == unique(Run)[1], 
                                                            as.character(feature)])
            common = single_sample_data[, list(n_common = .countCommonFeatures(feature, single_run_features)), 
                                        by = "Run"]
            common$fraction = common$n_common / max(common$n_common)
            overlap = common$fraction[-1]
            
            if (all(overlap > 0.5)) {
                has_fractions = FALSE
                is_risky = FALSE
            } else if (all(overlap < 0.5)) {
                has_fractions = TRUE
                is_risky = FALSE
            } else {
                has_fractions = FALSE
                is_risky = TRUE
            }
            
        }
    }
    list(has_fractions = has_fractions,
         is_risky = is_risky)
}


#' Add a Fraction column to the output of `MSstatsPreprocess`
#' @param input output of `MSstatsPreprocess`
#' @return data.table
#' @keywords internal
.addFractions = function(input) {
    Condition = BioReplicate = Run = CONDITION = Intensity = feature = Fraction = NULL
    
    input$Fraction = as.character(NA)
    run_info = unique(input[, list(Condition, BioReplicate, Run,
                                   CONDITION = paste(Condition, BioReplicate, sep = "_"))])
    single_condition = run_info[CONDITION == unique(CONDITION)[1], ]
    single_condition$Fraction = 1:nrow(single_condition)
    for (run_id in seq_along(single_condition$Run)) {
        features_in_run = as.character(unique(input[!is.na(Intensity) & Run == single_condition$Run[run_id], feature]))
        same_features = input[!is.na(Intensity) & feature %in% features_in_run, 
                              list(Condition, BioReplicate, Run, Intensity)]
        count_features = data.table::dcast(same_features, Run ~ Condition + BioReplicate,
                                           fun.aggregate = length, value.var = "Intensity")
        same_fraction = apply(count_features[, -1], 2, function(x) as.character(count_features[which.max(x), "Run"]))
        input[Run %in% same_fraction, "Fraction"] = single_condition[Run == single_condition$Run[run_id], Fraction]
    }
    input
}


#' Replace intensities of overlapped fractions with NA, keeping only one fraction
#' @param input output of `MSstatsPreprocess`
#' @return data.table
#' @keywords internal
.removeOverlappingFeatures = function(input) {
    Fraction = NULL

    if (data.table::uniqueN(input$Fraction) > 1) {
        measurement_count = input[
            (IsotopeLabelType == "L" | is.na(IsotopeLabelType)) &
            !is.na(Intensity) & Intensity > 0,
            .(n_obs = uniqueN(Run)),
            by = .(feature, Fraction)
        ]
        # Features absent from measurement_count have no L or NA observations.
        # Fall back to H observations so they are not silently dropped.
        h_only_features = setdiff(unique(input$feature), unique(measurement_count$feature))
        if (length(h_only_features) > 0) {
            h_count = input[
                feature %in% h_only_features &
                    IsotopeLabelType == "H" &
                    !is.na(Intensity) & Intensity > 0,
                .(n_obs = uniqueN(Run)),
                by = .(feature, Fraction)
            ]
            measurement_count = rbind(measurement_count, h_count)
        }
        measurement_count[, is_max := n_obs == max(n_obs), by = "feature"]
        max_fractions = measurement_count[(is_max)]

        fraction_map = .resolveFractionTies(input, max_fractions)
        input = input[fraction_map, on = .(feature, Fraction), nomatch = 0]
    }
    input
}


#' Resolve ties when multiple fractions share the maximum number of measurements
#' for a given feature. In the case of a tie, the fraction with the highest
#' mean intensity is selected.
#' @param input output of `MSstatsPreprocess`
#' @param max_fractions data.table of fractions that share the maximum number
#' of unique runs per feature, as produced by `.removeOverlappingFeatures`
#' @return data.table with columns `feature` and `Fraction`, containing one
#' selected fraction per feature
#' @keywords internal
.resolveFractionTies = function(input, max_fractions) {
    tie_features = max_fractions[, .(n_ties = .N), by = "feature"][n_ties > 1, feature]

    if (length(tie_features) > 0) {
        tied_fractions = max_fractions[feature %in% tie_features, .(feature, Fraction)]
        tied_input = input[tied_fractions, on = .(feature, Fraction), nomatch = 0]
        avg_abundance = tied_input[
            (IsotopeLabelType == "L" | is.na(IsotopeLabelType)) &
                !is.na(Intensity) & Intensity > 0,
            .(mean_abundance = mean(Intensity)),
            by = .(feature, Fraction)
        ]
        # H-only tied features have no L or NA rows; fall back to H for tie-breaking.
        h_only_tied = setdiff(tie_features, unique(avg_abundance$feature))
        if (length(h_only_tied) > 0) {
            h_avg = tied_input[
                feature %in% h_only_tied &
                    IsotopeLabelType == "H" &
                    !is.na(Intensity) & Intensity > 0,
                .(mean_abundance = mean(Intensity)),
                by = .(feature, Fraction)
            ]
            avg_abundance = rbind(avg_abundance, h_avg)
        }
        best_tied = avg_abundance[, .SD[which.max(mean_abundance)], by = "feature"]
        best_simple = max_fractions[
            !feature %in% tie_features,
            .(Fraction = Fraction[1]),
            by = "feature"
        ]
        rbind(best_simple[, .(feature, Fraction)],
              best_tied[, .(feature, Fraction)])
    } else {
        max_fractions[, .(Fraction = Fraction[1]), by = "feature"]
    }
}


#' Check if any features are measured in multiple fractions
#' @param input output of `MSstatsPreprocess`
#' @return data.table
#' @keywords internal
.checkOverlappedFeatures = function(input) {
    n_fractions = Fraction = Intensity = NULL 
    
    count_fractions = input[!is.na(Intensity), 
                            list(n_fractions = data.table::uniqueN(Fraction)),
                            by = "feature"]
    count_fractions = count_fractions[n_fractions > 1, ]
    if (nrow(count_fractions) > 0) {
        overlapped_features = unique(as.character(count_fractions$feature))
        msg = paste("** These features are measured across all fractionations.",
                    "Please keep only one intensity of listed features", 
                    "among fractionations from one sample.\n",
                    paste(overlapped_features, sep = ", ", collapse = ", "))
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    } else {
        input    
    }
}

# Data for testing fractions ----
## Simplest case:
data_for_fractions = data.table::data.table(
    ProteinName = "A",
    PeptideSequence = rep(rep(c("A", "B", "C", "D"), times = 1:4), each = 2),
    PrecursorCharge = 3,
    PSM = paste(rep(rep(c("A", "B", "C", "D"), times = 1:4), each = 2), 3, sep = "_"),
    Mixture = 1,
    TechRepMixture = 1,
    Run = as.character(rep(1:10, each = 2)),
    Channel = rep(c(1, 2), times = 2),
    Intensity = c(1, 1, 1, 1, 2, 2, 1, NA, 1, NA, 0, 2, 1, NA, 1, NA, 1, NA, -0.1, 1.1),
    Fraction = c(1, 1, 1, 1, 2, 2, 1, 1, 2, 2, 3, 3, 1, 1, 2, 2, 3, 3, 4, 4)
)
# Tests ----
## No fractions - do nothing
expect_identical(
    MSstatsConvert:::.handleFractions(data_for_fractions[1:2, ]),
    data_for_fractions[1:2, ]
)
## We find the right features
data_with_additional_cols = data_for_fractions
data_with_additional_cols$techrun = paste(data_with_additional_cols$Mixture, 
                                          data_with_additional_cols$TechRepMixture, 
                                          sep = "_")
.combine = MSstatsConvert:::.combine
data_with_additional_cols$feature = do.call(".combine", list(data_with_additional_cols$ProteinName,
                                                             data_with_additional_cols$PeptideSequence,
                                                             data_with_additional_cols$PrecursorCharge))
data_with_additional_cols$id = paste(data_with_additional_cols$feature, 
                                     data_with_additional_cols$Run, sep = "_")
expect_equal(
    MSstatsConvert:::.getOverlappingFeatures(data_with_additional_cols),
    c("A_B_3", "A_C_3", "A_D_3")
)
## Non-overlapped feature is not considered
expect_equal(
    MSstatsConvert:::.filterOverlapped(data_with_additional_cols, summary_function = mean,
                                       MSstatsConvert:::.getOverlappingFeatures(data_with_additional_cols)),
    data_with_additional_cols[-(c(3:4, 19:20)), ]
)
## Overlap check
### Fails, when overlapped
expect_error(
    MSstatsConvert:::.checkOverlappedFeatures(
        data.table::data.table(
            feature = "A",
            Fraction = c(1, 2, 3),
            Intensity = c(NA, 1, 2)
        )
    )
)
### OK, when not overlapped
fractions_ok = data.table::data.table(
    feature = "A",
    Fraction = c(1, 2, 2),
    Intensity = c(NA, 1, 2)
)
expect_identical(
    MSstatsConvert:::.checkOverlappedFeatures(fractions_ok),
    fractions_ok
)
## 
fractionated = data.table::data.table(
    feature = rep(c("A", "B"), each = 6),
    Fraction = rep(rep(c(1, 2), each = 3), times = 2),
    Run = 1:12,
    IsotopeLabelType = "L",
    Intensity = c(NA, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2)
)
### More observations win
expect_equal(
    unique(MSstatsConvert:::.removeOverlappingFeatures(fractionated[feature == "A"])$Fraction),
    2
)
### Higher average intensity wins on tie
expect_equal(
    unique(MSstatsConvert:::.removeOverlappingFeatures(fractionated[feature == "B"])$Fraction),
    2
)
### For full data
expect_identical(
    MSstatsConvert:::.removeOverlappingFeatures(data.table::copy(fractionated)),
    fractionated[Fraction == 2]
)
### Non-tied fraction with high mean intensity should not be selected over tied fractions
fractionated_third = data.table::data.table(
    feature = rep("A", 9),
    Fraction = c(rep(1, 3), rep(2, 3), rep(3, 3)),
    Run = 1:9,
    IsotopeLabelType = "L",
    Intensity = c(1, 1, 1,    # Fraction 1: 3 obs, mean = 1 (ties for max n_obs)
                  2, 2, 2,    # Fraction 2: 3 obs, mean = 2 (ties for max n_obs)
                  10, NA, NA) # Fraction 3: 1 obs, mean = 10 (loses on n_obs but mean would win w/o fix)
)
expect_equal(
    unique(MSstatsConvert:::.removeOverlappingFeatures(fractionated_third)$Fraction),
    2
)
### L/H isotope label types: fraction selection uses only L obs, output keeps both L and H;
### NA-only features: fraction selection uses NA obs
# Feature "A" (L+H): F1 has 1 L obs (run 1) + 4 H obs (runs 1-4) = 4 unique runs total
#                    F2 has 3 L obs (runs 5-7) + 1 H obs (run 5)  = 3 unique runs total
#   → If counting all unique runs: F1 wins (4 > 3)
#   → If counting L unique runs only: F2 wins (3 > 1)
#   → Expected: F2 selected, confirming H obs do not influence fraction selection
#   → Both L and H rows from F2 are returned
# Feature "B" (NA only): F1 has 2 NA obs (runs 8-9), F2 has 4 NA obs (runs 10-13)
#   → NA obs: F1=2, F2=4 → F2 wins; all 4 F2 rows are kept
fractionated_lh = data.table::data.table(
    feature = c(rep("A", 9), rep("B", 6)),
    Fraction = c(rep(1, 5), rep(2, 4),
                 rep(1, 2), rep(2, 4)),
    Run = c(1, 1, 2, 3, 4,    # A F1: run 1 paired L+H, runs 2-4 H only
            5, 5, 6, 7,        # A F2: run 5 paired L+H, runs 6-7 L only
            8, 9, 10, 11, 12, 13),
    IsotopeLabelType = c("L", "H", "H", "H", "H",   # A F1: 1 L, 4 H
                         "L", "H", "L", "L",          # A F2: 3 L, 1 H
                         rep(NA_character_, 6)),
    Intensity = rep(1, 15)
)
### Feature A: F1 has more total obs but fewer L obs — F2 must win to confirm L-only logic
expect_equal(
    unique(MSstatsConvert:::.removeOverlappingFeatures(fractionated_lh[feature == "A"])$Fraction),
    2
)
### Both L and H rows are returned for the winning fraction
expect_true(
    all(c("L", "H") %in% MSstatsConvert:::.removeOverlappingFeatures(fractionated_lh[feature == "A"])$IsotopeLabelType)
)
### Feature B (NA only) selected via NA obs count
expect_equal(
    unique(MSstatsConvert:::.removeOverlappingFeatures(fractionated_lh[feature == "B"])$Fraction),
    2
)
### Full dataset: A keeps F2 (L+H rows), B keeps F2 (NA rows)
expect_identical(
    MSstatsConvert:::.removeOverlappingFeatures(data.table::copy(fractionated_lh)),
    fractionated_lh[(feature == "A" & Fraction == 2) | (feature == "B" & Fraction == 2)]
)
### H-only feature: falls back to H obs for fraction selection (not silently dropped)
# F1 has 1 H obs, F2 has 3 H obs → F2 wins
fractionated_h_only = data.table::data.table(
    feature = rep("A", 4),
    Fraction = c(1, 2, 2, 2),
    Run = 1:4,
    IsotopeLabelType = "H",
    Intensity = 1
)
expect_equal(
    unique(MSstatsConvert:::.removeOverlappingFeatures(fractionated_h_only)$Fraction),
    2
)
### H-only feature with tied obs count: higher mean H intensity breaks the tie
# F1: 2 obs, mean intensity 1; F2: 2 obs, mean intensity 3 → F2 wins
fractionated_h_tied = data.table::data.table(
    feature = rep("A", 4),
    Fraction = rep(c(1, 2), each = 2),
    Run = 1:4,
    IsotopeLabelType = "H",
    Intensity = c(1, 1, 3, 3)
)
expect_equal(
    unique(MSstatsConvert:::.removeOverlappingFeatures(fractionated_h_tied)$Fraction),
    2
)
fractionated_tmt = fractionated = data.table::data.table(
    feature = rep(c("A", "B"), each = 6),
    Fraction = rep(rep(c(1, 2), each = 3), times = 2),
    Run = rep(rep(c(1, 2), each = 3), times = 2),
    Intensity = c(NA, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2),
    Channel = rep(1:3, times = 4),
    Mixture = rep(c(1, 2), each = 6),
    TechRepMixture = 1
)
tmt_unoverlapped = MSstatsConvert:::.handleFractionsTMT(data.table::copy(fractionated_tmt))
expect_identical(
    tmt_unoverlapped[, .(Mixture, TechRepMixture, Channel, feature, Intensity)],
    fractionated_tmt[Fraction == 2, .(Mixture, TechRepMixture, Channel, feature, Intensity)]
)

#' Import OpenMS files
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param input name of MSstats input report from OpenMS, which includes feature(peptide ion)-level data.
#' @param annotation name of 'annotation.txt' data which includes Condition, BioReplicate, Run. 
#' Run should be the same as filename.
#' @param ... additional parameters to `data.table::fread`.
#' 
#' @return data.frame in the MSstats required format.
#' 
#' @author Meena Choi, Olga Vitek. 
#' 
#' @export
#' 
#' 
#' @examples 
#' openms_raw = data.table::fread(system.file("tinytest/raw_data/OpenMS/openms_input.csv", 
#'                                            package = "MSstatsConvert"))
#' openms_imported = OpenMStoMSstatsFormat(openms_raw, use_log_file = FALSE)
#' head(openms_imported)
#' 
OpenMStoMSstatsFormat = function(
        input, annotation = NULL, useUniquePeptide = TRUE, removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE, summaryforMultipleRows = max,
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "OpenMS", ...)
    input = MSstatsConvert::MSstatsClean(input)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge", 
                        "FragmentIon", "ProductCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = summaryforMultipleRows))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  remove_few = removeFewMeasurements)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

#' Generate MSstatsTMT required input format from Protein Prospector output
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param input Input txt peptide report file from Protein Prospector with 
#' "Keep Replicates", "Mods in Peptide", and "Protein Mods" options selected.
#' @param annotation data frame which contains column Run, Fraction, 
#' TechRepMixture, Mixture, Channel, BioReplicate, Condition. 
#' 
#' @return data.frame of class "MSstatsTMT"
#' 
#' @export
#' 
#' @examples
#' input = system.file("tinytest/raw_data/ProteinProspector/Prospector_TotalTMT.txt",
#'     package = "MSstatsConvert")
#' input = data.table::fread(input)
#' annot = system.file("tinytest/raw_data/ProteinProspector/Annotation.csv",
#'                                 package = "MSstatsConvert")
#' annot = data.table::fread(annot)
#' output <- ProteinProspectortoMSstatsTMTFormat(input, annot)
#' head(output)
#' 
ProteinProspectortoMSstatsTMTFormat = function(
        input, annotation, useUniquePeptide = TRUE, removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE, summaryforMultipleRows = sum,
        use_log_file = TRUE, append = FALSE, verbose = TRUE, 
        log_file_path = NULL
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsTMT_converter_log_")
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstatsTMT", "ProteinProspector")
    input = MSstatsConvert::MSstatsClean(input)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = summaryforMultipleRows)
        )
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  fix_missing = "zero_to_na")
    data.table::setnames(input, "PrecursorCharge", "Charge", skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the proteinSummarization function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test ProteinProspectortoMSstatsTMTFormat ---------------------------
input = data.table::fread(system.file("tinytest/raw_data/ProteinProspector/Prospector_TotalTMT.txt",
                                      package = "MSstatsConvert"))
annot = data.table::fread(system.file("tinytest/raw_data/ProteinProspector/Annotation.csv",
                                      package = "MSstatsConvert"))
output = ProteinProspectortoMSstatsTMTFormat(input, annot, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 528)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("Charge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("TechRepMixture" %in% colnames(output))
expect_true("PSM" %in% colnames(output))
expect_true("Mixture" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Channel" %in% colnames(output))

# Test ProteinProspectortoMSstatsTMTFormat with missing value ------------
zero_value_entry = 
    output[output$PeptideSequence == "DINKVAEDLESEGLMAEEVQAVQQQEVYGAMPR" 
       & output$BioReplicate == "S1",]$Intensity
expect_true(is.na(zero_value_entry))

# Verify output intensities are present in input
intensity_cols = grep("^Int ", colnames(input), value = TRUE)
input_intensities = as.character(unlist(input[, intensity_cols, with = FALSE]))
output_intensities = as.character(output$Intensity[!is.na(output$Intensity)])
expect_true(all(output_intensities %in% input_intensities))

#' Import OpenSWATH files
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param input name of MSstats input report from OpenSWATH, which includes feature-level data.
#' @param annotation name of 'annotation.txt' data which includes Condition, BioReplicate, Run. 
#' Run should be the same as filename.
#' @param filter_with_mscore TRUE(default) will filter out the features that have greater than mscore_cutoff in m_score column. Those features will be removed.
#' @param mscore_cutoff Cutoff for m_score. Default is 0.01.
#' @param ... additional parameters to `data.table::fread`.
#'  
#' @return data.frame in the MSstats required format.
#' 
#' @author Meena Choi, Olga Vitek. 
#' 
#' @export
#' 
#' @examples 
#' os_raw = system.file("tinytest/raw_data/OpenSWATH/openswath_input.csv", 
#'                              package = "MSstatsConvert")
#' annot = system.file("tinytest/raw_data/OpenSWATH/annot_os.csv", 
#'                     package = "MSstatsConvert")
#' os_raw = data.table::fread(os_raw) 
#' annot = data.table::fread(annot)
#' 
#' os_imported = OpenSWATHtoMSstatsFormat(os_raw, annot, use_log_file = FALSE)
#' head(os_imported)
#' 
OpenSWATHtoMSstatsFormat = function(
        input, annotation, filter_with_mscore = TRUE, mscore_cutoff = 0.01,
        useUniquePeptide = TRUE, removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE, summaryforMultipleRows = max, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "OpenSWATH", ...)
    input = MSstatsConvert::MSstatsClean(input)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    m_score_filter = list(score_column = "m_score", 
                          score_threshold = mscore_cutoff, 
                          direction = "smaller", 
                          behavior = "remove", 
                          handle_na = "remove", 
                          fill_value = NA,
                          filter = filter_with_mscore, 
                          drop_column = TRUE)
    
    decoy_filter = list(col_name = "decoy", 
                        filter_symbols = 1, 
                        behavior = "remove",
                        fill_value = NULL,
                        filter = TRUE, 
                        drop_column = TRUE)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge", "FragmentIon")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = summaryforMultipleRows),
        score_filtering = list(ms_filter = m_score_filter),
        exact_filtering = list(decoy = decoy_filter),
        columns_to_fill = c("ProductCharge" = NA, 
                            "IsotopeLabelType" = "L"))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns, 
                                                  fix_missing = "na_to_zero",
                                                  remove_few = removeFewMeasurements)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test OpenSWATHtoMSstatsFormat ---------------------------
os_raw = system.file("tinytest/raw_data/OpenSWATH/openswath_input.csv",
                             package = "MSstatsConvert")
annot = system.file("tinytest/raw_data/OpenSWATH/annot_os.csv",
                    package = "MSstatsConvert")
os_raw = data.table::fread(os_raw)
annot = data.table::fread(annot)

output = OpenSWATHtoMSstatsFormat(os_raw, annot, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 96)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))

#' Prepare data for anomaly model, including feature engineering
#' @noRd
.prepareSpectronautAnomalyInput = function(input,
                                           quality_metrics,
                                           run_order,
                                           n_feat=100,
                                           missing_run_count=.5){
    input = as.data.table(input)
    
    input$Fragment = paste(input$PeptideSequence,
                           input$PrecursorCharge, 
                           input$FragmentIon,
                           input$ProductCharge, sep="_")
    input$PSM = paste(input$PeptideSequence,
                           input$PrecursorCharge, sep="_")

    # Remove fragments with more than 50% missing
    runs = length(unique(input$Run))
    remove = input[is.na(Intensity), .(count = .N / runs), by = Fragment
                   ][count > missing_run_count, Fragment]
    input = input[!(Fragment %in% remove)]
    
    # Select top N features
    feature_counts = input[
        , .(MeanAbundance = mean(as.numeric(Intensity), na.rm = TRUE)),
        by = .(ProteinName, Fragment)]
    feature_counts[, feature_rank := frank(-MeanAbundance, ties.method = "min"),
                   by = ProteinName]
    feature_counts = feature_counts[feature_rank <= n_feat,
                                    .(ProteinName, Fragment)]
    
    input = input[feature_counts, on = .(ProteinName, Fragment)]
    
    # Model at PSM level
    input = merge(input, run_order, by="Run", 
                  all.x=TRUE, all.y=FALSE)
    cols=c("ProteinName", "PSM", "Run", "Order", quality_metrics)
    feature_data = unique(
        input[, ..cols])
    setorder(feature_data, PSM, Order)
    
    # Scale quality metrics
    feature_data[, (quality_metrics) := lapply(
        .SD, function(x) as.numeric(scale(x))), 
        by = PSM, .SDcols = quality_metrics]
    
    # Apply add_features to each metric (avoids for loop)
    feature_data = feature_data[, {
        feature_list = lapply(.SD, .add_features)  # Apply add_features to each column
        feature_data = do.call(cbind, feature_list)  # Combine results
        feature_data[, Order := Order] # Ensure order is returned for join
        feature_data
    }, by = PSM, .SDcols = quality_metrics]
    
    # Add metrics back to df
    input = merge(input, feature_data,
                       all.x=TRUE, all.y=FALSE, 
                       by=c("PSM", "Order"))
    
    return(input)
}

#' Add moving window features given a quality metric vector
#' @noRd
.add_features = function(quality_vector){
    
    mean_increase = .add_mean_increase(quality_vector)
    mean_decrease = .add_mean_decrease(quality_vector)
    dispersion_increase = .add_dispersion_increase(quality_vector)

    
    feature_df = data.table(
        "mean_increase"=mean_increase,
        "mean_decrease"=mean_decrease,
        "dispersion_increase"=dispersion_increase
    )
    return(feature_df)
}

#' Calculate mean increase
#' @noRd
.add_mean_increase = function(quality_vector) {

    n = length(quality_vector)
    mean_increase = rep(NA_real_, n)

    d = 0.5
    h = 5

    state = 0
    mean_increase[1] = if (is.na(quality_vector[1])) NA_real_ else 0

    if (n >= 2) {
        for (k in 2:n) {

            if (is.na(quality_vector[k])) {
                mean_increase[k] = NA_real_   # output NA at missing
                next                          # state unchanged (carry forward)
            }

            if (state > h) {
                state = max(0, quality_vector[k] - d)
            } else {
                state = max(0, quality_vector[k] - d + state)
            }

            mean_increase[k] = state
        }
    }

    mean_increase
}

#' Calculate mean decrease
#' @noRd
.add_mean_decrease = function(quality_vector) {

    n = length(quality_vector)
    mean_decrease = rep(NA_real_, n)

    d = -0.5
    h = 5

    state = 0
    mean_decrease[1] =
        if (is.na(quality_vector[1])) NA_real_ else 0

    if (n >= 2) {
        for (k in 2:n) {

            if (is.na(quality_vector[k])) {
                mean_decrease[k] = NA_real_   # mark missing
                next                          # carry state forward
            }

            if (state > h) {
                state = max(0, d - quality_vector[k])
            } else {
                state = max(0, d - quality_vector[k] + state)
            }

            mean_decrease[k] = state
        }
    }

    mean_decrease
}

#' Calculate dispersion increase
#' @noRd
.add_dispersion_increase = function(quality_vector) {

    n = length(quality_vector)
    dispersion_increase = rep(NA_real_, n)
    v = rep(NA_real_, n)

    d = 0.5
    h = 5

    # initialize state
    state = 0

    if (!is.na(quality_vector[1])) {
        v[1] = (sqrt(abs(quality_vector[1])) - 0.822) / 0.349
        dispersion_increase[1] = 0
    } else {
        v[1] = NA_real_
        dispersion_increase[1] = NA_real_
    }

    if (n >= 2) {
        for (k in 2:n) {

            if (is.na(quality_vector[k])) {
                v[k] = NA_real_
                dispersion_increase[k] = NA_real_  # mark missing
                next                               # carry state forward
            }

            v[k] = (sqrt(abs(quality_vector[k])) - 0.822) / 0.349

            if (state > h) {
                state = max(0, v[k] - d)
            } else {
                state = max(0, v[k] - d + state)
            }

            dispersion_increase[k] = state
        }
    }

    return(dispersion_increase)
}

#' Train isolation forest model in parallel
#' @import parallel
#' @import Rcpp
#' 
#' @noRd
.runAnomalyModel = function(input_data, 
                            n_trees, 
                            max_depth, 
                            cores,
                            split_column,
                            quality_metrics){
    
    function_environment = environment()
    cl = parallel::makeCluster(cores)
    
    parallel::clusterExport(cl, c(
        "calculate_anomaly_score",
        "as.data.table",
        "max_depth",
        "n_trees",
        "split_column",
        "quality_metrics"), 
        envir = function_environment)
    
    psm_list = unique(input_data[, get(split_column)])
    num_psm = length(psm_list)
    
    cat(paste0("Number of PSMs to process: ", num_psm), 
        sep = "\n", file = "MSstats_anomaly_model_progress.log")
    
    model_input = unique(input_data[, c(split_column, quality_metrics), with = FALSE])

    model_results = parallel::parLapply(
        cl, seq_len(num_psm), 
        function(i){
            if (i %% 100 == 0) {
                cat("Finished processing an additional 100 PSMs", 
                    sep = "\n", file = "MSstats_anomaly_model_progress.log", append = TRUE)
            }
            single_psm = model_input[model_input[[split_column]] == psm_list[[i]], 
                                    ..quality_metrics]
            
            if (max_depth == "auto"){
                max_depth = round(log2(nrow(single_psm)))
            }
            
            forest = calculate_anomaly_score(
                single_psm, n_trees, max_depth)
            forest$anomaly_score
        }
    )
    
    model_input$AnomalyScores = unlist(model_results)

    input_data = merge(input_data, model_input, by = c(split_column, quality_metrics), all.x = TRUE)
      
    return(input_data)
}

# Comprehensive Unit Tests for Anomaly Detection Function using tinytest

# Helper function to create test data with quality metrics
run_quality_metrics = function(df, mean_increase, mean_decrease, dispersion_increase) {
    df$QualityMetric.mean_increase = mean_increase
    df$QualityMetric.mean_decrease = mean_decrease
    df$QualityMetric.dispersion_increase = dispersion_increase
    anomaly_output = MSstatsConvert:::.runAnomalyModel(df, 
                                                        n_trees=100, 
                                                        max_depth="auto", 
                                                        cores=1,
                                                        split_column="PSM",
                                                        quality_metrics=c("QualityMetric.mean_increase",
                                                                          "QualityMetric.mean_decrease",
                                                                          "QualityMetric.dispersion_increase"))
    return(anomaly_output)
}

# Create base dataframe for testing
create_base_df = function(n_rows) {
    data.table::data.table(
        PSM = rep("PSM1", n_rows)
    )
}

# Test 1: Property-Based Testing - Monotonicity with Cumulative Sums
base_df_10 = create_base_df(10)

# Baseline data with low values
baseline_scores = run_quality_metrics(
    base_df_10,
    rep(0.1, 10),  # mean_increase
    rep(0.1, 10),  # mean_decrease  
    rep(0.1, 10)   # dispersion_increase
)

# Data with progressively higher cumulative sums
high_scores = run_quality_metrics(
    base_df_10,
    c(seq(0, 0.1, length.out = 5), seq(2.0, 5.0, length.out = 5)),  # mean_increase
    c(seq(0, 0.1, length.out = 5), seq(2.0, 5.0, length.out = 5)),  # mean_decrease
    c(seq(0, 0.1, length.out = 5), seq(2.0, 5.0, length.out = 5))   # dispersion_increase
)

# The last 5 rows (with high values) should have higher mean anomaly scores
expect_true(mean(high_scores$AnomalyScores[6:10]) > mean(high_scores$AnomalyScores[1:5]),
            info = "Higher cumulative sum values should produce higher anomaly scores")

# Test 2: Extreme Value Testing - Obvious Outliers
base_df_20 = create_base_df(20)

extreme_scores = run_quality_metrics(
    base_df_20,
    c(seq(0, 0.1, length.out = 19), 10.0),  # Last value is extreme
    c(seq(0, 0.1, length.out = 19), 8.0),   # Last value is extreme
    c(seq(0, 0.1, length.out = 19), 12.0)   # Last value is extreme
)

# The extreme outlier (last row) should have the highest anomaly score
expect_true(extreme_scores$AnomalyScores[20] == max(extreme_scores$AnomalyScores),
            info = "Extreme outlier should have highest anomaly score")

# The outlier should score significantly higher than the median
expect_true(extreme_scores$AnomalyScores[20] > median(extreme_scores$AnomalyScores[1:19]) * 2,
            info = "Outlier should score significantly higher than median")

# Test 3: Consistency/Reproducibility Testing
base_df_20_orig = create_base_df(20)

# Run multiple times with original test data
scores1 = run_quality_metrics(
    base_df_20_orig,
    c(0.00, 0.00, 0.26, 0.00, 0.00, 
      0.00, 0.50, 0.00, 0.00, 0.00, 
      0.00, 0.00, 0.00, 0.00, 0.55, 
      0.87, 1.11, 1.42, 1.71, 2.94),
    c(0.00, 0.00, 0.00, 0.29, 0.50, 
      0.56, 0.00, 1.16, 0.99, 0.15, 
      0.00, 0.00, 1.27, 1.84, 0.29, 
      0.00, 0.00, 0.00, 0.00, 0.00),
    c(0.00, 0.00, 0.00, 0.00, 0.00, 
      0.00, 0.01, 0.85, 0.00, 0.00, 
      0.00, 0.00, 0.96, 1.07, 1.15, 
      0.89, 0.50, 0.22, 0.00, 0.91)
)

scores2 = run_quality_metrics(
    base_df_20_orig,
    c(0.00, 0.00, 0.26, 0.00, 0.00, 
      0.00, 0.50, 0.00, 0.00, 0.00, 
      0.00, 0.00, 0.00, 0.00, 0.55, 
      0.87, 1.11, 1.42, 1.71, 2.94),
    c(0.00, 0.00, 0.00, 0.29, 0.50, 
      0.56, 0.00, 1.16, 0.99, 0.15, 
      0.00, 0.00, 1.27, 1.84, 0.29, 
      0.00, 0.00, 0.00, 0.00, 0.00),
    c(0.00, 0.00, 0.00, 0.00, 0.00, 
      0.00, 0.01, 0.85, 0.00, 0.00, 
      0.00, 0.00, 0.96, 1.07, 1.15, 
      0.89, 0.50, 0.22, 0.00, 0.91)
)

# Correlation between runs should be high (>0.7) even if not identical
correlation = cor(scores1$AnomalyScores, scores2$AnomalyScores)
expect_true(correlation > 0.95,
            info = "Multiple runs should produce correlated results")

# Top anomalies should be largely consistent
top5_run1 = order(scores1$AnomalyScores, decreasing = TRUE)[1:5]
top5_run2 = order(scores2$AnomalyScores, decreasing = TRUE)[1:5]
overlap = length(intersect(top5_run1, top5_run2))
expect_true(overlap >= 3,
            info = "At least 3 of top 5 anomalies should overlap between runs")

# Test 4: Expected Anomaly Identification from Original Data
original_anomaly_scores = run_quality_metrics(
    base_df_20_orig,
    c(0.00, 0.00, 0.26, 0.00, 0.00, 
      0.00, 0.50, 0.00, 0.00, 0.00, 
      0.00, 0.00, 0.00, 0.00, 0.55, 
      0.87, 1.11, 1.42, 1.71, 2.94),
    c(0.00, 0.00, 0.00, 0.29, 0.50, 
      0.56, 0.00, 1.16, 0.99, 0.15, 
      0.00, 0.00, 1.27, 1.84, 0.29, 
      0.00, 0.00, 0.00, 0.00, 0.00),
    c(0.00, 0.00, 0.00, 0.00, 0.00, 
      0.00, 0.01, 0.85, 0.00, 0.00, 
      0.00, 0.00, 0.96, 1.07, 1.15, 
      0.89, 0.50, 0.22, 0.00, 0.91)
)

# Row 20 has highest mean_increase (2.94) - should be highly anomalous
top_anomalies = order(original_anomaly_scores$AnomalyScores, decreasing = TRUE)[1:5]
expect_true(20 %in% top_anomalies,
            info = "Row 20 with highest mean_increase should be in top anomalies")

# Create test data vectors for comparison
mean_increase_vals = c(0.00, 0.00, 0.26, 0.00, 0.00, 
                        0.00, 0.50, 0.00, 0.00, 0.00, 
                        0.00, 0.00, 0.00, 0.00, 0.55, 
                        0.87, 1.11, 1.42, 1.71, 2.94)
mean_decrease_vals = c(0.00, 0.00, 0.00, 0.29, 0.50, 
                        0.56, 0.00, 1.16, 0.99, 0.15, 
                        0.00, 0.00, 1.27, 1.84, 0.29, 
                        0.00, 0.00, 0.00, 0.00, 0.00)
dispersion_vals = c(0.00, 0.00, 0.00, 0.00, 0.00, 
                     0.00, 0.01, 0.85, 0.00, 0.00, 
                     0.00, 0.00, 0.96, 1.07, 1.15, 
                     0.89, 0.50, 0.22, 0.00, 0.91)

# Rows with all zeros should score lower than rows with high values
zero_rows = which(mean_increase_vals == 0 & mean_decrease_vals == 0 & dispersion_vals == 0)
high_value_rows = c(14, 16, 17, 18, 19, 20)  # Rows with notably high values

expect_true(mean(original_anomaly_scores$AnomalyScores[high_value_rows]) > 
                mean(original_anomaly_scores$AnomalyScores[zero_rows]),
            info = "High value rows should score higher than zero rows")

# Test 5: Edge Cases and Boundary Conditions
base_df_10_edge = create_base_df(10)

# Test with all identical values
identical_scores = run_quality_metrics(
    base_df_10_edge,
    rep(1.0, 10),  # All identical
    rep(1.0, 10),  # All identical
    rep(1.0, 10)   # All identical
)

expect_true(is.data.frame(identical_scores) && nrow(identical_scores) == 10,
            info = "Function should handle identical values gracefully")

# Test with minimum viable dataset
base_df_3 = create_base_df(3)
minimal_scores = run_quality_metrics(
    base_df_3,
    c(0.1, 0.2, 5.0),  # Clear outlier
    c(0.1, 0.2, 0.1),
    c(0.1, 0.1, 0.1)
)

# The outlier (row 3) should have highest score
expect_equal(which.max(minimal_scores$AnomalyScores), 3,
             info = "Clear outlier should have highest score in minimal dataset")

# Test 6: Scale Invariance Testing
base_df_5 = create_base_df(5)

# Original data
original_scale_scores = run_quality_metrics(
    base_df_5,
    c(0.1, 0.2, 0.3, 1.0, 2.0),
    c(0.1, 0.1, 0.5, 0.2, 0.1),
    c(0.1, 0.3, 0.1, 0.8, 0.2)
)

# Scaled data (multiply by 10)
scaled_scale_scores = run_quality_metrics(
    base_df_5,
    c(0.1, 0.2, 0.3, 1.0, 2.0) * 10,
    c(0.1, 0.1, 0.5, 0.2, 0.1) * 10,
    c(0.1, 0.3, 0.1, 0.8, 0.2) * 10
)

# Relative ranking should be preserved under scaling
original_ranking = order(original_scale_scores$AnomalyScores, decreasing = TRUE)
scaled_ranking = order(scaled_scale_scores$AnomalyScores, decreasing = TRUE)

# Rankings should be identical or very similar
expect_true(cor(original_ranking, scaled_ranking, method = "spearman") > 0.95,
            info = "Relative rankings should be preserved under scaling")

# Test 7: Regression Testing with Original Data
regression_scores = run_quality_metrics(
    base_df_20_orig,
    c(0.00, 0.00, 0.26, 0.00, 0.00, 
      0.00, 0.50, 0.00, 0.00, 0.00, 
      0.00, 0.00, 0.00, 0.00, 0.55, 
      0.87, 1.11, 1.42, 1.71, 2.94),
    c(0.00, 0.00, 0.00, 0.29, 0.50, 
      0.56, 0.00, 1.16, 0.99, 0.15, 
      0.00, 0.00, 1.27, 1.84, 0.29, 
      0.00, 0.00, 0.00, 0.00, 0.00),
    c(0.00, 0.00, 0.00, 0.00, 0.00, 
      0.00, 0.01, 0.85, 0.00, 0.00, 
      0.00, 0.00, 0.96, 1.07, 1.15, 
      0.89, 0.50, 0.22, 0.00, 0.91)
)

# Basic sanity checks
expect_equal(nrow(regression_scores), 20,
             info = "Output should have same number of rows as input")
expect_true(all(regression_scores$AnomalyScores >= 0),
            info = "All anomaly scores should be non-negative")
expect_true(all(is.finite(regression_scores$AnomalyScores)),
            info = "All anomaly scores should be finite")

# Row 20 (highest mean_increase = 2.94) should be highly anomalous
expect_true(regression_scores$AnomalyScores[20] > quantile(regression_scores$AnomalyScores, 0.8),
            info = "Row with highest mean_increase should be highly anomalous")

# Store results for regression testing (compare against future runs)
cat("Top 5 anomalous rows:", order(regression_scores$AnomalyScores, decreasing = TRUE)[1:5], "\n")
cat("Score range:", range(regression_scores$AnomalyScores), "\n")

# Test 8: Zero/Null Data Handling
base_df_10_zero = create_base_df(10)

# All zeros data
zero_scores = run_quality_metrics(
    base_df_10_zero,
    rep(0, 10),  # All zeros
    rep(0, 10),  # All zeros
    rep(0, 10)   # All zeros
)

expect_true(is.data.frame(zero_scores) && nrow(zero_scores) == 10,
            info = "Function should handle all-zero data gracefully")

# All scores should be similar/low for identical data
expect_true(sd(zero_scores$AnomalyScores) < 0.1,
            info = "All-zero data should have low variance in scores")

# Test 9: Relative Ranking Preservation
base_df_6_rank = create_base_df(6)

# Create data with obvious ranking: Row 6 > Row 5 > Row 4 > Rows 1,2,3
ranking_scores = run_quality_metrics(
    base_df_6_rank,
    c(0.1, 0.11, 0.12, 1.0, 2.0, 5.0),
    c(0.1, 0.11, 0.12, 1.0, 2.0, 5.0),
    c(0.1, 0.11, 0.12, 1.0, 2.0, 5.0)
)

# Row 5 should have highest score, Row 4 second highest, etc.
expect_true(ranking_scores$AnomalyScores[6] > ranking_scores$AnomalyScores[5],
            info = "Row 6 should score higher than Row 5")
expect_true(ranking_scores$AnomalyScores[5] > ranking_scores$AnomalyScores[4],
            info = "Row 5 should score higher than Row 4")
expect_true(ranking_scores$AnomalyScores[4] > max(ranking_scores$AnomalyScores[1:3]),
            info = "Row 4 should score higher than Rows 1-3")

# Test 10: Original Quality Metrics Calculation Test (from the beginning of the file)
# Test add_increase, add_decrease, add_dispersion
quality_vector = c(
    1.31, 0.05, 0.76, -0.79, -0.71, 
    -0.56, 1.00, -1.66, -0.33,  0.34, 
    0.00, -0.40, -1.77, -1.07, 1.05,
    0.82, 0.74, 0.81, 0.79, 1.73
)

mean_increase = MSstatsConvert:::.add_mean_increase(quality_vector)
mean_decrease = MSstatsConvert:::.add_mean_decrease(quality_vector)
dispersion_increase = MSstatsConvert:::.add_dispersion_increase(quality_vector)

expect_equal(
    mean_increase,
    c(0.00, 0.00, 0.26, 0.00, 0.00, 
      0.00, 0.50, 0.00, 0.00, 0.00, 
      0.00, 0.00, 0.00, 0.00, 0.55, 
      0.87, 1.11, 1.42, 1.71, 2.94),
    info = "Mean increase calculation should match expected values"
)

expect_equal(
    mean_decrease,
    c(0.00, 0.00, 0.00, 0.29, 0.50, 
      0.56, 0.00, 1.16, 0.99, 0.15, 
      0.00, 0.00, 1.27, 1.84, 0.29, 
      0.00, 0.00, 0.00, 0.00, 0.00),
    info = "Mean decrease calculation should match expected values"
)

expect_equal(
    dispersion_increase,
    c(0.00, 0.00, 0.00, 0.00, 0.00, 
      0.00, 0.01, 0.85, 0.00, 0.00, 
      0.00, 0.00, 0.96, 1.07, 1.15, 
      0.89, 0.50, 0.22, 0.00, 0.91),
    tolerance = 1e-2,
    info = "Dispersion increase calculation should match expected values"
)

cat("All anomaly detection tests completed successfully!\n")

# Test n_feat and missing_run_count parameters in .prepareSpectronautAnomalyInput
spectronaut_raw = data.table::data.table(
    ProteinName = c(rep("Q9UFW8", 10), rep("Q96S19", 15)),
    PeptideSequence = c(rep("AEFEEQNVR", 5), rep("TALYVTPLDR", 5),
                        rep("AFPLAEWQPSDVDQR", 5), rep("ASGLLLER", 5),
                        rep("LowAbundancePeptide", 5)),
    PrecursorCharge = rep(2, 25),
    FragmentIon = rep("y3", 25),
    ProductCharge = rep(1, 25),
    IsotopeLabelType = rep("L", 25),
    Condition = rep(c("A", "A", "A", "B", "B"), 5),
    BioReplicate = rep(seq(1:5), 5),
    Run = rep(paste0("Run", seq(1:5)), 5),
    Intensity = c(1000, 1500, 2000, 2500, 3000,
                  1100, 1600, 2100, 2600, 3100,
                  1200, NA, 2200, NA, NA,
                  1300, 1800, NA, 2800, NA,
                  100, 200, 300, 400, 500),
    QualityMetric1 = rep(0.5, 25)
)

temporal = spectronaut_raw[, .(Condition = unique(Condition), 
                                    Run = unique(Run)), 
                                by = BioReplicate]
temporal$Order = seq(1:nrow(temporal))
temporal = temporal[, c("Run", "Order")]

missing_run_excluded = MSstatsConvert:::.prepareSpectronautAnomalyInput(
    spectronaut_raw, 
    quality_metrics = c("QualityMetric1"),
    run_order = temporal,
    n_feat = 2,
    missing_run_count = 0.5)
expect_false("AFPLAEWQPSDVDQR" %in% missing_run_excluded$PeptideSequence)
expect_true("LowAbundancePeptide" %in% missing_run_excluded$PeptideSequence)

low_abundance_excluded = MSstatsConvert:::.prepareSpectronautAnomalyInput(
    spectronaut_raw, 
    quality_metrics = c("QualityMetric1"),
    run_order = temporal,
    n_feat = 2,
    missing_run_count = 0.95)
expect_true("AFPLAEWQPSDVDQR" %in% low_abundance_excluded$PeptideSequence)
expect_false("LowAbundancePeptide" %in% low_abundance_excluded$PeptideSequence)


# Test 11: Testing duplicity of quality metrics, applicable considering
# multiple fragments share the same precursor level metrics

# Data with progressively higher cumulative sums
duplicate_metrics = run_quality_metrics(
    base_df_10,
    c(rep(0.1, 5), seq(2.0, 4.0, length.out = 5)),  # mean_increase
    c(rep(0.1, 5), seq(2.0, 4.0, length.out = 5)),  # mean_decrease
    c(rep(0.1, 5), seq(2.0, 4.0, length.out = 5))   # dispersion_increase
)

# The last 5 rows (with high values) should have lower mean anomaly scores
# Since they are all clumped between 2 and 4, whereas 0.1 is by itself
expect_true(mean(duplicate_metrics$AnomalyScores[6:10]) < mean(duplicate_metrics$AnomalyScores[1:5]),
            info = "Rows 6-10 (values clumped 2-4) should have lower 
            anomaly scores than rows 1-5 (isolated value of 0.1)")

#' Import Proteome Discoverer files
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param input PD report or a path to it.
#' @param annotation name of 'annotation.txt' or 'annotation.csv' data which includes Condition, BioReplicate, 
#' Run information. 'Run' will be matched with 'Spectrum.File'.
#' @param useNumProteinsColumn TRUE removes peptides which have more than 1 in # Proteins column of PD output.
#' @param which.quantification Use 'Precursor.Area'(default) column for quantified intensities. 'Intensity' or 'Area' can be used instead.
#' @param which.proteinid Use 'Protein.Accessions'(default) column for protein name. 'Master.Protein.Accessions' can be used instead.
#' @param which.sequence Use 'Sequence'(default) column for peptide sequence. 'Annotated.Sequence' can be used instead.
#' @param ... additional parameters to `data.table::fread`.
#' 
#' @return data.frame in the MSstats required format.
#' 
#' @author Meena Choi, Olga Vitek
#' 
#' @export
#' 
#' @examples 
#' 
#' pd_raw = system.file("tinytest/raw_data/PD/pd_input.csv", 
#'                      package = "MSstatsConvert")
#' annot = system.file("tinytest/raw_data/PD/annot_pd.csv", 
#'                     package = "MSstatsConvert")
#' pd_raw = data.table::fread(pd_raw)
#' annot = data.table::fread(annot)
#' 
#' pd_imported = PDtoMSstatsFormat(pd_raw, annot, use_log_file = FALSE)
#' head(pd_imported)
#' 
PDtoMSstatsFormat = function(
        input, annotation, useNumProteinsColumn = FALSE, useUniquePeptide = TRUE,
        summaryforMultipleRows = max, removeFewMeasurements = TRUE,
        removeOxidationMpeptides = FALSE, removeProtein_with1Peptide = FALSE,
        which.quantification = 'Precursor.Area', 
        which.proteinid = 'Protein.Group.Accessions', which.sequence = 'Sequence',
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "ProteomeDiscoverer", ...)
    input = MSstatsConvert::MSstatsClean(
        input, 
        quantification_column = which.quantification, 
        protein_id_column = which.proteinid,
        sequence_column = which.sequence, 
        remove_shared = useNumProteinsColumn)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    oxidation_filter = list(col_name = "PeptideSequence", 
                            pattern = "Oxidation", 
                            filter = removeOxidationMpeptides, 
                            drop_column = FALSE)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Peptide,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = summaryforMultipleRows),
        pattern_filtering = list(oxidation = oxidation_filter),
        columns_to_fill = list("FragmentIon" = NA, 
                               "ProductCharge" = NA,
                               "IsotopeLabelType" = "L"))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  remove_few = removeFewMeasurements)
    data.table::setnames(input, "PeptideSequence", "PeptideModifiedSequence",
                         skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test PDtoMSstatsFormat ---------------------------
pd_raw = system.file("tinytest/raw_data/PD/pd_input.csv",
                     package = "MSstatsConvert")
annot = system.file("tinytest/raw_data/PD/annot_pd.csv",
                    package = "MSstatsConvert")
pd_raw = data.table::fread(pd_raw)
annot = data.table::fread(annot)
output = PDtoMSstatsFormat(pd_raw, annot, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 360)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideModifiedSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))

#' Convert Philosopher (Fragpipe) output to MSstatsTMT format.
#' 
#' @param input data.frame of `msstats.csv` file produced by Philosopher
#' @param annotation annotation with Run, Fraction, TechRepMixture, Mixture, Channel, 
#' BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column. Channel column should be 
#' consistent with the channel columns (Ignore the prefix "Channel ") in msstats.csv file. Run column should be consistent with the Spectrum.File columns in msstats.csv file.
#' @param protein_id_col Use 'Protein'(default) column for protein name. 
#' 'Master.Protein.Accessions' can be used instead to get the protein ID with single protein.
#' @param peptide_id_col Use 'Peptide.Sequence'(default) column for peptide sequence.
#'  'Modified.Peptide.Sequence' can be used instead to get the modified peptide sequence.
#' @param Purity_cutoff Cutoff for purity. Default is 0.6
#' @param PeptideProphet_prob_cutoff Cutoff for the peptide identification probability. Default is 0.7. 
#' The probability is confidence score determined by PeptideProphet and higher values indicate greater confidence.
#' @param rmPSM_withfewMea_withinRun TRUE (default) will remove the features that have 1 or 2 measurements within each Run.
#' @param rmPeptide_OxidationM TRUE (default) will remove the peptides including oxidation (M) sequence.
#' @param rmProtein_with1Feature TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.
#' @param ... additional parameters to `data.table::fread`.
#' @inheritParams .sharedParametersAmongConverters
#' 
#' @return `data.frame` of class `MSstatsTMT`
#' 
#' @examples
#' input_file_path = system.file("tinytest/raw_data/Philosopher/msstats.csv", 
#'                      package = "MSstatsConvert")
#' annotation_file_path = system.file("tinytest/raw_data/Philosopher/MSstatsTMT_annotation.csv", 
#'                     package = "MSstatsConvert")
#' input = data.table::fread(input_file_path)
#' annotation = data.table::fread(annotation_file_path)
#' msstats_format = PhilosophertoMSstatsTMTFormat(input, annotation)
#' head(msstats_format)
#' 
#' @export
PhilosophertoMSstatsTMTFormat = function(
        input, annotation, protein_id_col = "Protein", 
        peptide_id_col = "Peptide.Sequence", Purity_cutoff = 0.6, 
        PeptideProphet_prob_cutoff = 0.7, useUniquePeptide = TRUE, 
        rmPSM_withfewMea_withinRun = TRUE, rmPeptide_OxidationM = TRUE, 
        rmProtein_with1Feature = FALSE, summaryforMultipleRows = sum, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL, ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsTMT_converter_log_")
    checkmate::assertTRUE(!is.null(input))
    
    input[["Is.Unique"]] = as.logical(input[["Is.Unique"]])
    mixture_files = list(Mixture1=data.table(input))
    
    input = MSstatsConvert::MSstatsImport(c(mixture_files,
                                            list(annotation = annotation)), 
                                          type = "MSstatsTMT",
                                          tool = "Philosopher")
    channels = unique(annotation[["Channel"]])
    input = MSstatsConvert::MSstatsClean(input, protein_id_col, peptide_id_col,
                                         channels, useUniquePeptide)
    annotation = MSstatsMakeAnnotation(input, annotation)
    
    purity_filter = list(score_column = "Purity", 
                         score_threshold = Purity_cutoff, 
                         direction = "greater",
                         behavior = "remove", 
                         handle_na = "keep", 
                         fill_value = NULL,
                         filter = TRUE, 
                         drop_column = FALSE)
    probability_filter = list(score_column = "PeptideProphetProbability", 
                              score_threshold = PeptideProphet_prob_cutoff, 
                              direction = "greater",
                              behavior = "remove", 
                              handle_na = "keep", 
                              fill_value = NULL,
                              filter = TRUE, 
                              drop_column = FALSE)
    oxidation_filter = list(col_name = "PeptideSequence",
                            pattern = "Oxidation", 
                            filter = rmPeptide_OxidationM, 
                            drop_column = FALSE)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge") 
    input = MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = rmProtein_with1Feature,
        score_filtering = list(pur = purity_filter, prob = probability_filter),
        pattern_filtering = list(ox = oxidation_filter),
        feature_cleaning = list(remove_features_with_few_measurements = rmPSM_withfewMea_withinRun,
                                summarize_multiple_psms = summaryforMultipleRows)
    )
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  fix_missing = "zero_to_na")
    data.table::setnames(input, "PrecursorCharge", "Charge", skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the proteinSummarization function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
} 


#' Convert Philosopher parameters to consistent format
#' @inheritParams PhilosophertoMSstatsTMTFormat
#' @param path character. Path to a file or directory containing msstats.csv output(s) from Philosopher. Used when \code{input} is NULL.
#' @param folder logical. If TRUE, \code{path} is treated as a directory and all msstats files within it are read. If FALSE, \code{path} is treated as a single file path.
#' @keywords internal
.getPhilosopherInput = function(input, path, folder) {
    if (!is.null(input)) {
        mixture_files = input
    } else {
        if (folder) {
            mixture_files = list.files(path, pattern = "msstats", 
                                       full.names = TRUE)
        } else {
            mixture_files = path
        }
    }
    mixture_files = as.list(mixture_files)
    names(mixture_files) = paste0("Mixture", seq_along(mixture_files))    
    mixture_files
}

#' Perform by-feature operations.
#' @param input `data.table` preprocessed by one of the cleanRaw* functions.
#' @param feature_columns character vector of names of columns that define features.
#' @param cleaning_control named list of two or three elements. 
#' See the documentation for `MSstatsImport` for details.
#' @param anomaly_metrics character vector of quality metric column names to be 
#' used as features in an anomaly detection model.
#' @return `data.table`
#' @keywords internal 
.cleanByFeature = function(input, feature_columns, 
                           cleaning_control, anomaly_metrics = c()) {
    if (is.element("Channel", colnames(input))) {
        input = .filterFewMeasurements(
            input, 0, 
            cleaning_control[["remove_features_with_few_measurements"]],
            unique(c("PSM", "Run")))
        input = .aggregatePSMstoPeptideIons(
            input, feature_columns, 
            cleaning_control[["summarize_multiple_psms"]])
        input
    } else {
        summary_str = deparse(cleaning_control[["summarize_multiple_psms"]])
        summary_str = ifelse(grepl("sum", summary_str), "sum", "max")
        
        input = .summarizeMultipleMeasurements(
            input, cleaning_control[["summarize_multiple_psms"]],
            c(feature_columns, "Run"),
            anomaly_metrics)
        msg = paste("** Multiple measurements in a feature and a run",
                    "are summarized by summaryforMultipleRows:", summary_str)
        getOption("MSstatsLog")("INFO", msg)
        getOption("MSstatsMsg")("INFO", msg)
        input = .filterFewMeasurements(
            input, 0, 
            cleaning_control[["remove_features_with_few_measurements"]],
            feature_columns)
    }
    input
}


#' Remove features with a small number of (non-missing) measurements across runs
#' @param input `data.table` pre-processed by one of the .cleanRaw* functions.
#' @param min_intensity minimum intensity that will be considered non-missing.
#' @param remove_few logical, if TRUE, features that have less than three 
#' measurements will be removed. If FALSE, only features with all missing runs
#' will be removed.
#' @param feature_columns chr, vector of names of columns that define features. 
#' @return data.table
#' @keywords internal
.filterFewMeasurements = function(input, min_intensity, remove_few,
                                  feature_columns = NULL) {
    Intensity = n_obs = NULL
    
    if (is.null(feature_columns)) {
        if (is.element("Channel", colnames(input))) {
            feature_columns = c("PSM", "Run")
        } else {
            feature_columns = intersect(colnames(input),
                                        c("PeptideModifiedSequence", "Charge",
                                          "PeptideSequence", "PrecursorCharge",
                                          "FragmentIon", "ProductCharge"))
        }
    }
    feature_columns = setdiff(feature_columns, "IsotopeLabelType")
    
    input[, n_obs := sum(Intensity > min_intensity, na.rm = TRUE),
          by = feature_columns]
    
    what = ifelse(is.element("Channel", colnames(input)),
                  "channels within each run", "runs")
    if (remove_few) {
        cutoff = 2
        msg = paste("** Features with one or two measurements across", 
                    what, "are removed.")
    } else {
        cutoff = 0
        msg = paste("** Features with all missing measurements across", 
                    what, "are removed.")
    }
    input = input[n_obs > cutoff]
    getOption("MSstatsLog")("INFO", msg)
    getOption("MSstatsMsg")("INFO", msg)
    input[, colnames(input) != "n_obs", with = FALSE]
}


#' Summarize multiple measurements per feature in a single run
#' @param input `data.table` pre-processed by one of the .cleanRaw* functions.
#' @param aggregator function that will be used to aggregate duplicated values.
#' @param feature_columns chr, vector of names of columns that define features.
#' @param anomaly_metrics character vector of quality metric column names 
#' to be used as features in an anomaly detection model.
#' @return `data.table`
#' @keywords internal
.summarizeMultipleMeasurements = function(input, aggregator, 
                                          feature_columns, anomaly_metrics = c()) {
    Intensity = isZero = NULL
    
    info = unique(input[, intersect(colnames(input), 
                                    c("StandardType", "ProteinName", 
                                      "PeptideModifiedSequence", "Charge",
                                      "PeptideSequence", "PrecursorCharge",
                                      "IsotopeLabelType")), 
                        with = FALSE])
    
    if (is.element("isZero", colnames(input)) & length(anomaly_metrics) == 0) {
        input = input[, list(Intensity = aggregator(Intensity, na.rm = TRUE),
                             isZero = all(isZero | is.na(Intensity)) &
                                 !all(is.na(Intensity))), 
                      by = feature_columns]
    } else if (length(anomaly_metrics) > 0){
        
        input[, row_id := .I]
        max_per_group = input[, .(max_intensity = max(Intensity, na.rm = TRUE)),
                              by = feature_columns]
        
        input = input[max_per_group, 
                         on = c(feature_columns, "Intensity" = "max_intensity"),
                         nomatch = 0L, mult = "first"]
        input[, row_id := NULL]
        
    } else {
        input = input[, list(Intensity = aggregator(Intensity, na.rm = TRUE)), 
                      by = c(feature_columns)]
    }
    merge(input, info, 
          by = intersect(colnames(input), colnames(info)), sort = FALSE)
}


#' Remove proteins only identified by a single feature
#' @param input `data.table` pre-processed by one of the .cleanRaw* functions.
#' @param remove_single_feature lgl, if TRUE, proteins with a single feature
#' will be removed.
#' @return `data.table`
#' @keywords internal
.handleSingleFeaturePerProtein = function(input, remove_single_feature) {
    feature_count = feature = NULL
    
    if (remove_single_feature) {
        feature_columns = intersect(c("PeptideSequence", "PrecursorCharge",
                                      "FragmentIon", "ProductCharge", "Charge"),
                                    colnames(input))
        input[, feature := do.call(".combine", .SD), .SDcols = feature_columns]
        input[, feature_count := uniqueN(feature), by = "ProteinName"]
        input = input[feature_count > 1]
        input = input[, !(colnames(input) %in% c("feature_count", "feature")), 
                      with = FALSE]
        getOption("MSstatsLog")("INFO", 
                                "Proteins with a single feature are removed.")
        getOption("MSstatsMsg")("INFO", 
                                "Proteins with a single feature are removed.")
    }
    input
}


#' @keywords internal
.combine = function(...) {
    paste(..., sep = "_")  
} 

#' Aggregate multiple PSMs to a single peptide ion.
#' @param input data.table preprocessed by one of the cleanRaw* functions.
#' @param feature_columns chr, names of columns that define features.
#' @param summary_function function that will be used to aggregate intensities
#' if needed.
#' @return data.table
#' @keywords internal
.aggregatePSMstoPeptideIons = function(input, feature_columns, 
                                       summary_function = sum
) {
    keep = n_psms = PSM = Intensity = NULL
    
    feature_columns = unique(c(feature_columns, "Run"))
    input[, n_psms := data.table::uniqueN(PSM), by = feature_columns]
    
    if (any(input$n_psms > 1)) {
        input_duplicates = input[n_psms != 1]
        input = input[n_psms == 1]
        if (nrow(input_duplicates) > 0) {
            cols = intersect(colnames(input_duplicates),
                             c("PSM", "Channel", "Intensity", "Run", "Score",
                               "IsolationInterference", "IonsScore", "n_psms",
                               "Purity", "PeptideProphetProbability"))
            input_duplicates[, keep := .summarizeMultiplePSMs(.SD, 
                                                              summary_function),
                             by = feature_columns, .SDcols = cols]
        }
        input = rbind(input, input_duplicates, fill = TRUE)
        cols = intersect(colnames(input),
                         c("PSM", "Channel", "Intensity", "Run", "Score",
                           "IsolationInterference", "IonsScore", "n_psms",
                           "Purity", "PeptideProphetProbability"))
        input[, keep := .summarizeMultiplePSMs(.SD, summary_function), 
              by = feature_columns, .SDcols = cols]
        input = input[(PSM == keep) | is.na(keep), 
                      !(colnames(input) %in% c("keep", "feature")), 
                      with = FALSE]
        input[, n_psms := data.table::uniqueN(PSM), by = feature_columns]
        if (any(input$n_psms > 1)) {
            input = input[, list(Intensity = mean(Intensity, na.rm = TRUE)),
                          by = setdiff(colnames(input), 
                                       c("PSM", "Intensity", "n_psms",
                                         "IsolationInterference", "Score",
                                         "IonsScore", "Purity",
                                         "Purity", "PeptideProphetProbability"))]
        }
    }
    input[, PSM := do.call(".combine", .SD), 
          .SDcols = c("PeptideSequence", "PrecursorCharge")]
    msg = "** PSMs have been aggregated to peptide ions."
    getOption("MSstatsLog")("INFO", msg)
    getOption("MSstatsMsg")("INFO", msg)
    input[, colnames(input) != "n_psms", with = FALSE]
}

#' Pick one PSM from a data.table of several PSMs.
#' @param input data.table preprocessed by one of the .cleanRaw* functions.
#' @param summary_function function that will be used to aggregate intensities
#' if needed.
#' @return character - label of a chosen PSM
#' @keywords internal
.summarizeMultiplePSMs = function(input, summary_function) {
    Intensity = Score = IsolationInterference = IonsScore = PSM = NULL
    
    if (all(unique(input$n_psms) == 1)) {
        return(unique(input$PSM))
    } else {
        nonmissing_counts = input[, list(n_nonmissing = sum(!is.na(Intensity))),
                                  by = c("PSM")]
        is_max = nonmissing_counts$n_nonmissing == max(nonmissing_counts$n_nonmissing, 
                                                       na.rm = TRUE)
        if (sum(is_max, na.rm = TRUE) == 1) {
            return(nonmissing_counts$PSM[which.max(nonmissing_counts$n_nonmissing)])
        } else {
            input = input[PSM %in% unique(nonmissing_counts$PSM[is_max])]
        }
        
        if ("Score" %in% colnames(input)) {
            by_score = input[, list(score = unique(Score)),
                             by = c("PSM")]
            is_max = by_score$score == max(by_score$score, na.rm = TRUE)
            if (sum(is_max, na.rm = TRUE) == 1) {
                return(by_score$PSM[which.max(by_score$score)])
            } else {
                input = input[PSM %in% unique(by_score$PSM[is_max])]
            }
        }
        
        if ("IsolationInterference" %in% colnames(input) &
            !any(is.na(input$IsolationInterference))) {
            by_score = input[, list(score = unique(IsolationInterference)),
                             by = c("PSM")]
            is_min = by_score$score == min(by_score$score, na.rm = TRUE)
            if (sum(is_min, na.rm = TRUE) == 1) {
                return(by_score$PSM[which.min(by_score$score)])
            } else {
                input = input[PSM %in% unique(by_score$PSM[is_min])]
            }
        }
        
        if ("IonsScore" %in% colnames(input) & !any(is.na(input$IonsScore))) {
            by_score = input[, list(score = unique(IonsScore)),
                             by = c("PSM")]
            is_max = by_score$score == max(by_score$score, na.rm = TRUE)
            if (sum(is_max, na.rm = TRUE) == 1) {
                return(by_score$PSM[which.max(by_score$score)])
            } else {
                input = input[PSM %in% unique(by_score$PSM[is_max])]
            }
        }
        
        if ("Purity" %in% colnames(input) & !any(is.na(input$Purity))) {
            by_score = input[, list(score = unique(Purity)),
                             by = c("PSM")]
            is_max = by_score$score == max(by_score$score, na.rm = TRUE)
            if (sum(is_max, na.rm = TRUE) == 1) {
                return(by_score$PSM[which.max(by_score$score)])
            } else {
                input = input[PSM %in% unique(by_score$PSM[is_max])]
            }
        }
        
        if ("PeptideProphet.Probability" %in% colnames(input) &
            !any(is.na(input$PeptideProphet.Probability))) {
            by_score = input[, list(score = unique(PeptideProphet.Probability)),
                             by = c("PSM")]
            is_max = by_score$score == max(by_score$score, na.rm = TRUE)
            if (sum(is_max, na.rm = TRUE) == 1) {
                return(by_score$PSM[which.max(by_score$score)])
            } else {
                input = input[PSM %in% unique(by_score$PSM[is_max])]
            }
        }
        
        
        by_max = input[, list(Intensity = summary_function(Intensity, 
                                                           na.rm = TRUE)),
                       by = c("PSM")]
        is_max = by_max$Intensity == max(by_max$Intensity, na.rm = TRUE)
        if (sum(is_max, na.rm = TRUE) == 1) {
            return(by_max$PSM[which.max(by_max$Intensity)])
        } else {
            return(NA)
        }
    }
}


#' Classify IsotopeLabelType from peptide sequence patterns.
#'
#' Shared core logic for protein turnover workflows in both Spectronaut and
#' DIANN converters.  Each peptide is classified as heavy (\code{"H"}), light
#' (\code{"L"}), or unlabeled (\code{NA}) by matching regex patterns against
#' the \code{PeptideSequence} column.
#'
#' Two modes are supported:
#' \describe{
#'   \item{Spectronaut mode}{Pass \code{labeled_aa_regex}.  The sequence is
#'     first stripped of all bracket modifications; light is inferred when the
#'     bare labeled amino acid is present but the heavy bracket form is absent.}
#'   \item{DIANN mode}{Pass \code{light_regex}.  Both heavy and light patterns
#'     are matched directly against the modified sequence; absence of either
#'     yields \code{NA}.}
#' }
#'
#' @param dt \code{data.table} with a \code{PeptideSequence} column.
#' @param heavy_regex Perl-compatible regex matching heavy-labeled sequences.
#' @param light_regex Perl-compatible regex explicitly matching light-labeled
#'   sequences.  Required for DIANN mode; must be non-\code{NULL}.
#' @param labeled_aa_regex Perl-compatible regex for bare labeled amino acids
#'   applied to the bracket-stripped sequence to detect light peptides.
#'   Required for Spectronaut mode; must be non-\code{NULL}.
#'   Exactly one of \code{light_regex} and \code{labeled_aa_regex} must be
#'   supplied.
#' @return \code{dt} with \code{IsotopeLabelType} column added or updated.
#' @keywords internal
#' @noRd
.classifyIsotopeLabelType = function(dt, heavy_regex,
                                      light_regex = NULL,
                                      labeled_aa_regex = NULL) {
    IsotopeLabelType = PeptideSequence = StrippedSequence = NULL

    if (!is.null(labeled_aa_regex)) {
        dt[, StrippedSequence := gsub("\\[.*?\\]", "", PeptideSequence)]
        dt[, IsotopeLabelType := data.table::fcase(
            grepl(heavy_regex, PeptideSequence, perl = TRUE), "H",
            grepl(labeled_aa_regex, StrippedSequence, perl = TRUE), "L",
            default = NA_character_
        )]
        dt[, StrippedSequence := NULL]
    } else if (!is.null(light_regex)) {
        dt[, IsotopeLabelType := data.table::fcase(
            grepl(heavy_regex, PeptideSequence, perl = TRUE), "H",
            grepl(light_regex, PeptideSequence, perl = TRUE), "L",
            default = NA_character_
        )]
    }

    dt
}


#' Fix invalid intensities: infinite to NA, between 0 and 1 to 0
#' @param input data.table
#' @return data.table
#' @keywords internal
.adjustIntensities = function(input) {
    Intensity = isZero = NULL
    
    if (is.element("isZero", colnames(input))) {
        input[, isZero := ifelse(Intensity > 0 & Intensity <= 1, TRUE, isZero)]
    }
    input[, Intensity := ifelse(is.finite(Intensity), Intensity, NA)]
    input[, Intensity := ifelse(Intensity > 0 & Intensity <= 1, 0, Intensity)]
}

# Example data ----
## Without channels
to_filter = data.table::data.table(
    PeptideSequence = sort(rep(letters[1:20], times = 3)),
    PrecursorCharge = rep(rep(c(2, 3), each = 15), times = 2),
    Run = rep(rep(1:3, times = 5), times = 2),
    Intensity = seq(0.1, 10, length.out = 60)
)
## With channels
to_filter_channel = data.table::data.table(
    ProteinName = "A",
    PeptideSequence = sort(rep(rep(letters[1:10], times = 3), each = 5)),
    PrecursorCharge = 3,
    Run = rep(rep(1:3, each = 5), 10),
    Channel = rep(1:5, times = 30),
    Fraction = 1,
    Intensity = seq(0.1, 10, length.out = 150)
)
to_filter_channel$Intensity[1:6] = NA
to_filter_channel$PSM = paste(to_filter_channel$PeptideSequence,
                              to_filter_channel$PrecursorCharge,
                              sep = "_")
# Remove feature with few measurements ----
# Filtering for TMT
## Filter few works for TMT data - removes PSMs with less than 3 observations
## across channels
expect_equal(
    MSstatsConvert:::.filterFewMeasurements(
        data.table::copy(to_filter_channel[1:15]), 0, TRUE, c("PSM", "Run")),
    to_filter_channel[(PeptideSequence == "a") & Run %in% c(2, 3), ]
)
## Nothing is missing
expect_equal(
    nrow(MSstatsConvert:::.filterFewMeasurements(
        data.table::copy(to_filter)[, Intensity := 0.1], 0, TRUE, c("PeptideSequence", "PrecursorCharge"))),
    nrow(to_filter)
)
## With a threshold:
expect_equal(
    MSstatsConvert:::.filterFewMeasurements(
        data.table::copy(to_filter), 1, TRUE, c("PeptideSequence", "PrecursorCharge")),
    to_filter[!(PeptideSequence %in% c("a", "b"))]
)
## With missing values
to_filter$Intensity = ifelse(to_filter$PeptideSequence %in% c("a", "b", "c") &
                                 to_filter$Run %in% 1:2, NA, to_filter$Intensity)
to_filter$Intensity[3] = NA
## Remove only features that have all missing values
expect_equal(
    MSstatsConvert:::.filterFewMeasurements(
        data.table::copy(to_filter), 0, FALSE, c("PeptideSequence", "PrecursorCharge")),
    to_filter[PeptideSequence != "a", ]
)
## Remove all features with less than three non-missing values
expect_equal(
    MSstatsConvert:::.filterFewMeasurements(
        data.table::copy(to_filter), 0, TRUE, c("PeptideSequence", "PrecursorCharge")),
    to_filter[!(PeptideSequence %in% c("a", "b", "c"))]
)
# Aggregating / summarizing features ----
to_aggregate = data.table::data.table(PeptideSequence = "A", 
                                      Run = rep(1:3, each = 2),
                                      Intensity = 1:6)
to_aggregate$Intensity[5] = NA
expect_equal(
    MSstatsConvert:::.summarizeMultipleMeasurements(to_aggregate, max, c("PeptideSequence", "Run")),
    to_aggregate[c(2, 4, 6), ]
)
## Zeros are coded correctly
### 
with_zeros = data.table::data.table(
    ProteinName = "A",
    PeptideSequence = "a",
    FragmentIon = rep(c("precursor", "precursor [M+1]", "precursor [M+2]"),
                      times = 4),
    Run = rep(1:4, each = 3),
    Intensity = c(0, 2, 3, NA, NA, NA, 0, 0, NA, 0, 0, 0),
    isZero = c(TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE, FALSE,
               TRUE, TRUE, TRUE)
)
correct_result = data.table::data.table(
    PeptideSequence = "a",
    FragmentIon = NA,
    ProductCharge = NA,
    Run = 1:4,
    Intensity = c(5, 0, 0, 0),
    isZero = c(FALSE, FALSE, TRUE, TRUE),
    ProteinName = "A"
)
MSstatsConvert:::.checkDDA(with_zeros)
expect_equal(
    MSstatsConvert:::.summarizeMultipleMeasurements(with_zeros, sum, c("PeptideSequence", "Run")),
    correct_result[, -(2:3), with = FALSE]
)
expect_equal(
    MSstatsConvert:::.handleIsotopicPeaks(with_zeros, TRUE),
    correct_result[, c(7, 1, 4, 5, 6, 2, 3), with = FALSE]
)
expect_equal(
    MSstatsConvert:::.handleIsotopicPeaks(with_zeros, FALSE),
    with_zeros
)
## TMT data
to_aggregate_channel = data.table::copy(to_filter_channel)
to_aggregate_channel$Run = rep(1:6, each = 25)
to_aggregate_channel$PSM = paste(to_aggregate_channel$PeptideSequence,
                                 to_aggregate_channel$PrecursorCharge,
                                 rep(1:30, each = 5), sep = "_")
to_aggregate_channel$Intensity[1:15] = 1
to_aggregate_channel$Feature = MSstatsConvert:::.combine(to_aggregate_channel$PeptideSequence,
                                                         to_aggregate_channel$PrecursorCharge)
to_aggregate_channel[, n_psms := data.table::uniqueN(PSM), 
                     by = c("PeptideSequence", "PrecursorCharge")]
## PSM summarization works as expected
### Case 1: there is a score
to_aggregate_channel$Score = 1
to_aggregate_channel$Score[16:30] = rep(1:3, each = 5)
expect_equal(
    MSstatsConvert:::.summarizeMultiplePSMs(to_aggregate_channel[16:30, ], max),
    "b_3_6"
)
### Case 3: there is isolation interference
to_aggregate_channel$IsolationInterference = 1
to_aggregate_channel$IsolationInterference[31:45] = rep(1:3, each = 5)
expect_equal(
    MSstatsConvert:::.summarizeMultiplePSMs(to_aggregate_channel[31:45, ], max),
    "c_3_7"
)
### Case 4: there is IonsScore
to_aggregate_channel$IonsScore = 1
to_aggregate_channel$IonsScore[46:60] = rep(1:3, each = 5)
expect_equal(
    MSstatsConvert:::.summarizeMultiplePSMs(to_aggregate_channel[46:60, ], max),
    "d_3_12"
)
### Case 5: number of nonmissing values
to_aggregate_channel$Intensity[c(61:63, 66:67, 71)] = NA 
expect_equal(
    MSstatsConvert:::.summarizeMultiplePSMs(to_aggregate_channel[61:75, ], max),
    "e_3_15"
)
### Case 6: maximal intensity
to_aggregate_channel$Intensity[76:90] = 1:15
expect_equal(
    MSstatsConvert:::.summarizeMultiplePSMs(to_aggregate_channel[76:90, ], max),
    "f_3_18"
)
### Case 7: cannot decide - average intensities
to_aggregate_channel2 = data.table::copy(to_aggregate_channel[PeptideSequence == "b"][1:10])
to_aggregate_channel2$Intensity = 1
to_aggregate_channel2[, IsolationInterference := NULL]
to_aggregate_channel2[, IonsScore := NULL]
to_aggregate_channel2[, Score := NULL]
expect_equal(
    MSstatsConvert:::.summarizeMultiplePSMs(to_aggregate_channel2, max),
    NA
)
to_compare_agg_psm = to_aggregate_channel2[1:10, list(Intensity = mean(Intensity, na.rm = TRUE),
                                                      PSM = "b_3"),
                                           by = c("ProteinName", "PeptideSequence", "PrecursorCharge",
                                                  "Run", "Channel", "Fraction", "Feature")]
# to_compare_agg_psm$PSM = to_compare_agg_psm$Feature
expect_equal(
    MSstatsConvert:::.aggregatePSMstoPeptideIons(
        data.table::copy(to_aggregate_channel2[1:10, ]),
        c("PeptideSequence", "PrecursorCharge")),
    to_compare_agg_psm
)
## Aggregation works as expected
aggregated = MSstatsConvert:::.aggregatePSMstoPeptideIons(to_aggregate_channel, c("PeptideSequence", "PrecursorCharge"), summary_function = sum)
expect_equal(nrow(aggregated), 5 * (data.table::uniqueN(to_aggregate_channel[, .(PeptideSequence, PrecursorCharge, Run)])))
# Removing proteins with just a single feature ----
to_filter_channel2 = to_filter_channel[, !(colnames(to_filter_channel) %in% c("feature", "feature_count")), 
                                       with = FALSE]
dataset_tmt2 = rbind(to_filter_channel2, 
                     data.table::data.table(ProteinName = "B", PeptideSequence = "z",
                                            PSM = "z_3", PrecursorCharge = 3, Run = rep(1:3, each = 5),
                                            Fraction = 1, Channel = rep(1:5, times = 3), Intensity = 1))
## Do nothing
expect_equal(
    MSstatsConvert:::.handleSingleFeaturePerProtein(to_filter_channel2, FALSE),
    to_filter_channel2
)
## Filter when needed
expect_equal(
    MSstatsConvert:::.handleSingleFeaturePerProtein(to_filter_channel2, TRUE),
    data.table::setkey(to_filter_channel2[!(ProteinName == "B"), ], NULL)
)

# Test .classifyIsotopeLabelType ----
# DIANN mode: explicit SILAC-H / SILAC-L suffix tags
dt_diann = data.table::data.table(PeptideSequence = c(
    "PEPTIDEK(SILAC-K-H)",  # heavy
    "PEPTIDEK(SILAC-K-L)",  # light
    "PEPTIDEAC"             # neither -> NA
))
result_diann = MSstatsConvert:::.classifyIsotopeLabelType(
    dt_diann,
    heavy_regex = "\\(SILAC-(?:K)-H\\)",
    light_regex = "\\(SILAC-(?:K)-L\\)"
)
expect_equal(result_diann$IsotopeLabelType, c("H", "L", NA_character_))

# DIANN mode with multiple labeled amino acids (K and R)
dt_multi = data.table::data.table(PeptideSequence = c(
    "PEPTIDEK(SILAC-K-H)",  # heavy K
    "PEPTIDER(SILAC-R-H)",  # heavy R
    "PEPTIDEK(SILAC-K-L)",  # light K
    "PEPTIDER(SILAC-R-L)",  # light R
    "PEPTIDEAC"             # NA
))
result_multi = MSstatsConvert:::.classifyIsotopeLabelType(
    dt_multi,
    heavy_regex = "\\(SILAC-(?:K|R)-H\\)",
    light_regex = "\\(SILAC-(?:K|R)-L\\)"
)
expect_equal(result_multi$IsotopeLabelType, c("H", "H", "L", "L", NA_character_))

# Utility function ----
expect_equal(MSstatsConvert:::.combine(c("A", "B"), c("A", "B")),
             c("A_A", "B_B"))
# Top-level function works
clean_lf = MSstatsConvert:::.cleanByFeature(
    to_filter, 
    c("PeptideSequence", "PrecursorCharge"),
    list(remove_features_with_few_measurements = TRUE,
         summarize_multiple_psms = max))
clean_tmt = MSstatsConvert:::.cleanByFeature(
    to_filter_channel, 
    c("PeptideSequence", "PrecursorCharge"),
    list(remove_features_with_few_measurements = TRUE,
         summarize_multiple_psms = max))
expect_equal(class(clean_lf)[1], "data.table")
expect_equal(class(clean_tmt)[1], "data.table")

#' Import Spectronaut files
#'
#' @param input name of Spectronaut output, which is long-format. ProteinName, PeptideSequence, PrecursorCharge, FragmentIon, ProductCharge, IsotopeLabelType, Condition, BioReplicate, Run, Intensity, F.ExcludedFromQuantification are required. Rows with F.ExcludedFromQuantification=True will be removed.
#' @param annotation name of 'annotation.txt' data which includes Condition, BioReplicate, Run. If annotation is already complete in Spectronaut, use annotation=NULL (default). It will use the annotation information from input.
#' @param intensity Intensity column to use. Accepts legacy enum values
#'   \code{'PeakArea'} (default, uses F.PeakArea), \code{'NormalizedPeakArea'}
#'   (uses F.NormalizedPeakArea).  Can also be any raw
#'   Spectronaut column name passed as a string (e.g.
#'   \code{"FG.MS1Quantity"}); the column name is standardized internally.
#'   For protein turnover workflows the recommended default is
#'   \code{"FG.MS1Quantity"}.
#' @param peptideSequenceColumn Name of the Spectronaut column that contains the
#' peptide sequence.  Defaults to \code{"EG.ModifiedSequence"}. The value is 
#' standardized internally (dots and spaces removed) before column lookup.
#' @param heavyLabels Character list identifying the heavy isotope labels as it
#'   appears inside square brackets in the peptide sequence column, e.g.
#'   \code{c("Lys6")} matches peptides containing \code{[Lys6]}.  
#'   \code{c("Lys6", "Arg10")} matches peptides containing either \code{[Lys6]} or \code{[Arg10]}.
#'   Supports any novel label name reported by Spectronaut (e.g. \code{"Leu6"},
#'   \code{"Phe10"}).  When provided, peptides are
#'   classified as heavy (\code{IsotopeLabelType = "H"}), light
#'   (\code{IsotopeLabelType = "L"}), or unlabeled
#'   (\code{IsotopeLabelType = NA}) based on its labeled sequence.  When
#'   \code{NULL} (default) all peptides receive \code{IsotopeLabelType = "L"}.
#'   Useful for protein turnover experiments.
#' @param excludedFromQuantificationFilter Remove rows with F.ExcludedFromQuantification=TRUE Default is TRUE.
#' @param filter_with_Qvalue FALSE(default) will not perform any filtering. TRUE will filter out the intensities that have greater than qvalue_cutoff in EG.Qvalue column. Those intensities will be replaced with zero and will be considered as censored missing values for imputation purpose.
#' @param qvalue_cutoff Cutoff for EG.Qvalue. default is 0.01.
#' @param calculateAnomalyScores Default is FALSE. If TRUE, will run anomaly detection model and calculate anomaly scores for each feature. Used downstream to weigh measurements in differential analysis.
#' @param anomalyModelFeatures character vector of quality metric column names to be used as features in the anomaly detection model. List must not be empty if calculateAnomalyScores=TRUE.
#' @param anomalyModelFeatureTemporal character vector of temporal direction corresponding to columns passed to anomalyModelFeatures. Values must be one of: `mean_decrease`, `mean_increase`, `dispersion_increase`, or NULL (to perform no temporal feature engineering). Default is empty vector. If calculateAnomalyScores=TRUE, vector must have as many values as anomalyModelFeatures (even if all NULL).
#' @param removeMissingFeatures Remove features with missing values in more than this fraction of runs. Default is 0.5. Only used if calculateAnomalyScores=TRUE.
#' @param anomalyModelFeatureCount Feature selection for anomaly model. Anomaly detection works on the precursor-level and can be much slower if all features used. We will by default filter to the top-100 highest intensity features. This can be adjusted as necessary. To turn feature-selection off, set this value to a high number (e.g. 10000). Only used if calculateAnomalyScores=TRUE.
#' @param runOrder Temporal order of MS runs. Should be a two column data.table with columns `Run` and `Order`, where `Run` matches the run name output by Spectronaut and `Order` is an integer. Used to engineer the temporal features defined in anomalyModelFeatureTemporal.
#' @param n_trees Number of trees to use in isolation forest when calculateAnomalyScores=TRUE. Default is 100.
#' @param max_depth Max tree depth to use in isolation forest when calculateAnomalyScores=TRUE. Default is "auto" which calculates depth as log2(N) where N is the number of runs. Otherwise must be an integer.
#' @param numberOfCores Number of cores for parallel processing anomaly detection model. When > 1, a logfile named 'MSstats_anomaly_model_progress.log' is created to track progress. Only works for Linux & Mac OS. Default is 1.
#' @param ... additional parameters to `data.table::fread`.
#' @inheritParams .sharedParametersAmongConverters
#' 
#' @return data.frame in the MSstats required format.
#' 
#' @author Meena Choi, Olga Vitek
#' 
#' @export
#' 
#' @examples 
#' spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
#'                               package = "MSstatsConvert")
#' spectronaut_raw = data.table::fread(spectronaut_raw)
#' spectronaut_imported = SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE)
#' head(spectronaut_imported)
#' 
SpectronauttoMSstatsFormat = function(
        input, annotation = NULL,
        intensity = 'PeakArea',
        peptideSequenceColumn = "EG.ModifiedSequence",
        heavyLabels = NULL,
        excludedFromQuantificationFilter = TRUE,
        filter_with_Qvalue = FALSE, qvalue_cutoff = 0.01, 
        useUniquePeptide = TRUE, removeFewMeasurements=TRUE,
        removeProtein_with1Feature = FALSE,
        summaryforMultipleRows = max,
        calculateAnomalyScores=FALSE, anomalyModelFeatures=c(),
        anomalyModelFeatureTemporal=c(), removeMissingFeatures=.5,
        anomalyModelFeatureCount=100,
        runOrder=NULL, n_trees=100, max_depth="auto", numberOfCores=1, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, 
        log_file_path = NULL, ...
) {

    validation_config = list(
        input = input,
        annotation = annotation,
        intensity = intensity,
        excludedFromQuantificationFilter = excludedFromQuantificationFilter,
        filter_with_Qvalue = filter_with_Qvalue, 
        qvalue_cutoff = qvalue_cutoff, 
        useUniquePeptide = useUniquePeptide, 
        removeFewMeasurements = removeFewMeasurements,
        removeProtein_with1Feature = removeProtein_with1Feature, 
        summaryforMultipleRows = summaryforMultipleRows, 
        calculateAnomalyScores = calculateAnomalyScores,
        anomalyModelFeatures = anomalyModelFeatures, 
        anomalyModelFeatureTemporal = anomalyModelFeatureTemporal, 
        removeMissingFeatures = removeMissingFeatures,
        anomalyModelFeatureCount = anomalyModelFeatureCount, 
        runOrder = runOrder, 
        n_trees = n_trees, 
        max_depth = max_depth, 
        numberOfCores = numberOfCores,
        use_log_file = use_log_file, 
        append = append, 
        verbose = verbose, 
        log_file_path = log_file_path
    )
    
    .validateMSstatsConverterParameters(validation_config)
    
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    anomalyModelFeatures = .standardizeColnames(anomalyModelFeatures)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "Spectronaut", ...)

    input = MSstatsConvert::MSstatsClean(input, intensity = intensity,
                                         calculateAnomalyScores,
                                         anomalyModelFeatures,
                                         peptideSequenceColumn = peptideSequenceColumn,
                                         heavyLabels = heavyLabels)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    pq_filter = list(score_column = "PGQvalue", 
                     score_threshold = 0.01, 
                     direction = "smaller", 
                     behavior = "fill", 
                     handle_na = "keep", 
                     fill_value = NA_real_,
                     filter = filter_with_Qvalue, 
                     drop_column = TRUE)
    qval_filter = list(score_column = "EGQvalue", 
                       score_threshold = qvalue_cutoff, 
                       direction = "smaller", 
                       behavior = "fill", 
                       handle_na = "keep", 
                       fill_value = NA_real_, 
                       filter = filter_with_Qvalue, 
                       drop_column = TRUE)
    excluded_quant_filter = list(
        col_name    = "FExcludedFromQuantification",
        filter_symbols = TRUE,
        behavior    = "fill",
        fill_value  = NA_real_,
        filter      = excludedFromQuantificationFilter,
        drop_column = TRUE
    )
    
    feature_columns = c("PeptideSequence", "PrecursorCharge",
                        "FragmentIon", "ProductCharge")
    preprocess_feature_columns = if ("IsotopeLabelType" %in% colnames(input))
        c(feature_columns, "IsotopeLabelType") else feature_columns
    fill_isotope_label_type = if ("IsotopeLabelType" %in% colnames(input)) 
        list() else list("IsotopeLabelType" = "L") 
    
    input = MSstatsConvert::MSstatsPreprocess(
        input,
        annotation,
        preprocess_feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        feature_cleaning = list(remove_features_with_few_measurements = removeFewMeasurements,
                                summarize_multiple_psms = summaryforMultipleRows),
        score_filtering = list(pgq = pq_filter,
                               psm_q = qval_filter),
        exact_filtering = list(excluded_quant = excluded_quant_filter),
        columns_to_fill = fill_isotope_label_type,
        anomaly_metrics = anomalyModelFeatures)
    input[, Intensity := ifelse(Intensity == 0, NA, Intensity)]
    
    input = MSstatsConvert::MSstatsBalancedDesign(
        input, feature_columns, 
        remove_few = removeFewMeasurements,
        anomaly_metrics = anomalyModelFeatures)
    
    if (calculateAnomalyScores){
        input = MSstatsConvert::MSstatsAnomalyScores(
            input, anomalyModelFeatures, anomalyModelFeatureTemporal,
            removeMissingFeatures, anomalyModelFeatureCount, runOrder, n_trees, 
            max_depth, numberOfCores)
    }
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test SpectronauttoMSstatsFormat ---------------------------
spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
output = SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 372)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))
expect_true(all(spectronaut_raw$IsotopeLabelType == "L"))

# Test SpectronauttoMSstatsFormat Missing Columns ---------------------------
spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_raw$F.ExcludedFromQuantification = NULL
expect_silent(
    SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE)
)

spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_raw$F.FrgLossType = NULL
expect_silent(
    SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE)
)

spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_raw$PG.ProteinGroups = NULL
expect_silent(
    SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE)
)

spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_raw$PG.ProteinGroups = NULL
spectronaut_raw$PG.ProteinAccessions = NULL
expect_error(
    SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE),
    "The following columns are missing from the input data: PGProteinGroups"
)

spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_raw$FG.Charge = NULL
expect_error(
    SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE),
    "The following columns are missing from the input data: FGCharge"
)


# Test SpectronauttoMSstatsFormat Quality Metrics ---------------------------
spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_raw$`FG.ShapeQualityScore (MS2)` = 1
spectronaut_raw$`FG.ShapeQualityScore (MS1)` = 1
spectronaut_raw$`EG.ApexRT` = 1
spectronaut_raw$`F.PossibleInterference` = TRUE
temporal = spectronaut_raw[, .(R.Replicate, R.Condition), by = R.FileName]
temporal = unique(temporal)
data.table::setnames(temporal, "R.FileName", "Run")
temporal$Order = seq(1:nrow(temporal))
temporal = temporal[, c("Run", "Order")]

msstats_format = SpectronauttoMSstatsFormat(
    spectronaut_raw,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
)

expect_true("Run" %in% colnames(msstats_format))
expect_true("ProteinName" %in% colnames(msstats_format))
expect_true("PeptideSequence" %in% colnames(msstats_format))
expect_true("PrecursorCharge" %in% colnames(msstats_format))
expect_true("Intensity" %in% colnames(msstats_format))
expect_true("FragmentIon" %in% colnames(msstats_format))
expect_true("ProductCharge" %in% colnames(msstats_format))
expect_true("IsotopeLabelType" %in% colnames(msstats_format))
expect_true("Condition" %in% colnames(msstats_format))
expect_true("BioReplicate" %in% colnames(msstats_format))
expect_true("Fraction" %in% colnames(msstats_format))
expect_true("AnomalyScores" %in% colnames(msstats_format))
expect_true("FGShapeQualityScore(MS2)" %in% colnames(msstats_format))
expect_true("FGShapeQualityScore(MS1)" %in% colnames(msstats_format))
expect_true("EGApexRT" %in% colnames(msstats_format))
expect_true("FGShapeQualityScore(MS2).mean_decrease" %in% colnames(msstats_format))
expect_true("FGShapeQualityScore(MS1).mean_increase" %in% colnames(msstats_format))
expect_true("EGApexRT.dispersion_increase" %in% colnames(msstats_format))


# Spectronaut quality features error handling -------------------------------

# runOrder not provided
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = NULL,
    n_trees = 100,
    max_depth = "auto"
))

# runOrder is not a data frame
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = seq(1:nrow(annotation)),
    n_trees = 100,
    max_depth = "auto"
))

# n_trees is negative
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = -3,
    max_depth = "auto"
))

# FAIL n_trees is a decimal
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100.5,
    max_depth = "auto"
))

# n_trees is a string
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = "string",
    max_depth = "auto"
))

# max_depth is a string
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "string"
))

# FAIL max_depth is a decimal
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = 5.5
))

# FAIL max_depth is a negative number
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c("FG.ShapeQualityScore (MS2)",
                             "FG.ShapeQualityScore (MS1)",
                             "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = -5
))

# anomalyModelFeatures is empty
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures = c(),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
))

# anomalyModelFeatureTemporal contains unrecognized string
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures =c("FG.ShapeQualityScore (MS2)",
                            "FG.ShapeQualityScore (MS1)",
                            "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("string",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
))

# anomalyModelFeatureTemporal is empty
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures =c("FG.ShapeQualityScore (MS2)",
                            "FG.ShapeQualityScore (MS1)",
                            "EG.ApexRT"),
    anomalyModelFeatureTemporal = c(),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
))

# FAIL removeMissingFeatures is a percentage above 1
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures =c("FG.ShapeQualityScore (MS2)",
                            "FG.ShapeQualityScore (MS1)",
                            "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 1.5,
    anomalyModelFeatureCount = 100,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
))

# FAIL anomalyModelFeatureCount is not an integer
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures =c("FG.ShapeQualityScore (MS2)",
                            "FG.ShapeQualityScore (MS1)",
                            "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = 100.5,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
))

# FAIL anomalyModelFeatureCount is a string
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures =c("FG.ShapeQualityScore (MS2)",
                            "FG.ShapeQualityScore (MS1)",
                            "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = "string",
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
))

# anomalyModelFeatureCount is a negative number
expect_error(SpectronauttoMSstatsFormat(
    spectronaut_raw,
    annotation = annotation,
    calculateAnomalyScores = TRUE,
    anomalyModelFeatures =c("FG.ShapeQualityScore (MS2)",
                            "FG.ShapeQualityScore (MS1)",
                            "EG.ApexRT"),
    anomalyModelFeatureTemporal = c("mean_decrease",
                                    "mean_increase",
                                    "dispersion_increase"),
    removeMissingFeatures = 0.5,
    anomalyModelFeatureCount = -5,
    runOrder = temporal,
    n_trees = 100,
    max_depth = "auto"
))


# --- Heavy Label Testing --------------------------------

boxcar_path = system.file(
    "tinytest/raw_data/Spectronaut/boxcar_protein_turnover_input.csv",
    package = "MSstatsConvert")
boxcar_raw = data.table::fread(boxcar_path)

output_heavy = SpectronauttoMSstatsFormat(
    boxcar_raw,
    intensity    = "FG.MS1Quantity",
    peptideSequenceColumn = "FG.LabeledSequence",
    heavyLabel   = c("K[Lys6]"),
    use_log_file = FALSE
)

expect_true("Run"              %in% colnames(output_heavy))
expect_true("ProteinName"      %in% colnames(output_heavy))
expect_true("PeptideSequence"  %in% colnames(output_heavy))
expect_true("PrecursorCharge"  %in% colnames(output_heavy))
expect_true("Intensity"        %in% colnames(output_heavy))
expect_true("FragmentIon"      %in% colnames(output_heavy))
expect_true("ProductCharge"    %in% colnames(output_heavy))
expect_true("IsotopeLabelType" %in% colnames(output_heavy))
expect_true("Condition"        %in% colnames(output_heavy))
expect_true("BioReplicate"     %in% colnames(output_heavy))

expect_true(all(c("0d", "8d", "32d") %in% unique(output_heavy$Condition)))
expect_true(nrow(output_heavy) > 0)
expect_false(any(is.na(output_heavy$ProteinName)))
expect_true("H" %in% unique(output_heavy$IsotopeLabelType))
expect_true("L" %in% unique(output_heavy$IsotopeLabelType))
heavy_rows = subset(output_heavy, IsotopeLabelType == "H")
expect_true(all(grepl("K", heavy_rows$PeptideSequence, fixed = TRUE)))
light_rows = subset(output_heavy, IsotopeLabelType == "L")
expect_true(all(grepl("K", light_rows$PeptideSequence, fixed = TRUE)))
na_rows = subset(output_heavy, is.na(IsotopeLabelType))
expect_false(any(grepl("K", na_rows$PeptideSequence, fixed = TRUE)))
output_leu = SpectronauttoMSstatsFormat(
    boxcar_raw,
    peptideSequenceColumn = "FG.LabeledSequence",
    intensity    = "FG.MS1Quantity",
    heavyLabel   = c("L[Leu6]"),
    use_log_file = FALSE
)
expect_false("H" %in% unique(output_leu$IsotopeLabelType))

# Verify intensity values for ANGYTTEYSASVK are preserved from FG.MS1Quantity
output_heavy = data.table::as.data.table(output_heavy)
angyt_input_intensities = sort(
    boxcar_raw[PEP.StrippedSequence == "ANGYTTEYSASVK", FG.MS1Quantity])
angyt_output_intensities = sort(
    output_heavy[grepl("ANGYTTEYSASVK", PeptideSequence, fixed = TRUE), Intensity])
expect_equivalent(angyt_input_intensities, angyt_output_intensities)

# Verify row count for ANGYTTEYSASVK is 2 * number of bioreplicates
# (one row per bioreplicate for heavy label, one for light label)
n_bioreplicates = length(unique(boxcar_raw$R.Replicate))
angyt_rows = subset(output_heavy, grepl("ANGYTTEYSASVK", PeptideSequence, fixed = TRUE))
expect_equal(nrow(angyt_rows), 2L * n_bioreplicates)

#' Handle PSM/proteins scores
#' @param input `data.table` preprocessed by one of the .cleanRaw* functions.
#' @param score_filtering list of by-score filtering controls.
#' @param exact_filtering list of exact filtering controls.
#' @param pattern_filtering list of by-pattern filtering controls.
#' @return data.table
#' @keywords internal
.handleFiltering = function(input, score_filtering, exact_filtering,
                            pattern_filtering) {
    for (filtering in exact_filtering) {
        input = .filterExact(input, filtering[["col_name"]], 
                             filtering[["filter_symbols"]],
                             filtering[["behavior"]], filtering[["fill_value"]],
                             filtering[["filter"]], filtering[["drop_column"]])
    }
    for (filtering in score_filtering) {
        input = .filterByScore(input, filtering[["score_column"]], 
                               filtering[["score_threshold"]], 
                               filtering[["direction"]], filtering[["behavior"]], 
                               filtering[["handle_na"]], filtering[["fill_value"]],
                               filtering[["filter"]], filtering[["drop_column"]])
    }
    for (filtering in pattern_filtering) {
        input = .filterByPattern(input, filtering[["col_name"]], 
                                 filtering[["pattern"]], 
                                 filtering[["filter"]], 
                                 filtering[["drop_column"]])
    }
    input
}


#' Filter PSMs / proteins by a given score column.
#' @param input `data.table` preprocessed by one of the .cleanRaw* functions.
#' @param score_column chr, name of the column that contains scores.
#' @param score_threshold num, values below or above this threshold will be
#' removed from the data.
#' @param direction chr, if "greater" only values above the threshold will be
#' retained, if "smaller" - below the threshold.
#' @param behavior chr, if "remove", values below/above the threshold will be 
#' removed, if "replace", they will be set to `fill_value`.
#' @param fill_value if `behavior` = "replace", values below/above the threshold
#' will be replaced with `fill_value`. Defaults to `NA`.
#' @param filter If TRUE, filtering will be performed.
#' @param drop if TRUE, `score_column` will be removed.
#' @return data.table
#' @keywords internal
.filterByScore = function(input, score_column, score_threshold, direction,
                          behavior, handle_na = "keep", fill_value = NA,
                          filter = TRUE, drop = TRUE) {
    if (!is.element(score_column, colnames(input))) {
        msg = paste("**", score_column, "not found in input columns.")
        getOption("MSstatsLog")("WARN", msg)
        getOption("MSstatsMsg")("WARN", msg)
        return(input)
    } else {
        if (filter) {
            if (direction == "greater") {
                score_filter = input[[score_column]] >= score_threshold
            } else {
                score_filter = input[[score_column]] <= score_threshold
            }
            
            if (handle_na == "keep") {
                score_filter = score_filter | is.na(input[[score_column]])
            } else{
                score_filter = score_filter & !is.na(input[[score_column]])
            }
            
            if (behavior == "remove") {
                input = input[score_filter, ]    
            } else {
                input[!score_filter, "Intensity"] = fill_value
            }
            
            msg = .makeScoreFilterMessage(score_column, score_threshold, direction,
                                          behavior, fill_value)
            getOption("MSstatsLog")("INFO", msg)
            getOption("MSstatsMsg")("INFO", msg)
        }
        
        if (drop) {
            input = input[, !(colnames(input) == score_column), with = FALSE]
        }
    }
    input
}


#' Handle filtering by pattern
#' @param input `data.table` preprocessed by one of the .cleanRaw* functions.
#' @param col_name chr, name of the column with peptide sequences.
#' @param patterns chr, regular expression - matching peptides will be 
#' removed from the data.
#' @param filter lgl, if TRUE, peptides will be actually filtered.
#' @param drop lgl, if TRUE, the `column` will be dropped.
#' @return data.table
#' @keywords internal
.filterByPattern = function(input, col_name, patterns, filter, drop) {
    if (!is.element(col_name, colnames(input))) {
        msg = paste(col_name, "not found in input columns.")
        getOption("MSstatsLog")("WARN", msg)
        getOption("MSstatsMsg")("WARN", msg)
        return(input)
    } else {
        if (filter) {
            msg = paste("** Sequences containing", 
                        paste(patterns, sep = ", ", collapse = ", "), "are removed.")
            getOption("MSstatsLog")("INFO", msg)
            getOption("MSstatsMsg")("INFO", msg)
            pattern_filter = rep(TRUE, nrow(input))
            for (pattern in patterns) {
                pattern_filter = pattern_filter & !grepl(pattern, input[[col_name]])
            }
        } else {
            pattern_filter = rep(TRUE, nrow(input))
        }
        
        if (drop) {
            input[pattern_filter, !(colnames(input) == col_name), with = FALSE]
        } else {
            input[pattern_filter, ]
        }
    }
    
}


#' Filter out specified symbols.
#' @param input data.table preprocessed by one of the .cleanRaw* functions.
#' @param col_name chr, name of the column that will be the base for filtering
#' @param filter_symbols character vector of symbols that will be removed
#' @param behavior chr, if "remove", values below/above the threshold will be 
#' removed, if "replace", they will be set to `fill_value`.
#' @param fill_value if `behavior` = "replace", values below/above the threshold
#' will be replaced with `fill_value`. Defaults to `NA`.
#' @param filter lgl, if TRUE, decoy proteins will be removed from the data.
#' @param drop lgl, if TRUE, column that contains decoy proteins will be dropped.
#' @return data.table
#' @keywords internal
.filterExact = function(input, col_name, filter_symbols, behavior, 
                        fill_value, filter, drop
) {
    if (!is.element(col_name, colnames(input))) {
        msg = paste(col_name, "not found in input columns.")
        getOption("MSstatsLog")("WARN", msg)
        getOption("MSstatsMsg")("WARN", msg)
        return(input)
    } else {
        find_col = colnames(input) == col_name
        if (filter) {
            exact_filter = !(input[[col_name]] %in% filter_symbols)
            msg = .makeExactFilterMessage(col_name, filter_symbols, 
                                          behavior, fill_value)
            getOption("MSstatsLog")("INFO", msg)
            getOption("MSstatsMsg")("INFO", msg)
        } else {
            exact_filter = rep(TRUE, nrow(input))
        }
        if (behavior == "remove") {
            input = input[exact_filter]
        } else {
            input$Intensity = ifelse(exact_filter, input$Intensity, fill_value)
        }
        if (drop) {
            input = input[, !find_col, with = FALSE]    
        }
        input
    }
}


#' Filter rows that contain specifed symbols in multiple columns.
#' @param input data.table preprocessed by one of the `cleanRaw*` functions.
#' @param filter_columns chr, names of columns in which elements will be matched
#' and removed.
#' @param filter_symbols chr, vector of strings. Rows with corresponding elements 
#' in `filter_columns` will be removed.
#' @return data.table
#' @keywords internal
.filterManyColumns = function(input, filter_columns, filter_symbols) {
    for (col in intersect(filter_columns, colnames(input))) {
        has_col = is.element(col, colnames(input))
        input = .filterExact(input, col, filter_symbols, "remove", NULL,
                             has_col, has_col)        
    }
    input
}

# Ceate data ----
data_to_filter = data.table::data.table(
    ProteinName = rep(LETTERS[1:5], 10),
    PeptideSequence = rep(letters[1:10], 5),
    PeptideSequenceModified = rep(letters[1:10], 5),
    Intensity = seq(0, 1, length.out = 50),
    Score = seq(0, 1, length.out = 50),
    Symbol_1 = c(rep("+", 25), rep("-", 25)),
    Symbol_2 = c(rep("+", 25), rep("-", 25))
)
data_to_filter$Score[c(1, 13, 15, 27, 38)] = NA
# Filter by pattern ----
## Warn when the column isn't there
expect_stdout(
    MSstatsConvert:::.filterByPattern(data_to_filter, "Not_there", "\\+", TRUE, FALSE)
)
## Filter and don't drop the column
expect_identical(
    MSstatsConvert:::.filterByPattern(data_to_filter, "Symbol_1", "\\+", TRUE, FALSE),
    data_to_filter[26:50, ]
)
## Filter and drop the column
expect_identical(
    MSstatsConvert:::.filterByPattern(data_to_filter, "Symbol_1", "\\+", TRUE, TRUE),
    data_to_filter[26:50, -6]
)
## Don't filter and don't drop the column
expect_identical(
    MSstatsConvert:::.filterByPattern(data_to_filter, "Symbol_1", "\\+", FALSE, FALSE),
    data_to_filter
)
# Don't filter but drop the column 
expect_identical(
    MSstatsConvert:::.filterByPattern(data_to_filter, "Symbol_1", "\\+", FALSE, TRUE),
    data_to_filter[, -6]
)
# Filter by exact values ----
## Warn when the column isn't there
expect_stdout(MSstatsConvert:::.filterExact(data_to_filter, "Not_there", 
                                             "-", "remove", NULL, TRUE, TRUE))
## Filter and drop column
expect_identical(MSstatsConvert:::.filterExact(data_to_filter, "Symbol_1", 
                                               "-", "remove", NULL, TRUE, TRUE),
                 data_to_filter[1:25, -6])
## Filter so that nothing's left
expect_identical(MSstatsConvert:::.filterExact(data_to_filter, "Symbol_1", 
                                               c("+", "-"), "remove", NULL, TRUE, TRUE),
                 data_to_filter[FALSE, -6])
## Filter, but don't drop the column
expect_identical(MSstatsConvert:::.filterExact(data_to_filter, "Symbol_1", 
                                               "+", "remove", NULL, TRUE, FALSE),
                 data_to_filter[26:50, ])
## Don't filter, but drop column
expect_identical(MSstatsConvert:::.filterExact(data_to_filter, "Symbol_1", 
                                               c("+", "-"), "remove", NULL, FALSE, TRUE),
                 data_to_filter[, -6])
## Don't filter and don't drop column
expect_identical(MSstatsConvert:::.filterExact(data_to_filter, "Symbol_1", 
                                               c("+", "-"), "remove", NULL, FALSE, FALSE),
                 data_to_filter)
## Filter and drop, but the symbol is not there
expect_identical(MSstatsConvert:::.filterExact(data_to_filter, "Symbol_1", 
                                               "R", "remove", 
                                               NULL, TRUE, TRUE),
                 data_to_filter[, -6])
# Filter multiple columns by exact values ----
## Warn when the column isn't there
expect_stdout(
    MSstatsConvert:::.filterManyColumns(data_to_filter, c("Not_there", "Not_there"), "+")
)
## Filter
expect_identical(
    MSstatsConvert:::.filterManyColumns(data_to_filter, c("Symbol_1", "Symbol_2"), "+"),
    data_to_filter[26:50, 1:5]
)
## When the values aren't there just drop columns
expect_identical(
    MSstatsConvert:::.filterManyColumns(data_to_filter, c("Symbol_1", "Symbol_2"), "X"),
    data_to_filter[, 1:5]
)
# Filter by a numerical score ----
## Warn when the column is not there
expect_stdout(MSstatsConvert:::.filterByScore(data_to_filter, "Not_there", 0.3, "smaller",
                                               "fill", handle_na = "keep", fill_value = NA,
                                               filter = TRUE, drop = TRUE)
)
## Don't filter and don't drop
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                 "remove", handle_na = "keep", fill_value = NA,
                                                 filter = FALSE, drop = FALSE),
                 data_to_filter)
## Don't filter but drop
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                 "remove", handle_na = "keep", fill_value = NA,
                                                 filter = FALSE, drop = TRUE),
                 data_to_filter[, -5])
## Keep only scores bigger than 0.3, don't drop the column, keep NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                 "remove", handle_na = "keep", fill_value = NA,
                                                 filter = TRUE, drop = FALSE),
                 data_to_filter[Score > 0.3 | is.na(Score), ])
## Keep only scores bigger than 0.3, don't drop the column, remove NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                 "remove", handle_na = "remove", fill_value = NA,
                                                 filter = TRUE, drop = FALSE),
                 data_to_filter[Score > 0.3 & !is.na(Score), ])
## Keep only scores bigger than 0.3, drop the column, keep NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                 "remove", handle_na = "keep", fill_value = NA,
                                                 filter = TRUE, drop = TRUE),
                 data_to_filter[Score > 0.3 | is.na(Score), -5])
## Keep only scores bigger than 0.3, drop the column, remove NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                 "remove", handle_na = "remove", fill_value = NA,
                                                 filter = TRUE, drop = TRUE),
                 data_to_filter[Score > 0.3 & !is.na(Score), -5])
## Keep only scores smaller than 0.3, don't drop the column, keep NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "smaller",
                                                 "remove", handle_na = "keep", fill_value = NA,
                                                 filter = TRUE, drop = FALSE),
                 data_to_filter[Score < 0.3 | is.na(Score), ])
## Keep only scores smaller than 0.3, don't drop the column, remove NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "smaller",
                                                 "remove", handle_na = "remove", fill_value = NA,
                                                 filter = TRUE, drop = FALSE),
                 data_to_filter[Score < 0.3 & !is.na(Score), ])
## Keep only scores smaller than 0.3, drop the column, keep NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "smaller",
                                                 "remove", handle_na = "keep", fill_value = NA,
                                                 filter = TRUE, drop = TRUE),
                 data_to_filter[Score < 0.3 | is.na(Score), -5])
## Keep only scores smaller than 0.3, drop the column, remove NA
expect_identical(MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "smaller",
                                                 "remove", handle_na = "remove", fill_value = NA,
                                                 filter = TRUE, drop = TRUE),
                 data_to_filter[Score < 0.3 & !is.na(Score), -5])
## Keep only scores greater than 0.3, replace other Intensities with NA,
## drop the column, keep NA
fill_smaller_than_0_3 = MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                        "fill", handle_na = "keep", fill_value = NA,
                                                        filter = TRUE, drop = TRUE)
expect_identical(ncol(fill_smaller_than_0_3), 6L)
expect_identical(fill_smaller_than_0_3$Intensity, 
                 ifelse(data_to_filter$Score > 0.3 | is.na(data_to_filter$Score),
                        data_to_filter$Intensity, NA))
## Keep only scores bigger than 0.3, drop the column, remove NA, 
## replace other Intensities with NA
fill_smaller_than_0_3_2 = MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "greater",
                                                          "fill", handle_na = "remove", fill_value = NA,
                                                          filter = TRUE, drop = TRUE)
expect_identical(ncol(fill_smaller_than_0_3_2), 6L)
expect_identical(fill_smaller_than_0_3_2$Intensity, 
                 ifelse(data_to_filter$Score > 0.3 & !is.na(data_to_filter$Score),
                        data_to_filter$Intensity, NA))

## Keep only scores smaller than 0.3, replace other Intensities with NA,
## drop the column, keep NA
fill_smaller_than_0_3_s = MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "smaller",
                                                          "fill", handle_na = "keep", fill_value = NA,
                                                          filter = TRUE, drop = TRUE)
expect_identical(ncol(fill_smaller_than_0_3_s), 6L)
expect_identical(fill_smaller_than_0_3_s$Intensity, 
                 ifelse(data_to_filter$Score < 0.3 | is.na(data_to_filter$Score),
                        data_to_filter$Intensity, NA))
## Keep only scores smaller than 0.3, drop the column, remove NA, 
## replace other Intensities with NA
fill_smaller_than_0_3_2_s = MSstatsConvert:::.filterByScore(data_to_filter, "Score", 0.3, "smaller",
                                                            "fill", handle_na = "remove", fill_value = NA,
                                                            filter = TRUE, drop = TRUE)
expect_identical(ncol(fill_smaller_than_0_3_2_s), 6L)
expect_identical(fill_smaller_than_0_3_2_s$Intensity, 
                 ifelse(data_to_filter$Score < 0.3 & !is.na(data_to_filter$Score),
                        data_to_filter$Intensity, NA))
# Full filtering function ----
## Nothing to filter
expect_equal(
    MSstatsConvert:::.handleFiltering(data_to_filter, 
                                      list(), 
                                      list(), 
                                      list()),
    data_to_filter
)
## Filter and drop
expect_identical(
    MSstatsConvert:::.handleFiltering(data_to_filter, 
                                      list(list(score_column = "Score", score_threshold = 0.3, 
                                                direction = "greater", behavior = "remove", 
                                                handle_na = "keep", fill_value = NA,
                                                filter = FALSE, drop_column = TRUE)), 
                                      list(list(col_name = "Symbol_1", 
                                                filter_symbols = "+", filter = TRUE,
                                                behavior = "remove", fill_value = NULL,
                                                drop_column = TRUE)),
                                      list(list(col_name = "Symbol_2", pattern = "\\+", 
                                                filter = TRUE, drop_column = TRUE))),
    data_to_filter[(Score > 0.3 | is.na(Score)) & Symbol_1 == "-", 1:4]
)
## Filter, but don't drop
expect_equal(
    MSstatsConvert:::.handleFiltering(data_to_filter, 
                                      list(list(score_column = "Score", score_threshold = 0.3, 
                                                direction = "greater", behavior = "remove", 
                                                handle_na = "keep", fill_value = NA,
                                                filter = FALSE, drop_column = TRUE)), 
                                      list(list(col_name = "Symbol_1", 
                                                filter_symbols = "+", filter = TRUE, 
                                                behavior = "remove", fill_value = NULL,
                                                drop_column = FALSE)),
                                      list(list(col_name = "Symbol_1", pattern = "\\+", 
                                                filter = TRUE, drop_column = TRUE))),
    data_to_filter[(Score > 0.3 | is.na(Score)) & Symbol_1 == "-", c(1:4, 7)]
)

#' Clean raw output from MaxQuant
#' @param msstats_object object that inherits from MSstatsInputFiles class.
#' @param protein_id_col character, name of a column with names of proteins.
#' @param remove_by_site logical, if TRUE, proteins only identified by site will
#' be removed.
#' @param channel_columns character, regular expression that identifies channel columns
#' in TMT data.
#' @return data.table
#' @keywords internal
.cleanRawMaxQuant = function(msstats_object, protein_id_col, 
                             remove_by_site = FALSE,
                             channel_columns = "Reporterintensitycorrected"
) {
  ProteinIDs = id = PSM = PeptideSequence = NULL
  ProteingroupIDs = PrecursorCharge = Intensity = NULL
  
  mq_input = getInputFile(msstats_object, "evidence")
  mq_pg = getInputFile(msstats_object, "protein_groups")
  
  filter_cols = c("Contaminant", "Potentialcontaminant", "Reverse")
  msg = paste("** + Contaminant, + Reverse, + Potential.contaminant",
              "proteins are removed.")
  if (remove_by_site) {
    filter_cols = c(filter_cols, "Onlyidentifiedbysite")
    msg = paste("** + Contaminant, + Reverse, + Potential.contaminant,", 
                "+ Only.identified.by.site proteins are removed.")
  }
  
  mq_input = .filterManyColumns(mq_input, filter_cols, "+")
  mq_pg = .filterManyColumns(mq_pg, filter_cols, "+")
  getOption("MSstatsLog")("INFO", msg)
  getOption("MSstatsMsg")("INFO", msg)
  
  suppressWarnings({
    mq_input[, ProteingroupIDs := as.integer(as.character(ProteingroupIDs))]
  })
  
  if (getDataType(msstats_object) == "MSstats") {
    mq_input = mq_input[ProteingroupIDs %in% unique(mq_pg[["id"]]), ]
  }
  
  mq_input = merge(mq_input, 
                   unique(mq_pg[, list(uniquefromProteinGroups = ProteinIDs,
                                       ProteingroupIDs = id)]),
                   by = "ProteingroupIDs", sort = FALSE)
  
  if (getDataType(msstats_object) == "MSstatsTMT") {
    protein_id = .findAvailable(c("Proteins", "Leadingproteins", 
                                  "Leadingrazorprotein", "Genenames"),
                                protein_id_col, "Proteins")
    channels = .getChannelColumns(colnames(mq_input), channel_columns)
  } else {
    protein_id = ifelse(protein_id_col == "Proteins", 
                        "uniquefromProteinGroups",
                        "Leadingrazorprotein")
    channels = character(0)
  }
  data.table::setnames(
    mq_input, 
    c(protein_id, "Modifiedsequence", "Charge", "Rawfile"), 
    c("ProteinName", "PeptideSequence", "PrecursorCharge", "Run"),
    skip_absent = TRUE)
  mq_input[["PeptideSequence"]] = gsub("_", "", mq_input[["PeptideSequence"]])
  mq_cols = c("ProteinName", "PeptideSequence", "Modifications", 
              "PrecursorCharge", "Run", "Intensity", 
              "Fraction", "TechReplicate", "Run", "BioReplicate",
              "PSM", "Score")
  mq_cols = intersect(c(mq_cols, channels),
                      colnames(mq_input))
  mq_input = mq_input[, mq_cols, with = FALSE]
  mq_input = unique(mq_input)
  
  if (getDataType(msstats_object) == "MSstatsTMT") {
    mq_input[, PSM := paste(PeptideSequence, PrecursorCharge, 
                            1:nrow(mq_input), sep = "_")]
    mq_input = melt(mq_input, measure.vars = channels,
                    id.vars = c("ProteinName", "PeptideSequence", 
                                "PrecursorCharge", "PSM", "Run", "Score"),
                    variable.name = "Channel", value.name = "Intensity")
    mq_input$Channel = gsub(channel_columns, "channel", mq_input$Channel)
    mq_input$Channel = .standardizeColnames(mq_input$Channel)
    suppressWarnings({
      mq_input$Intensity = ifelse(mq_input$Intensity == 0, NA,
                                  mq_input$Intensity)
    })
    mq_input = .filterFewMeasurements(mq_input, 0, FALSE, 
                                      c("PeptideSequence", 
                                        "PrecursorCharge", "Run"))
  }
  
  mq_input = mq_input[!is.na(Intensity), ]
  .logSuccess("MaxQuant", "clean")
  mq_input
}

.checkMissing = function(input){
    
    missing_percent = sum(is.na(input$Intensity)) / nrow(input)
    msg = paste0(as.character(round(missing_percent,4)*100),
                "% of the values are missing across all intensities")
    getOption("MSstatsMsg")("INFO", msg)
    
    return(missing_percent)
}

.checkIntensityDistribution = function(input){
    log_intensities = log2(input$Intensity)
    log_intensities = log_intensities[!is.na(log_intensities)]
    mean_int = mean(log_intensities, na.rm=TRUE) #quantile
    sd_int = sd(log_intensities, na.rm=TRUE)
    
    if (mean_int - (2*sd_int) < 0){
        below_zero_warning = TRUE
        
        msg = paste0("Intensity distribution indicates zero truncated data.", 
                     " More missing values than normal should be expected.")
        getOption("MSstatsMsg")("INFO", msg)
    } else {
        below_zero_warning = FALSE
        
        msg = paste0("Intensity distribution standard truncated data.")
        getOption("MSstatsMsg")("INFO", msg)
    }
    
    return(below_zero_warning)
    
}

.checkFeatureSD = function(input){
    sd_data = input[, .(sd_Intensity = sd(log2(Intensity), na.rm = TRUE),
                        mean_Intensity = mean(log2(Intensity), na.rm = TRUE),
                        ratio = sd(Intensity, na.rm = TRUE) / mean(
                            Intensity, na.rm = TRUE)), 
                    by = Feature]
    
    high_ratio = nrow(sd_data[sd_data$ratio > .5]) / nrow(sd_data) # by condition
    msg = paste0(as.character(round(high_ratio,4)*100),
                  "% of features have a ratio of standard deviation to mean > 0.5.",
                 " A high value here could indicate problematic measurements.")
    getOption("MSstatsMsg")("INFO", msg)
    
    return(sd_data)
}

.checkFeatureOutliers = function(input, feature_data){
    
    full_dt = merge(input, feature_data, by = "Feature", all.x = TRUE)
    full_dt[, is_outlier := abs(
        log2(Intensity) - mean_Intensity) > 2 * sd_Intensity] # switch to quantile
    percent_outlier = sum(full_dt$is_outlier == TRUE, na.rm=TRUE
        ) / nrow(full_dt[!is.na(full_dt$Intensity)])
    
    outlier_info = full_dt[, .(outliers = sum(is_outlier, na.rm=TRUE)), 
                           by = Feature]
    feature_data = merge(feature_data, outlier_info, 
                         by = "Feature", all.x = TRUE)
    
    msg = paste0(as.character(round(percent_outlier,4)*100),
                 "% of all measured intensities fall within the outlier range")
    getOption("MSstatsMsg")("INFO", msg)
    
    return(list(feature_data, percent_outlier))
}

.checkFeatureCoverage = function(input, feature_data){
    n_run = length(unique(input$Run))
    missing_check = input[, .(percent_missing = sum(is.na(Intensity)) / n_run), 
                          by=Feature]
    feature_data = merge(feature_data, missing_check, 
                         by = "Feature", all.x = TRUE)
    
    high_missing = nrow(feature_data[feature_data$percent_missing > .5]
                        ) / nrow(feature_data)
    msg = paste0(as.character(round(high_missing, 4)*100),
                 "% of features are missing in more than half the runs.",
                 " These features may need to be removed.")
    getOption("MSstatsMsg")("INFO", msg)
    
    return(feature_data)
}

pearson_skewness = function(x) {
    n = length(x)
    mean_x = mean(x)
    sd_x = sd(x)  # sample standard deviation
    skewness = sum((x - mean_x)^3) / n / (sd_x^3)
    return(skewness)
}

.checkAnomalySkew = function(input){
    
    input$PSM = paste(input$PeptideSequence,
                      input$PrecursorCharge, sep="_")

    skew_results = input[, .(skew = pearson_skewness(AnomalyScores)), by = PSM]
    
    return(skew_results)
    
}

# Test data setup
create_test_data = function() {
    data.table::data.table(
        ProteinName = c(rep("Q9UFW8", 10), rep("Q96S19", 15)),
        PeptideSequence = c(rep("AEFEEQNVR", 5), rep("TALYVTPLDR", 5),
                            rep("AFPLAEWQPSDVDQR", 5), rep("ASGLLLER", 5),
                            rep("LowAbundancePeptide", 5)),
        PrecursorCharge = rep(2, 25),
        FragmentIon = rep("y3", 25),
        ProductCharge = rep(1, 25),
        IsotopeLabelType = rep("L", 25),
        Condition = rep(c("A", "A", "A", "B", "B"), 5),
        BioReplicate = rep(seq(1:5), 5),
        Run = rep(paste0("Run", seq(1:5)), 5),
        Intensity = c(1000, 1500, 2000, 2500, 3000,
                      1100, 1600, 2100, 2600, 3100,
                      1200, 1700, 2200, 2000, 1700,
                      1300, 1800, 1200, 2800, 1800,
                      100, 200, 300, 400, 500),
        AnomalyScores = rep(c(0.01,0.01,0.01,0.7,0.5), 5)
    )
}

# Test .checkMissing function
test_data = create_test_data()

# Test with no missing values
result = MSstatsConvert:::.checkMissing(test_data)
expect_equal(result, 0, info = ".checkMissing returns 0 for no missing values")
expect_true(is.numeric(result), info = ".checkMissing returns numeric value")

# Test with some missing values
test_data_missing = test_data
test_data_missing$Intensity[1:5] = NA
result_missing = MSstatsConvert:::.checkMissing(test_data_missing)
expect_equal(result_missing, 5/25, info = ".checkMissing correctly calculates missing percentage")

# Test with all missing values
test_data_all_missing = test_data
test_data_all_missing$Intensity = NA
result_all_missing = MSstatsConvert:::.checkMissing(test_data_all_missing)
expect_equal(result_all_missing, 1, info = ".checkMissing returns 1 for all missing values")

# Test .checkIntensityDistribution function
test_data = create_test_data()

# Test normal distribution
result = MSstatsConvert:::.checkIntensityDistribution(test_data)
expect_true(is.logical(result), info = ".checkIntensityDistribution returns logical value")

# Test with low intensities that would trigger zero truncation warning
test_data_low = test_data
test_data_low$Intensity = c(rep(1, 10), rep(2, 15))
result_low = MSstatsConvert:::.checkIntensityDistribution(test_data_low)
expect_true(is.logical(result_low), info = ".checkIntensityDistribution handles low intensity data")

# Test .checkFeatureSD function
test_data = create_test_data()
test_data$Feature = paste(test_data$PeptideSequence,
                           test_data$PrecursorCharge,
                           test_data$FragmentIon,
                           test_data$ProductCharge, sep="_")

result = MSstatsConvert:::.checkFeatureSD(test_data)

# Check structure
expect_true(inherits(result, "data.table"), info = ".checkFeatureSD returns data.table")
expect_true("Feature" %in% names(result), info = ".checkFeatureSD includes Feature column")
expect_true("sd_Intensity" %in% names(result), info = ".checkFeatureSD includes sd_Intensity column")
expect_true("mean_Intensity" %in% names(result), info = ".checkFeatureSD includes mean_Intensity column")
expect_true("ratio" %in% names(result), info = ".checkFeatureSD includes ratio column")

# Check that we get one row per unique feature
expected_features = unique(test_data$Feature)
expect_equal(nrow(result), length(expected_features), info = ".checkFeatureSD returns one row per feature")

# Check that calculations make sense
expect_true(all(result$mean_Intensity > 0, na.rm = TRUE), info = ".checkFeatureSD mean intensities are positive")
expect_true(all(result$sd_Intensity >= 0, na.rm = TRUE), info = ".checkFeatureSD standard deviations are non-negative")
expect_true(all(result$ratio >= 0, na.rm = TRUE), info = ".checkFeatureSD ratios are non-negative")

# Test .checkFeatureOutliers function
test_data = create_test_data()
test_data$Feature = paste(test_data$PeptideSequence,
                           test_data$PrecursorCharge,
                           test_data$FragmentIon,
                           test_data$ProductCharge, sep="_")

feature_data = MSstatsConvert:::.checkFeatureSD(test_data)
result = MSstatsConvert:::.checkFeatureOutliers(test_data, feature_data)

# Check structure
expect_true(is.list(result), info = ".checkFeatureOutliers returns list")
expect_equal(length(result), 2, info = ".checkFeatureOutliers returns list of length 2")

enhanced_feature_data = result[[1]]
outlier_percent = result[[2]]

expect_true("outliers" %in% names(enhanced_feature_data), info = ".checkFeatureOutliers adds outliers column")
expect_true(inherits(enhanced_feature_data, "data.table"), info = ".checkFeatureOutliers returns data.table as first element")
expect_true(is.numeric(outlier_percent), info = ".checkFeatureOutliers returns numeric outlier percentage")
expect_true(outlier_percent >= 0 && outlier_percent <= 1, info = ".checkFeatureOutliers outlier percentage is between 0 and 1")

# Test .checkFeatureCoverage function
test_data = create_test_data()
test_data$Feature = paste(test_data$PeptideSequence,
                           test_data$PrecursorCharge,
                           test_data$FragmentIon,
                           test_data$ProductCharge, sep="_")

feature_data = MSstatsConvert:::.checkFeatureSD(test_data)
result = MSstatsConvert:::.checkFeatureCoverage(test_data, feature_data)

# Check structure
expect_true(inherits(result, "data.table"), info = ".checkFeatureCoverage returns data.table")
expect_true("percent_missing" %in% names(result), info = ".checkFeatureCoverage includes percent_missing column")

# With no missing values, percent_missing should be 0
expect_true(all(result$percent_missing == 0), info = ".checkFeatureCoverage shows 0% missing for complete data")

# Test with missing values
test_data_missing = test_data
test_data_missing$Intensity[1:3] = NA
result_missing = MSstatsConvert:::.checkFeatureCoverage(test_data_missing, feature_data)

# First feature should have some missing percentage
first_feature = result_missing$Feature[1]
first_feature_missing = result_missing[Feature == first_feature]$percent_missing
expect_true(first_feature_missing > 0, info = ".checkFeatureCoverage detects missing values")

# Test pearson_skewness function
# Test symmetric distribution (should be near 0)
symmetric_data = c(1, 2, 3, 4, 4, 3, 2, 1, 5)
skew_symmetric = MSstatsConvert:::pearson_skewness(symmetric_data)
expect_true(abs(skew_symmetric) < 0.5, info = "pearson_skewness returns near-zero for symmetric data")

# Test right-skewed distribution (should be positive)
right_skewed = c(1, 1, 1, 1, 2, 2, 3, 4, 10)
skew_right = MSstatsConvert:::pearson_skewness(right_skewed)
expect_true(skew_right > 0, info = "pearson_skewness returns positive for right-skewed data")

# Test left-skewed distribution (should be negative)
left_skewed = c(10, 10, 10, 10, 9, 9, 8, 7, 1)
skew_left = MSstatsConvert:::pearson_skewness(left_skewed)
expect_true(skew_left < 0, info = "pearson_skewness returns negative for left-skewed data")

# Test .checkAnomalySkew function
test_data = create_test_data()
result = MSstatsConvert:::.checkAnomalySkew(test_data)

# Check structure
expect_true(inherits(result, "data.table"), info = ".checkAnomalySkew returns data.table")
expect_true("PSM" %in% names(result), info = ".checkAnomalySkew includes PSM column")
expect_true("skew" %in% names(result), info = ".checkAnomalySkew includes skew column")

# Check that we get one row per unique PSM
expected_psms = unique(paste(test_data$PeptideSequence, 
                              test_data$PrecursorCharge, sep="_"))
expect_equal(nrow(result), length(expected_psms), info = ".checkAnomalySkew returns one row per PSM")

# Check that skewness values are numeric and finite (FAIL)
expect_true(all(is.numeric(result$skew)), info = ".checkAnomalySkew returns numeric skewness values")
expect_true(all(is.finite(result$skew)), info = ".checkAnomalySkew returns finite skewness values")

# Verify PSM creation
expected_psm_1 = paste("AEFEEQNVR", "2", sep="_")
expect_true(expected_psm_1 %in% result$PSM, info = ".checkAnomalySkew creates correct PSM identifiers")

# Test CheckDataHealth main function
test_data = create_test_data()
result = CheckDataHealth(test_data)

# Check structure
expect_true(is.list(result), info = "CheckDataHealth returns list")
expect_equal(length(result), 2, info = "CheckDataHealth returns list of length 2")

feature_data = result[[1]]
skew_results = result[[2]]

# Check feature_data structure
expect_true(inherits(feature_data, "data.table"), info = "CheckDataHealth returns feature_data as data.table")
expected_cols = c("Feature", "sd_Intensity", "mean_Intensity", 
                   "ratio", "outliers", "percent_missing")
expect_true(all(expected_cols %in% names(feature_data)), info = "CheckDataHealth feature_data has all expected columns")

# Check skew_results structure
expect_true(inherits(skew_results, "data.table"), info = "CheckDataHealth returns skew_results as data.table")
expect_true(all(c("PSM", "skew") %in% names(skew_results)), info = "CheckDataHealth skew_results has expected columns")

# Verify Feature creation in main function
expected_feature = paste("AEFEEQNVR", "2", "y3", "1", sep="_")
expect_true(expected_feature %in% feature_data$Feature, info = "CheckDataHealth creates correct Feature identifiers")

# Test with all same intensities (zero variance)
test_data_same = create_test_data()
test_data_same$Intensity = 1000
result_same = CheckDataHealth(test_data_same)
expect_true(is.list(result_same), info = "CheckDataHealth handles zero variance data")

# Check that SD is 0 for features with same intensities
feature_data_same = result_same[[1]]
expect_true(all(feature_data_same$sd_Intensity == 0), info = "CheckDataHealth correctly calculates zero SD for identical intensities")

# Test CheckDataHealth with different anomaly score patterns
test_data = create_test_data()

# Test with highly skewed anomaly scores
test_data_skewed = test_data
test_data_skewed$AnomalyScores = c(rep(0.01, 20), rep(0.99, 5))
result_skewed = CheckDataHealth(test_data_skewed)

skew_results = result_skewed[[2]]
expect_true(inherits(skew_results, "data.table"), info = "CheckDataHealth handles skewed anomaly scores")
expect_true(all(c("PSM", "skew") %in% names(skew_results)), info = "CheckDataHealth skew results maintain structure with skewed data")

# Test with uniform anomaly scores
test_data_uniform = test_data
test_data_uniform$AnomalyScores = 0.5
result_uniform = CheckDataHealth(test_data_uniform)

skew_uniform = result_uniform[[2]]
# Uniform distribution should have skewness near 0 (but will be NaN for truly uniform)
expect_true(all(is.finite(skew_uniform$skew) | is.nan(skew_uniform$skew)), info = "CheckDataHealth handles uniform anomaly scores appropriately")

# Test additional edge case: different anomaly score distributions per PSM
test_data_mixed = create_test_data()
# Create more varied anomaly scores to test skewness calculation
test_data_mixed$AnomalyScores[1:5] = c(0.1, 0.2, 0.3, 0.8, 0.9)  # Right skewed for first PSM
test_data_mixed$AnomalyScores[6:10] = c(0.9, 0.8, 0.3, 0.2, 0.1)  # Left skewed for second PSM
result_mixed = CheckDataHealth(test_data_mixed)

skew_mixed = result_mixed[[2]]
expect_equal(nrow(skew_mixed), 5, info = "CheckDataHealth calculates skewness for all unique PSMs")
expect_true(any(skew_mixed$skew > 0) || any(skew_mixed$skew < 0), info = "CheckDataHealth detects skewness in anomaly scores")

#' Import DIA-Umpire files 
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param raw.frag name of FragSummary_date.xls data, which includes feature-level data.
#' @param raw.pep name of PeptideSummary_date.xls data, which includes selected fragments information.
#' @param raw.pro name of ProteinSummary_date.xls data, which includes selected peptides information.
#' @param annotation name of annotation data which includes Condition, BioReplicate, Run information.
#' @param useSelectedFrag TRUE will use the selected fragment for each peptide. 'Selected_fragments' column is required.
#' @param useSelectedPep TRUE will use the selected peptide for each protein. 'Selected_peptides' column is required.
#' @param ... additional parameters to `data.table::fread`.
#' 
#' @return data.frame in the MSstats required format.
#'
#' @author Meena Choi, Olga Vitek 
#'
#' @export
#' 
#' @examples 
#' diau_frag = system.file("tinytest/raw_data/DIAUmpire/dia_frag.csv", 
#'                              package = "MSstatsConvert")
#' diau_pept = system.file("tinytest/raw_data/DIAUmpire/dia_pept.csv", 
#'                              package = "MSstatsConvert")
#' diau_prot = system.file("tinytest/raw_data/DIAUmpire/dia_prot.csv", 
#'                              package = "MSstatsConvert")
#' annot = system.file("tinytest/raw_data/DIAUmpire/annot_diau.csv", 
#'                     package = "MSstatsConvert")
#' diau_frag = data.table::fread(diau_frag) 
#' diau_pept = data.table::fread(diau_pept) 
#' diau_prot = data.table::fread(diau_prot) 
#' annot = data.table::fread(annot)
#' diau_frag = diau_frag[, lapply(.SD, function(x) if (is.integer(x)) as.numeric(x) else x)]
#' # In case numeric columns are not interpreted correctly
#' 
#' diau_imported = DIAUmpiretoMSstatsFormat(diau_frag, diau_pept, diau_prot, 
#'                                          annot, use_log_file = FALSE)
#' head(diau_imported)
#' 
DIAUmpiretoMSstatsFormat = function(
        raw.frag, raw.pep, raw.pro, annotation, useSelectedFrag = TRUE,
        useSelectedPep = TRUE, removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE, summaryforMultipleRows = max, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(Fragments = raw.frag, 
                                               Peptides = raw.pep, 
                                               Proteins = raw.pro), 
                                          type = "MSstats", 
                                          tool = "DIAUmpire", ...)
    input = MSstatsConvert::MSstatsClean(input, 
                                         use_frag = useSelectedFrag, 
                                         use_pept = useSelectedPep)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    feature_columns = c("PeptideSequence", "FragmentIon")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation,
        feature_columns,
        remove_shared_peptides = TRUE, 
        remove_single_feature_proteins = removeProtein_with1Feature,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = summaryforMultipleRows),
        columns_to_fill = list("PrecursorCharge" = NA,
                               "ProductCharge" = NA,
                               "IsotopeLabelType" = "L"))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  remove_few = removeFewMeasurements)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test DIAUmpiretoMSstatsFormat ---------------------------

diau_frag = system.file("tinytest/raw_data/DIAUmpire/dia_frag.csv",
                             package = "MSstatsConvert")
diau_pept = system.file("tinytest/raw_data/DIAUmpire/dia_pept.csv",
                             package = "MSstatsConvert")
diau_prot = system.file("tinytest/raw_data/DIAUmpire/dia_prot.csv",
                             package = "MSstatsConvert")
annot = system.file("tinytest/raw_data/DIAUmpire/annot_diau.csv",
                    package = "MSstatsConvert")
diau_frag = data.table::fread(diau_frag)
diau_pept = data.table::fread(diau_pept)
diau_prot = data.table::fread(diau_prot)
annot = data.table::fread(annot)
diau_frag = diau_frag[, lapply(.SD, function(x) if (is.integer(x)) as.numeric(x) else x)]

output = MSstatsConvert:::DIAUmpiretoMSstatsFormat(diau_frag, diau_pept, diau_prot,
                                         annot, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 480)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))

#' Generate MSstatsTMT required input format for OpenMS output
#' @param input MSstatsTMT report from OpenMS
#' @param rmPSM_withfewMea_withinRun TRUE (default) will remove the features that have 1 or 2 measurements within each Run.
#' @param rmProtein_with1Feature TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.
#' @param summaryforMultiplePSMs sum(default) or max - when there are multiple measurements for certain feature in certain run, select the feature with the largest summation or maximal value.
#' @param ... additional parameters to `data.table::fread`.
#' @inheritParams .sharedParametersAmongConverters
#'  
#' @return `data.frame` of class `MSstatsTMT`.
#' 
#' @export
#' 
#' @examples
#' raw.om = data.table::fread(system.file("tinytest/raw_data/OpenMSTMT/openmstmt_input.csv",
#'                                       package = "MSstatsConvert"))
#' input.om <- OpenMStoMSstatsTMTFormat(raw.om)
#' head(input.om)
#' 
OpenMStoMSstatsTMTFormat = function(
        input, useUniquePeptide = TRUE, rmPSM_withfewMea_withinRun = TRUE, 
        rmProtein_with1Feature = FALSE, summaryforMultiplePSMs = sum, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL, ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsTMT_converter_log_")
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstatsTMT", "OpenMS", ...)
    input = MSstatsConvert::MSstatsClean(input)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        NULL, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = rmProtein_with1Feature,
        feature_cleaning = list(remove_features_with_few_measurements = rmPSM_withfewMea_withinRun,
                                summarize_multiple_psms = summaryforMultiplePSMs)
    )
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  fix_missing = "zero_to_na")
    
    data.table::setnames(input, "PrecursorCharge", "Charge", skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the proteinSummarization function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test OpenMStoMSstatsTMTFormat ---------------------------
raw = data.table::fread(system.file("tinytest/raw_data/OpenMSTMT/openmstmt_input.csv",
                                    package = "MSstatsConvert"))
output = OpenMStoMSstatsTMTFormat(raw, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_true(nrow(output) > 0)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("Charge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("TechRepMixture" %in% colnames(output))
expect_true("PSM" %in% colnames(output))
expect_true("Mixture" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Channel" %in% colnames(output))

# Verify output intensities are present in input
input_intensities = as.character(raw$Intensity)
output_intensities = as.character(output$Intensity[!is.na(output$Intensity)])
expect_true(all(output_intensities %in% input_intensities))

#' Import Progenesis files
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param input name of Progenesis output, which is wide-format. 'Accession', 'Sequence', 'Modification', 'Charge' and one column for each run are required.
#' @param annotation name of 'annotation.txt' or 'annotation.csv' data which includes Condition, BioReplicate, Run information. It will be matched with the column name of input for MS runs.
#' @param ... additional parameters to `data.table::fread`.
#'
#' @return data.frame in the MSstats required format.
#' 
#' @author Meena Choi, Olga Vitek, Ulrich Omasits
#' 
#' @export
#' 
#' @examples 
#' progenesis_raw = system.file("tinytest/raw_data/Progenesis/progenesis_input.csv", 
#'                              package = "MSstatsConvert")
#' annot = system.file("tinytest/raw_data/Progenesis/progenesis_annot.csv", 
#'                     package = "MSstatsConvert")
#' progenesis_raw = data.table::fread(progenesis_raw) 
#' annot = data.table::fread(annot)
#' 
#' progenesis_imported = ProgenesistoMSstatsFormat(progenesis_raw, annot,
#'                                                 use_log_file = FALSE)
#' head(progenesis_imported)
#' 
ProgenesistoMSstatsFormat = function(
        input, annotation, useUniquePeptide = TRUE, summaryforMultipleRows = max,
        removeFewMeasurements = TRUE, removeOxidationMpeptides = FALSE, 
        removeProtein_with1Peptide = FALSE, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose,
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "Progenesis", ...)
    input = MSstatsConvert::MSstatsClean(input, 
                                         unique(as.character(annotation$Run)), 
                                         fix_colnames = TRUE)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    oxidation_filter = list(col_name = "PeptideSequence", 
                            pattern = "Oxidation", 
                            filter = removeOxidationMpeptides, 
                            drop_column = FALSE)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Peptide,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = summaryforMultipleRows),
        pattern_filtering = list(oxidation = oxidation_filter),
        columns_to_fill = list("FragmentIon" = NA, 
                               "ProductCharge" = NA,
                               "IsotopeLabelType" = "L"))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  remove_few = removeFewMeasurements)
    data.table::setnames(input, "PeptideSequence", "PeptideModifiedSequence",
                         skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test ProgenesistoMSstatsFormat ---------------------------

progenesis_raw = system.file("tinytest/raw_data/Progenesis/progenesis_input.csv",
                             package = "MSstatsConvert")
annot = system.file("tinytest/raw_data/Progenesis/progenesis_annot.csv",
                    package = "MSstatsConvert")
progenesis_raw = data.table::fread(progenesis_raw)
annot = data.table::fread(annot)

output = ProgenesistoMSstatsFormat(progenesis_raw, annot,
                                                use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 180)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideModifiedSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))

## ----include = FALSE----------------------------------------------------------
  knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
  )

## ----logs---------------------------------------------------------------------
library(MSstatsConvert)
# default - creates a new file
MSstatsLogsSettings(use_log_file = TRUE, append = FALSE) 

# default - creates a new file
MSstatsLogsSettings(use_log_file = TRUE, append = TRUE, 
                    log_file_path = "log_file.log") 

# switches logging off
MSstatsLogsSettings(use_log_file = FALSE, append = FALSE) 

# switches off logs and messages
MSstatsLogsSettings(use_log_file = FALSE, verbose = FALSE) 

## -----------------------------------------------------------------------------
MSstatsSaveSessionInfo()

## -----------------------------------------------------------------------------
maxquant_evidence = read.csv(system.file("tinytest/raw_data/MaxQuant/mq_ev.csv",
                                         package = "MSstatsConvert"))
maxquant_proteins = read.csv(system.file("tinytest/raw_data/MaxQuant/mq_pg.csv",
                                         package = "MSstatsConvert"))
maxquant_imported = MSstatsImport(list(evidence = maxquant_evidence,
                                       protein_groups = maxquant_proteins),
                                  type = "MSstats", tool = "MaxQuant")
is(maxquant_imported)

openms_input = read.csv(system.file(
  "tinytest/raw_data/OpenMSTMT/openmstmt_input.csv",
  package = "MSstatsConvert"
))
openms_imported = MSstatsImport(list(input = openms_input),
                                "MSstatsTMT", "OpenMS")
is(openms_imported)

## -----------------------------------------------------------------------------
getInputFile(maxquant_imported, "evidence")[1:5, 1:5]

## -----------------------------------------------------------------------------
maxquant_cleaned = MSstatsClean(maxquant_imported, protein_id_col = "Proteins")
head(maxquant_cleaned)

openms_cleaned = MSstatsClean(openms_imported)
head(openms_cleaned)

## -----------------------------------------------------------------------------
maxquant_annotation = read.csv(system.file(
  "tinytest/raw_data/MaxQuant/annotation.csv",
  package = "MSstatsConvert"
))
maxquant_annotation = MSstatsMakeAnnotation(maxquant_cleaned,
                                            maxquant_annotation,
                                            Run = "Rawfile")
m_filter = list(col_name = "PeptideSequence", 
                pattern = "M", 
                filter = TRUE, 
                drop_column = FALSE)

oxidation_filter = list(col_name = "Modifications", 
                        pattern = "Oxidation", 
                        filter = TRUE, 
                        drop_column = TRUE)

feature_columns = c("PeptideSequence", "PrecursorCharge")
maxquant_processed = MSstatsPreprocess(
  maxquant_cleaned, 
  maxquant_annotation,
  feature_columns,
  remove_shared_peptides = TRUE, 
  remove_single_feature_proteins = FALSE,
  pattern_filtering = list(oxidation = oxidation_filter,
                           m = m_filter),
  feature_cleaning = list(remove_features_with_few_measurements = TRUE,
                          summarize_multiple_psms = max),
  columns_to_fill = list("FragmentIon" = NA,
                         "ProductCharge" = NA,
                         "IsotopeLabelType" = "L"))
head(maxquant_processed)

# OpenMS - TMT data
feature_columns_tmt = c("PeptideSequence", "PrecursorCharge")
openms_processed = MSstatsPreprocess(
  openms_cleaned, 
  NULL, 
  feature_columns_tmt,
  remove_shared_peptides = TRUE,
  remove_single_feature_proteins = TRUE,
  feature_cleaning = list(remove_features_with_few_measurements = TRUE,
                          summarize_multiple_psms = max)
)
head(openms_processed)

## ----eval = FALSE-------------------------------------------------------------
# list(
#   list(score_column = "Intensity", score_threshold = 1,
#        direction = "greater", behavior = "remove",
#        handle_na = "remove", fill_value = NA, filter = TRUE, drop = FALSE
#        )
# )

## -----------------------------------------------------------------------------
maxquant_balanced = MSstatsBalancedDesign(maxquant_processed, feature_columns)
head(maxquant_balanced)
dim(maxquant_balanced)
dim(maxquant_processed)

openms_balanced = MSstatsBalancedDesign(openms_processed, feature_columns_tmt)
head(openms_balanced)
dim(openms_balanced)
dim(openms_processed)

#' Read file from a provided path or convert given data.frame to data.table
#' @param input report from a signal processing tool or a path to it
#' @param ... additional parameters for data.table::fread
#' @importFrom data.table as.data.table fread
#' @return data.table
#' @keywords internal
.getDataTable = function(input, ...) {
    checkmate::checkTRUE(is.character(input) | inherits(input, "data.frame"))
    if (inherits(input, "data.frame")) {
        input = data.table::as.data.table(input)
    } else {
        input = data.table::fread(input, showProgress = FALSE, ...)
    }
    colnames(input) = .standardizeColnames(colnames(input))
    input
}


#' Remove underscores from sequences and change intensity type to numeric
#' @param input data.table
#' @return data.table
#' @keywords internal
.fixBasicColumns = function(input) {
  Intensity = PeptideSequence = FragmentIon = NULL
  
  input[, Intensity := as.numeric(Intensity)]
  input[, PeptideSequence := stringi::stri_replace_all(PeptideSequence, 
                                                       "", fixed = "_")]
  input[, FragmentIon := stringi::stri_replace_all(FragmentIon, "",
                                                   fixed = "_")]
  input
}


#' Change classes of multiple columns
#' @param input data.table preprocessed by one of the `cleanRaw*` functions.
#' @param numeric_columns chr, vector of names of columns that will be 
#' converted to numeric.
#' @param character_columns chr, vector of names of colums taht will be 
#' converted to character.
#' @param factor_columns chr, vector of names of columns that will be 
#' converted to factor.
#' @return data.table
#' @keywords internal
.fixColumnTypes = function(input, numeric_columns = NULL, 
                           character_columns = NULL,
                           factor_columns = NULL) {
    for (column in factor_columns) {
        input[[column]] = factor(input[[column]])
    }
    for (column in numeric_columns) {
        input[[column]] = as.numeric(as.character(input[[column]]))
    }
    for (column in character_columns) {
        input[[column]] = as.character(input[[column]])
    }
    input
}


#' Set column to a single value
#' @param input data.table preprocessed by one of the `cleanRaw*` functions.
#' @param fill_list named list, names correspond to column names, elements 
#' to values that will be used in the columns.
#' @return data.table
#' @keywords internal
.fillValues = function(input, fill_list) {
    if (length(fill_list) > 0) {
        input[, names(fill_list) := as.list(unname(fill_list))]
    }
}


#' Change column names to match read.table/read.csv/read.delim conventions
#' @param col_names chr, vector of column names
#' @return character vector
#' @keywords internal
.standardizeColnames = function(col_names) {
    col_names_unique = unique(col_names)
    col_names_transformed = stringi::stri_replace_all(col_names_unique, fixed = " ", replacement = ".")
    col_names_transformed = stringi::stri_replace_all(col_names_transformed, regex = "\\[|\\]|\\%", replacement = ".")
    col_names_transformed = stringi::stri_replace_all(col_names_transformed, fixed = "/", replacement = "")
    col_names_transformed = stringi::stri_replace_all(col_names_transformed, fixed = "+", replacement = "")
    col_names_transformed = stringi::stri_replace_all(col_names_transformed, fixed = "#", replacement = "X.")
    col_names_transformed = stringi::stri_replace_all(col_names_transformed, regex = "[\\.]+", replacement = "")
    lookup_table = c()
    lookup_table[col_names_unique] = col_names_transformed
    col_names = lookup_table[col_names]
    return(unname(col_names))
}


#' Get intensity columns from wide-format data
#' @param col_names names of columns, where some of the columns store intensity
#' value for different channels
#' @param ... varying number of strings that define channel columns.
#' @return character vector of column names that correspond to channel intensities
#' @keywords internal
.getChannelColumns = function(col_names, ...) {
    all_patterns = unlist(list(...))
    channel_filter = rep(TRUE, length(col_names))
    for (pattern in all_patterns) {
        channel_filter = channel_filter & grepl(pattern, col_names, 
                                                fixed = TRUE)
    }
    col_names[channel_filter]
}


#' Select columns for MSstats format
#' @param input data.table
#' @return data.table
#' @keywords internal
.selectMSstatsColumns = function(input, anomaly_metrics) {
    Condition = NULL

    standard_columns = c(
        "ProteinName", "PeptideSequence", "PeptideModifiedSequence", 
        "PrecursorCharge", "FragmentIon", "ProductCharge", "IsotopeLabelType",
        "Condition", "BioReplicate", "Run", "TechReplicate", "StandardType", 
        "Fraction", "DetectionQValue", "Intensity", anomaly_metrics
      )
    
    standard_columns_tmt = c("ProteinName", "PeptideSequence", "PrecursorCharge", 
                             "PSM", "Mixture", "TechRepMixture", "Run", 
                             "Channel", "BioReplicate", "Condition", "Intensity")
    
    if (is.element("Channel", colnames(input))) {
        cols = standard_columns_tmt
        character_cols = c("ProteinName", "PeptideSequence", "PrecursorCharge",
                           "PSM", "Mixture", "TechRepMixture", "Run",
                           "Channel", "BioReplicate")
        input[, c("ProteinName", "PeptideSequence", "PrecursorCharge",
                  "PSM", "Mixture", "TechRepMixture", "Run",
                  "Channel", "BioReplicate") := lapply(.SD, as.character),
              .SDcols = character_cols]
        input[, Condition := factor(as.character(Condition))]
    } else {
        cols = standard_columns
    }
    input[, intersect(cols, colnames(input)), with = FALSE]
}

test_path = system.file("tinytest", "get_dt_test.csv", package = "MSstatsConvert")
test_df = data.table::fread(test_path)

# Import and conversion ----
## data.table from path
expect_equal(class(MSstatsConvert:::.getDataTable(test_path))[1], "data.table")
## data.table from data.frame
expect_equal(class(MSstatsConvert:::.getDataTable(test_df))[1], "data.table")
## When it's not a data.frame, a test 
expect_error(MSstatsConvert:::.getDataTable(numeric(10)))
## Remove unnecessary symbols from column names
expect_equal(MSstatsConvert:::.standardizeColnames("Col.name"), "Colname")
expect_equal(MSstatsConvert:::.standardizeColnames("[Col%name]"), "Colname")
expect_equal(MSstatsConvert:::.standardizeColnames(c("Col+/name", "#Col.name", "#Col.name")), c("Colname", "XColname", "XColname"))
### Does not cover ":" - is it bad?
expect_equal(MSstatsConvert:::.getChannelColumns(c("Ab:1", "Ab:2"), "Ab"), c("Ab:1", "Ab:2"))
# Additional columns
updated_df = MSstatsConvert:::.fillValues(test_df, list(z = NA))
expect_true(all(is.na(updated_df$z)) & is.element("z", colnames(updated_df)))
# Column types ---
updated_col_types = updated_df[, list(x = as.character(x), y = as.factor(y), z = as.numeric(z))]
fixed_column_types = MSstatsConvert:::.fixColumnTypes(updated_col_types, numeric_columns = "x", 
                                                      character_columns = "y",
                                                      factor_columns = "z")
expect_equal(class(fixed_column_types$x), "numeric")
expect_equal(class(fixed_column_types$y), "character")
expect_equal(class(fixed_column_types$z), "factor")

#' Import Skyline files
#'
#' @inheritParams .sharedParametersAmongConverters
#' @param input name of MSstats input report from Skyline, which includes feature-level data.
#' @param annotation name of 'annotation.txt' data which includes Condition, BioReplicate, Run. If annotation is already complete in Skyline, use annotation=NULL (default). It will use the annotation information from input.
#' @param removeiRT TRUE (default) will remove the proteins or peptides which are labeled 'iRT' in 'StandardType' column. FALSE will keep them.
#' @param filter_with_Qvalue TRUE(default) will filter out the intensities that have greater than qvalue_cutoff in DetectionQValue column. Those intensities will be replaced with zero and will be considered as censored missing values for imputation purpose.
#' @param qvalue_cutoff Cutoff for DetectionQValue. default is 0.01.
#' @param ... additional parameters to `data.table::fread`.
#' 
#' @return data.frame in the MSstats required format.
#' 
#' @author Meena Choi, Olga Vitek
#' 
#' @export
#' 
#' @examples 
#' skyline_raw = system.file("tinytest/raw_data/Skyline/skyline_input.csv",
#'                           package = "MSstatsConvert")
#' skyline_raw = data.table::fread(skyline_raw)
#' skyline_imported = SkylinetoMSstatsFormat(skyline_raw)
#' head(skyline_imported)
#' 
SkylinetoMSstatsFormat = function(
        input, annotation = NULL, removeiRT = TRUE, filter_with_Qvalue = TRUE,
        qvalue_cutoff = 0.01, useUniquePeptide = TRUE, removeFewMeasurements = TRUE,
        removeOxidationMpeptides = FALSE, removeProtein_with1Feature = FALSE,
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "Skyline", ...)
    input = MSstatsConvert::MSstatsClean(input)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, 
                                                       annotation, 
                                                       Run = "FileName")
    
    decoy_filter = list(col_name = "ProteinName",
                        pattern = c("DECOY", "Decoys"),
                        filter = TRUE, 
                        drop_column = FALSE)
    
    irt_filter = list(col_name = "StandardType", 
                      filter_symbols = "iRT",
                      filter = removeiRT, 
                      behavior = "remove",
                      fill_value = NULL,
                      drop_column = FALSE)
    
    oxidation_filter = list(col_name = "PeptideSequence",
                            pattern = "\\+16", 
                            filter = removeOxidationMpeptides, 
                            drop_column = FALSE)
    
    truncated_filter = list(col_name = "Truncated", 
                            filter_symbols = "TRUE",
                            behavior = "fill",
                            fill_value = NA_real_,
                            filter = TRUE, 
                            drop_column = TRUE)
    
    qval_filter = list(score_column = "DetectionQValue", 
                       score_threshold = qvalue_cutoff, 
                       direction = "smaller",
                       behavior = "fill", 
                       fill_value = NA_real_, 
                       handle_na = "keep",
                       filter = filter_with_Qvalue, 
                       drop_column = TRUE)
    
    feature_columns = c("IsotopeLabelType", "PeptideSequence", "PrecursorCharge", 
                        "FragmentIon", "ProductCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        score_filtering = list(qval = qval_filter),
        pattern_filtering = list(decoy = decoy_filter, 
                                 oxidation = oxidation_filter),
        exact_filtering = list(irt = irt_filter,
                               truncated = truncated_filter),
        aggregate_isotopic = TRUE,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = sum))
    input[, Intensity := ifelse(Intensity == 0, NA, Intensity)]
    
    input = MSstatsBalancedDesign(input, c("PeptideSequence", "PrecursorCharge", 
                                           "FragmentIon", "ProductCharge"),
                                  remove_few = removeFewMeasurements)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test SkylinetoMSstatsFormat ---------------------------
skyline_raw = system.file("tinytest/raw_data/Skyline/skyline_input.csv",
                          package = "MSstatsConvert")
skyline_raw = data.table::fread(skyline_raw)
output = SkylinetoMSstatsFormat(skyline_raw)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 705)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))

#' Class to model files that describe a single MS dataset.
#' 
#' @slot files named list of files generated by a signal processing tools. 
#' In most cases, this will be a single file named `input`. 
#' In some cases, multiple files are used, for example `MaxQuant` outputs 
#' `evidence` and `proteinGroups` files.
#' @slot type character: "MSstats" or "MSstatsTMT".
#' @slot tool character: name of a signal processing tools that generated the
#' output. Possible values are: DIAUmpire, MaxQuant, OpenMS, OpenSWATH, 
#' Progenesis, ProteomeDiscoverer, Skyline, SpectroMine, Spectronaut.
#' @slot version description of a software version of the signal processing tool.
#' Not implemented yet.
#' @rdname MSstatsInputFiles
setClass("MSstatsInputFiles", 
         slots = c(files = "list", type = "character", 
                   tool = "character", version = "ANY"))

#' MSstatsDIAUmpireFiles: class for DIAUmpire files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsDIAUmpireFiles", contains = "MSstatsInputFiles")
#' MSstatsMaxQuantFiles: class for MaxQuant files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsMaxQuantFiles", contains = "MSstatsInputFiles")
#' MSstatsOpenMSFiles: class for OpenMS files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsOpenMSFiles", contains = "MSstatsInputFiles")
#' MSstatsOpenSWATHFiles: class for OpenSWATH files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsOpenSWATHFiles", contains = "MSstatsInputFiles")
#' MSstatsProgenesisFiles: class for Progenesis files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsProgenesisFiles", contains = "MSstatsInputFiles")
#' MSstatsProteomeDiscovererFiles: class for ProteomeDiscoverer files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsProteomeDiscovererFiles", contains = "MSstatsInputFiles")
#' MSstatsSkylineFiles: class for Skyline files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsSkylineFiles", contains = "MSstatsInputFiles")
#' MSstatsSkylineFiles: class for SpectroMine files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsSpectroMineFiles", contains = "MSstatsInputFiles")
#' MSstatsSpectronautFiles: class for Spectronaut files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsSpectronautFiles", contains = "MSstatsInputFiles")
#' MSstatsPhilosopherFiles: class for Philosopher files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsPhilosopherFiles", contains = "MSstatsInputFiles")
#' MSstatsDIANNFiles: class for DIA-NN files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsDIANNFiles", contains = "MSstatsInputFiles")
#' MSstatsFragPipeFiles: class for FragPipe files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsFragPipeFiles", contains = "MSstatsInputFiles")
#' MSstatsMetamorpheusFiles: class for Metamorpheus files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsMetamorpheusFiles", contains = "MSstatsInputFiles")
#' MSstatsProteinProspectorFiles: class for ProteinProspector files.
#' @rdname MSstatsInputFiles
#' @keywords internal
setClass("MSstatsProteinProspectorFiles", contains = "MSstatsInputFiles")


#' Get one of files contained in an instance of `MSstatsInputFiles` class.
#' @rdname getInputFile
#' @return data.table
#' @export
#' @examples
#' evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
#'                             package = "MSstatsConvert")
#' pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
#'                       package = "MSstatsConvert")
#' evidence = read.csv(evidence_path)
#' pg = read.csv(pg_path)
#' imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
#'                          "MSstats", "MaxQuant")
#' class(imported)
#' head(getInputFile(imported, "evidence"))
setGeneric("getInputFile", 
           function(msstats_object, file_type) standardGeneric("getInputFile"), 
           signature = "msstats_object")
#' @param msstats_object object that inherits from `MSstatsInputFiles` class.
#' @param file_type character name of a type file. Usually equal to "input".
#' @return data.table
#' @export 
#' @rdname getInputFile
setMethod("getInputFile", "MSstatsInputFiles", 
          function(msstats_object, file_type = "input") 
              msstats_object@files[[file_type]])
#' @param msstats_object object that inherits from `MSstatsPhilosopherFiles` class.
#' @param file_type character name of a type file. Usually equal to "input".
#' @return data.table
#' @rdname getInputFile
setMethod("getInputFile", "MSstatsPhilosopherFiles",
          function(msstats_object, file_type = "input") {
              if (file_type == "annotation") {
                  msstats_object@files[["annotation"]]
              } else {
                  list_names = names(msstats_object@files)
                  data.table::rbindlist(msstats_object@files[list_names != "annotation"])
              }
          })


#' Get type of dataset from an MSstatsInputFiles object.
#' @rdname getDataType
#' @keywords internal
#' @export
#' @return character - label of a data type. Currently, "MSstats" or "MSstatsTMT"
#' @examples
#' evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
#'                             package = "MSstatsConvert")
#' pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
#'                       package = "MSstatsConvert")
#' evidence = read.csv(evidence_path)
#' pg = read.csv(pg_path)
#' imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
#'                          "MSstats", "MaxQuant")
#' class(imported)
#' getDataType(imported) # "MSstats"
#' 
setGeneric("getDataType", 
           function(msstats_object) standardGeneric("getDataType"))
#' @param msstats_object object that inherits from `MSstatsInputFiles` class.
#' @return character "MSstats" or "MSstatsTMT".
#' @export
#' @rdname getDataType
setMethod("getDataType", "MSstatsInputFiles",
          function(msstats_object) msstats_object@type)


#' Import files from signal processing tools.
#' 
#' @param input_files list of paths to input files or `data.frame` objects.
#' Interpretation of this parameter depends on values of parameters `type` and `tool`.
#' @param type chr, "MSstats" or "MSstatsTMT".
#' @param tool chr, name of a signal processing tool that generated input files.
#' @param tool_version not implemented yet. In the future, this parameter will allow
#' handling different versions of each signal processing tools.
#' @param ... optional additional parameters to `data.table::fread`.
#' 
#' @return an object of class `MSstatsInputFiles`.
#' @export
#' 
#' @examples 
#' evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
#'                             package = "MSstatsConvert")
#' pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
#'                       package = "MSstatsConvert")
#' evidence = read.csv(evidence_path)
#' pg = read.csv(pg_path)
#' imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
#'                          "MSstats", "MaxQuant")
#' class(imported)
#' head(getInputFile(imported, "evidence"))
#' 
MSstatsImport = function(input_files, type, tool, tool_version = NULL, ...) {
    checkmate::assertChoice(type, c("MSstats", "MSstatsTMT"))
    checkmate::assertTRUE(!is.null(names(input_files)))
    
    input_files = as.list(input_files)
    input_files = lapply(input_files, .getDataTable, ...)
    
    msstats_object = methods::new("MSstatsInputFiles", files = input_files,
                                  type = type, tool = tool, 
                                  version = tool_version)
    class = paste0("MSstats", tool, "Files")
    .logSuccess(tool, "import")
    methods::new(class, msstats_object)
}

#' Clean files generated by a signal processing tools.
#' @param msstats_object object that inherits from `MSstatsInputFiles` class.
#' @param ... additional parameter to specific cleaning functions.
#' @rdname MSstatsClean
#' @export
#' @return data.table
#' 
#' @examples 
#' evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
#'                             package = "MSstatsConvert")
#' pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
#'                       package = "MSstatsConvert")
#' evidence = read.csv(evidence_path)
#' pg = read.csv(pg_path)
#' imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
#'                          "MSstats", "MaxQuant")
#' cleaned_data = MSstatsClean(imported, protein_id_col = "Proteins")
#' head(cleaned_data)
#' 
setGeneric("MSstatsClean", function(msstats_object, ...) {
    standardGeneric("MSstatsClean")
})
#' Clean DIAUmpire files
#' @include clean_DIAUmpire.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawDIAUmpire
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsDIAUmpireFiles", 
          .cleanRawDIAUmpire)
#' Clean MaxQuant files
#' @include clean_MaxQuant.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawMaxQuant
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsMaxQuantFiles", 
          .cleanRawMaxQuant)
#' Clean OpenMS files
#' @include clean_OpenMS.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawOpenMS
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsOpenMSFiles", 
          .cleanRawOpenMS)
#' Clean OpenSWATH files
#' @include clean_OpenSWATH.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawOpenSWATH
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsOpenSWATHFiles", 
          .cleanRawOpenSWATH)
#' Clean Progenesis files
#' @include clean_Progenesis.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawProgenesis
setMethod("MSstatsClean", signature = "MSstatsProgenesisFiles", 
          .cleanRawProgenesis)
#' Clean ProteomeDiscoverer files
#' @include clean_ProteomeDiscoverer.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawPD
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsProteomeDiscovererFiles", 
          .cleanRawPD)
#' Clean Skyline files
#' @include clean_Skyline.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawSkyline
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsSkylineFiles", 
          .cleanRawSkyline)
#' Clean SpectroMine files
#' @include clean_SpectroMine.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawSpectroMineTMT
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsSpectroMineFiles", 
          .cleanRawSpectroMineTMT)
#' Clean Spectronaut files
#' @include clean_Spectronaut.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawSpectronaut
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsSpectronautFiles", 
          .cleanRawSpectronaut)
#' Clean Philosopher files
#' @include clean_Philosopher.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawPhilosopher
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsPhilosopherFiles", 
          .cleanRawPhilosopher)
#' Clean DIA-NN files
#' @include clean_DIANN.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawDIANN
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsDIANNFiles", 
          .cleanRawDIANN)
#' Clean Metamorpheus files
#' @include clean_Metamorpheus.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawMetamorpheus
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsMetamorpheusFiles", 
          .cleanRawMetamorpheus)
#' Clean Protein Prospector files
#' @include clean_ProteinProspector.R
#' @rdname MSstatsClean
#' @inheritParams .cleanRawProteinProspector
#' @return data.table
setMethod("MSstatsClean", signature = "MSstatsProteinProspectorFiles", 
          .cleanRawProteinProspector)


#' Preprocess outputs from MS signal processing tools for analysis with MSstats
#' 
#' @param input data.table processed by the MSstatsClean function.
#' @param annotation annotation file generated by a signal processing tool.
#' @param feature_columns character vector of names of columns that 
#' define spectral features.
#' @param remove_shared_peptides logical, if TRUE shared peptides will be removed.
#' @param remove_single_feature_proteins logical, if TRUE, proteins that only have
#' one feature will be removed.
#' @param feature_cleaning named list with maximum two (for `MSstats` converters)
#' or three (for `MSstatsTMT` converter) elements. If `handle_few_measurements` is
#' set to "remove", feature with less than three measurements will be removed 
#' (otherwise it should be equal to "keep"). `summarize_multiple_psms` is a function
#' that will be used to aggregate multiple feature measurements in a run. It should
#' return a scalar and accept an `na.rm` parameter. For `MSstatsTMT` converters,
#' setting `remove_psms_with_any_missing` will remove features which have missing
#' values in a run from that run. 
#' @param score_filtering a list of named lists that specify filtering options.
#' Details are provided in the vignette.
#' @param exact_filtering a list of named lists that specify filtering options.
#' Details are provided in the vignette.
#' @param pattern_filtering a list of named lists that specify filtering options.
#' Details are provided in the vignette.
#' @param columns_to_fill a named list of scalars. If provided, columns with
#' names defined by the names of this list and values corresponding to its elements
#' will be added to the output `data.frame`.
#' @param aggregate_isotopic logical. If `TRUE`, isotopic peaks will by summed.
#' @param anomaly_metrics character vector of names of columns with quality metrics. Default is missing and is not required if anomaly model not run.
#' @param ... additional parameters to `data.table::fread`.
#' 
#' @return data.table
#' @export
#' 
#' @examples 
#' evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
#'                             package = "MSstatsConvert")
#' pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
#'                       package = "MSstatsConvert")
#' evidence = read.csv(evidence_path)
#' pg = read.csv(pg_path)
#' imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
#'                          "MSstats", "MaxQuant")
#' cleaned_data = MSstatsClean(imported, protein_id_col = "Proteins")
#' annot_path = system.file("tinytest/raw_data/MaxQuant/annotation.csv", 
#'                          package = "MSstatsConvert")
#' mq_annot = MSstatsMakeAnnotation(cleaned_data, read.csv(annot_path),
#'                                  Run = "Rawfile")
#'                                
#' # To filter M-peptides and oxidatin peptides 
#' m_filter = list(col_name = "PeptideSequence", pattern = "M", 
#'                 filter = TRUE, drop_column = FALSE)
#' oxidation_filter = list(col_name = "Modifications", pattern = "Oxidation", 
#'                         filter = TRUE, drop_column = TRUE)
#' msstats_format = MSstatsPreprocess(
#' cleaned_data, mq_annot, 
#' feature_columns = c("PeptideSequence", "PrecursorCharge"),
#' columns_to_fill = list(FragmentIon = NA, ProductCharge = NA),
#' pattern_filtering = list(oxidation = oxidation_filter, m = m_filter)
#' )
#' # Output in the standard MSstats format
#' head(msstats_format)
#' 
MSstatsPreprocess = function(
    input, annotation, feature_columns, remove_shared_peptides = TRUE,
    remove_single_feature_proteins = TRUE,
    feature_cleaning = list(remove_features_with_few_measurements = TRUE,
                            summarize_multiple_psms = max),
    score_filtering = list(), exact_filtering = list(), 
    pattern_filtering = list(), columns_to_fill = list(), 
    aggregate_isotopic = FALSE, anomaly_metrics = c(), ...
) {
    .checkMSstatsParams(input, annotation, feature_columns,
                        remove_shared_peptides,
                        remove_single_feature_proteins,
                        feature_cleaning)
    .logConverterOptions(
        feature_columns, remove_shared_peptides, remove_single_feature_proteins,
        feature_cleaning, is.element("Channel", colnames(input))
    )
    
    input = .fixBasicColumns(input)
    input = .handleFiltering(input, score_filtering, 
                             exact_filtering, pattern_filtering)
    input = .handleIsotopicPeaks(input, aggregate_isotopic)
    input = .filterFewMeasurements(input, 1, FALSE)
    input = .handleSharedPeptides(input, remove_shared_peptides)
    input = .cleanByFeature(input, feature_columns, 
                            feature_cleaning, anomaly_metrics)
    input = .handleSingleFeaturePerProtein(input, 
                                           remove_single_feature_proteins)
    input = .mergeAnnotation(input, annotation)
    .fillValues(input, columns_to_fill)
    .adjustIntensities(input)
    input
}


#' Creates balanced design by removing overlapping fractions and filling incomplete rows
#' 
#' @param input `data.table` processed by the `MSstatsPreprocess` function
#' @param feature_columns str, names of columns that define spectral features
#' @param fill_incomplete if TRUE (default), ensures that rows with missing data 
#' for specific features are added as NA. For example, if the y10 ion of 
#' peptideA is measured in the "disease" samples but entirely missing for the 
#' "healthy" samples, rows with NA values will be created for the y10 ion of 
#' peptideA in the "healthy" group. This process increases the number of 
#' rows to account for all possible feature-sample combinations.
#' @param handle_fractions if TRUE (default), overlapping fractions will be resolved
#' @param fix_missing str, optional. Defaults to NULL, which means no action.
#' If not NULL, must be one of the options: "zero_to_na" or "na_to_zero".
#' If "zero_to_na", Intensity values equal exactly to 0 will be converted to NA.
#' If "na_to_zero", missing values will be replaced by zeros.
#' @param remove_few lgl, if TRUE, features with one or two measurements 
#' across runs will be removed.
#' @param anomaly_metrics character vector of names of columns with quality metrics
#' 
#' @export
#' @return data.frame of class `MSstatsValidated`
#' 
#' @examples
#' unbalanced_data = system.file("tinytest/raw_data/unbalanced_data.csv", 
#'                               package = "MSstatsConvert")
#' unbalanced_data = data.table::as.data.table(read.csv(unbalanced_data))
#' balanced = MSstatsBalancedDesign(unbalanced_data, 
#'                                  c("PeptideSequence", "PrecursorCharge",
#'                                    "FragmentIon", "ProductCharge"))
#' dim(balanced) # Now balanced has additional rows (with Intensity = NA)
#' # for runs that were not included in the unbalanced_data table
#' 
MSstatsBalancedDesign = function(input, feature_columns, fill_incomplete = TRUE,
                                 handle_fractions = TRUE, fix_missing = NULL,
                                 remove_few = TRUE, anomaly_metrics = c()) {
    feature = NULL
    
    input[, feature := do.call(".combine", .SD), .SDcols = feature_columns]
    if (handle_fractions) {
        input = .handleFractions(input)
        input = .filterFewMeasurements(input, 1, 
                                       remove_few, 
                                       feature_columns)
        msg_fractions = "** Fractionation handled."
        getOption("MSstatsLog")("INFO", msg_fractions)
        getOption("MSstatsMsg")("INFO", msg_fractions)
    } 
    input = .makeBalancedDesign(input, fill_incomplete, anomaly_metrics)
    msg_balanced = paste("** Updated quantification data to make balanced design.",
                         "Missing values are marked by NA")
    getOption("MSstatsLog")("INFO", msg_balanced)
    getOption("MSstatsMsg")("INFO", msg_balanced)
    input = .fixMissingValues(input, fix_missing)
    input = input[, !(colnames(input) %in% c("feature", "isZero")), 
                  with = FALSE]
    
    getOption("MSstatsLog")("INFO", "\n")
    .MSstatsFormat(input, anomaly_metrics)
}


#' Create annotation
#' 
#' @param input data.table preprocessed by the MSstatsClean function
#' @param annotation data.table 
#' @param ... key-value pairs, where keys are names of columns of `annotation` 
#' 
#' @return data.table
#' @export
#' 
#' @examples 
#' evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
#'                             package = "MSstatsConvert")
#' pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
#'                       package = "MSstatsConvert")
#' evidence = read.csv(evidence_path)
#' pg = read.csv(pg_path)
#' imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
#'                          "MSstats", "MaxQuant")
#' cleaned_data = MSstatsClean(imported, protein_id_col = "Proteins")
#' annot_path = system.file("tinytest/raw_data/MaxQuant/annotation.csv", 
#'                          package = "MSstatsConvert")
#' mq_annot = MSstatsMakeAnnotation(cleaned_data, read.csv(annot_path),
#'                                  Run = "Rawfile")
#' head(mq_annot)
#' 
MSstatsMakeAnnotation = function(input, annotation, ...) {
    all_columns = unlist(list(...))
    if (!is.null(annotation)) {
        annotation = .getDataTable(annotation)
        msg = "** Using provided annotation."
        getOption("MSstatsLog")("INFO", msg)
        getOption("MSstatsMsg")("INFO", msg)
    } else {
        cols = c("Run", "Channel", "Condition", "BioReplicate", "TechReplicate",
                 "Mixture", "TechRepMixture", "Fraction", unname(all_columns))
        cols = intersect(cols, colnames(input))
        annotation = unique(input[, cols, with = FALSE])
        msg = "** Using annotation extracted from quantification data."
        getOption("MSstatsLog")("INFO", msg)
        getOption("MSstatsMsg")("INFO", msg)
    }
    if (length(all_columns) > 0) {
        data.table::setnames(annotation, 
                             unname(all_columns),
                             names(all_columns),
                             skip_absent = TRUE)
    }
    annotation = annotation[, !duplicated(colnames(annotation)), 
                            with = FALSE]
    .checkAnnotation(input, annotation)
    if (is.element("Channel", colnames(annotation))) {
        annotation$Channel = .standardizeColnames(annotation$Channel)
        labels_msg = "Run and Channel"
    } else {
        labels_msg = "Run"
    }
    annotation$Run = .standardizeColnames(annotation$Run)
    msg = paste("**", labels_msg, "labels were standardized to remove",
                "symbols such as '.' or '%'.")
    getOption("MSstatsLog")("INFO", msg)
    getOption("MSstatsMsg")("INFO", msg)
    annotation
}

#' Run Anomaly Model
#'
#' Detects anomalous measurements in mass spectrometry data using an isolation forest algorithm.
#' This function identifies unusual precursor measurements based on quality metrics and their 
#' temporal patterns. For features with insufficient quality metric data, it assigns anomaly 
#' scores based on the median score of similar features (same peptide and charge combination).
#' The model supports parallel processing for improved performance on large datasets.
#' 
#' @param input data.table preprocessed by the MSstatsBalancedDesign function
#' @param quality_metrics character vector of quality metrics to use in the model
#' @param temporal_direction character vector of same length as quality_metrics indicating temporal feature to create.
#' @param missing_run_count numeric, maximum allowed fraction of missing runs per feature.
#' @param n_feat numeric, maximum number of features per protein to use in the model.
#' @param run_order data.frame with two columns: Run and Order. Order should be numeric and indicate the order of runs.
#' @param n_trees numeric, number of trees to use in the isolation forest model. Default is 100.
#' @param max_depth numeric or "auto", maximum depth of each tree. Default is "auto" which sets depth to log2(N) where N is the number of runs.
#' @param cores numeric, number of cores to use for parallel processing. Default is 1.
#' @useDynLib MSstatsConvert, .registration = TRUE
#' 
#' @return data.table
#' @export
MSstatsAnomalyScores = function(input, quality_metrics, temporal_direction,
                                missing_run_count, n_feat, run_order, n_trees, 
                                max_depth, cores){

    input = .prepareSpectronautAnomalyInput(input, quality_metrics, 
                                            run_order, n_feat, 
                                            missing_run_count)
    input$PSM = paste0(input$PeptideSequence, input$PrecursorCharge)
    
    for (i in seq_along(quality_metrics)){
        if (temporal_direction[i] != FALSE){
            quality_metrics = c(quality_metrics,
                                paste0(quality_metrics[i], ".",
                                       temporal_direction[i]))
        }
    }

    input = .runAnomalyModel(input, 
                             n_trees=n_trees, 
                             max_depth=max_depth, 
                             cores=cores,
                             split_column="PSM",
                             quality_metrics=quality_metrics)

    subset_cols = c("Run", "ProteinName", "PeptideSequence", 
                    "PrecursorCharge", "FragmentIon", 
                    "ProductCharge", "IsotopeLabelType", 
                    "Condition", "BioReplicate", 
                    "Fraction", "Intensity", "AnomalyScores",
                    quality_metrics)
    
    subset_cols = subset_cols[subset_cols %in% names(input)]
    input = input[, ..subset_cols]

    ordered_runs = .standardizeColnames(run_order$Run[order(run_order$Order)])
    input$Run = factor(input$Run, levels = ordered_runs)

    return(input)

}

#' Takes as input the output of the SpectronauttoMSstatsFormat function and calculates various quality metrics to assess the health of the data. Requires Anomaly Detection model to be fit.
#' 
#' @param input MSstats input which is the output of Spectronaut converter
#' @return list of two data.tables
#' 
#' @export
CheckDataHealth = function(input){
    
    input = as.data.table(input)
    
    # All intensity characteristics
    missing_percent = .checkMissing(input)
    zero_truncated = .checkIntensityDistribution(input)
    
    # Feature specific characteristics
    input$Feature = paste(input$PeptideSequence,
                          input$PrecursorCharge,
                          input$FragmentIon,
                          input$ProductCharge, sep="_")
    feature_data = .checkFeatureSD(input)
    outlier_info = .checkFeatureOutliers(input, feature_data)
    feature_data = outlier_info[[1]]
    outlier_summary = outlier_info[[2]]
    feature_data = .checkFeatureCoverage(input, feature_data)
    
    skew_results = .checkAnomalySkew(input)
    
    return(list(feature_data, skew_results))
}

openms = system.file("tinytest/raw_data/OpenMS/openms_input.csv",
                     package = "MSstatsConvert")
openms = read.csv(openms)

imported = MSstatsConvert::MSstatsImport(list(input = openms),
                                         "MSstats", "OpenMS")
expect_identical(is(imported), 
                           c("MSstatsOpenMSFiles", "MSstatsInputFiles"))
cleaned = MSstatsConvert::MSstatsClean(imported)
expect_identical(class(cleaned), 
                           c("data.table", "data.frame"))
annotation = MSstatsConvert::MSstatsMakeAnnotation(cleaned, NULL)
expect_identical(class(annotation), 
                           c("data.table", "data.frame"))
processed = MSstatsConvert::MSstatsPreprocess(
    cleaned, annotation, 
    feature_columns = c("PeptideSequence", "PrecursorCharge",
                        "FragmentIon", "ProductCharge")
)
expect_identical(class(processed), 
                           c("data.table", "data.frame"))
expect_true(nrow(processed) > 0)
expect_true(ncol(processed) == 10)
balanced = MSstatsConvert::MSstatsBalancedDesign(
    processed, 
    feature_columns = c("PeptideSequence", "PrecursorCharge",
                        "FragmentIon", "ProductCharge")
)
expect_true(nrow(balanced) > 0)
expect_true(ncol(balanced) == 11)

dataset = data.table::data.table(
    Run = 1:5,
    Condition = 1,
    BioReplicate = 1
)
dataset_tmt = data.table::data.table(
    Run = as.character(rep(1:5, each = 2)),
    Channel = rep(c("a", "b"), times = 5)
)
annotation_1 = data.table::data.table(
    Run = 1:5,
    Condition = 1,
    BioReplicate = 1
)
annotation_2 = data.table::data.table(
    Rawfile = 1:4,
    Condition = 1,
    BioReplicate = 1
)
annotation_3 = data.table::data.table(
    Rawfile = rep(1:5, each = 2),
    Condition = rep(c("A", "B"), times = 5),
    Channel = rep(c("a", "b"), times = 5),
    Mixture = 1,
    TechRepMixture = 1,
    Fraction = 1,
    BioReplicate = 1
)
annotation_4 = data.table::data.table(
    Rawfile = c(rep(1:4, each = 2), 5),
    TechRep = 1:9,
    Condition = c(rep(c("A", "B"), times = 4), "A"),
    Channel = c(rep(c("a", "b"), times = 4), "c"),
    Mixture = 1,
    TechRepMixture = 1,
    Fraction = 1,
    BioReplicate = 1
)
# Create annotation ----
## No annotation - return a subset of original data
unique_runs = unique(dataset)
unique_runs$Run = as.character(unique_runs$Run)
expect_equal(MSstatsConvert:::MSstatsMakeAnnotation(dataset, NULL),
             unique_runs)
## No additional information
expect_equal(MSstatsConvert:::MSstatsMakeAnnotation(
    dataset, data.table::data.table(Run = 1:5,
                                    Condition = 1,
                                    BioReplicate = 1)),
    data.table::data.table(Run = as.character(1:5),
                           Condition = 1,
                           BioReplicate = 1))
## Annotation provided - return annotation
annotation_1$Run = as.character(annotation_1$Run)
expect_identical(MSstatsConvert:::MSstatsMakeAnnotation(dataset, annotation_1),
                 annotation_1)
## Column names are updated
expect_true(
    is.element("Run",
               colnames(MSstatsConvert:::MSstatsMakeAnnotation(dataset, 
                                                               annotation_2,
                                                               Run = "Rawfile")))
)
# Merge annotation ----
## MSstats version: no new information in annotation
dataset_chr = dataset
dataset_chr$Run = as.character(dataset_chr$Run)
expect_identical(
    MSstatsConvert:::.mergeAnnotation(dataset, NULL),
    dataset_chr
)
## MSstats: annotation provided
expect_identical(
    MSstatsConvert:::.mergeAnnotation(dataset, annotation_1),
    merge(dataset_chr, annotation_1, sort = FALSE)
)
## MSstatsTMT: all is OK
tmt_annotation = MSstatsConvert:::MSstatsMakeAnnotation(dataset_tmt, annotation_3, Run = "Rawfile")
expect_identical(
    MSstatsConvert:::.mergeAnnotation(dataset_tmt, tmt_annotation),
    merge(dataset_tmt, tmt_annotation, sort = FALSE)
)
## MSstats: missing condition
missing_condition = MSstatsConvert:::MSstatsMakeAnnotation(dataset, annotation_2, Run = "Rawfile")
expect_stdout(MSstatsConvert:::.mergeAnnotation(dataset, missing_condition))
## MSstatsTMT: missing channel
missing_channel = MSstatsConvert:::MSstatsMakeAnnotation(dataset_tmt, annotation_4, Run = "Rawfile")
expect_error(MSstatsConvert:::.mergeAnnotation(dataset_tmt, missing_channel))
## Missing column: error
expect_error(MSstatsConvert::MSstatsMakeAnnotation(
    dataset, 
    annotation2[, .(Rawfile, BioReplicate)],
    Run = "Rawfile"))
## Multiple conditions in a Run
expect_error(MSstatsConvert::MSstatsMakeAnnotation(
    dataset, 
    rbind(annotation_2, data.table::data.table(Rawfile = 1, Condition = 2, BioReplicate = 2)),
    Run = "Rawfile"))

#' Import data from SpectroMine
#'
#' @param input data name of SpectroMine PSM output. Read PSM sheet.
#' @param annotation data frame which contains column Run, Fraction, TechRepMixture, Mixture, Channel, BioReplicate, Condition. Refer to the example 'annotation.mine' for the meaning of each column.
#' @param filter_with_Qvalue TRUE(default) will filter out the intensities that have greater than qvalue_cutoff in EG.Qvalue column. Those intensities will be replaced with NA and will be considered as censored missing values for imputation purpose.
#' @param qvalue_cutoff Cutoff for EG.Qvalue. default is 0.01.
#' @param rmPSM_withfewMea_withinRun TRUE (default) will remove the features that have 1 or 2 measurements within each Run.
#' @param rmProtein_with1Feature TRUE will remove the proteins which have only 1 peptide and charge. Defaut is FALSE.
#' @param ... additional parameters to `data.table::fread`.
#' @inheritParams .sharedParametersAmongConverters
#'  
#' @return `data.frame` of class `MSstatsTMT`
#' 
#' @export 
#' @examples
#' raw.mine = data.table::fread(system.file("tinytest/raw_data/SpectroMine/spectromine_input.csv",
#'                                       package = "MSstatsConvert"))
#' annotation.mine = data.table::fread(system.file("tinytest/raw_data/SpectroMine/spectromine_annotation.csv",
#'                                       package = "MSstatsConvert"))
#' head(raw.mine)
#' head(annotation.mine)
#' input.mine <- SpectroMinetoMSstatsTMTFormat(raw.mine, annotation.mine)
#' head(input.mine)
#' 
SpectroMinetoMSstatsTMTFormat <- function(
        input, annotation, filter_with_Qvalue = TRUE, qvalue_cutoff = 0.01,
        useUniquePeptide = TRUE, rmPSM_withfewMea_withinRun = TRUE, 
        rmProtein_with1Feature = FALSE, summaryforMultipleRows = sum,
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL, ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsTMT_converter_log_")
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstatsTMT", "SpectroMine", ...)
    input = MSstatsConvert::MSstatsClean(input)
    annotation = MSstatsMakeAnnotation(input, annotation)
    
    pq_filter = list(score_column = "PGQValue", 
                     score_threshold = 0.01, 
                     direction = "smaller",
                     behavior = "fill", 
                     handle_na = "keep", 
                     fill_value = NA,
                     filter = TRUE, 
                     drop_column = TRUE)
    
    qval_filter = list(score_column = "Qvalue", 
                       score_threshold = qvalue_cutoff, 
                       direction = "smaller",
                       behavior = "fill", 
                       handle_na = "keep", 
                       fill_value = NA,
                       filter = filter_with_Qvalue, 
                       drop_column = TRUE)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge") 
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = rmProtein_with1Feature,
        score_filtering = list(pgq = pq_filter, psm_q = qval_filter),
        feature_cleaning = list(remove_features_with_few_measurements = rmPSM_withfewMea_withinRun,
                                summarize_multiple_psms = summaryforMultipleRows)
    )
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  fix_missing = "zero_to_na")
    data.table::setnames(input, "PrecursorCharge", "Charge", skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the proteinSummarization function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

#' Import MaxQuant files
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param evidence name of 'evidence.txt' data, which includes feature-level data.
#' @param annotation name of 'annotation.txt' data which includes Raw.file, Condition, BioReplicate, Run, IsotopeLabelType information.
#' @param proteinGroups name of 'proteinGroups.txt' data. It needs to matching protein group ID. If proteinGroups=NULL, use 'Proteins' column in 'evidence.txt'.
#' @param proteinID 'Proteins'(default) or 'Leading.razor.protein' for Protein ID.
#' @param ... additional parameters to `data.table::fread`.
#' 
#' @return data.frame in the MSstats required format.
#'  
#' @note Warning: MSstats does not support for metabolic labeling or iTRAQ experiments.
#' 
#' @author Meena Choi, Olga Vitek. 
#' 
#' @export
#' 
#' @examples 
#' mq_ev = data.table::fread(system.file("tinytest/raw_data/MaxQuant/mq_ev.csv",
#'                                       package = "MSstatsConvert"))
#' mq_pg = data.table::fread(system.file("tinytest/raw_data/MaxQuant/mq_pg.csv",
#'                                       package = "MSstatsConvert"))
#' annot = data.table::fread(system.file("tinytest/raw_data/MaxQuant/annotation.csv",
#'                                       package = "MSstatsConvert"))
#' maxq_imported = MaxQtoMSstatsFormat(mq_ev, annot, mq_pg, use_log_file = FALSE)
#' head(maxq_imported)
#' 
MaxQtoMSstatsFormat = function(
        evidence, annotation, proteinGroups, proteinID = "Proteins", 
        useUniquePeptide = TRUE, summaryforMultipleRows = max, 
        removeFewMeasurements = TRUE, removeMpeptides = FALSE,
        removeOxidationMpeptides = FALSE, removeProtein_with1Peptide = FALSE, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(evidence = evidence, 
                                               protein_groups = proteinGroups), 
                                          type = "MSstats",
                                          tool = "MaxQuant", ...)
    input = MSstatsConvert::MSstatsClean(input, 
                                         protein_id_col = proteinID, 
                                         remove_by_site = TRUE)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, 
                                                       annotation, 
                                                       "Run" = "Rawfile")
    
    m_filter = list(col_name = "PeptideSequence", 
                    pattern = "M", 
                    filter = removeMpeptides, 
                    drop_column = FALSE)
    
    oxidation_filter = list(col_name = "Modifications", 
                            pattern = "Oxidation", 
                            filter = removeOxidationMpeptides, 
                            drop_column = TRUE)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation,
        feature_columns,
        remove_shared_peptides = useUniquePeptide, 
        remove_single_feature_proteins = removeProtein_with1Peptide,
        pattern_filtering = list(oxidation = oxidation_filter,
                                 m = m_filter),
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = summaryforMultipleRows),
        columns_to_fill = list("FragmentIon" = NA,
                               "ProductCharge" = NA,
                               "IsotopeLabelType" = "L"))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  remove_few = removeFewMeasurements)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test MaxQtoMSstatsFormat ---------------------------
mq_ev = data.table::fread(system.file("tinytest/raw_data/MaxQuant/mq_ev.csv",
                                      package = "MSstatsConvert"))
mq_pg = data.table::fread(system.file("tinytest/raw_data/MaxQuant/mq_pg.csv",
                                      package = "MSstatsConvert"))
annot = data.table::fread(system.file("tinytest/raw_data/MaxQuant/annotation.csv",
                                      package = "MSstatsConvert"))
output = MaxQtoMSstatsFormat(mq_ev, annot, mq_pg, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 1590)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))

#' Clean raw Spectronaut output.
#' @param msstats_object an object of class `MSstatsSpectronautFiles`.
#' @inheritParams SpectronauttoMSstatsFormat
#' @return `data.table`
#' @keywords internal
.cleanRawSpectronaut = function(msstats_object, intensity,
                                calculateAnomalyScores,
                                anomalyModelFeatures,
                                peptideSequenceColumn = "EG.ModifiedSequence",
                                heavyLabels = NULL) {
  FFrgLossType = FExcludedFromQuantification = NULL

  spec_input = getInputFile(msstats_object, "input")
  .validateSpectronautInput(spec_input, peptideSequenceColumn)
  spec_input = .addSpectronautColumnsIfMissing(spec_input)
  spec_input = spec_input[FFrgLossType == "noloss", ]
  
  f_charge_col = .findAvailable(c("FCharge", "FFrgZ"), colnames(spec_input), fall_back = "FCharge")
  pg_qval_col = .findAvailable(c("PGQvalue"), colnames(spec_input))
  interference_col = .findAvailable(c("FPossibleInterference"),
                                    colnames(spec_input))
  exclude_col = .findAvailable(c("FExcludedFromQuantification"),
                               colnames(spec_input))
  intensity_col = .resolveSpectronautIntensityColumn(intensity, colnames(spec_input))
  peptide_col = .findAvailable(.standardizeColnames(peptideSequenceColumn),
                               colnames(spec_input))
  protein_col = .findAvailable(c("PGProteinGroups", "PGProteinAccessions"),
                               colnames(spec_input), fall_back = "PGProteinGroups")

  cols = c(protein_col, peptide_col, "FGCharge", "FFrgIon",
           f_charge_col, "RFileName", "RCondition", "RReplicate",
           "EGQvalue", pg_qval_col, interference_col, exclude_col,
           intensity_col)
  if (calculateAnomalyScores){
    cols = c(cols, anomalyModelFeatures)
  }
  cols = intersect(cols, colnames(spec_input))
  spec_input = spec_input[, cols, with = FALSE]

  data.table::setnames(
    spec_input,
    c(protein_col, peptide_col, "FGCharge", "FFrgIon",
      f_charge_col, "RFileName", intensity_col,
      "RCondition", "RReplicate"),
    c("ProteinName", "PeptideSequence", "PrecursorCharge", "FragmentIon",
      "ProductCharge", "Run", "Intensity", "Condition", "BioReplicate"),
    skip_absent = TRUE)

  spec_input = .assignSpectronautIsotopeLabelType(
    spec_input, heavyLabels)

  .logSuccess("Spectronaut", "clean")
  spec_input
}

#' Helper method to validate input has necessary columns
#' @param spec_input dataframe input
#' @param peptideSequenceColumn character, name of the column containing peptide 
#' sequences, passed from user
#' @noRd
.validateSpectronautInput = function(spec_input, peptideSequenceColumn) {
    # Only FGCharge is truly required
    required_columns = c("FGCharge")
    missing_columns = setdiff(required_columns, colnames(spec_input))
    if (length(missing_columns) > 0) {
        msg = paste("The following columns are missing from the input data:",
                    paste(missing_columns, sep = ", ", collapse = ", "))
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
    # Ensure at least one protein name column is present
    if (!any(c("PGProteinGroups", "PGProteinAccessions") %in% colnames(spec_input))) {
        msg = paste("The following columns are missing from the input data:",
                    "PGProteinGroups")
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
    # Ensure at least one protein name column is present
    if (!(.standardizeColnames(peptideSequenceColumn) %in% colnames(spec_input))) {
        msg = paste("The following column are missing from the input data:",
                    peptideSequenceColumn)
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
}

#' Add synthetic columns that are absent in protein turnover Spectronaut reports.
#'
#' Spectronaut's protein turnover export omits several columns that the
#' standard MSstats pipeline expects.  Rather than requiring callers to patch
#' the data frame before passing it in (the "hacks" documented in the design
#' document), we synthesize sensible defaults here so the rest of the pipeline
#' sees a consistent schema.
#'
#' Columns synthesized when absent:
#' \describe{
#'   \item{FFrgLossType}{"noloss" — the filter \code{spec_input[FFrgLossType == "noloss"]}
#'     keeps all rows, which is the correct behavior when the column is missing.}
#'   \item{FExcludedFromQuantification}{FALSE — no rows are excluded.}
#'   \item{FFrgIon}{NA — fragment ion identity is not available at the precursor
#'     level used by MS1-based protein turnover workflows.}
#'   \item{FCharge}{NA — product charge is not applicable when fragment-level
#'     columns are absent.}
#' }
#'
#' @param spec_input `data.table` with standardized column names.
#' @return `data.table` with missing columns added.
#' @keywords internal
#' @noRd
.addSpectronautColumnsIfMissing = function(spec_input) {
  if (!("FFrgLossType" %in% colnames(spec_input))) {
    spec_input[, FFrgLossType := "noloss"]
  }
  if (!("FExcludedFromQuantification" %in% colnames(spec_input))) {
    spec_input[, FExcludedFromQuantification := FALSE]
  }
  if (!("FFrgIon" %in% colnames(spec_input))) {
    spec_input[, FFrgIon := NA_character_]
  }
  if (!("FCharge" %in% colnames(spec_input))) {
    spec_input[, FCharge := NA_integer_]
  }
  spec_input
}


#' Resolve the Spectronaut intensity column from user input.
#'
#' Accepts either a legacy enum alias or a raw (standardized) column name.
#' The legacy aliases map to their canonical standardized column names so that
#' old code continues to work unchanged.  When the user passes a raw column
#' name (e.g. \code{"FGMS1Quantity"} or \code{"FGMS2Quantity"}), it is used
#' directly after verifying that the column exists.
#'
#' @param intensity Character scalar: enum alias or raw column name.
#' @param available_cols Character vector of available standardized column names.
#' @return The resolved standardized column name.
#' @keywords internal
#' @noRd
.resolveSpectronautIntensityColumn = function(intensity, available_cols) {
  legacy_mapping = c(
    "PeakArea"           = "FPeakArea",
    "NormalizedPeakArea" = "FNormalizedPeakArea"
  )

  if (intensity %in% names(legacy_mapping)) {
    resolved = legacy_mapping[[intensity]]
  } else {
    resolved = .standardizeColnames(intensity)
  }

  if (!(resolved %in% available_cols)) {
    stop(paste0(
      "Intensity column '", intensity, "' not found in input data. ", collapse = ", ")
    )
  }
  resolved
}


#' Assign IsotopeLabelType based on heavy label detection.
#'
#' When \code{heavyLabel} is provided, each row is classified as heavy
#' (\code{"H"}), light (\code{"L"}), or unlabeled (\code{NA}) by inspecting
#' the labeled sequence column for the presence of the label tag.
#'
#' In Spectronaut protein turnover reports, heavy peptides appear in
#' \code{FG.LabeledSequence} with a bracketed modification, e.g.
#' \code{_PEPTIDEK[Lys6]_}.  Any sequence that contains
#' \code{[<heavyLabel>]} is classified as heavy; all others are light.
#' Sequences that do not have amino acids that can carry the label
#' are classified as \code{NA}.  For example, if \code{heavyLabels} is 
#' \code{"Lys6"}, then \code{PEPTIDEZ} is classified as NA since it
#' has no lysine residues that could be labeled.
#' When \code{heavyLabel} is \code{NULL} the column is left untouched so
#' that the downstream \code{columns_to_fill} default of \code{"L"} applies,
#' preserving backwards compatibility.
#'
#' @param spec_input `data.table` after column renaming.
#' @param heavyLabels Character scalar heavy label name (e.g. \code{"Lys6"}),
#'   or \code{NULL}.
#' @return `data.table` with \code{IsotopeLabelType} column added or updated.
#' @keywords internal
#' @noRd
.assignSpectronautIsotopeLabelType = function(spec_input, heavyLabels) {
    PeptideSequence = NULL
    if (is.null(heavyLabels)) {
        return(spec_input)
    }

    bare_amino_acids = sub("\\[.*\\]", "", heavyLabels)
    labeled_aa_regex = paste0(bare_amino_acids, collapse = "|")
    heavy_regex = paste(
        gsub("([\\[\\]])", "\\\\\\1", heavyLabels, perl = TRUE),
        collapse = "|"
    )

    spec_input = .classifyIsotopeLabelType(spec_input, heavy_regex,
                                            labeled_aa_regex = labeled_aa_regex)

    for (i in seq_along(heavyLabels)) {
        escaped = gsub("([\\[\\]])", "\\\\\\1", heavyLabels[i], perl = TRUE)
        spec_input[, PeptideSequence := gsub(escaped, bare_amino_acids[i],
                                             PeptideSequence, perl = TRUE)]
    }
    spec_input
}

# Test intensity parameter
spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_raw$FG.MS1Quantity = 100000
msstats_input = MSstatsConvert::MSstatsImport(
    list(input = spectronaut_raw), "MSstats", "Spectronaut")
output = MSstatsConvert:::.cleanRawSpectronaut(msstats_input, intensity = 'FG.MS1Quantity', 
                                    calculateAnomalyScores = FALSE, 
                                    anomalyModelFeatures = c())
expect_true(all(output$Intensity == 100000))

expect_error(MSstatsConvert:::.cleanRawSpectronaut(msstats_input, intensity = 'invalid',
                                                   calculateAnomalyScores = FALSE,
                                                   anomalyModelFeatures = c()),
             pattern = "not found in input data")

# Tests for .assignSpectronautIsotopeLabelType
make_spec_input = function(peptides) {
    data.table::data.table(PeptideSequence = peptides)
}

dt = make_spec_input(c(
    "_PEPTIDEK[Lys6]_",   # heavy via Lys6
    "_PEPTIDER[Arg10]_",  # heavy via Arg10
    "_PEPTIDEK_",         # light (has K, but no Lys6 label)
    "_PEPTIDER_",         # light (has R, but no Arg10 label)
    "_PEPTIDEAC_"         # NA (no K or R)
))
result = MSstatsConvert:::.assignSpectronautIsotopeLabelType(
    dt, heavyLabels = c("K[Lys6]", "R[Arg10]"))
expect_equal(result$IsotopeLabelType, c("H", "H", "L", "L", NA_character_))
expect_equal(result$PeptideSequence,
             c("_PEPTIDEK_", "_PEPTIDER_", "_PEPTIDEK_", "_PEPTIDER_", "_PEPTIDEAC_"))

dt = make_spec_input(c(
    "_PEPTM[Oxidation]IDEK_",   
    "_S[Kmodification]PEPTIDEK_", 
    "_PEPTM[Oxidation]IDEK[Lys6]_",
    "_ACDEGFHI_"
))
result = MSstatsConvert:::.assignSpectronautIsotopeLabelType(
    dt, heavyLabels = "K[Lys6]")
expect_equal(result$IsotopeLabelType, c("L", "L", "H", NA_character_))
# Non-heavy brackets and heavy label brackets are stripped from PeptideSequence
expect_equal(result$PeptideSequence,
             c("_PEPTM[Oxidation]IDEK_", "_S[Kmodification]PEPTIDEK_",
               "_PEPTM[Oxidation]IDEK_", "_ACDEGFHI_"))

dt = make_spec_input(c("_PEPTIDEK_", "_PEPTIDER_"))
result = MSstatsConvert:::.assignSpectronautIsotopeLabelType(dt, heavyLabels = NULL)
expect_equal(result, dt)

#' Clean raw output from OpenMS
#' @param msstats_object an object of class `MSstatsSpectroMineFiles`.
#' @return data.table
#' @keywords internal
.cleanRawOpenMS = function(msstats_object) {
  Intensity = PSM = NULL
  
  om_input = getInputFile(msstats_object, "input")
  om_input[, Intensity := as.numeric(as.character(Intensity))]
  om_input = unique(om_input)
  
  if (getDataType(msstats_object) == "MSstats") {
    if (!is.element("IsotopeLabelType", colnames(om_input))) {
      om_input = .fillValues(om_input, c("IsotopeLabelType" = "L"))
    }
  } else {
    om_input[, PSM := do.call(".combine", .SD), 
             .SDcols = c("PeptideSequence", "Charge", "RetentionTime")]
    om_input[, Intensity := ifelse(Intensity == 0, NA, Intensity)]
  }
  
  all_cols = c("ProteinName", "PeptideSequence", "PrecursorCharge", "Charge",
               "FragmentIon", "ProductCharge", "PSM", "IsotopeLabelType",
               "Condition", "BioReplicate", "Run", "Channel", "Intensity",
               "Mixture", "TechRepMixture", "TechReplicate", "Fraction")
  cols = intersect(all_cols, colnames(om_input))
  om_input = om_input[, cols, with = FALSE]
  data.table::setnames(om_input, "Charge", "PrecursorCharge", 
                       skip_absent = TRUE)
  .logSuccess("OpenMS", "clean")
  unique(om_input)
}

#' Clean raw DIAUmpire files
#' @param msstats_object Object that inherits from MSstatsInputFiles class.
#' @param use_frag TRUE will use the selected fragment for each peptide. 
#' 'Selected_fragments' column is required.
#' @param use_pept TRUE will use the selected fragment for each protein 
#' 'Selected_peptides' column is required.
#' @return data.table
#' @keywords internal
.cleanRawDIAUmpire = function(msstats_object, use_frag, use_pept) {
    FragmentIon = ProteinName = PeptideSequence = Selected_peptides = NULL
    Selected_fragments = Proteins = ProteinKey = FragmentKey = Run = NULL
    frag_input = getInputFile(msstats_object, "Fragments")
    pept_input = getInputFile(msstats_object, "Peptides")
    prot_input = getInputFile(msstats_object, "Proteins")
    
    if (!is.element("Selected_fragments", colnames(pept_input)) | 
        !is.element("Selected_peptides", colnames(prot_input))) {
        msg = "** Selected_fragments column is required. Please check it."
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
    
    pept_cols = c("PeptideKey", "Proteins", "Selected_fragments")
    pept_input = pept_input[, pept_cols, with = FALSE]
    pept_input = pept_input[Proteins != "", ]
    pept_input[, Selected_fragments := as.character(Selected_fragments)]
    data.table::setnames(
        pept_input,
        colnames(pept_input), 
        c("PeptideSequence", "ProteinName", "FragmentIon"),
        skip_absent = TRUE)
    pept_input = pept_input[
        , lapply(list(FragmentIon), 
                 function(x) unlist(data.table::tstrsplit(x, "|", 
                                                          fixed = TRUE))),
        by = c("ProteinName", "PeptideSequence")
        ] 
    data.table::setnames(pept_input, "V1", "FragmentIon", skip_absent = TRUE)
    pept_input = pept_input[pept_input[["FragmentIon"]] != "", ]
    pept_input[["ProteinName"]] = as.character(pept_input[["ProteinName"]])
    
    prot_input = prot_input[, c("ProteinKey", "Selected_peptides"), 
                            with = FALSE]
    prot_input = prot_input[ProteinKey != "", ] 
    prot_input[, Selected_peptides := as.character(Selected_peptides)]
    prot_input[, Selected_peptides := trimws(Selected_peptides, "both")]
    data.table::setnames(prot_input, colnames(prot_input),
                         c("ProteinName", "PeptideSequence"),
                         skip_absent = TRUE)
    prot_input = prot_input[
        , lapply(list(PeptideSequence),
                 function(x) (unlist(data.table::tstrsplit(x, "|", 
                                                           fixed = TRUE)))),
        by = c("ProteinName")]
    data.table::setnames(prot_input, "V1", "PeptideSequence", 
                         skip_absent = TRUE)
    prot_input = prot_input[PeptideSequence != "", ]
    prot_input[, ProteinName := gsub(";", "", ProteinName)]
    
    if (use_frag & use_pept) {
        input = merge(pept_input[, c(FALSE, TRUE, TRUE), with = FALSE], 
                      prot_input, all.x = TRUE, by = "PeptideSequence",
                      sort = FALSE)
        getOption("MSstatsMsg")("INFO", "** Using selected fragments and peptides.")
    } else if (use_frag & !use_pept) { 
        input = pept_input
        input$ProteinName = gsub(";", "", input$ProteinName)
        getOption("MSstatsMsg")("INFO", "** Using selected fragments.")
    } else {
        msg = "** MSstats recommends to use at least selected fragments."
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
    input[, FragmentIon := gsub("\\+", "_", FragmentIon)]
    
    intensity_cols = grepl("Intensity", colnames(frag_input))
    frag_col_names = c("FragmentKey", "Protein", "Peptide", "Fragment")
    key_cols = colnames(frag_input) %in% frag_col_names
    frag_cols = intensity_cols | key_cols
    frag_input = frag_input[, frag_cols, with = FALSE]
    new_names = c("ProteinName", "PeptideSequence", "FragmentIon")
    data.table::setnames(frag_input, frag_col_names[2:4], new_names,
                         skip_absent = TRUE)
    frag_input = .fixColumnTypes(
        frag_input, character_columns = c(new_names, "FragmentKey"))
    
    if (!grepl("\\+", frag_input[["FragmentIon"]][1])) {
        last_char = nchar(frag_input[["FragmentKey"]])
        frag_input[, FragmentIon := paste(FragmentIon, 
                                          substr(FragmentKey, last_char, 
                                                 last_char), sep = "_")]
    } else {
        frag_input[, FragmentIon := gsub("\\+", "_", FragmentIon)]
    }
    frag_input = frag_input[, (colnames(frag_input) != "FragmentKey"),
                            with = FALSE]
    
    setkey(input, ProteinName, PeptideSequence, FragmentIon)
    setkey(frag_input, ProteinName, PeptideSequence, FragmentIon)
    to_join = unique(frag_input[input, which = TRUE, 
                                allow.cartesian = TRUE, drop = FALSE])
    frag_input = frag_input[to_join]
    frag_input = melt(frag_input, 
                      id.vars = new_names, variable.name = "Run", 
                      value.name = "Intensity", value.factor = FALSE)
    frag_input[, Run := gsub("_Intensity", "", Run)]
    getOption("MSstatsMsg")("INFO", paste("** Extracted the data from selected",
                                          "fragments and/or peptides."))
    .logSuccess("DIAUmpire", "clean")
    frag_input[!(is.na(ProteinName))]
}

#' Clean raw Diann files
#' @param msstats_object an object of class `MSstatsDIANNFiles`.
#' @inheritParams DIANNtoMSstatsFormat
#' @return data.table
#' @importFrom stats na.omit
#' @keywords internal
.cleanRawDIANN <- function(msstats_object, MBR = TRUE,
                           quantificationColumn = "FragmentQuantCorrected",
                           global_qvalue_cutoff = 0.01,
                           qvalue_cutoff = 0.01,
                           pg_qvalue_cutoff = 0.01,
                           calculateAnomalyScores = FALSE,
                           anomalyModelFeatures = c(),
                           labeledAminoAcids = NULL) {
    dn_input <- getInputFile(msstats_object, "input")
    dn_input <- data.table::as.data.table(dn_input)

    # Process quantification columns
    quantificationColumn <- .cleanDIANNProcessQuantificationColumns(dn_input, quantificationColumn)

    # Add missing columns
    dn_input <- .cleanDIANNAddMissingColumns(dn_input)

    has_channel <- !is.null(labeledAminoAcids) && "Channel" %in% colnames(dn_input)

    # Select required columns
    dn_input <- .cleanDIANNSelectRequiredColumns(dn_input, quantificationColumn, MBR,
                                                 calculateAnomalyScores,
                                                 anomalyModelFeatures,
                                                 has_channel = has_channel)

    # Split concatenated values
    dn_input <- .cleanDIANNSplitConcatenatedValues(dn_input, quantificationColumn)

    # Process fragment information
    dn_input <- .cleanDIANNProcessFragmentInfo(dn_input, quantificationColumn)

    # Clean and filter data
    dn_input <- .cleanDIANNCleanAndFilterData(dn_input, MBR, quantificationColumn,
                                              global_qvalue_cutoff,
                                              qvalue_cutoff,
                                              pg_qvalue_cutoff)

    # Rename columns
    dn_input <- .cleanDIANNRenameColumns(dn_input, quantificationColumn)

    # Assign IsotopeLabelType for protein turnover
    dn_input <- .assignDIANNIsotopeLabelType(dn_input, labeledAminoAcids, has_channel)

    .logSuccess("DIANN", "clean")
    dn_input
}

#' Process quantification columns for DIANN 2.0 format
#' @param dn_input data.table input
#' @param quantificationColumn quantification column name
#' @return updated quantification column name
#' @noRd
.cleanDIANNProcessQuantificationColumns <- function(dn_input, quantificationColumn) {
    if (quantificationColumn == "auto") {
        fragment_columns <- grep("^Fr[0-9]+Quantity$", names(dn_input), value = TRUE)
        if (length(fragment_columns) == 0) {
            stop("No fragment quantification columns found. Please check your input.")
        }
        dn_input[, FragmentQuantCorrected := do.call(paste, c(.SD, sep = ";")),
                 .SDcols = fragment_columns]
        quantificationColumn <- "FragmentQuantCorrected"
    }
    return(quantificationColumn)
}

#' Add missing required columns
#' @param dn_input data.table input
#' @return data.table with missing columns added
#' @noRd
.cleanDIANNAddMissingColumns <- function(dn_input) {
    if (!is.element("PrecursorMz", colnames(dn_input))) {
        dn_input[, PrecursorMz := NA]
    }
    if (!is.element('FragmentInfo', colnames(dn_input))) {
        dn_input[, FragmentInfo := NA]
    }
    return(dn_input)
}

#' Select required columns based on MBR setting
#' @param dn_input data.table input
#' @param quantificationColumn quantification column name
#' @param MBR logical indicating if match between runs was used
#' @return data.table with selected columns
#' @noRd
.cleanDIANNSelectRequiredColumns <- function(dn_input, quantificationColumn, MBR,
                                             calculateAnomalyScores,
                                             anomalyModelFeatures,
                                             has_channel = FALSE) {
    base_cols <- c('ProteinNames', 'StrippedSequence', 'ModifiedSequence',
                   'PrecursorCharge', quantificationColumn, 'QValue',
                   'PrecursorMz', 'FragmentInfo', 'Run')

    if (has_channel) {
        base_cols <- c(base_cols, 'Channel')
    }

    mbr_cols <- if (MBR) {
        c('LibQValue', 'LibPGQValue')
    } else {
        c('GlobalQValue', 'GlobalPGQValue')
    }

    qual_cols <- if (calculateAnomalyScores) {
        anomalyModelFeatures
    } else {
        c()
    }

    req_cols <- c(base_cols, mbr_cols, qual_cols)
    return(dn_input[, req_cols, with = FALSE])
}

#' Split concatenated values in quantification and fragment info columns
#' @param dn_input data.table input
#' @param quantificationColumn quantification column name
#' @return data.table with split values
#' @noRd
.cleanDIANNSplitConcatenatedValues <- function(dn_input, quantificationColumn) {
    split_cols <- c(quantificationColumn, "FragmentInfo")
    by_cols <- setdiff(colnames(dn_input), split_cols)
    
    dn_input <- dn_input[, lapply(.SD, function(x) unlist(tstrsplit(x, ";"))),
                         .SDcols = split_cols, 
                         by = by_cols]
    return(dn_input)
}

#' Process fragment information and add derived columns
#' @param dn_input data.table input
#' @param quantificationColumn quantification column name
#' @return data.table with processed fragment info
#' @noRd
.cleanDIANNProcessFragmentInfo <- function(dn_input, quantificationColumn) {
    # Generate fragment info if missing
    if (all(is.na(dn_input[["FragmentInfo"]]))) {
        dn_input[, FragmentInfo := paste0("Frag", 1:.N),
                 by = c("ProteinNames", "ModifiedSequence", "PrecursorCharge", "Run")]
    }
    
    # Convert quantification column to numeric
    dn_input[, (quantificationColumn) := lapply(.SD, as.numeric), 
             .SDcols = quantificationColumn]
    
    # Process fragment ion information
    dn_input[, FragmentIon := sub('\\^\\.\\*', '', FragmentInfo)]
    
    # Extract product charge
    if (any(grepl("/", dn_input$FragmentInfo))) {
        dn_input[, ProductCharge := .cleanDIANNExtractProductCharge(FragmentInfo), by = FragmentInfo]
    } else {
        dn_input[, ProductCharge := 1]
    }
    
    return(dn_input)
}

#' Extract product charge from fragment info
#' @param fragment_info fragment information string
#' @return numeric product charge
#' @noRd
.cleanDIANNExtractProductCharge <- function(fragment_info) {
    charge_part <- unlist(strsplit(fragment_info, split = "/"))[[1]]
    return(strtoi(sub("\\.\\*\\^", "", charge_part)))
}

#' Clean and filter data by removing unwanted fragments and NA values
#' @param dn_input data.table input
#' @inheritParams DIANNtoMSstatsFormat
#' @return cleaned data.table
#' @noRd
.cleanDIANNCleanAndFilterData <- function(dn_input, MBR, quantificationColumn,
                                         global_qvalue_cutoff,
                                         qvalue_cutoff, 
                                         pg_qvalue_cutoff) {
    # Remove NH3 and H2O loss fragments & remove rows with NA in quant column
    dn_input <- dn_input[!grepl("NH3|H2O", FragmentIon) & 
                             !is.na(get(quantificationColumn))]
    
    msg = paste0('** Filtering on Q.Value < ', global_qvalue_cutoff)
    getOption("MSstatsLog")("INFO", msg)
    getOption("MSstatsMsg")("INFO", msg)
    
    dn_input = dn_input[QValue >= global_qvalue_cutoff, (quantificationColumn) := 0]
    if (MBR) {
        msg = '** MBR was used to analyze the data. Now setting names and filtering'
        msg_1_mbr = paste0('-- LibPGQValue < ', pg_qvalue_cutoff)
        msg_2_mbr = paste0('-- LibQValue < ', qvalue_cutoff)
        dn_input = dn_input[LibPGQValue >= pg_qvalue_cutoff, (quantificationColumn) := 0]
        dn_input = dn_input[LibQValue >= qvalue_cutoff, (quantificationColumn) := 0]
        getOption("MSstatsLog")("INFO", msg)
        getOption("MSstatsMsg")("INFO", msg)
        getOption("MSstatsLog")("INFO", msg_1_mbr)
        getOption("MSstatsMsg")("INFO", msg_1_mbr)
        getOption("MSstatsLog")("INFO", msg_2_mbr)
        getOption("MSstatsMsg")("INFO", msg_2_mbr)
        # getOption("MSstatsLog")("INFO", "\n")
    } else{
        msg = '** MBR was not used to analyze the data. Now setting names and filtering'
        msg_1 = paste0('-- Filtering on GlobalPGQValue < ', pg_qvalue_cutoff)
        msg_2 = paste0('-- Filtering on GlobalQValue < ', qvalue_cutoff)
        dn_input = dn_input[GlobalPGQValue >= pg_qvalue_cutoff, (quantificationColumn) := 0]
        dn_input = dn_input[GlobalQValue >= qvalue_cutoff, (quantificationColumn) := 0]
        getOption("MSstatsLog")("INFO", msg)
        getOption("MSstatsMsg")("INFO", msg)
        getOption("MSstatsLog")("INFO", msg_1)
        getOption("MSstatsMsg")("INFO", msg_1)
        getOption("MSstatsLog")("INFO", msg_2)
        getOption("MSstatsMsg")("INFO", msg_2)
        # getOption("MSstatsLog")("INFO", "\n")
    }
    
    return(dn_input)
}

#' Assign IsotopeLabelType for DIANN protein turnover workflows.
#'
#' Dispatches to one of two classification paths depending on \code{has_channel}:
#'
#' \strong{Channel-based path} (\code{has_channel = TRUE}): \code{Channel}
#' values are mapped directly to \code{IsotopeLabelType} (\code{"H"} →
#' \code{"H"}, \code{"L"} → \code{"L"}, anything else → \code{NA}), and the
#' \code{Channel} column is then dropped.  \code{labeledAminoAcids} acts solely
#' as the opt-in flag that enables this path; the amino acid codes are
#' \strong{not} used to validate or filter \code{ModifiedSequence}.
#'
#' \strong{ModifiedSequence-parsing path} (\code{has_channel = FALSE}):
#' \code{PeptideSequence} (the retained \code{ModifiedSequence}) is matched
#' against SILAC suffixes of the form \code{(SILAC-<AA>-H)} or
#' \code{(SILAC-<AA>-L)}, where \code{<AA>} is any code in
#' \code{labeledAminoAcids}.  Sequences carrying neither suffix receive
#' \code{IsotopeLabelType = NA}.  The SILAC suffix is stripped from
#' \code{PeptideSequence} after classification.
#'
#' @param dn_input \code{data.table} after column renaming.
#' @param labeledAminoAcids Character vector of single-letter amino acid codes
#'   (e.g. \code{c("K")} or \code{c("K", "R")}), or \code{NULL} to skip
#'   protein-turnover mode entirely (backwards-compatible default).
#' @param has_channel Logical; \code{TRUE} when the raw input contained a
#'   \code{Channel} column that was retained through
#'   \code{.cleanDIANNSelectRequiredColumns}.
#' @return \code{dn_input} with \code{IsotopeLabelType} column added (and
#'   \code{Channel} removed when \code{has_channel} is \code{TRUE}).
#' @keywords internal
#' @noRd
.assignDIANNIsotopeLabelType <- function(dn_input, labeledAminoAcids, has_channel) {
    IsotopeLabelType = Channel = PeptideSequence = NULL

    if (is.null(labeledAminoAcids)) {
        return(dn_input)
    }

    if (has_channel) {
        dn_input[, IsotopeLabelType := data.table::fcase(
            Channel == "H", "H",
            Channel == "L", "L",
            default = NA_character_
        )]
        dn_input[, Channel := NULL]
    } else {
        aa_pattern <- paste0(labeledAminoAcids, collapse = "|")
        heavy_regex <- paste0("\\(SILAC-(?:", aa_pattern, ")-H\\)")
        light_regex <- paste0("\\(SILAC-(?:", aa_pattern, ")-L\\)")
        strip_regex <- paste0("\\(SILAC-(?:", aa_pattern, ")-[HL]\\)")

        dn_input <- .classifyIsotopeLabelType(dn_input, heavy_regex, light_regex)
        dn_input[, PeptideSequence := gsub(strip_regex, "", PeptideSequence, perl = TRUE)]
    }

    if (all(is.na(dn_input[["IsotopeLabelType"]]))) {
        warning("labeledAminoAcids was provided but no rows were classified as H or L. ",
                "Check that the input contains either a Channel column with H/L values ",
                "or ModifiedSequence entries with (SILAC-<AA>-H)/(SILAC-<AA>-L) suffixes.")
    }

    dn_input
}


#' Rename columns to standardized names
#' @param dn_input data.table input
#' @param quantificationColumn quantification column name
#' @return data.table with renamed columns
#' @noRd
.cleanDIANNRenameColumns <- function(dn_input, quantificationColumn) {
    old_names <- c('ProteinNames', 'StrippedSequence', 'ModifiedSequence',
                   'PrecursorCharge', quantificationColumn, 'QValue', 
                   'PrecursorMz', 'FragmentIon', 'Run', 'ProductCharge')
    
    new_names <- c('ProteinName', 'PeptideSequence', 'PeptideModifiedSequence',
                   'PrecursorCharge', 'Intensity', 'DetectionQValue', 
                   'PrecursorMz', 'FragmentIon', 'Run', 'ProductCharge')
    
    data.table::setnames(dn_input, old = old_names, new = new_names, skip_absent = TRUE)
    
    # Clean up peptide sequence columns
    dn_input[, PeptideSequence := NULL]
    setnames(dn_input, "PeptideModifiedSequence", "PeptideSequence")
    
    return(dn_input)
}

# Test DIANN .cleanRawDIANN ---------------------------

.validateOutput = function(output) {
    expect_equal(ncol(output), 12)
    expect_true(nrow(output) > 0)
    expect_true("Run" %in% colnames(output))
    expect_true("ProteinName" %in% colnames(output))
    expect_true("PeptideSequence" %in% colnames(output))
    expect_true("PrecursorCharge" %in% colnames(output))
    expect_true("Intensity" %in% colnames(output))
    expect_true("DetectionQValue" %in% colnames(output))
    expect_true("PrecursorMz" %in% colnames(output))
    expect_true("LibQValue" %in% colnames(output))
    expect_true("LibPGQValue" %in% colnames(output))
    expect_true("FragmentInfo" %in% colnames(output))
    expect_true("FragmentIon" %in% colnames(output))
    expect_true("ProductCharge" %in% colnames(output))
    expect_equal(class(output$Intensity), "numeric")
}

file_path = system.file("tinytest/processed_data/DIANN/MSstatsDIANNFilesObject.rds", package="MSstatsConvert")
input = readRDS(file_path)
output = MSstatsConvert:::.cleanRawDIANN(input)
.validateOutput(output)
output = MSstatsConvert:::.cleanRawDIANN(input, quantificationColumn = "FragmentQuantRaw")
.validateOutput(output)

# Q-value filtering
expect_qvalue_cutoff <- function(output, col, cutoff) {
    expect_equal(
        sum(output[[col]] > cutoff),
        sum(output[["Intensity"]] == 0 & output[[col]] > cutoff),
        info = sprintf(
            "All rows with %s > %s should have %s == 0",
            col, cutoff, "Intensity"
        )
    )
    expect_equal(
        sum(output[[col]] <= cutoff),
        nrow(output) - sum(output[[col]] > cutoff),
        info = sprintf(
            "Rows with %s <= %s should account for all rows not above the cutoff",
            col, cutoff
        )
    )
}
output <- MSstatsConvert:::.cleanRawDIANN(input, global_qvalue_cutoff = 0.005)
expect_qvalue_cutoff(output, "DetectionQValue", 0.005)
output <- MSstatsConvert:::.cleanRawDIANN(input, qvalue_cutoff = 0.00001)
expect_qvalue_cutoff(output, "LibQValue", 0.00001)
output <- MSstatsConvert:::.cleanRawDIANN(input, pg_qvalue_cutoff = 0.001)
expect_qvalue_cutoff(output, "LibPGQValue", 0.001)
output <- MSstatsConvert:::.cleanRawDIANN(input, MBR = FALSE, qvalue_cutoff = 0.001)
expect_qvalue_cutoff(output, "GlobalQValue", 0.001)
output <- MSstatsConvert:::.cleanRawDIANN(input, MBR = FALSE, pg_qvalue_cutoff = 0.0002)
expect_qvalue_cutoff(output, "GlobalPGQValue", 0.0002)

# Test .assignDIANNIsotopeLabelType ---------------------------
# Channel path: Channel column is mapped to IsotopeLabelType, then dropped
dt_channel = data.table::data.table(
    PeptideSequence = c("PEPTIDEK", "PEPTIDEK", "PEPTIDEK"),
    Channel = c("H", "L", "other")
)
result_channel = MSstatsConvert:::.assignDIANNIsotopeLabelType(
    dt_channel, labeledAminoAcids = c("K"), has_channel = TRUE
)
expect_equal(result_channel$IsotopeLabelType, c("H", "L", NA_character_))
expect_false("Channel" %in% colnames(result_channel))

# NULL path: labeledAminoAcids = NULL returns input unchanged (no IsotopeLabelType added)
dt_null = data.table::data.table(
    PeptideSequence = c("PEPTIDEK(SILAC-K-H)", "PEPTIDEK(SILAC-K-L)")
)
result_null = MSstatsConvert:::.assignDIANNIsotopeLabelType(
    dt_null, labeledAminoAcids = NULL, has_channel = FALSE
)
expect_false("IsotopeLabelType" %in% colnames(result_null))
expect_equal(nrow(result_null), 2L)

#' Import Metamorpheus files
#' 
#' @param input name of Metamorpheus output file, which is tabular format. Use the AllQuantifiedPeaks.tsv file from the Metamorpheus output.
#' @param annotation name of 'annotation.txt' data which includes Condition, BioReplicate.
#' @param MBR If TRUE, the function will include peaks detected by MBR
#' @param qvalue_cutoff The q-value cutoff for filtering peaks detected by MBR 
#' @inheritParams .sharedParametersAmongConverters
#' 
#' @return data.frame in the MSstats required format.
#' 
#' @author Anthony Wu
#' 
#' @export
#' 
#' @examples 
#' input = system.file("tinytest/raw_data/Metamorpheus/QuantifiedPeaks.tsv", 
#'                                 package = "MSstatsConvert")
#' input = data.table::fread(input)
#' annot = system.file("tinytest/raw_data/Metamorpheus/annotation.csv", 
#'                                 package = "MSstatsConvert")
#' annot = data.table::fread(annot)
#' metamorpheus_imported = MSstatsConvert:::MetamorpheusToMSstatsFormat(input, annotation = annot)
#' head(metamorpheus_imported)
#' 
MetamorpheusToMSstatsFormat = function(
        input, annotation = NULL, MBR = TRUE, qvalue_cutoff = 0.05, 
        useUniquePeptide = TRUE, removeFewMeasurements = TRUE,
        removeProtein_with1Feature = FALSE, summaryforMultipleRows = max,
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    input = MSstatsConvert::MSstatsImport(list(input = input), 
                                          "MSstats", "Metamorpheus", ...)
    input = MSstatsConvert::MSstatsClean(input, MBR, qvalue_cutoff)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        feature_cleaning = list(remove_features_with_few_measurements = removeFewMeasurements,
                                summarize_multiple_psms = summaryforMultipleRows),
        columns_to_fill = list("FragmentIon" = NA,
                               "ProductCharge" = NA,
                               "IsotopeLabelType" = "L"))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  remove_few = removeFewMeasurements)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    return(input)
}

# Test MetamorpheusToMSstatsFormat ---------------------------

input_file_path = system.file("tinytest/raw_data/Metamorpheus/QuantifiedPeaks.tsv", package="MSstatsConvert")
annotation_file_path = system.file("tinytest/raw_data/Metamorpheus/annotation.csv", package = "MSstatsConvert")
input = data.table::fread(input_file_path)
annotation = data.table::fread(annotation_file_path)
output = MSstatsConvert:::MetamorpheusToMSstatsFormat(input, annotation = annotation)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 20)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))

#' Clean raw SpectroMine TMT data
#' @param msstats_object an object of class `MSstatsSpectroMineFiles`.
#' @importFrom data.table melt setnames
#' @return `data.table`
#' @keywords internal
.cleanRawSpectroMineTMT = function(msstats_object) {
  PSM = PeptideSequence = PrecursorCharge = ProteinName = NULL
  
  sm_input = getInputFile(msstats_object, "input")
  channels = .getChannelColumns(colnames(sm_input), "PSM", "Raw")
  if (length(channels) == 0L) {
    msg = paste("There is no channel intensity column in the input data,", 
                "which should start with 'PSM' and end with 'Raw'.")
    getOption("MSstatsMsg")("ERROR", msg)
    stop(msg)
  }
  pg_qval_column = grep("pgqvalue", colnames(sm_input), ignore.case = TRUE, value = TRUE)
  data.table::setnames(sm_input, pg_qval_column, "PGQValue")
  sm_input = sm_input[, c("PGProteinAccessions", "PMoleculeID", "PPCharge",
                          "PGQValue", "PSMQvalue", "RFileName", channels),
                      with = FALSE]
  data.table::setnames(sm_input, 
                       c("PGProteinAccessions", "PMoleculeID", 
                         "PPCharge", "PSMQvalue", "RFileName"),
                       c("ProteinName", "PeptideSequence", "PrecursorCharge",
                         "Qvalue", "Run"), skip_absent = TRUE)
  sm_input = sm_input[(ProteinName != "") & (!is.na(ProteinName)), ]
  sm_input = unique(sm_input)
  sm_input[, PSM := paste(PeptideSequence, PrecursorCharge, 
                          1:nrow(sm_input), sep = "_")]
  sm_input = melt(sm_input, measure.vars = channels,
                  id.vars = setdiff(colnames(sm_input), channels),
                  variable.name = "Channel", value.name = "Intensity")
  sm_input$Channel = gsub("PSM", "", sm_input$Channel)
  sm_input$Channel = gsub("Raw", "", sm_input$Channel)
  sm_input$Channel = gsub(".", "", sm_input$Channel, fixed = TRUE)
  sm_input$Intensity = ifelse(sm_input$Intensity == 0, NA, sm_input$Intensity)
  .logSuccess("SpectroMine", "clean")
  sm_input
}

#' Clean raw Metamorpheus files
#' @param msstats_object an object of class `MSstatsMetamorpheusFiles`.
#' @param MBR If TRUE, the function will include peaks detected by MBR
#' @param qvalue_cutoff The q-value cutoff for filtering peaks detected by MBR
#' @return data.table
#' @keywords internal
.cleanRawMetamorpheus = function(msstats_object, MBR = TRUE, qvalue_cutoff = 0.05) {
    metamorpheus_input = getInputFile(msstats_object, "input")
    metamorpheus_input = data.table::as.data.table(metamorpheus_input)
    # Remove peptide decoys 
    if ("DecoyPeptide" %in% colnames(metamorpheus_input)) {
        metamorpheus_input <- metamorpheus_input[!metamorpheus_input$DecoyPeptide, ]
    }
    # Remove peak decoys
    if ("RandomRT" %in% colnames(metamorpheus_input)) {
        metamorpheus_input <- metamorpheus_input[!metamorpheus_input$RandomRT, ]
    }
    if (MBR) {
        metamorpheus_input = metamorpheus_input[
            metamorpheus_input$PeakDetectionType == "MSMS" | 
                metamorpheus_input$`PIPQ-Value` <= qvalue_cutoff , ]
    } else {
        metamorpheus_input = metamorpheus_input[
            metamorpheus_input$PeakDetectionType == "MSMS", ]
    }
    req_cols = c('FileName', 'ProteinGroup', 'FullSequence', 
                 'PrecursorCharge', 'Peakintensity')
    metamorpheus_input = metamorpheus_input[, req_cols, with = FALSE]
    data.table::setnames(
        metamorpheus_input, 
        c("ProteinGroup", "FullSequence", "PrecursorCharge", "FileName", "Peakintensity"),
        c("ProteinName", "PeptideSequence", "PrecursorCharge", "Run", "Intensity"), 
        skip_absent = TRUE)
    .logSuccess("Metamorpheus", "clean")
    metamorpheus_input
}

# Test Metamorpheus .cleanRawMetamorpheus ---------------------------

file_path = system.file("tinytest/processed_data/Metamorpheus/MSstatsMetamorpheusFilesObject.rds", package="MSstatsConvert")
input = readRDS(file_path)
output = MSstatsConvert:::.cleanRawMetamorpheus(input)
expect_equal(ncol(output), 5)
expect_true(nrow(output) > 0)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))

#' Merge annotation with feature data
#' @param data.table preprocessed by one of the .cleanRaw functions.
#' @param annotation data.table with annotation
#'
#' @return data.table
#' @keywords internal 
.mergeAnnotation = function(input, annotation) {
    input$Run = .standardizeColnames(input$Run)
    if (!is.null(annotation)) {
        if (is.element("Channel", colnames(input))) {
            input$Channel = .standardizeColnames(input$Channel)
            if (!all(unique(annotation$Channel) %in% unique(input$Channel))) {
                msg = paste("** Please check the annotation file.",
                            "The channel name must be matched",
                            "with that in input data.")
                getOption("MSstatsLog")("ERROR", msg)
                stop(msg)
            }
        }
        cols = intersect(colnames(input), colnames(annotation))
        cols = setdiff(cols, c("Run", "Channel"))
        annotation_cols = intersect(c("Run", "Channel"), colnames(input))
        input = merge(input[, !(colnames(input) %in% cols), with = FALSE], 
                      annotation, 
                      by = annotation_cols, all.x = TRUE, sort = FALSE)
        msg = "** Run annotation merged with quantification data."
        getOption("MSstatsLog")("INFO", msg)
        getOption("MSstatsMsg")("INFO", msg)
    }
    if (any(is.na(input$Condition))) {
        msg = "** Condition in the input file must match condition in annotation."
        getOption("MSstatsLog")("WARN", msg)
        getOption("MSstatsMsg")("WARN", msg)
    }
    input
}


#' Check if the annotation is valid
#' @param input data processed by the MSstatsClean
#' @param annotation annotation created by the MSstatsMakeAnnotation function
#' @return TRUE invisibly if the annotation is correct, throws an error otherwise
#' @keywords internal
.checkAnnotation = function(input, annotation) {
    if (is.element("Channel", colnames(input))) {
        missing_cols = setdiff(
            c("Run", "TechRepMixture", "Fraction", "Mixture", 
              "Channel", "Condition", "BioReplicate"),
            colnames(annotation)
        )
    } else {
        missing_cols = setdiff(c("Run", "Condition", "BioReplicate"),
                               colnames(annotation))
    }
    if (length(missing_cols) > 0) {
        msg = paste("** Columns", paste(missing_cols, sep = ", ", 
                                        collapse = ", "),
                    "missing in the annotation.", 
                    "Please check the annotation file.")
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
    if (!is.element("Channel", colnames(input))) {
        counts = annotation[, list(n_rows = .N), by = "Run"]
        if (any(counts$n_rows > 1)) {
            msg = paste("** Please check annotation.",
                        "Each MS run (Raw.file) can\'t have multiple",
                        "conditions or BioReplicates.")
            getOption("MSstatsLog")("ERROR", msg)
            stop(msg)
        }
    }
    invisible(TRUE)
}

#' Appender used not to write messages
#' 
#' A convenience function written to save time on checking if messages should
#' be printed or logs should be written to a file.
#' 
#' @param level log level 
#' @param ... messages - ignored
#' @return NULL invisibly
#' @keywords internal
.nullAppender = function(level, ...) {
    invisible(NULL)
}


#' Format one log line: level, timestamp, message
#'
#' Message parts are joined with paste0, never sprintf, so a literal percent
#' sign is written verbatim.
#' @keywords internal
.formatLogMessage = function(level, ...) {
    timestamp = format(Sys.time(), "%Y-%m-%d %H:%M:%S")
    level_field = formatC(level, width = 5L, flag = "-")
    paste0(level_field, " [", timestamp, "] ", paste0(..., collapse = ""), "\n")
}


#' File appender: returns a function(level, ...) that writes one line to a file
#' @keywords internal
.fileAppender = function(log_file_path, append = TRUE) {
    force(log_file_path)
    force(append)
    function(level, ...) {
        cat(.formatLogMessage(level, ...), file = log_file_path, sep = "",
            append = append)
    }
}


#' Console appender: returns a function(level, ...) that writes one line to console
#' @keywords internal
.consoleAppender = function() {
    function(level, ...) {
        cat(.formatLogMessage(level, ...))
    }
}


#' Set default logging object when package is loaded
#' @param ... ignored
#' @return none, sets options called MSstatsLog and MSstatsMsg
#' @keywords internal
.onLoad = function(...) {
    logs = getOption("MSstatsLog")
    msgs = getOption("MSstatsMsg")
    time_now = Sys.time()
    path = paste0("./MSstats_log_", gsub("[ :\\-]", "_", time_now), ".log")
    
    if (is.null(logs)) {
        ms_logs = .fileAppender(path)
        options(MSstatsLog = ms_logs)
    }
    if (is.null(msgs)) {
        ms_messages = .consoleAppender()
        options(MSstatsMsg = ms_messages)
    }
}


#' Set how MSstats will log information from data processing
#' 
#' @param use_log_file logical. If TRUE, information about data processing
#' will be saved to a file.
#' @param append logical. If TRUE, information about data processing will be added
#' to an existing log file.
#' @param verbose logical. If TRUE, information about data processing wil be printed
#' to the console.
#' @param log_file_path character. Path to a file to which information about 
#' data processing will be saved. 
#' If not provided, such a file will be created automatically.
#' If `append = TRUE`, has to be a valid path to a file.
#' @param base start of the file name.
#' @param pkg_name currently "MSstats", "MSstatsPTM" or "MSstatsTMT". 
#' Each package can use its own separate log settings.
#' 
#' @return TRUE invisibly in case of successful logging setup.
#' @export
#' 
#' @examples 
#' # No logging and no messages
#' MSstatsLogsSettings(FALSE, FALSE, FALSE)
#' # Log, but do not display messages
#' MSstatsLogsSettings(TRUE, FALSE, FALSE)
#' # Log to an existing file
#' file.create("new_log.log")
#' MSstatsLogsSettings(TRUE, TRUE, log_file_path = "new_log.log")
#' # Do not log, but display messages
#' MSstatsLogsSettings(FALSE)
#' 
MSstatsLogsSettings = function(use_log_file = TRUE, append = FALSE,
                               verbose = TRUE, log_file_path = NULL,
                               base = "MSstats_log_", pkg_name = "MSstats") {
    checkmate::assertLogical(use_log_file)
    checkmate::assertLogical(append)
    checkmate::assertLogical(verbose)
    checkmate::assertCharacter(log_file_path, null.ok = TRUE)
    checkmate::assertTRUE(!(append & !use_log_file))
    checkmate::assertTRUE(!(append & is.null(log_file_path)))
    if (!is.null(log_file_path)) {
        checkmate::assertPathForOutput(log_file_path, overwrite = TRUE)
    }
    
    if (use_log_file) {
        if (!is.null(log_file_path)) {
            file_logger = .fileAppender(log_file_path, append = append)
        } else {
            time_now = Sys.time()
            log_file_path = paste0(base, gsub("[ :\\-]", "_", time_now), 
                                   ".log")
            file_logger = .fileAppender(log_file_path)
        }
    } else {
        file_logger = .nullAppender
    }

    if (verbose) {
        console_logger = .consoleAppender()
    } else {
        console_logger = .nullAppender
    }
    loggers = list(file_logger, console_logger)
    names(loggers) = paste0(pkg_name, c("Log", "Msg"))
    do.call(options, loggers)
    invisible(TRUE)
}


#' Make a message about filtering based on fixed values
#' @inheritParams .filterExact
#' @return character - message
#' @keywords internal
.makeExactFilterMessage = function(col_name, filter_symbols, 
                                   behavior, fill_value
) {
    if (behavior == "remove") {
        subject = "Rows"
        event = "are removed"
        what = ""
    } else {
        subject = "Intensities"
        event = "are replaced with"
        what = fill_value
    }
    paste("**", subject, "with values of", col_name, "equal to", 
          paste(filter_symbols, sep = ", ", collapse = ", "),
          event, what)
}


#' Make a message about filtering based on a score
#' @inheritParams .filterByScore
#' @return character - message
#' @keywords internal
.makeScoreFilterMessage = function(score_column, score_threshold, direction,
                                   behavior, fill_value) {
    if (behavior == "remove") {
        subject = "Rows"
        event = "are removed"
        what = ""
    } else {
        subject = "Intensities"
        event = "are replaced with"
        what = fill_value
    } 
    msg = paste("**", subject, "with values not", direction, "than", score_threshold,
                "in", score_column, event, what)
    msg
}


#' Log information about converter options
#' @inheritParams MSstatsPreprocess
#' @param is_tmt If TRUE, the dataset comes from a TMT experiment
#' @return TRUE invisibly if message was logged 
#' @keywords internal
.logConverterOptions = function(feature_columns, remove_shared_peptides,
                                remove_single_feature_proteins,
                                feature_cleaning, is_tmt = FALSE) {
    init = "** The following options are used:"
    features = paste("  - Features will be defined by the columns:",
                     paste(feature_columns, sep = ", ", collapse = ", "))
    if (remove_shared_peptides) {
        shared = "  - Shared peptides will be removed."
    } else {
        shared = "  - Shared peptides will not be removed."
    }
    if (remove_single_feature_proteins) {
        single = "  - Proteins with a single feature will be removed."
    } else {
        single = "  - Proteins with single feature will not be removed."
    }
    what = ifelse(is_tmt, "within each run", "across runs")
    if (feature_cleaning[["remove_features_with_few_measurements"]]) {
        few = paste("  - Features with less than 3 measurements", 
                    what, "will be removed.")
    } else {
        few = paste("  - Features with less than 3 measurements", 
                    what, "will be kept.")
    }
    msg = paste(init, features, shared, single, few, sep = "\n")
    getOption("MSstatsLog")("INFO", msg)    
    getOption("MSstatsMsg")("INFO", msg)
    invisible(TRUE)
}


#' Make a message about successful data cleaning/importing
#' @param tool name of a signal processing tool
#' @return TRUE invisibly if logging was sucessful
#' @keywords internal
.logSuccess = function(tool, event) {
    if (event == "clean") {
        what = "cleaned successfully."
    } else {
        what = "imported successfully."
    }
    msg = paste("** Raw data from", tool, what)
    getOption("MSstatsLog")("INFO", msg)
    getOption("MSstatsMsg")("INFO", msg)
    invisible(TRUE)
}

#' Save session information
#' 
#' @param path optional path to output file. If not provided, "MSstats_session_info" 
#' and current timestamp will be used as a file name
#' @param append if TRUE and file given by the `path` parameter already exists,
#' session info will be appended to the file
#' @param base beginning of a file name
#' @return TRUE invisibly after session info was saved
#' @export
#' @importFrom utils sessionInfo
#' @examples 
#' MSstatsSaveSessionInfo("session_info.txt")
#' MSstatsSaveSessionInfo("session_info.txt", base = "MSstatsTMT_session_info_")
#' 
MSstatsSaveSessionInfo = function(path = NULL, append = TRUE, 
                                  base = "MSstats_session_info_") {
    if (is.null(path)) {
        time_now = Sys.time()
        path = paste0(base, 
                      gsub("[ :\\-]", "_", time_now), ".log")
    }
    session_info = utils::sessionInfo()
    sink(path, append = append)
    print(session_info)
    sink()
    invisible(TRUE)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

calculate_anomaly_score <- function(df, n_trees, max_depth) {
    .Call(`_MSstatsConvert_calculate_anomaly_score`, df, n_trees, max_depth)
}

#' Clean raw Philosopher files
#' @param msstats_object object of class MSstatsPhilosopherFiles
#' @param protein_id_col character name of a column that identifies proteins
#' @param peptide_id_col character name of a column that identifies peptides
#' @param channels character vector of channel labels
#' @param remove_shared_peptides logical, if TRUE, shared peptides will be 
#' removed based on the IsUnique column from Philosopher output
#' @keywords internal
#' @return data.table
.cleanRawPhilosopher = function(msstats_object, protein_id_col,
                                peptide_id_col, channels,
                                remove_shared_peptides) {
  ProteinName = ModifiedPeptideSequence = PeptideSequence = NULL
  PrecursorCharge = Intensity = IsUnique = PSM = NULL
  
  channels = .standardizeColnames(channels)
  input = getInputFile(msstats_object, "psm_data")  
  protein_col = .standardizeColnames(protein_id_col)
  peptide_col = .standardizeColnames(peptide_id_col)
  
  input[, ModifiedPeptideSequence := ifelse(is.na(ModifiedPeptideSequence),
                                            PeptideSequence, 
                                            ModifiedPeptideSequence)]
  data.table::setnames(input, stringi::stri_replace_all(colnames(input), 
                                                        fixed = "Channel", replacement = ""))
  cols = c(protein_col, peptide_col, "Charge", "SpectrumName", 
           "PeptideProphetProbability", "Purity", "Modifications",
           "IsUnique", channels)
  cols = intersect(cols, colnames(input))
  input = input[, cols, with = FALSE]
  data.table::setnames(input, c(protein_col, peptide_col, "SpectrumName", "Charge"),
                       c("ProteinName", "PeptideSequence", "Run", "PrecursorCharge"))
  if (remove_shared_peptides) {
    input = input[(IsUnique)]
    input[, IsUnique := NULL]
  }
  input[, PSM := paste(PeptideSequence, PrecursorCharge, 
                       1:nrow(input), sep = "_")]
  input = data.table::melt(input, measure.vars = channels,
                           variable.name = "Channel", value.name = "Intensity",
                           variable.factor = FALSE)
  input[, Intensity := ifelse(Intensity == 0, NA, Intensity)]
  input[, Run := stringi::stri_split(Run, regex = "\\.", simplify = TRUE)[, 1]]
  input = input[!is.na(ProteinName) & ProteinName != ""]
  input
}

#' Generate MSstatsTMT required input format from MaxQuant output
#' 
#' @param evidence name of 'evidence.txt' data, which includes feature-level data.
#' @param proteinGroups name of 'proteinGroups.txt' data.
#' @param annotation data frame which contains column Run, Fraction, TechRepMixture, Mixture, Channel, BioReplicate, Condition. Refer to the example 'annotation.mq' for the meaning of each column.
#' @param which.proteinid Use 'Proteins' (default) column for protein name. 'Leading.proteins' or 'Leading.razor.proteins' or 'Gene.names' can be used instead to get the protein ID with single protein. However, those can potentially have the shared peptides.
#' @param rmProt_Only.identified.by.site TRUE will remove proteins with '+' in 'Only.identified.by.site' column from proteinGroups.txt, which was identified only by a modification site. FALSE is the default.
#' @param rmPSM_withfewMea_withinRun TRUE (default) will remove the features that have 1 or 2 measurements within each Run.
#' @param rmProtein_with1Feature TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.
#' @param ... additional parameters to `data.table::fread`.
#' @inheritParams .sharedParametersAmongConverters
#' 
#' @return data.frame of class "MSstatsTMT"
#' 
#' @export
#' 
#' @examples
#' evidence = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_ev.csv",
#'                                       package = "MSstatsConvert"))
#' proteinGroups = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_pg.csv",
#'                                       package = "MSstatsConvert"))
#' annotation.mq = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_annotation.csv",
#'                                       package = "MSstatsConvert"))
#' input.mq <- MaxQtoMSstatsTMTFormat(evidence, proteinGroups, annotation.mq)
#' head(input.mq)
#' 
MaxQtoMSstatsTMTFormat = function(
        evidence, proteinGroups, annotation, which.proteinid = 'Proteins',
        rmProt_Only.identified.by.site = FALSE, useUniquePeptide = TRUE,
        rmPSM_withfewMea_withinRun = TRUE, rmProtein_with1Feature = FALSE, 
        summaryforMultipleRows = sum, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL,
        ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsTMT_converter_log_")
    
    input = MSstatsConvert::MSstatsImport(list(evidence = evidence,
                                               protein_groups = proteinGroups), 
                                          "MSstatsTMT", "MaxQuant", ...)
    input = MSstatsConvert::MSstatsClean(
        input,
        protein_id_col = which.proteinid, 
        remove_by_site = rmProt_Only.identified.by.site,
        channel_columns = "Reporterintensitycorrected")
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input, 
        annotation, 
        feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = rmProtein_with1Feature,
        feature_cleaning = list(remove_features_with_few_measurements = rmPSM_withfewMea_withinRun,
                                summarize_multiple_psms = summaryforMultipleRows))
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  fix_missing = "zero_to_na")
    data.table::setnames(input, "PrecursorCharge", "Charge", skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the proteinSummarization function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test MaxQtoMSstatsTMTFormat ---------------------------
mq_ev = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_ev.csv",
                                      package = "MSstatsConvert"))
mq_pg = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_pg.csv",
                                      package = "MSstatsConvert"))
annot = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_annotation.csv",
                                      package = "MSstatsConvert"))
output = MaxQtoMSstatsTMTFormat(mq_ev, mq_pg, annot, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_true(nrow(output) > 0)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("Charge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("TechRepMixture" %in% colnames(output))
expect_true("PSM" %in% colnames(output))
expect_true("Mixture" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Channel" %in% colnames(output))

bad_annotation = data.table::fread(system.file("tinytest/raw_data/PDTMT/pd_annotation.csv",
                                              package = "MSstatsConvert"))

expect_error(MaxQtoMSstatsTMTFormat(evidence = mq_ev, proteinGroups = mq_pg,
                                    annotation = bad_annotation)) # wrong annotation file

# Verify output intensities are present in input
intensity_cols = grep("Reporter.intensity.corrected", colnames(mq_ev), value = TRUE)
input_intensities = as.character(unlist(mq_ev[, intensity_cols, with = FALSE]))
output_intensities = as.character(output$Intensity[!is.na(output$Intensity)])
expect_true(all(output_intensities %in% input_intensities))

#' Clean raw Progenesis output
#' @param msstats_object an object of class `MSstatsSpectroMineFiles`.
#' @param runs chr, vector of Run labels.
#' @param fix_colnames lgl, if TRUE, one of the rows will be used as colnames.
#' @return data.table
#' @keywords internal
.cleanRawProgenesis = function(msstats_object, runs, fix_colnames = TRUE) {
    Useinquantitation = Intensity = NULL
    
    prog_input = getInputFile(msstats_object, "input")
    raw_abundance_col_id = which(colnames(prog_input) == "Rawabundance")
    if (is.element("Rawabundance", colnames(prog_input)) &
        is.element("Normalizedabundance", colnames(prog_input))) {
        norm_abundance_col_id = which(colnames(prog_input) == "Normalizedabundance")
        if ((raw_abundance_col_id - norm_abundance_col_id) != length(runs)) {
            msg = paste("** Please check annotation file. The numbers of MS runs", 
                        "in annotation and output are not matched.")
            getOption("MSstatsLog")("ERROR", msg)
            stop(msg)
        }
    }
    raw_abundances_col_ids = seq(raw_abundance_col_id,
                                 raw_abundance_col_id + length(runs) - 1)
    
    if (fix_colnames) {
        if (all(unique(prog_input[[1]][1:2]) == "")) {
            skip = 1:2
        } else {
            skip = 1
        }
        prog_input = prog_input[-skip, ]
        colnames(prog_input) = as.character(unlist(prog_input[1, ]))
        prog_input = prog_input[-1, ]
    }
    colnames(prog_input) = .standardizeColnames(colnames(prog_input))
    protein_col = .findAvailable(c("Protein", "Accession"), 
                                 colnames(prog_input))
    cols = which(colnames(prog_input) %in% c(protein_col, "Modifications", 
                                             "Sequence", "Charge", "Ions",
                                             "Useinquantitation"))
    cols = c(cols, raw_abundances_col_ids)
    prog_input = prog_input[, cols, with = FALSE]
    data.table::setnames(prog_input, "Ions", "Charge", skip_absent = TRUE)
    data.table::setnames(prog_input, 
                         c(protein_col, "Charge"), 
                         c("ProteinName", "PrecursorCharge"),
                         skip_absent = TRUE)
    
    nonmissing_prot = !is.na(prog_input$ProteinName) & prog_input$ProteinName != ""
    nonmissing_pept = !is.na(prog_input$Sequence) & prog_input$Sequence != ""
    prog_input = prog_input[nonmissing_prot & nonmissing_pept, ]
    prog_input$PeptideSequence = paste(prog_input$Sequence,
                                       prog_input$Modifications,
                                       sep = "")
    if (is.element("Useinquantitation", colnames(prog_input))) {
        if (!is.logical(prog_input$Useinquantitation)) {
            prog_input$Useinquantitation = prog_input$Useinquantitation == "True"
        }
        prog_input = prog_input[(Useinquantitation), ]
        prog_input = prog_input[, !(colnames(prog_input) == "Useinquantitation"),
                                with = FALSE]
    }
    feature_cols = intersect(colnames(prog_input), 
                             c("ProteinName", "PeptideSequence",
                               "PrecursorCharge", "Fraction"))
    prog_cols = intersect(colnames(prog_input), 
                          c(feature_cols, as.character(runs)))
    prog_input = prog_input[, prog_cols, with = FALSE]
    prog_input = data.table::melt(prog_input,
                                  id.vars = feature_cols,
                                  measure.vars = runs,
                                  variable.name = "Run",
                                  value.name = "Intensity",
                                  value.factor = FALSE)
    prog_input[, Intensity := as.numeric(as.character(Intensity))]
    .logSuccess("Progenesis", "clean")
    prog_input
}

#' Clean raw Proteome Discoverer data
#' @inheritParams .cleanRawPDTMT
#' @inheritParams .cleanRawPDMSstats
#' @return data.table
.cleanRawPD = function(msstats_object, quantification_column, protein_id_column,
                       sequence_column, remove_shared, 
                       remove_protein_groups = TRUE,
                       intensity_columns_regexp = "Abundance") {
  if (getDataType(msstats_object) == "MSstatsTMT") {
    cleaned_pd = .cleanRawPDTMT(msstats_object, remove_shared,
                                remove_protein_groups, 
                                protein_id_column, intensity_columns_regexp)
  } else {
    cleaned_pd = .cleanRawPDMSstats(msstats_object, quantification_column, 
                                    protein_id_column, sequence_column, 
                                    remove_shared)
  }
  .logSuccess("ProteomeDiscoverer", "clean")
  cleaned_pd
}

#' Clean raw PD output
#' @param msstats_object an object of class `MSstatsSpectroMineFiles`.
#' @param quantification_column chr, name of a column used for quantification.
#' @param protein_id_column chr, name of a column with protein IDs.
#' @param sequence_column chr, name of a column with peptide sequences.
#' @param remove_shared lgl, if TRUE, shared peptides will be removed.
#' @return data.table
#' @keywords internal
.cleanRawPDMSstats = function(msstats_object, quantification_column, 
                              protein_id_column, sequence_column, remove_shared,
                              run_column = "SpectrumFile"
) {
  XProteins = NULL
  
  pd_input = getInputFile(msstats_object, "input")
  protein_id_column = .standardizeColnames(protein_id_column)
  sequence_column = .standardizeColnames(sequence_column)
  quantification_column = .standardizeColnames(quantification_column)
  
  if (remove_shared & is.element("XProteins", colnames(pd_input))) {
    pd_input = pd_input[XProteins == "1", ]
  }
  pd_cols = c(protein_id_column, sequence_column, 
              "Modifications", "Charge", run_column, quantification_column)
  if (any(is.element(colnames(pd_input), "Fraction"))) {
    pd_cols = c(pd_cols, "Fraction")
  }
  pd_input = pd_input[, pd_cols, with = FALSE]
  data.table::setnames(
    pd_input,
    c(protein_id_column, sequence_column, run_column, 
      quantification_column, "Charge"),
    c("ProteinName", "PeptideSequence", "Run", 
      "Intensity", "PrecursorCharge"),
    skip_absent = TRUE)
  pd_input[["PeptideSequence"]] = paste(pd_input[["PeptideSequence"]], 
                                        pd_input[["Modifications"]], 
                                        sep = "_")
  pd_input[, !(colnames(pd_input) == "Modifications"), with = FALSE]
}


#' Clean raw TMT data from Proteome Discoverer
#' @inheritParams .cleanRawPDMSstats
#' @param remove_protein_groups if TRUE, proteins with numProteins > 1 will be removed.
#' @param intensity_columns_regexp regular expressions that defines intensity columns.
#' Defaults to "Abundance", which means that columns that contain the word "Abundance"
#' will be treated as corresponding to intensities for different channels.
#' @importFrom data.table melt
#' @return `data.table`
#' @keywords internal
.cleanRawPDTMT = function(msstats_object, remove_shared = TRUE, 
                          remove_protein_groups = TRUE,
                          protein_id_column = "ProteinAccessions",
                          intensity_columns_regexp = "Abundance",
                          run_column = "SpectrumFile") {
  ProteinName = numProtein = QuanInfo = NULL
  
  pd_input = getInputFile(msstats_object, "input")
  protein_id_column = .standardizeColnames(protein_id_column)
  if (!is.element(protein_id_column, colnames(pd_input))) {
    protein_id_column = .findAvailable(c("ProteinAccessions", 
                                         "MasterProteinAccessions"),
                                       colnames(pd_input), 
                                       "ProteinAccessions")
  }
  if (protein_id_column == "ProteinAccessions") {
    num_proteins = .findAvailable(c("XProteins", 
                                    "NumberofProteins"),
                                  colnames(pd_input), 
                                  "XProteins")
  } else {
    num_proteins = .findAvailable(c("XProteinGroups", 
                                    "NumberofProteinGroups"),
                                  colnames(pd_input), 
                                  "XProteinGroups")
  }
  
  channels = .getChannelColumns(colnames(pd_input), intensity_columns_regexp)
  .validatePDTMTInputColumns(pd_input, protein_id_column, num_proteins, run_column, channels)
  
  pd_cols = intersect(c(protein_id_column, num_proteins, "AnnotatedSequence", 
                        "Charge", "PrecursorCharge", "IonsScore", 
                        run_column, "QuanInfo", 
                        "IsolationInterference", channels),
                      colnames(pd_input))
  pd_input = pd_input[, pd_cols, with = FALSE]
  data.table::setnames(pd_input,
                       c(protein_id_column, num_proteins, "AnnotatedSequence", 
                         run_column, "Charge"),
                       c("ProteinName", "numProtein", "PeptideSequence", 
                         "Run", "PrecursorCharge"),
                       skip_absent = TRUE)
  pd_input = unique(pd_input)
  pd_input$PSM = paste(pd_input$PeptideSequence, pd_input$PrecursorCharge,
                       1:nrow(pd_input), sep = "_")
  pd_input = melt(pd_input, measure.vars = channels, 
                  id.vars = setdiff(colnames(pd_input), channels),
                  variable.name = "Channel", value.name = "Intensity")
  pd_input$Channel = .standardizeColnames(pd_input$Channel)
  pd_input$Channel = gsub(intensity_columns_regexp, "", pd_input$Channel)
  pd_input$Channel = gsub(":", "", pd_input$Channel)
  pd_input$Intensity = ifelse(pd_input$Intensity == 0, NA, pd_input$Intensity)
  pd_input = pd_input[(ProteinName != "") & (!is.na(ProteinName)), ]
  if (remove_protein_groups) {
    pd_input = pd_input[numProtein == 1]
  }
  if (remove_shared) {
    if ("UNIQUE" %in% toupper(pd_input[["QuanInfo"]])) {
      pd_input = pd_input[toupper(QuanInfo) == 'UNIQUE', ]
    }
  }
  pd_input = pd_input[, !(colnames(pd_input) %in% "QuanInfo"), with = FALSE]
  pd_input
}


#' Helper method to validate input has necessary columns
#' @param pd_input data.frame input
#' @param protein_id_column column name for protein passed from user
#' @param num_proteins_column column name for number of protein groups passed from user
#' @param run_column column name for Run ID, depends on PD version
#' @param channels list of column names for channels
#' @noRd
.validatePDTMTInputColumns = function(pd_input, 
                                protein_id_column, 
                                num_proteins_column, 
                                run_column,
                                channels
) {
    required_columns = c(protein_id_column, num_proteins_column, "AnnotatedSequence", 
      run_column)
    missing_columns = setdiff(required_columns, colnames(pd_input))
    if (length(missing_columns) > 0) {
        msg = paste("The following columns are missing from the input data:", 
                    paste(missing_columns, sep = ", ", collapse = ", "))
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
    if (length(channels) == 0L) {
        msg = paste("There is no channel intensity column in the input data,",
                    "which should start with the string provided in the",
                    "intensity_columns_regexp parameter (default: 'Abundance')")
        getOption("MSstatsLog")("ERROR", msg)
        stop(msg)
    }
}

.testHappyPath = function(input) {
    output = MSstatsConvert:::.cleanRawPDTMT(input)
    expect_equal(ncol(output), 10)
    expect_true(nrow(output) > 0)
    expected_column_names = c(
        "ProteinName",
        "numProtein",
        "PeptideSequence",
        "PrecursorCharge",
        "IonsScore",
        "Run",
        "IsolationInterference",
        "PSM",
        "Channel",
        "Intensity"
    )
    missing_columns = setdiff(expected_column_names, colnames(output))
    expect_equal(length(missing_columns), 0)
}

# Test PD .cleanRawPDTMT ---------------------------

file_path = system.file("tinytest/processed_data/PD/MSstatsProteomeDiscovererFilesObjectTMT.rds", package="MSstatsConvert")
input = readRDS(file_path)
.testHappyPath(input)

# Test PD .cleanRawPDTMT Column Validation Error ---------------------------
file_path = system.file("tinytest/processed_data/PD/MSstatsProteomeDiscovererFilesObjectTMTBadInput.rds", 
                        package="MSstatsConvert")
input_with_missing_column = readRDS(file_path)
expect_error(MSstatsConvert:::.cleanRawPDTMT(input_with_missing_column),
             pattern = "The following columns are missing from the input data")

# Test PD .cleanRawPDTMT for PD 3.1+, where "# Proteins" & "# Protein Groups" 
# columns are renamed to "Number of Proteins" & "Number of Protein Groups"
file_path = system.file("tinytest/processed_data/PD/MSstatsProteomeDiscovererFilesObjectTMT3-1Format.rds", package="MSstatsConvert")
input = readRDS(file_path)
.testHappyPath(input)

#' Fill missing rows to create balanced design
#' @param input output of `MSstatsPreprocess`
#' @param fill_missing if TRUE, missing Intensities values will be added to data 
#' @param anomaly_metrics character vector of quality metric column names to be 
#' used as features in an anomaly detection model.
#' and marked as NA
#' @return data.table
#' @keywords internal
.makeBalancedDesign = function(input, fill_missing, anomaly_metrics = c()) {
    feature = NULL
    
    is_tmt = is.element("Channel", colnames(input))
    
    if (fill_missing) {
        cols = intersect(colnames(input), 
                         c("ProteinName", "feature", "PeptideSequence", "PSM", 
                           "PrecursorCharge", "FragmentIon", "ProductCharge"))
        annotation_cols = intersect(colnames(input), 
                                    c("Run", "Condition", "BioReplicate", 
                                      "Channel", "Mixture", "TechRepMixture", 
                                      "TechReplicate"))
        intensity_ids = intersect(c("feature", "Run", 
                                    "Channel", "IsotopeLabelType",
                                    "Fraction"), colnames(input))
        if (is_tmt) {
            group_col = "Run"
            measurement_col = "Channel"
            cols = intersect(c(cols, "Fraction"),
                             colnames(input))
        } else {
            group_col = "Fraction"
            measurement_col = "Run"
        }
        all_possibilities = .getFullDesign(input, group_col, "feature",
                                           measurement_col, is_tmt)
        all_possibilities = merge(
            all_possibilities, 
            unique(input[, c(cols, group_col), with = FALSE]), 
            all.x = TRUE, by = c("feature", group_col), allow.cartesian = TRUE)
        all_possibilities = merge(
            all_possibilities, 
            unique(input[, annotation_cols, with = FALSE]),
            all.x = TRUE, by = unique(c("Run", measurement_col)))
        intensities = intersect(c(intensity_ids, "Intensity",
                                  "isZero", anomaly_metrics), 
                                colnames(input))
        input = merge(all_possibilities, 
                      unique(input[, intensities, with = FALSE]),
                      all.x = TRUE, by = intensity_ids)
    } else {
        if (!is_tmt) {
            any_missing = as.character(unique(.getMissingRunsPerFeature(input)[, feature]))
            msg = paste("The following features have missing values in at least one run.",
                        paste(any_missing, sep = ",\n ", collapse = ",\n "))
            getOption("MSstatsLog")("WARN", msg)
            getOption("MSstatsMsg")("WARN", msg)
        }
    }
    input
}


#' Create a data.frame of each combination of values for given variables
#' @param input output of `MSstatsPreprocess`
#' @param group_col name of column in `input`. Combination of values of 
#' `feature_col` and `measurement_col` will be created within each unique value
#' of this column
#' @param `feature_col` name of the column that labels features
#' @param `measurement_col` name of a column with measurement labels - Runs in
#' label-free case, Channels in TMT case.
#' @param is_tmt if TRUE, data will be treated as coming from TMT experiment.
#' @importFrom data.table rbindlist
#' @return data.table
#' @keywords internal
.getFullDesign = function(input, group_col, feature_col, 
                          measurement_col, is_tmt
) {
    if (is_tmt) {
        labels = "L"
        groups = unique(input[[group_col]])
        by_group = vector("list", length(groups))
        measurements = unique(input[[measurement_col]])
        for (group_id in seq_along(groups)) {
            group = groups[group_id]
            group_filter = input[[group_col]] == group
            by_group[[group_id]] = data.table::as.data.table(
                expand.grid(labels = labels,
                            features = unique(input[[feature_col]][group_filter]),
                            measurements = measurements))
            by_group[[group_id]]$group = group
        }
        result = data.table::rbindlist(by_group)
        colnames(result) = c("IsotopeLabelType", feature_col, 
                             measurement_col, group_col)
        result[, 2:4, with = FALSE]
    } else {
        labels = na.omit(unique(input[["IsotopeLabelType"]]))
        groups = unique(input[[group_col]])
        by_group = vector("list", length(groups))
        measurements = unique(input[[measurement_col]])
        for (group_id in seq_along(groups)) {
            group = groups[group_id]
            group_filter = input[[group_col]] == group
            non_na_filter = group_filter & !is.na(input[["IsotopeLabelType"]])
            na_filter = group_filter & is.na(input[["IsotopeLabelType"]])
            non_na_features = unique(input[[feature_col]][non_na_filter])
            na_label_features = unique(input[[feature_col]][na_filter])
            parts = list()
            if (length(non_na_features) > 0) {
                parts[[length(parts) + 1]] = data.table::as.data.table(
                    expand.grid(
                        labels = labels,
                        features = non_na_features,
                        measurements = unique(input[[measurement_col]][group_filter])
                    ))
            }
            if (length(na_label_features) > 0) {
                parts[[length(parts) + 1]] = data.table::as.data.table(
                    expand.grid(
                        labels = NA,
                        features = na_label_features,
                        measurements = unique(input[[measurement_col]][group_filter])
                    ))
            }
            by_group[[group_id]] = data.table::rbindlist(parts)
            by_group[[group_id]]$group = group
        }
        result = data.table::rbindlist(by_group)
        colnames(result) = c("IsotopeLabelType", feature_col, 
                             measurement_col, group_col)
        result
    }
}


#' Get names of missing runs
#' @param input output of `MSstatsPreprocess`
#' @importFrom data.table uniqueN
#' @return data.table
#' @keywords internal
.getMissingRunsPerFeature = function(input) {
    n_measurements = Run = NULL
    
    grouping_cols = intersect(c("Fraction", "IsotopeLabelType", "feature"),
                              colnames(input))
    
    n_runs = data.table::uniqueN(input$Run)
    any_missing = input[, list(n_measurements = data.table::uniqueN(Run)),
                        by = grouping_cols]
    any_missing = any_missing[n_measurements < n_runs]
    any_missing
}

#' Check if there are duplicated measurements within run
#' @param input output of `MSstatsPreprocess`
#' @return character vector of feature labels
#' @keywords internal
.checkDuplicatedMeasurements = function(input) {
    n_measurements = feature = NULL
    counts = input[, list(n_measurements = .N),
                   by = c("Fraction", "ProteinName", "IsotopeLabelType", 
                          "feature", "Run")]
    counts = unique(counts[n_measurements > 1, as.character(feature)])
    if (length(counts) > 0) {
        msg = paste("** The following features have duplicated measurements",
                    "in some runs: please remove the duplicates.", "\n",
                    paste(counts, sep = ",\n ", collapse = ",\n "))
        # TODO: report separately for L and H
        getOption("MSstatsLog")("WARN", msg)
        stop(msg)
    }
}


#' Change labels for missing values
#' @param input output of `MSstatsPreprocess`
#' @param fix_missing missing values can be labeled by `NA`, `0` or both.
#' If `NULL`, data were processed by Skyline, so missing values will be denoted
#' by both `NA` and `0`. If "na_to_zero", `NA` values will be replaced by `0`.
#' If "zero_to_na", `0` values will be replaced by `NA`
#' @return data.table
#' @keywords internal
.fixMissingValues = function(input, fix_missing = NULL) {
    Intensity = isZero = NULL
    
    if (is.element("isZero", colnames(input))) {
        input[, Intensity := ifelse(Intensity == 0 & !is.na(Intensity), 
                                    ifelse(isZero, 0, NA), Intensity)]
        input[, Intensity := ifelse(!is.na(Intensity) & Intensity > 0 & 
                                        Intensity < 1,
                                    0, Intensity)]
    } else {
        if (!is.null(fix_missing)) {
            if (fix_missing == "zero_to_na") {
                input[, Intensity := ifelse(Intensity == 0 & !is.na(Intensity), 
                                            NA, Intensity)]
            } else {
                input[, Intensity := ifelse(is.na(Intensity), 0, Intensity)]
            }
        }
    }
    input
}

# Balanced design
filter_NA = function(dt) {
    dt[!is.na(Intensity)]
}
## Label-free
### Example data
n_runs = 5
n_labels = 2
n_features = 5
test_data_1 = data.table::data.table(
    ProteinName = "A",
    feature = rep(letters[1:n_features], each = n_runs * n_labels),
    Run = rep(rep(1:n_runs, times = n_features), each = n_labels),
    IsotopeLabelType = rep(c(c("L", "H")[1:n_labels]), times = n_features * n_runs),
    Fraction = 1,
    Intensity = runif(n_runs * n_labels * n_features)
)
### Some rows are missing
test1_na = data.table::copy(test_data_1)[order(ProteinName, feature, 
                                               IsotopeLabelType, Run, Fraction)]
test1_na[, Intensity := ifelse(feature == "a" & Run %in% c(1, 3, 4) & 
                                   IsotopeLabelType == "L", 
                               NA, Intensity)]
test1_nona = filter_NA(test1_na)
expect_equal(
    MSstatsConvert:::.makeBalancedDesign(test1_nona, TRUE)[order(ProteinName, feature, 
                                                                 IsotopeLabelType, Run, Fraction), Intensity],
    test1_na$Intensity
)
### List of missing values is returned when fill_missing = FALSE
expect_stdout(MSstatsConvert:::.makeBalancedDesign(test1_nona, FALSE))
expect_equal(MSstatsConvert:::.getMissingRunsPerFeature(test1_nona)$feature,
             "a")
### All rows in one label are missing
test2_na = data.table::copy(test_data_1)[order(ProteinName, feature, 
                                               IsotopeLabelType, Run, Fraction)]
test2_na[, Intensity := ifelse(feature == "a" & IsotopeLabelType == "L", 
                               NA, Intensity)]
test2_nona = filter_NA(test2_na)
expect_equal(
    MSstatsConvert:::.makeBalancedDesign(test2_nona, TRUE)[order(ProteinName, feature, 
                                                                 IsotopeLabelType, Run, Fraction), Intensity],
    test2_na$Intensity
)
## Labeled
n_labels = 4
test_data_tmt = data.table::data.table(
    ProteinName = "A",
    feature = rep(letters[1:n_features], each = n_runs * n_labels),
    Run = rep(rep(1:n_runs, times = n_features), each = n_labels),
    Channel = rep(c(letters[1:n_labels]), times = n_features * n_runs),
    Intensity = runif(n_runs * n_labels * n_features)
)
test_data_tmt_na = data.table::copy(test_data_tmt)[order(ProteinName, feature, 
                                                         Run, Channel)]
set.seed(100)
test_data_tmt_na$Intensity[sample(1:nrow(test_data_tmt), 15)] = NA
test_data_tmt_nona = filter_NA(test_data_tmt_na)
expect_equal(
    MSstatsConvert:::.makeBalancedDesign(test_data_tmt_nona, TRUE)[order(ProteinName, feature, 
                                                                         Run, Channel), Intensity],
    test_data_tmt_na$Intensity
)
# Duplicated measurements
## No error when there are no duplicates
no_duplicates = data.table::data.table(
    ProteinName = "A",
    IsotopeLabelType = "L",
    Fraction = 1,
    feature = "a",
    Run = 1:6,
    Intensity = 1:6
)
expect_silent(MSstatsConvert:::.checkDuplicatedMeasurements(no_duplicates))
expect_error(MSstatsConvert:::.checkDuplicatedMeasurements(
    rbind(no_duplicates,
          no_duplicates[6, ])
))
# Missing values
no_duplicates2 = data.table::copy(no_duplicates)
no_duplicates2$Intensity[c(1, 6)] = 0
no_duplicates3 = data.table::copy(no_duplicates)
no_duplicates3$Intensity[c(1, 6)] = NA
no_duplicates4 = data.table::copy(no_duplicates)
no_duplicates4$Intensity[c(1, 6)] = 0
no_duplicates4$isZero = c(TRUE, rep(FALSE, 5))
## Zeros are converted to NA
expect_equal(MSstatsConvert:::.fixMissingValues(data.table::copy(no_duplicates2), "zero_to_na"),
             no_duplicates3)
## NAs are converted to 0
expect_equal(MSstatsConvert:::.fixMissingValues(data.table::copy(no_duplicates3), "na_to_zero"),
             no_duplicates2)
## For Skyline, 0 that are a result of sum(NA, na.rm = T) are replaced by NA
expect_equal(MSstatsConvert:::.fixMissingValues(data.table::copy(no_duplicates4))$Intensity,
             c(0, 2:5, NA))
## Do nothing if no isZero column and fix_missing = NULL
expect_equal(MSstatsConvert:::.fixMissingValues(data.table::copy(no_duplicates3), NULL)$Intensity,
             no_duplicates3$Intensity)

# .getFullDesign tests

# Test 1: H/L features expand over all runs in the group even if a label was only
# observed in a subset of runs. NA-labeled features also expand to all group runs.
# "b" has H only in run 1 and L only in run 2; "a" (NA) only in run 3.
# All three should be filled out to all runs {1, 2, 3} in the group.
gfd_mixed = data.table::data.table(
    feature          = c("b", "b", "a"),
    Run              = c(  1,   2,   3),
    IsotopeLabelType = c("H", "L",  NA),
    Fraction         = 1L
)
gfd_result = MSstatsConvert:::.getFullDesign(
    gfd_mixed, group_col = "Fraction", feature_col = "feature",
    measurement_col = "Run", is_tmt = FALSE)
a_rows = gfd_result[gfd_result$feature == "a", ]
expect_true(all(is.na(a_rows$IsotopeLabelType)))
expect_equal(nrow(a_rows), 3L)
b_rows = gfd_result[gfd_result$feature == "b", ]
expect_equal(nrow(b_rows), 6L)
expect_true(all(sort(unique(b_rows$IsotopeLabelType)) == c("H", "L")))

# Test 2: All features have NA IsotopeLabelType — full design contains only NA labels,
# not the empty H/L set that na.omit() would strip from the labels vector.
gfd_all_na = data.table::data.table(
    feature          = c("x", "x", "y", "y"),
    Run              = c(  1,   2,   1,   2),
    IsotopeLabelType = NA_character_,
    Fraction         = 1L
)
gfd_na_result = MSstatsConvert:::.getFullDesign(
    gfd_all_na, group_col = "Fraction", feature_col = "feature",
    measurement_col = "Run", is_tmt = FALSE)
expect_true(all(is.na(gfd_na_result$IsotopeLabelType)))
expect_equal(nrow(gfd_na_result), 4L)

# Test 3: NA-labeled features expand over ALL runs in the group, not just the runs
# where they were observed. Feature "p" appears in runs 1 and 2 with H/L;
# feature "q" appears only in run 3 with NA, but should be filled out to runs 1-3.
gfd_separate_runs = data.table::data.table(
    feature          = c("p", "p", "p", "p", "q"),
    Run              = c(  1,   1,   2,   2,   3),
    IsotopeLabelType = c("L", "H", "L", "H",  NA),
    Fraction         = 1L
)
gfd_sep_result = MSstatsConvert:::.getFullDesign(
    gfd_separate_runs, group_col = "Fraction", feature_col = "feature",
    measurement_col = "Run", is_tmt = FALSE)
p_rows = gfd_sep_result[gfd_sep_result$feature == "p", ]
expect_equal(nrow(p_rows), 6)
q_rows = gfd_sep_result[gfd_sep_result$feature == "q", ]
expect_equal(nrow(q_rows), 3L)
expect_true(all(is.na(q_rows$IsotopeLabelType)))
expect_equal(sort(q_rows$Run), c(1L, 2L, 3L))

#' Import Diann files
#' 
#' @inheritParams .sharedParametersAmongConverters
#' @param input name of MSstats input report from Diann, which includes fragment-level data.  
#' Output fragment data with --export-quant flag in DIA-NN 2.0
#' @param annotation name of 'annotation.txt' data which includes Condition, BioReplicate, Run. 
#' @param MBR True if analysis was done with match between runs
#' @param global_qvalue_cutoff The qvalue cutoff for the Q.Value column, i.e. 
#' the run-specific precursor q-value. Default is 0.01.
#' @param qvalue_cutoff If MBR is false, the qvalue cutoff for the Global.Q.Value 
#' column, i.e. global precursor q-value.  If MBR is true, the qvalue cutoff for the 
#' Lib.Q.Value column, i.e. the q-value for the library created after the first MBR pass.
#' Default is 0.01.
#' @param pg_qvalue_cutoff If MBR is false, the qvalue cutoff for the Global.PG.Q.Value 
#' column, i.e. the global q-value for the protein group.  If MBR is true, the
#' qvalue cutoff for the Lib.PG.Q.Value column, i.e. the protein group q-value for 
#' the library created after the first MBR pass. Default is 0.01.
#' @param useUniquePeptide should unique peptides be removed
#' @param removeFewMeasurements should proteins with few measurements be removed
#' @param removeOxidationMpeptides should peptides with oxidation be removed
#' @param removeProtein_with1Feature should proteins with a single feature be removed
#' @param labeledAminoAcids Character vector of single-letter amino acid codes
#' that carry the SILAC label in protein turnover experiments, e.g.
#' \code{c("K")} or \code{c("K", "R")}.  Supplying this vector opts in to
#' protein-turnover mode; the exact amino acids determine behaviour only in the
#' \code{ModifiedSequence}-parsing path described below.
#'
#' \strong{Channel-based path} (DIA-NN 2.x exports that include a
#' \code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
#' \emph{and} the input contains a \code{Channel} column, \code{Channel} values
#' are mapped directly to \code{IsotopeLabelType} (\code{"H"} → \code{"H"},
#' \code{"L"} → \code{"L"}, anything else → \code{NA}).  The amino acid codes
#' in \code{labeledAminoAcids} are \strong{not} used to validate or filter
#' \code{ModifiedSequence} in this path.
#'
#' \strong{ModifiedSequence-parsing path} (DIA-NN 1.x exports without a
#' \code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
#' and no \code{Channel} column is present, each \code{ModifiedSequence} is
#' inspected for SILAC suffixes of the form \code{(SILAC-<AA>-H)} or
#' \code{(SILAC-<AA>-L)}, where \code{<AA>} is one of the supplied amino acid
#' codes.  Matching sequences are classified as \code{"H"} or \code{"L"};
#' sequences carrying neither suffix receive \code{IsotopeLabelType = NA}.
#' The SILAC suffix is then stripped from \code{PeptideSequence}.
#'
#' When \code{NULL} (default), protein-turnover mode is disabled and all
#' peptides receive \code{IsotopeLabelType = "Light"}.
#' @param quantificationColumn Use 'FragmentQuantCorrected'(default) column for quantified intensities for DIANN 1.8.x.
#' Use 'FragmentQuantRaw' for quantified intensities for DIANN 1.9.x.
#' Use 'auto' for quantified intensities for DIANN 2.x where each fragment intensity is a separate column, e.g. Fr0Quantity.
#' @param calculateAnomalyScores Default is FALSE. If TRUE, will run anomaly detection model and calculate anomaly scores for each feature. Used downstream to weigh measurements in differential analysis.
#' @param anomalyModelFeatures character vector of quality metric column names to be used as features in the anomaly detection model. List must not be empty if calculateAnomalyScores=TRUE.
#' @param anomalyModelFeatureTemporal character vector of temporal direction corresponding to columns passed to anomalyModelFeatures. Values must be one of: `mean_decrease`, `mean_increase`, `dispersion_increase`, or NULL (to perform no temporal feature engineering). Default is empty vector. If calculateAnomalyScores=TRUE, vector must have as many values as anomalyModelFeatures (even if all NULL).
#' @param removeMissingFeatures Remove features with missing values in more than this fraction of runs. Default is 0.5. Only used if calculateAnomalyScores=TRUE.
#' @param anomalyModelFeatureCount Feature selection for anomaly model. Anomaly detection works on the precursor-level and can be much slower if all features used. We will by default filter to the top-100 highest intensity features. This can be adjusted as necessary. To turn feature-selection off, set this value to a high number (e.g. 10000). Only used if calculateAnomalyScores=TRUE.
#' @param runOrder Temporal order of MS runs. Should be a two column data.table with columns `Run` and `Order`, where `Run` matches the run name output by DIA-NN and `Order` is an integer. Used to engineer the temporal features defined in anomalyModelFeatureTemporal.
#' @param n_trees Number of trees to use in isolation forest when calculateAnomalyScores=TRUE. Default is 100.
#' @param max_depth Max tree depth to use in isolation forest when calculateAnomalyScores=TRUE. Default is "auto" which calculates depth as log2(N) where N is the number of runs. Otherwise must be an integer.
#' @param numberOfCores Number of cores for parallel processing anomaly detection model. When > 1, a logfile named 'MSstats_anomaly_model_progress.log' is created to track progress. Only works for Linux & Mac OS. Default is 1.
#' @param ... additional parameters to `data.table::fread`.
#'  
#' @return data.frame in the MSstats required format.
#' 
#' @author Elijah Willie
#' 
#' @export
#' 
#' @examples 
#' input_file_path = system.file("tinytest/raw_data/DIANN/diann_input.tsv", 
#'                                 package="MSstatsConvert")
#' annotation_file_path = system.file("tinytest/raw_data/DIANN/annotation.csv", 
#'                                 package = "MSstatsConvert")
#' input = data.table::fread(input_file_path)
#' annot = data.table::fread(annotation_file_path)
#' output = DIANNtoMSstatsFormat(input, annotation = annot, MBR = FALSE, 
#'                                 use_log_file = FALSE)
#' head(output)
#' 
#' # For DIANN 2.0, set quantificationColumn = 'auto'
#' input_file_path_2_0 = system.file("tinytest/raw_data/DIANN/diann_2.0.parquet", 
#'                                 package="MSstatsConvert")
#' annotation_file_path_2_0 = system.file("tinytest/raw_data/DIANN/annotation_diann_2.0.csv", 
#'                                 package = "MSstatsConvert")
#' input_2_0 = arrow::read_parquet(input_file_path_2_0)
#' annot_2_0 = data.table::fread(annotation_file_path_2_0)
#' output_2_0 = DIANNtoMSstatsFormat(input_2_0, annotation = annot_2_0, MBR = FALSE, 
#'                                 use_log_file = FALSE, quantificationColumn = 'auto')
#' head(output_2_0)
DIANNtoMSstatsFormat = function(
    input, annotation = NULL,
    global_qvalue_cutoff = 0.01,
    qvalue_cutoff = 0.01,
    pg_qvalue_cutoff = 0.01,
    useUniquePeptide = TRUE,
    removeFewMeasurements = TRUE,
    removeOxidationMpeptides = TRUE,
    removeProtein_with1Feature = TRUE,
    MBR = TRUE,
    labeledAminoAcids = NULL,
    quantificationColumn = "FragmentQuantCorrected",
    calculateAnomalyScores=FALSE, anomalyModelFeatures=c(),
    anomalyModelFeatureTemporal=c(), removeMissingFeatures=.5,
    anomalyModelFeatureCount=100,
    runOrder=NULL, n_trees=100, max_depth="auto", numberOfCores=1,
    use_log_file = TRUE, append = FALSE,
    verbose = TRUE, log_file_path = NULL,
    ...) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path)
    
    anomalyModelFeatures = .standardizeColnames(anomalyModelFeatures)

    input = MSstatsConvert::MSstatsImport(list(input = input),
                                          "MSstats", "DIANN")
   
    input = MSstatsConvert::MSstatsClean(input, MBR = MBR,
                                         quantificationColumn = quantificationColumn,
                                         global_qvalue_cutoff = global_qvalue_cutoff,
                                         qvalue_cutoff = qvalue_cutoff,
                                         pg_qvalue_cutoff = pg_qvalue_cutoff,
                                         calculateAnomalyScores = calculateAnomalyScores,
                                         anomalyModelFeatures = anomalyModelFeatures,
                                         labeledAminoAcids = labeledAminoAcids)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)

    decoy_filter = list(col_name = "ProteinName",
                        pattern = c("DECOY", "Decoys"),
                        filter = T,
                        drop_column = FALSE)
    oxidation_filter = list(col_name = "PeptideSequence",
                            pattern = "\\(UniMod\\:35\\)",
                            filter = removeOxidationMpeptides,
                            drop_column = FALSE)

    feature_columns = c("PeptideSequence", "PrecursorCharge",
                        "FragmentIon", "ProductCharge")
    preprocess_feature_columns = if ("IsotopeLabelType" %in% colnames(input))
        c(feature_columns, "IsotopeLabelType") else feature_columns
    fill_isotope_label_type = if ("IsotopeLabelType" %in% colnames(input))
        list() else list("IsotopeLabelType" = "Light")

    input = MSstatsConvert::MSstatsPreprocess(
        input,
        annotation,
        preprocess_feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = removeProtein_with1Feature,
        exact_filtering = NULL,
        pattern_filtering = list(decoy = decoy_filter,
                                 oxidation = oxidation_filter),
        aggregate_isotopic = FALSE,
        feature_cleaning = list(
            remove_features_with_few_measurements = removeFewMeasurements,
            summarize_multiple_psms = max),
        columns_to_fill = c(list(Fraction = 1), fill_isotope_label_type),
        anomaly_metrics = anomalyModelFeatures)
    input[, Intensity := ifelse(Intensity == 0, NA, Intensity)]
    # browser()
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns, 
                                                  fill_incomplete = TRUE,
                                                  handle_fractions = FALSE,
                                                  remove_few = removeFewMeasurements,
                                                  anomaly_metrics = anomalyModelFeatures
    )
    # browser()
    if (calculateAnomalyScores){
        input = MSstatsConvert::MSstatsAnomalyScores(
            input, anomalyModelFeatures, anomalyModelFeatureTemporal,
            removeMissingFeatures, anomalyModelFeatureCount, runOrder, n_trees, 
            max_depth, numberOfCores)
    }
    # browser()
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the dataProcess function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test DIANNtoMSstatsFormat SILAC protein turnover ---------------------------
# Dataset source: MSV000097050
input_file_path_silac = system.file("tinytest/raw_data/DIANN/diann_input_silac.csv",
                                    package = "MSstatsConvert")
annotation_file_path_silac = system.file("tinytest/raw_data/DIANN/annotation_silac.csv",
                                         package = "MSstatsConvert")
input_silac = data.table::fread(input_file_path_silac)
annot_silac = data.table::fread(annotation_file_path_silac)
output_silac = DIANNtoMSstatsFormat(input_silac, annotation = annot_silac,
                                    labeledAminoAcids = c("K"),
                                    removeFewMeasurements = FALSE,
                                    use_log_file = FALSE)
output_silac_dt = data.table::as.data.table(output_silac)

# Basic structure
expect_equal(ncol(output_silac), 11)
expect_equal(nrow(output_silac), 1020)
expect_true("Run" %in% colnames(output_silac))
expect_true("ProteinName" %in% colnames(output_silac))
expect_true("PeptideSequence" %in% colnames(output_silac))
expect_true("PrecursorCharge" %in% colnames(output_silac))
expect_true("Intensity" %in% colnames(output_silac))
expect_true("FragmentIon" %in% colnames(output_silac))
expect_true("ProductCharge" %in% colnames(output_silac))
expect_true("IsotopeLabelType" %in% colnames(output_silac))
expect_true("Condition" %in% colnames(output_silac))
expect_true("BioReplicate" %in% colnames(output_silac))
expect_true("Fraction" %in% colnames(output_silac))

# IsotopeLabelType is classified as H/L/NA — not the "Light" default from
# the labeledAminoAcids=NULL path
expect_false("Light" %in% output_silac$IsotopeLabelType)
label_counts = table(output_silac$IsotopeLabelType, useNA = "ifany")
expect_equal(unname(label_counts["H"]), 350L)
expect_equal(unname(label_counts["L"]), 350L)
expect_equal(sum(is.na(output_silac$IsotopeLabelType)), 320L)

# SILAC modification tags are stripped from PeptideSequence
expect_false(any(grepl("SILAC", output_silac$PeptideSequence)))

# Peptides without a labeled K get NA IsotopeLabelType
unlabeled_rows = output_silac_dt[PeptideSequence %in% c("AAAAADLANR", "AAAADGEPLHNEEER")]
expect_true(all(is.na(unlabeled_rows$IsotopeLabelType)))

# Peptides with a labeled K get only H or L
k_pep_rows = output_silac_dt[PeptideSequence %in% c("AAAAAAAAQMHTK", "AAAAAAAK",
                                                     "AAAAAAK", "AAAAAGGK")]
expect_true(all(k_pep_rows$IsotopeLabelType %in% c("H", "L")))

# Annotation merges correctly: run dAL_AT_Long_B1 maps to Condition 10
cond_b1 = unique(output_silac_dt[Run == "dAL_AT_Long_B1", Condition])
expect_equal(as.character(cond_b1), "10")

# Heavy fragment intensities trace back to the heavy input rows.
# Input: AAAAAAAAQMHTK(SILAC-K-H), Run dAL_AT_Long_B1 has
#   Fragment.Quant.Corrected = "0;18378.38477;17024.44141;15426.27441;38014.78906;"
#   Fragment.Info ions (after NH3 removal):
#     y8^1/863.4499512 -> 18378.38477, y9^1/934.4870605 -> 17024.44141,
#     y6^1/721.3757324 -> 15426.27441
heavy_b1 = output_silac_dt[PeptideSequence == "AAAAAAAAQMHTK" &
                            IsotopeLabelType == "H" &
                            Run == "dAL_AT_Long_B1"]
heavy_y8 = heavy_b1[FragmentIon == "y8^1/863.4499512", Intensity]
expect_equal(heavy_y8, 18378.38477, tolerance = 1)
heavy_y9 = heavy_b1[FragmentIon == "y9^1/934.4870605", Intensity]
expect_equal(heavy_y9, 17024.44141, tolerance = 1)
heavy_y6 = heavy_b1[FragmentIon == "y6^1/721.3757324", Intensity]
expect_equal(heavy_y6, 15426.27441, tolerance = 1)

# Light fragment intensities trace back to the light input rows.
# Input: AAAAAAAAQMHTK(SILAC-K-L), Run dAL_AT_Long_B1 has
#   Fragment.Quant.Corrected = "120170.8125;39879.93359;13874.50293;33449.03516;0;33788.10156;"
#   Fragment.Info ions (after NH3 removal):
#     y5^1/644.3184814 -> 120170.8125, y8^1/857.4298096 -> 39879.93359,
#     y10^1/999.5040283 -> 13874.50293, y9^1/928.4669189 -> 33449.03516
light_b1 = output_silac_dt[PeptideSequence == "AAAAAAAAQMHTK" &
                            IsotopeLabelType == "L" &
                            Run == "dAL_AT_Long_B1"]
light_y5 = light_b1[FragmentIon == "y5^1/644.3184814", Intensity]
expect_equal(light_y5, 120170.8125, tolerance = 1)
light_y8 = light_b1[FragmentIon == "y8^1/857.4298096", Intensity]
expect_equal(light_y8, 39879.93359, tolerance = 1)
light_y10 = light_b1[FragmentIon == "y10^1/999.5040283", Intensity]
expect_equal(light_y10, 13874.50293, tolerance = 1)
light_y9 = light_b1[FragmentIon == "y9^1/928.4669189", Intensity]
expect_equal(light_y9, 33449.03516, tolerance = 1)

# Zero-intensity input fragments are converted to NA in output
# AAAAAAAAQMHTK(SILAC-K-L) B1 has y6^1/715.3555908 = 0 in input
light_y6_zero = light_b1[FragmentIon == "y6^1/715.3555908", Intensity]
expect_true(is.na(light_y6_zero))

# Verify cross-run consistency: B8 light intensities also trace to input
# Input: AAAAAAAAQMHTK(SILAC-K-L), Run dAL_AT_Long_B8 has
#   Fragment.Quant.Corrected = "27676.55078;5134.108887;4891.382324;3844.573975;3443.866699;8253.693359;"
light_b8 = output_silac_dt[PeptideSequence == "AAAAAAAAQMHTK" &
                            IsotopeLabelType == "L" &
                            Run == "dAL_AT_Long_B8"]
light_y5_b8 = light_b8[FragmentIon == "y5^1/644.3184814", Intensity]
expect_equal(light_y5_b8, 27676.55078, tolerance = 1)
light_y8_b8 = light_b8[FragmentIon == "y8^1/857.4298096", Intensity]
expect_equal(light_y8_b8, 5134.108887, tolerance = 1)

# Test DIANNtoMSstatsFormat PXD055942 (DIANN 2.0 + Channel-based SILAC) ------
# Dataset: PXD055942 (heart protein turnover, 0/8/32 day timepoints).
# Uses quantificationColumn = "auto" (Fr.N.Quantity columns) and the Channel
# column path for IsotopeLabelType (no SILAC suffix in ModifiedSequence).
input_file_path_pxd = system.file("tinytest/raw_data/DIANN/PXD055942.parquet",
                                   package = "MSstatsConvert")
annotation_file_path_pxd = system.file("tinytest/raw_data/DIANN/annotation_PXD055942.csv",
                                        package = "MSstatsConvert")
input_pxd = arrow::read_parquet(input_file_path_pxd)
annot_pxd = data.table::fread(annotation_file_path_pxd)
output_pxd = DIANNtoMSstatsFormat(input_pxd, annotation = annot_pxd,
                                   labeledAminoAcids = c("K"),
                                   removeFewMeasurements = FALSE,
                                   quantificationColumn = "auto",
                                   use_log_file = FALSE)
output_pxd_dt = data.table::as.data.table(output_pxd)

# Basic structure
expect_equal(ncol(output_pxd), 11)
expect_equal(nrow(output_pxd), 23568)
expect_true("Run" %in% colnames(output_pxd))
expect_true("ProteinName" %in% colnames(output_pxd))
expect_true("PeptideSequence" %in% colnames(output_pxd))
expect_true("PrecursorCharge" %in% colnames(output_pxd))
expect_true("Intensity" %in% colnames(output_pxd))
expect_true("FragmentIon" %in% colnames(output_pxd))
expect_true("ProductCharge" %in% colnames(output_pxd))
expect_true("IsotopeLabelType" %in% colnames(output_pxd))
expect_true("Condition" %in% colnames(output_pxd))
expect_true("BioReplicate" %in% colnames(output_pxd))
expect_true("Fraction" %in% colnames(output_pxd))

# Channel path used: H/L/NA counts, no "Light" default
expect_false("Light" %in% output_pxd$IsotopeLabelType)
pxd_label_counts = table(output_pxd$IsotopeLabelType, useNA = "ifany")
expect_equal(unname(pxd_label_counts["H"]), 9720L)
expect_equal(unname(pxd_label_counts["L"]), 9720L)
expect_equal(sum(is.na(output_pxd$IsotopeLabelType)), 4128L)

# Channel path does NOT strip (SILAC) from PeptideSequence — that notation is
# part of the peptide identity in this format, not a label indicator
expect_true(any(grepl("(SILAC)", output_pxd$PeptideSequence, fixed = TRUE)))

# Unlabeled peptides (no K, no SILAC) get NA IsotopeLabelType
pxd_unlabeled = output_pxd_dt[PeptideSequence %in% c("AAATFNPELITHILDGSPENTR", "AAASLDTAALSATDMALALNR",
                                                       "AIMHHEGHMDDGLNLSR")]
expect_true(all(is.na(pxd_unlabeled$IsotopeLabelType)))

# Annotation maps correctly to all three timepoint conditions
expect_true("0 day" %in% output_pxd$Condition)
expect_true("8 days" %in% output_pxd$Condition)
expect_true("32 days" %in% output_pxd$Condition)
run_0d = "20210805_BoxCarmax1st_wideMS1_JM_pSIL_pro_heart_0d_1"
expect_equal(as.character(unique(output_pxd_dt[Run == run_0d, Condition])), "0 day")

h_pxd_pep = "TAFDDAIAELDTLNEDSYK(SILAC)(SILAC)(SILAC)"
h_pxd_run = "20210805_BoxCarmax2nd_wideMS1_JM_pSIL_pro_heart_32d_1"
h_pxd_ints = output_pxd_dt[PeptideSequence == h_pxd_pep &
                             IsotopeLabelType == "H" &
                             Run == h_pxd_run, Intensity]
expect_equal(max(h_pxd_ints, na.rm = TRUE), 2453057.5, tolerance = 1)

l_pxd_pep = "TAFDDAIAELDTLNEDSYK(SILAC)(SILAC)(SILAC)"
l_pxd_run = "20210805_BoxCarmax2nd_wideMS1_JM_pSIL_pro_heart_0d_1"
l_pxd_ints = output_pxd_dt[PeptideSequence == l_pxd_pep &
                             IsotopeLabelType == "L" &
                             Run == l_pxd_run, Intensity]
expect_equal(max(l_pxd_ints, na.rm = TRUE), 4686039.5, tolerance = 1)

# Test DIANNtoMSstatsFormat ---------------------------
input_file_path = system.file("tinytest/raw_data/DIANN/diann_input.tsv", package="MSstatsConvert")
annotation_file_path = system.file("tinytest/raw_data/DIANN/annotation.csv", package = "MSstatsConvert")
input = data.table::fread(input_file_path)
annot = data.table::fread(annotation_file_path)
output = DIANNtoMSstatsFormat(input, annotation = annot, MBR = FALSE, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 348) 
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))
# When labeledAminoAcids = NULL (default), IsotopeLabelType is filled with "Light"
expect_true(all(output$IsotopeLabelType == "Light"))

# Test DIANNtoMSstatsFormat DIANN 2.0 ------------------------
input_file_path = system.file("tinytest/raw_data/DIANN/diann_2.0.parquet", package="MSstatsConvert")
annotation_file_path = system.file("tinytest/raw_data/DIANN/annotation_diann_2.0.csv", package = "MSstatsConvert")
input = arrow::read_parquet(input_file_path)
annot = data.table::fread(annotation_file_path)
output = DIANNtoMSstatsFormat(input, annotation = annot, MBR = FALSE, use_log_file = FALSE, quantificationColumn = 'auto')
expect_equal(ncol(output), 11)
expect_equal(nrow(output), 192)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("PrecursorCharge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("FragmentIon" %in% colnames(output))
expect_true("ProductCharge" %in% colnames(output))
expect_true("IsotopeLabelType" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Fraction" %in% colnames(output))
# When labeledAminoAcids = NULL (default), IsotopeLabelType is filled with "Light"
expect_true(all(output$IsotopeLabelType == "Light"))

#' Clean raw Protein Prospector data
#' @param msstats_object object that inherits from MSstatsInputFiles class.
#' @return data.table
#' @keywords internal
#' @noRd
.cleanRawProteinProspector = function(msstats_object) {
    PSM = PeptideSequence = PrecursorCharge = NULL
    protein_prospector_input = getInputFile(msstats_object, "input")
    protein_prospector_input = 
        data.table::as.data.table(protein_prospector_input)
    channels = .getChannelColumns(
        colnames(protein_prospector_input), "Int")
    req_cols = c('AccX', 'z', 'DBPeptide', 'Fraction', channels)
    protein_prospector_input = protein_prospector_input[, req_cols, with = FALSE]
    data.table::setnames(
        protein_prospector_input, 
        c("AccX", "DBPeptide", "z", "Fraction"),
        c("ProteinName", "PeptideSequence", "PrecursorCharge", "Run"), 
        skip_absent = TRUE)
    protein_prospector_input[, PSM := paste(PeptideSequence, PrecursorCharge, 
                            1:nrow(protein_prospector_input), sep = "_")]
    
    protein_prospector_input = melt(protein_prospector_input, 
                                    measure.vars = channels, 
                                    id.vars = setdiff(
                                        colnames(protein_prospector_input), 
                                        channels
                                    ),
                                    variable.name = "Channel", 
                                    value.name = "Intensity"
                                )
    protein_prospector_input$Channel = .standardizeColnames(
        protein_prospector_input$Channel
    )
    
    .logSuccess("ProteinProspector", "clean")
    protein_prospector_input
}

.testHappyPath = function(input) {
    output = MSstatsConvert:::.cleanRawProteinProspector(input)
    expect_equal(ncol(output), 7)
    expect_equal(nrow(output), 966)
    expected_column_names = c(
        "ProteinName",
        "PeptideSequence",
        "PrecursorCharge",
        "Run",
        "PSM",
        "Channel",
        "Intensity"
    )
    missing_columns = setdiff(expected_column_names, colnames(output))
    expect_equal(length(missing_columns), 0)
}

# Test PD .cleanRawPDTMT ---------------------------

file_path = system.file("tinytest/processed_data/ProtProspector/MSstatsProtProspectorFilesObjectTMT.rds", package="MSstatsConvert")
input = readRDS(file_path)
.testHappyPath(input)

#' Convert Proteome Discoverer output to MSstatsTMT format.
#' 
#' @param input PD report or a path to it.
#' @param annotation annotation with Run, Fraction, TechRepMixture, Mixture, Channel, 
#' BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column.
#' @param which.proteinid Use 'Protein.Accessions'(default) column for protein name. 'Master.Protein.Accessions' can be used instead to get the protein name with single protein.
#' @param useNumProteinsColumn logical, TRUE (default) removes shared peptides by information of # Proteins column in PSM sheet.
#' @param rmPSM_withfewMea_withinRun TRUE (default) will remove the features that have 1 or 2 measurements within each Run.
#' @param rmProtein_with1Feature TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.
#' @param ... additional parameters to `data.table::fread`.
#' @inheritParams .sharedParametersAmongConverters
#' 
#' @return `data.frame` of class `MSstatsTMT`
#' 
#' @export
#' 
#' @examples
#' raw.pd = data.table::fread(system.file("tinytest/raw_data/PDTMT/pdtmt_input.csv",
#'                                       package = "MSstatsConvert"))
#' annotation.pd = data.table::fread(system.file("tinytest/raw_data/PDTMT/pd_annotation.csv",
#'                                       package = "MSstatsConvert"))
#' head(raw.pd)
#' head(annotation.pd)
#' input.pd <- PDtoMSstatsTMTFormat(raw.pd, annotation.pd)
#' head(input.pd)
#' 
PDtoMSstatsTMTFormat <- function(
        input, annotation, which.proteinid = 'Protein.Accessions', 
        useNumProteinsColumn = TRUE, useUniquePeptide = TRUE, 
        rmPSM_withfewMea_withinRun = TRUE, rmProtein_with1Feature = FALSE, 
        summaryforMultipleRows = sum, 
        use_log_file = TRUE, append = FALSE, verbose = TRUE, log_file_path = NULL, ...
) {
    MSstatsConvert::MSstatsLogsSettings(use_log_file, append, verbose, 
                                        log_file_path, 
                                        base = "MSstatsTMT_converter_log_")
    
    input = MSstatsConvert::MSstatsImport(list(input = input),
                                          "MSstatsTMT", "ProteomeDiscoverer", ...)
    input = MSstatsConvert::MSstatsClean(
        input, 
        protein_id_column = which.proteinid,
        remove_shared = useNumProteinsColumn,
        remove_protein_groups = useNumProteinsColumn)
    annotation = MSstatsConvert::MSstatsMakeAnnotation(input, annotation)
    
    feature_columns = c("PeptideSequence", "PrecursorCharge")
    input = MSstatsConvert::MSstatsPreprocess(
        input,
        annotation, 
        feature_columns = feature_columns,
        remove_shared_peptides = useUniquePeptide,
        remove_single_feature_proteins = rmProtein_with1Feature,
        feature_cleaning = list(remove_features_with_few_measurements = rmPSM_withfewMea_withinRun,
                                summarize_multiple_psms = summaryforMultipleRows)
    )
    input = MSstatsConvert::MSstatsBalancedDesign(input, feature_columns,
                                                  fix_missing = "zero_to_na")
    data.table::setnames(input, "PrecursorCharge", "Charge", skip_absent = TRUE)
    
    msg_final = paste("** Finished preprocessing. The dataset is ready",
                      "to be processed by the proteinSummarization function.")
    getOption("MSstatsLog")("INFO", msg_final)
    getOption("MSstatsMsg")("INFO", msg_final)
    getOption("MSstatsLog")("INFO", "\n")
    input
}

# Test PDtoMSstatsTMTFormat ---------------------------
pd_raw = data.table::fread(system.file("tinytest/raw_data/PDTMT/pdtmt_input.csv",
                                       package = "MSstatsConvert"))
annot = data.table::fread(system.file("tinytest/raw_data/PDTMT/pd_annotation.csv",
                                      package = "MSstatsConvert"))
output = PDtoMSstatsTMTFormat(pd_raw, annot, use_log_file = FALSE)
expect_equal(ncol(output), 11)
expect_true(nrow(output) > 0)
expect_true("Run" %in% colnames(output))
expect_true("ProteinName" %in% colnames(output))
expect_true("PeptideSequence" %in% colnames(output))
expect_true("Charge" %in% colnames(output))
expect_true("Intensity" %in% colnames(output))
expect_true("TechRepMixture" %in% colnames(output))
expect_true("PSM" %in% colnames(output))
expect_true("Mixture" %in% colnames(output))
expect_true("Condition" %in% colnames(output))
expect_true("BioReplicate" %in% colnames(output))
expect_true("Channel" %in% colnames(output))

raw.pd = data.table::fread(system.file("tinytest/raw_data/PDTMT/pdtmt_input.csv",
                                       package = "MSstatsConvert"))
annotation.pd = data.table::fread(system.file("tinytest/raw_data/PDTMT/pd_annotation.csv",
                                              package = "MSstatsConvert"))

expect_error(PDtoMSstatsTMTFormat(input = pd_raw[, !colnames(pd_raw) == "Protein.Accessions"], # missing columns in input
                                  annotation = annot, use_log_file = FALSE))

expect_error(PDtoMSstatsTMTFormat(input = pd_raw,
                                  annotation = annot[, !colnames(annot) == "Condition"], # missing columns in annotation
                                  use_log_file = FALSE))

# Verify output intensities are present in input
intensity_cols = grep("^Abundance", colnames(pd_raw), value = TRUE)
input_intensities = as.character(unlist(pd_raw[, intensity_cols, with = FALSE]))
output_intensities = as.character(output$Intensity[!is.na(output$Intensity)])
expect_true(all(output_intensities %in% input_intensities))

---
title: "Working with MSstatsConvert"
output: rmarkdown::html_vignette
vignette: >
 %\VignetteIndexEntry{Working with MSstatsConvert}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---
  
```{r, include = FALSE}
  knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
  )
```
  
  
# Purpose of MSstatsConvert
  
The `MSstatsConvert` package is a member of the `MSstatst` family of packages, MSstats and MSstatsTMT.
It creates an abstraction for the steps in mass spectrometry (MS) data analysis 
that are required before a dataset can be used for statistical modeling. 
In short, the package is responsible for converting output from signal processing
tools such as `OpenMS` or `MaxQuant` into a format suitable for statistical analysis. 
This includes:
  
  - PSM- and protein-level filtering,
  - managing shared peptides,
  - removing decoy, iRT, and other irrelavant sequences,
  - removing features or proteins with a low number of measurements,
  - aggregating duplicated measurements,
  - handling fractionation by removing overlapped features,
  - creating balanced statistical design in the presence of missing data.
  
`MSstatsConvert` allows for transforming any MS quantification result into a format
required by `MSstats` and `MSstatsTMT` packages.
Additionally, it provides built-in cleaning functions for outputs of `DIAUmpire`, `MaxQuant`,
`OpenMS`, `OpenSWATH`, `Progenesis`, `ProteomeDiscoverer`, `Skyline`, `Spectromine`, and `Spectronaut`.
These functions serve as a base for converter functions (called `*toMSstatsFormat` or 
`*toMSstatsTMTFormat`) provided  by the `MSstats` and `MSstatsTMT` packages.
  
# MSstats data format
  
MSstats family packages works with label-free, SRM and TMT datasets. 
The following column are required.

  - `ProteinName`: column that indicates a protein ID. If the analysis is to be made 
  at the peptide-level, the column should store peptide IDs. Summarization performed 
  by `MSstats` and `MSstatsTMT` packages is done separately for each ID in this column,
  - `PeptideSequence`, `PrecursorCharge`, `FragmentIon` and `ProductCharge`: these 
  four columns define a spectral feature (transition in SRM case). If information 
  for any of the columns is not available, it should be set to a constant value
  (for example `NA`),
  - `IsotopeLabelType`: column that indicates whether the measurement is based on an
  endogenous peptide (indicated by value "L" or "light") or reference peptide (indicated by value "H" or "heavy"),
  - `Run`: column that stores IDs of mass spectrometry runs. If annotation describing
  biological conditions and replicates is provided via a separate table, the run IDs
  should match `Run` IDs in the annotation,
  - `Condition`: column that stores labels for biological conditions (groups of interest).
  For time-course experiments, this column will represent time points. If the design
  experiment includes both time points and distinct biological subjects, these labels
  should be a combination of subject and time point,
  - `BioReplicate`: this column should contain a unique identifier for each biological replicate in the experiment. 
  For example, in a clinical proteomic investigation this should be a unique  patient ID. 
  Patients from distinct groups (indicated by the `Condition` column) should have distinct IDs,
  - `Intensity`: column that stores untransformed (in particular, no log transformation) measurements of feature abundance 
  in a given Run (and Channel in TMT case). 
  They can be peak heights, peak areas under the curve, or other quantitative representations
  of feature abundance,
  - for TMT datasets, a `Channel` column is required. Similarly to the `Run` column,
  values in this column must correspond to values in the annotation file, if provided separately.
  
Additionally, if the experiment involves fractionation, `Fraction` column can be added
to store fraction labels.

# Logging
  
`MSstatsConvert` allows for flexible logging.
Information about preprocessing steps can be written to a file, to a console,
to both or to neither.
The `MSstatsLogsSettings` function helps manage log settings.
The user can pass a path to an existing file to the `log_file_path` parameter.
Combined with setting `append = TRUE`, this allows writing all information related to 
a specific data analysis to a single file.
If a user does not specify a file, a new file will be created automatically with a name
starting with "MSstats_log", followed by a timestamp.
  
```{r logs}
library(MSstatsConvert)
# default - creates a new file
MSstatsLogsSettings(use_log_file = TRUE, append = FALSE) 

# default - creates a new file
MSstatsLogsSettings(use_log_file = TRUE, append = TRUE, 
                    log_file_path = "log_file.log") 

# switches logging off
MSstatsLogsSettings(use_log_file = FALSE, append = FALSE) 

# switches off logs and messages
MSstatsLogsSettings(use_log_file = FALSE, verbose = FALSE) 
```
  
Additionally, session info generated by the `utils::sessionInfo()` function
can be saved to file with the `MSstatsSaveSessionInfo` function.

```{r }
MSstatsSaveSessionInfo()
```
  
By default, the output file name will start with "MSstats_session_info" and end with 
a current timestamp.
  
# Importing and cleaning data
  
MS data processing by `MSstatsConvert` starts with importing and cleaning data.
The `MSstatsImport` function produces a wrapper for possibly multiple files that
may describe a single dataset.
For example, `MaxQuant` output consists of two files, while `OpenMS` outputs just
a single file.
  
```{r }
maxquant_evidence = read.csv(system.file("tinytest/raw_data/MaxQuant/mq_ev.csv",
                                         package = "MSstatsConvert"))
maxquant_proteins = read.csv(system.file("tinytest/raw_data/MaxQuant/mq_pg.csv",
                                         package = "MSstatsConvert"))
maxquant_imported = MSstatsImport(list(evidence = maxquant_evidence,
                                       protein_groups = maxquant_proteins),
                                  type = "MSstats", tool = "MaxQuant")
is(maxquant_imported)

openms_input = read.csv(system.file(
  "tinytest/raw_data/OpenMSTMT/openmstmt_input.csv",
  package = "MSstatsConvert"
))
openms_imported = MSstatsImport(list(input = openms_input),
                                "MSstatsTMT", "OpenMS")
is(openms_imported)
```
  
The `getInputFile` method allows user to retrieve the files:

```{r }
getInputFile(maxquant_imported, "evidence")[1:5, 1:5]
```
  
As a next step of the analysis, input files are combined into a single `data.table`
with standardized column names by the `MSstatsClean` function.
It is a generic function with built-in support for outputs of tools listed in 
the "Purpose of the MSstatsConvert package" section.
The `type` parameter is equal to either `MSstats` or `MSstatsTMT` and indicates 
if the data comes from a labelled TMT experiment.

For some datasets, `MSstatsClean` may require additional parameters described in the respective help files.
For our example datasets, the following calls merge input files into a single table.

```{r }
maxquant_cleaned = MSstatsClean(maxquant_imported, protein_id_col = "Proteins")
head(maxquant_cleaned)

openms_cleaned = MSstatsClean(openms_imported)
head(openms_cleaned)
```

If a user wants to use `MSstatsConvert` package with data in a format that is not
currently supported, it is enough to first re-format the data into a `data.table`
with column ProteinName, PeptideSequence, PrecursorCharge, FragmentIon, ProductCharge (with the latter two possibly equal to NA), Run and IsotopeLabelType (in case of non-TMT data) or 
Channel (in case of TMT data). Moreover, the dataset may include any column that will be used for filtering the dataset (for example a column that stores q-values).
In our example, such additional columns are "Modifications" and "Score" from MaxQuant files.

Annotation columns should be called Condition and BioReplicate. For TMT data,
Mixture, TechRepMixture columns may be added.
Fractionation should be indicated by a Fraction column.

# Preprocessing
  
The goal of `MSstatsPreprocess` function is to transform cleaned MS data into a format
ready for statistical analysis with `MSstats` or `MSstatsTMT` packages.
This function accepts several parameters, and each corresponds to a preprocessing step.

  - `input` parameter is the dataset for preprocessing,
  - `annotation` is a description of biological conditions and replicates associated with
  MS runs (and channels in TMT case). If annotation is already included in the `input`,
  it should be equal to `NULL`. The annotation should be created by the
  `MSstatsMakeAnnotation` function,
  - `feature_columns` is a vector of column names that will denote features,
  - `remove_shared_peptides` is a logical parameter - if `TRUE`, shared peptides 
  will be removed from the analysis. Currently, `MSstats` assumes that only unique peptides
  are used, and presence of shared peptides may cause issues,
  - `remove_single_feature_proteins` is a logical parameter that indicates if proteins
  that only have a single feature should be removed from the analysis (`TRUE`),
  - `feature_cleaning` is a list, that currently consists of two named elements:
  `remove_features_with_few_measurements` should be equal to TRUE or FALSE. In the first case,
  feature that have less than three measurements across runs (or channels in a run for TMT data)
  will be removed. FALSE means that only features with no non-missing measurements will be removed. The `summarize_multiple_psms` element should be a function that will be used to aggregate multiple feature measurements within a single MS run,
  - `aggregate_isotopic` is a logical parameter - `TRUE` means that isotopic peaks will be aggregated (currently only used for Skyline input),
  - `columns_to_fill` is an optional named list with names corresponding to columns 
  and values correponding to values that will be used for these columns. For example,
  if the dataset is missing information about `ProductCharge`, such a column can be 
  added by passing `list(ProductCharge = NA)` to this parameter,
  - `score_filtering`, `exact_filtering` and `pattern_filtering` parameters are optional parameters that can be used to perform data filtering. An example is given below.
  

```{r }
maxquant_annotation = read.csv(system.file(
  "tinytest/raw_data/MaxQuant/annotation.csv",
  package = "MSstatsConvert"
))
maxquant_annotation = MSstatsMakeAnnotation(maxquant_cleaned,
                                            maxquant_annotation,
                                            Run = "Rawfile")
m_filter = list(col_name = "PeptideSequence", 
                pattern = "M", 
                filter = TRUE, 
                drop_column = FALSE)

oxidation_filter = list(col_name = "Modifications", 
                        pattern = "Oxidation", 
                        filter = TRUE, 
                        drop_column = TRUE)

feature_columns = c("PeptideSequence", "PrecursorCharge")
maxquant_processed = MSstatsPreprocess(
  maxquant_cleaned, 
  maxquant_annotation,
  feature_columns,
  remove_shared_peptides = TRUE, 
  remove_single_feature_proteins = FALSE,
  pattern_filtering = list(oxidation = oxidation_filter,
                           m = m_filter),
  feature_cleaning = list(remove_features_with_few_measurements = TRUE,
                          summarize_multiple_psms = max),
  columns_to_fill = list("FragmentIon" = NA,
                         "ProductCharge" = NA,
                         "IsotopeLabelType" = "L"))
head(maxquant_processed)

# OpenMS - TMT data
feature_columns_tmt = c("PeptideSequence", "PrecursorCharge")
openms_processed = MSstatsPreprocess(
  openms_cleaned, 
  NULL, 
  feature_columns_tmt,
  remove_shared_peptides = TRUE,
  remove_single_feature_proteins = TRUE,
  feature_cleaning = list(remove_features_with_few_measurements = TRUE,
                          summarize_multiple_psms = max)
)
head(openms_processed)
```
  
Annotation is created via the `MSstatsMakeAnnotation` function. It takes the cleaned dataset
and annotation file as input. Additionally, key-value pairs can be passed to this function
to change column names (not including dots and other symbols) in the annotation from names given by values to names given by keys.

For programmatic applications and consistency of the interface, filtering is done
with the help of lists.

For filtering based on numerical scores (for example q-value filtering), the list should 
consist of elements named 
  
  - `score_column`: name of a column that stores the score,
  - `score_threshold`: value above or below which measurements should be kept,
  - `direction`: if "greater", values greater than `score_threshold` will be kept; 
  if "smaller", values smaller than `score_threshold` will be kept;
  - `behavior`: if "remove", rows not below/above the threshold will be removed;
  if "replace", intensity in rows not below/above the threshold will be replaced by a given value,
  - `handle_na`: if "keep", `NA` in the score column will not be removed,
  - `fill_value`: value that will be used if `behavior = "replace"`,
  - `filter`: if `TRUE`, filtering will be performed (can be used for conditional filtering),
  - `drop_column`: if `TRUE`, column that stored the score will be removed.

For example, to remove intensities smaller than 1, we could pass the following list to
the `score_filtering` parameters:

```{r , eval = FALSE}
list(
  list(score_column = "Intensity", score_threshold = 1,
       direction = "greater", behavior = "remove", 
       handle_na = "remove", fill_value = NA, filter = TRUE, drop = FALSE
       )
)
```

For filtering based on patterns (for example, removing oxidation peptides), the list
should consist of elements named

  - `col_name`: name of a column that values that may be removed,
  - `filter_symbols`: vector of values - rows with these values in `col_name` will be removed
  or corresponding intensities will be replaced,
  - `behavior`: if "remove", rows that contain `filter_symbols` in `col_name` will be removed;
  if "replace", intensity in rows that contain `filter_symbols` in `col_name` will be replaced by a given value,
  - `fill_value`: value that will be used if `behavior = "replace"`,
  - `filter`: if `TRUE`, filtering will be performed (can be used for conditional filtering),
  - `drop_column`: if `TRUE`, column that stored the score will be removed.

For filtering based on exact values (for example, removing iRT proteins), the list
should consists of elements named

  - `col_name`: name of a column that stores strings that will be searched for given patterns,
  - `pattern`: vector of regular expressions - rows with matching values in `col_name` will be removed,
  - `filter`: if `TRUE`, filtering will be performed (can be used for conditional filtering),
  - `drop_column`: if `TRUE`, column that stored the values for filtering will be removed.

# Fractions and balanced design

Finally, after preprocessing, `MSstatsBalancedDesign` function can be applied to 
handle fractions and create balanced design.
For label-free and SRM data, it means that fractionation or technical replicates will be detected if these information is not provided. Features measured in multiple fractions (overlapped) will be assigned to a unique fraction. Then, the data will be adjusted so that within each fraction, every feature has a row for certain run. If the intensity value is missing, it will be denoted by `NA`.

For TMT data, a unique fraction will be selected for each overlapped feature and the 
data will adjusted so that within each run, every feature has a row for each channel.
If the intensity is missing for a channel, it will be denoted by `NA`.

```{r }
maxquant_balanced = MSstatsBalancedDesign(maxquant_processed, feature_columns)
head(maxquant_balanced)
dim(maxquant_balanced)
dim(maxquant_processed)

openms_balanced = MSstatsBalancedDesign(openms_processed, feature_columns_tmt)
head(openms_balanced)
dim(openms_balanced)
dim(openms_processed)
```

`MSstatsBalancedDesign` output is a `data.frame` of class `MSstatsValidated`.
Such a `data.frame` will be recognized by statistical processing functions from
`MSstats` and `MSstatsTMT` packages as a valid input, which will allow them to skip
checks and transformation necessary to fit data into this format.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_dt_operations.R
\name{.fixColumnTypes}
\alias{.fixColumnTypes}
\title{Change classes of multiple columns}
\usage{
.fixColumnTypes(
  input,
  numeric_columns = NULL,
  character_columns = NULL,
  factor_columns = NULL
)
}
\arguments{
\item{input}{data.table preprocessed by one of the \verb{cleanRaw*} functions.}

\item{numeric_columns}{chr, vector of names of columns that will be
converted to numeric.}

\item{character_columns}{chr, vector of names of colums taht will be
converted to character.}

\item{factor_columns}{chr, vector of names of columns that will be
converted to factor.}
}
\value{
data.table
}
\description{
Change classes of multiple columns
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_balanced_design.R
\name{.getMissingRunsPerFeature}
\alias{.getMissingRunsPerFeature}
\title{Get names of missing runs}
\usage{
.getMissingRunsPerFeature(input)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}
}
\value{
data.table
}
\description{
Get names of missing runs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.checkOverlappedFeatures}
\alias{.checkOverlappedFeatures}
\title{Check if any features are measured in multiple fractions}
\usage{
.checkOverlappedFeatures(input)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}
}
\value{
data.table
}
\description{
Check if any features are measured in multiple fractions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_DIAUmpire.R
\name{.cleanRawDIAUmpire}
\alias{.cleanRawDIAUmpire}
\title{Clean raw DIAUmpire files}
\usage{
.cleanRawDIAUmpire(msstats_object, use_frag, use_pept)
}
\arguments{
\item{msstats_object}{Object that inherits from MSstatsInputFiles class.}

\item{use_frag}{TRUE will use the selected fragment for each peptide.
'Selected_fragments' column is required.}

\item{use_pept}{TRUE will use the selected fragment for each protein
'Selected_peptides' column is required.}
}
\value{
data.table
}
\description{
Clean raw DIAUmpire files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_classes.R
\name{as.data.frame.MSstatsValidated}
\alias{as.data.frame.MSstatsValidated}
\title{Convert output of converters to data.frame}
\usage{
\method{as.data.frame}{MSstatsValidated}(x, ...)
}
\arguments{
\item{x}{object of class MSstatsValidated}

\item{...}{Additional arguments to be passed to or from other methods.}
}
\value{
data.frame
}
\description{
Convert output of converters to data.frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_SpectroMinetoMSstatsTMTFormat.R
\name{SpectroMinetoMSstatsTMTFormat}
\alias{SpectroMinetoMSstatsTMTFormat}
\title{Import data from SpectroMine}
\usage{
SpectroMinetoMSstatsTMTFormat(
  input,
  annotation,
  filter_with_Qvalue = TRUE,
  qvalue_cutoff = 0.01,
  useUniquePeptide = TRUE,
  rmPSM_withfewMea_withinRun = TRUE,
  rmProtein_with1Feature = FALSE,
  summaryforMultipleRows = sum,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{data name of SpectroMine PSM output. Read PSM sheet.}

\item{annotation}{data frame which contains column Run, Fraction, TechRepMixture, Mixture, Channel, BioReplicate, Condition. Refer to the example 'annotation.mine' for the meaning of each column.}

\item{filter_with_Qvalue}{TRUE(default) will filter out the intensities that have greater than qvalue_cutoff in EG.Qvalue column. Those intensities will be replaced with NA and will be considered as censored missing values for imputation purpose.}

\item{qvalue_cutoff}{Cutoff for EG.Qvalue. default is 0.01.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{rmPSM_withfewMea_withinRun}{TRUE (default) will remove the features that have 1 or 2 measurements within each Run.}

\item{rmProtein_with1Feature}{TRUE will remove the proteins which have only 1 peptide and charge. Defaut is FALSE.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
\code{data.frame} of class \code{MSstatsTMT}
}
\description{
Import data from SpectroMine
}
\examples{
raw.mine = data.table::fread(system.file("tinytest/raw_data/SpectroMine/spectromine_input.csv",
                                      package = "MSstatsConvert"))
annotation.mine = data.table::fread(system.file("tinytest/raw_data/SpectroMine/spectromine_annotation.csv",
                                      package = "MSstatsConvert"))
head(raw.mine)
head(annotation.mine)
input.mine <- SpectroMinetoMSstatsTMTFormat(raw.mine, annotation.mine)
head(input.mine)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{CheckDataHealth}
\alias{CheckDataHealth}
\title{Takes as input the output of the SpectronauttoMSstatsFormat function and calculates various quality metrics to assess the health of the data. Requires Anomaly Detection model to be fit.}
\usage{
CheckDataHealth(input)
}
\arguments{
\item{input}{MSstats input which is the output of Spectronaut converter}
}
\value{
list of two data.tables
}
\description{
Takes as input the output of the SpectronauttoMSstatsFormat function and calculates various quality metrics to assess the health of the data. Requires Anomaly Detection model to be fit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_dt_operations.R
\name{.fixBasicColumns}
\alias{.fixBasicColumns}
\title{Remove underscores from sequences and change intensity type to numeric}
\usage{
.fixBasicColumns(input)
}
\arguments{
\item{input}{data.table}
}
\value{
data.table
}
\description{
Remove underscores from sequences and change intensity type to numeric
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.resolveFractionTies}
\alias{.resolveFractionTies}
\title{Resolve ties when multiple fractions share the maximum number of measurements
for a given feature. In the case of a tie, the fraction with the highest
mean intensity is selected.}
\usage{
.resolveFractionTies(input, max_fractions)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}

\item{max_fractions}{data.table of fractions that share the maximum number
of unique runs per feature, as produced by \code{.removeOverlappingFeatures}}
}
\value{
data.table with columns \code{feature} and \code{Fraction}, containing one
selected fraction per feature
}
\description{
Resolve ties when multiple fractions share the maximum number of measurements
for a given feature. In the case of a tie, the fraction with the highest
mean intensity is selected.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_Skyline.R
\name{.handleIsotopicPeaks}
\alias{.handleIsotopicPeaks}
\title{Handle isotopic peaks}
\usage{
.handleIsotopicPeaks(input, aggregate = FALSE)
}
\arguments{
\item{input}{data.table preprocessed by one of the \verb{cleanRaw*} functions.}

\item{aggregate}{if TRUE, isotopic peaks will be summed.}
}
\value{
data.table
}
\description{
Handle isotopic peaks
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clean_features.R
\name{.summarizeMultiplePSMs}
\alias{.summarizeMultiplePSMs}
\title{Pick one PSM from a data.table of several PSMs.}
\usage{
.summarizeMultiplePSMs(input, summary_function)
}
\arguments{
\item{input}{data.table preprocessed by one of the .cleanRaw* functions.}

\item{summary_function}{function that will be used to aggregate intensities
if needed.}
}
\value{
character - label of a chosen PSM
}
\description{
Pick one PSM from a data.table of several PSMs.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.filterOverlapped}
\alias{.filterOverlapped}
\title{Remove overlapped features}
\usage{
.filterOverlapped(input, summary_function, overlapped_features)
}
\arguments{
\item{input}{data.table preprocessed by one of the .cleanRaw* functions and
merged with annotation.}

\item{summary_function}{summary function (mean, sum, max) that will be used
to pick one feature from multiple overlapping features}

\item{overlapped_features}{features that overlap.}
}
\value{
\code{data.table}
}
\description{
Remove overlapped features
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_checks.R
\name{.validateMSstatsConverterParameters}
\alias{.validateMSstatsConverterParameters}
\title{Generic parameter validation for all MSstats converters using configuration object}
\usage{
.validateMSstatsConverterParameters(config)
}
\arguments{
\item{config}{A list containing all converter parameters. See details for required structure.}
}
\value{
NULL (throws error if validation fails)
}
\description{
Generic parameter validation for all MSstats converters using configuration object
}
\details{
The config list should contain the input and optionally other parameters:
\itemize{
\item input: input data (required)
\item annotation: annotation data (optional)
\item intensity: intensity type (optional)
\item filter_with_Qvalue: Q-value filter setting (default: FALSE)
\item qvalue_cutoff: Q-value cutoff (default: 0.01)
\item useUniquePeptide: unique peptide setting (default: TRUE)
\item removeFewMeasurements: remove few measurements setting (default: TRUE)
\item removeProtein_with1Feature: remove single feature proteins setting (default: FALSE)
\item summaryforMultipleRows: aggregation function (default: max)
\item calculateAnomalyScores: anomaly detection setting (default: FALSE)
\item anomalyModelFeatures: anomaly model features (default: c())
\item anomalyModelFeatureTemporal: temporal features (default: c())
\item removeMissingFeatures: missing feature threshold (default: 0.5)
\item anomalyModelFeatureCount: feature count for anomaly model (default: 100)
\item runOrder: run order data (default: NULL)
\item n_trees: number of trees (default: 100)
\item max_depth: max tree depth (default: "auto")
\item numberOfCores: number of cores (default: 1)
\item use_log_file: logging setting (default: TRUE)
\item append: append setting (default: FALSE)
\item verbose: verbose setting (default: TRUE)
\item log_file_path: log file path (default: NULL)
\item excludedFromQuantificationFilter: filter setting (default: NULL)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clean_features.R
\name{.summarizeMultipleMeasurements}
\alias{.summarizeMultipleMeasurements}
\title{Summarize multiple measurements per feature in a single run}
\usage{
.summarizeMultipleMeasurements(
  input,
  aggregator,
  feature_columns,
  anomaly_metrics = c()
)
}
\arguments{
\item{input}{\code{data.table} pre-processed by one of the .cleanRaw* functions.}

\item{aggregator}{function that will be used to aggregate duplicated values.}

\item{feature_columns}{chr, vector of names of columns that define features.}

\item{anomaly_metrics}{character vector of quality metric column names
to be used as features in an anomaly detection model.}
}
\value{
\code{data.table}
}
\description{
Summarize multiple measurements per feature in a single run
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{MSstatsClean}
\alias{MSstatsClean}
\alias{MSstatsClean,MSstatsDIAUmpireFiles-method}
\alias{MSstatsClean,MSstatsMaxQuantFiles-method}
\alias{MSstatsClean,MSstatsOpenMSFiles-method}
\alias{MSstatsClean,MSstatsOpenSWATHFiles-method}
\alias{MSstatsClean,MSstatsProgenesisFiles-method}
\alias{MSstatsClean,MSstatsProteomeDiscovererFiles-method}
\alias{MSstatsClean,MSstatsSkylineFiles-method}
\alias{MSstatsClean,MSstatsSpectroMineFiles-method}
\alias{MSstatsClean,MSstatsSpectronautFiles-method}
\alias{MSstatsClean,MSstatsPhilosopherFiles-method}
\alias{MSstatsClean,MSstatsDIANNFiles-method}
\alias{MSstatsClean,MSstatsMetamorpheusFiles-method}
\alias{MSstatsClean,MSstatsProteinProspectorFiles-method}
\title{Clean files generated by a signal processing tools.}
\usage{
MSstatsClean(msstats_object, ...)

\S4method{MSstatsClean}{MSstatsDIAUmpireFiles}(msstats_object, use_frag, use_pept)

\S4method{MSstatsClean}{MSstatsMaxQuantFiles}(
  msstats_object,
  protein_id_col,
  remove_by_site = FALSE,
  channel_columns = "Reporterintensitycorrected"
)

\S4method{MSstatsClean}{MSstatsOpenMSFiles}(msstats_object)

\S4method{MSstatsClean}{MSstatsOpenSWATHFiles}(msstats_object)

\S4method{MSstatsClean}{MSstatsProgenesisFiles}(msstats_object, runs, fix_colnames = TRUE)

\S4method{MSstatsClean}{MSstatsProteomeDiscovererFiles}(
  msstats_object,
  quantification_column,
  protein_id_column,
  sequence_column,
  remove_shared,
  remove_protein_groups = TRUE,
  intensity_columns_regexp = "Abundance"
)

\S4method{MSstatsClean}{MSstatsSkylineFiles}(msstats_object)

\S4method{MSstatsClean}{MSstatsSpectroMineFiles}(msstats_object)

\S4method{MSstatsClean}{MSstatsSpectronautFiles}(
  msstats_object,
  intensity,
  calculateAnomalyScores,
  anomalyModelFeatures,
  peptideSequenceColumn = "EG.ModifiedSequence",
  heavyLabels = NULL
)

\S4method{MSstatsClean}{MSstatsPhilosopherFiles}(
  msstats_object,
  protein_id_col,
  peptide_id_col,
  channels,
  remove_shared_peptides
)

\S4method{MSstatsClean}{MSstatsDIANNFiles}(
  msstats_object,
  MBR = TRUE,
  quantificationColumn = "FragmentQuantCorrected",
  global_qvalue_cutoff = 0.01,
  qvalue_cutoff = 0.01,
  pg_qvalue_cutoff = 0.01,
  calculateAnomalyScores = FALSE,
  anomalyModelFeatures = c(),
  labeledAminoAcids = NULL
)

\S4method{MSstatsClean}{MSstatsMetamorpheusFiles}(msstats_object, MBR = TRUE, qvalue_cutoff = 0.05)

\S4method{MSstatsClean}{MSstatsProteinProspectorFiles}(msstats_object)
}
\arguments{
\item{msstats_object}{object that inherits from \code{MSstatsInputFiles} class.}

\item{...}{additional parameter to specific cleaning functions.}

\item{use_frag}{TRUE will use the selected fragment for each peptide.
'Selected_fragments' column is required.}

\item{use_pept}{TRUE will use the selected fragment for each protein
'Selected_peptides' column is required.}

\item{protein_id_col}{character, name of a column with names of proteins.}

\item{remove_by_site}{logical, if TRUE, proteins only identified by site will
be removed.}

\item{channel_columns}{character, regular expression that identifies channel columns
in TMT data.}

\item{runs}{chr, vector of Run labels.}

\item{fix_colnames}{lgl, if TRUE, one of the rows will be used as colnames.}

\item{quantification_column}{chr, name of a column used for quantification.}

\item{protein_id_column}{chr, name of a column with protein IDs.}

\item{sequence_column}{chr, name of a column with peptide sequences.}

\item{remove_shared}{lgl, if TRUE, shared peptides will be removed.}

\item{remove_protein_groups}{if TRUE, proteins with numProteins > 1 will be removed.}

\item{intensity_columns_regexp}{regular expressions that defines intensity columns.
Defaults to "Abundance", which means that columns that contain the word "Abundance"
will be treated as corresponding to intensities for different channels.}

\item{intensity}{Intensity column to use. Accepts legacy enum values
\code{'PeakArea'} (default, uses F.PeakArea), \code{'NormalizedPeakArea'}
(uses F.NormalizedPeakArea).  Can also be any raw
Spectronaut column name passed as a string (e.g.
\code{"FG.MS1Quantity"}); the column name is standardized internally.
For protein turnover workflows the recommended default is
\code{"FG.MS1Quantity"}.}

\item{calculateAnomalyScores}{Default is FALSE. If TRUE, will run anomaly detection model and calculate anomaly scores for each feature. Used downstream to weigh measurements in differential analysis.}

\item{anomalyModelFeatures}{character vector of quality metric column names to be used as features in the anomaly detection model. List must not be empty if calculateAnomalyScores=TRUE.}

\item{peptideSequenceColumn}{Name of the Spectronaut column that contains the
peptide sequence.  Defaults to \code{"EG.ModifiedSequence"}. The value is
standardized internally (dots and spaces removed) before column lookup.}

\item{heavyLabels}{Character list identifying the heavy isotope labels as it
appears inside square brackets in the peptide sequence column, e.g.
\code{c("Lys6")} matches peptides containing \code{[Lys6]}.
\code{c("Lys6", "Arg10")} matches peptides containing either \code{[Lys6]} or \code{[Arg10]}.
Supports any novel label name reported by Spectronaut (e.g. \code{"Leu6"},
\code{"Phe10"}).  When provided, peptides are
classified as heavy (\code{IsotopeLabelType = "H"}), light
(\code{IsotopeLabelType = "L"}), or unlabeled
(\code{IsotopeLabelType = NA}) based on its labeled sequence.  When
\code{NULL} (default) all peptides receive \code{IsotopeLabelType = "L"}.
Useful for protein turnover experiments.}

\item{peptide_id_col}{character name of a column that identifies peptides}

\item{channels}{character vector of channel labels}

\item{remove_shared_peptides}{logical, if TRUE, shared peptides will be
removed based on the IsUnique column from Philosopher output}

\item{MBR}{True if analysis was done with match between runs}

\item{quantificationColumn}{Use 'FragmentQuantCorrected'(default) column for quantified intensities for DIANN 1.8.x.
Use 'FragmentQuantRaw' for quantified intensities for DIANN 1.9.x.
Use 'auto' for quantified intensities for DIANN 2.x where each fragment intensity is a separate column, e.g. Fr0Quantity.}

\item{global_qvalue_cutoff}{The qvalue cutoff for the Q.Value column, i.e.
the run-specific precursor q-value. Default is 0.01.}

\item{qvalue_cutoff}{If MBR is false, the qvalue cutoff for the Global.Q.Value
column, i.e. global precursor q-value.  If MBR is true, the qvalue cutoff for the
Lib.Q.Value column, i.e. the q-value for the library created after the first MBR pass.
Default is 0.01.}

\item{pg_qvalue_cutoff}{If MBR is false, the qvalue cutoff for the Global.PG.Q.Value
column, i.e. the global q-value for the protein group.  If MBR is true, the
qvalue cutoff for the Lib.PG.Q.Value column, i.e. the protein group q-value for
the library created after the first MBR pass. Default is 0.01.}

\item{labeledAminoAcids}{Character vector of single-letter amino acid codes
that carry the SILAC label in protein turnover experiments, e.g.
\code{c("K")} or \code{c("K", "R")}.  Supplying this vector opts in to
protein-turnover mode; the exact amino acids determine behaviour only in the
\code{ModifiedSequence}-parsing path described below.

\strong{Channel-based path} (DIA-NN 2.x exports that include a
\code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
\emph{and} the input contains a \code{Channel} column, \code{Channel} values
are mapped directly to \code{IsotopeLabelType} (\code{"H"} → \code{"H"},
\code{"L"} → \code{"L"}, anything else → \code{NA}).  The amino acid codes
in \code{labeledAminoAcids} are \strong{not} used to validate or filter
\code{ModifiedSequence} in this path.

\strong{ModifiedSequence-parsing path} (DIA-NN 1.x exports without a
\code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
and no \code{Channel} column is present, each \code{ModifiedSequence} is
inspected for SILAC suffixes of the form \code{(SILAC-<AA>-H)} or
\code{(SILAC-<AA>-L)}, where \code{<AA>} is one of the supplied amino acid
codes.  Matching sequences are classified as \code{"H"} or \code{"L"};
sequences carrying neither suffix receive \code{IsotopeLabelType = NA}.
The SILAC suffix is then stripped from \code{PeptideSequence}.

When \code{NULL} (default), protein-turnover mode is disabled and all
peptides receive \code{IsotopeLabelType = "Light"}.}
}
\value{
data.table

data.table

data.table

data.table

data.table

data.table

data.table

data.table

data.table

data.table

data.table

data.table

data.table
}
\description{
Clean files generated by a signal processing tools.

Clean DIAUmpire files

Clean MaxQuant files

Clean OpenMS files

Clean OpenSWATH files

Clean Progenesis files

Clean ProteomeDiscoverer files

Clean Skyline files

Clean SpectroMine files

Clean Spectronaut files

Clean Philosopher files

Clean DIA-NN files

Clean Metamorpheus files

Clean Protein Prospector files
}
\examples{
evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
                            package = "MSstatsConvert")
pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
                      package = "MSstatsConvert")
evidence = read.csv(evidence_path)
pg = read.csv(pg_path)
imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
                         "MSstats", "MaxQuant")
cleaned_data = MSstatsClean(imported, protein_id_col = "Proteins")
head(cleaned_data)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.formatLogMessage}
\alias{.formatLogMessage}
\title{Format a log line as "LEVEL \link{timestamp} message"}
\usage{
.formatLogMessage(level, ...)
}
\description{
Message parts are joined with paste0, never sprintf, so a literal percent
sign is written verbatim.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_dt_operations.R
\name{.standardizeColnames}
\alias{.standardizeColnames}
\title{Change column names to match read.table/read.csv/read.delim conventions}
\usage{
.standardizeColnames(col_names)
}
\arguments{
\item{col_names}{chr, vector of column names}
}
\value{
character vector
}
\description{
Change column names to match read.table/read.csv/read.delim conventions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.removeOverlappingFeatures}
\alias{.removeOverlappingFeatures}
\title{Replace intensities of overlapped fractions with NA, keeping only one fraction}
\usage{
.removeOverlappingFeatures(input)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}
}
\value{
data.table
}
\description{
Replace intensities of overlapped fractions with NA, keeping only one fraction
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_filtering.R
\name{.filterExact}
\alias{.filterExact}
\title{Filter out specified symbols.}
\usage{
.filterExact(
  input,
  col_name,
  filter_symbols,
  behavior,
  fill_value,
  filter,
  drop
)
}
\arguments{
\item{input}{data.table preprocessed by one of the .cleanRaw* functions.}

\item{col_name}{chr, name of the column that will be the base for filtering}

\item{filter_symbols}{character vector of symbols that will be removed}

\item{behavior}{chr, if "remove", values below/above the threshold will be
removed, if "replace", they will be set to \code{fill_value}.}

\item{fill_value}{if \code{behavior} = "replace", values below/above the threshold
will be replaced with \code{fill_value}. Defaults to \code{NA}.}

\item{filter}{lgl, if TRUE, decoy proteins will be removed from the data.}

\item{drop}{lgl, if TRUE, column that contains decoy proteins will be dropped.}
}
\value{
data.table
}
\description{
Filter out specified symbols.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{MSstatsPreprocess}
\alias{MSstatsPreprocess}
\title{Preprocess outputs from MS signal processing tools for analysis with MSstats}
\usage{
MSstatsPreprocess(
  input,
  annotation,
  feature_columns,
  remove_shared_peptides = TRUE,
  remove_single_feature_proteins = TRUE,
  feature_cleaning = list(remove_features_with_few_measurements = TRUE,
    summarize_multiple_psms = max),
  score_filtering = list(),
  exact_filtering = list(),
  pattern_filtering = list(),
  columns_to_fill = list(),
  aggregate_isotopic = FALSE,
  anomaly_metrics = c(),
  ...
)
}
\arguments{
\item{input}{data.table processed by the MSstatsClean function.}

\item{annotation}{annotation file generated by a signal processing tool.}

\item{feature_columns}{character vector of names of columns that
define spectral features.}

\item{remove_shared_peptides}{logical, if TRUE shared peptides will be removed.}

\item{remove_single_feature_proteins}{logical, if TRUE, proteins that only have
one feature will be removed.}

\item{feature_cleaning}{named list with maximum two (for \code{MSstats} converters)
or three (for \code{MSstatsTMT} converter) elements. If \code{handle_few_measurements} is
set to "remove", feature with less than three measurements will be removed
(otherwise it should be equal to "keep"). \code{summarize_multiple_psms} is a function
that will be used to aggregate multiple feature measurements in a run. It should
return a scalar and accept an \code{na.rm} parameter. For \code{MSstatsTMT} converters,
setting \code{remove_psms_with_any_missing} will remove features which have missing
values in a run from that run.}

\item{score_filtering}{a list of named lists that specify filtering options.
Details are provided in the vignette.}

\item{exact_filtering}{a list of named lists that specify filtering options.
Details are provided in the vignette.}

\item{pattern_filtering}{a list of named lists that specify filtering options.
Details are provided in the vignette.}

\item{columns_to_fill}{a named list of scalars. If provided, columns with
names defined by the names of this list and values corresponding to its elements
will be added to the output \code{data.frame}.}

\item{aggregate_isotopic}{logical. If \code{TRUE}, isotopic peaks will by summed.}

\item{anomaly_metrics}{character vector of names of columns with quality metrics. Default is missing and is not required if anomaly model not run.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.table
}
\description{
Preprocess outputs from MS signal processing tools for analysis with MSstats
}
\examples{
evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
                            package = "MSstatsConvert")
pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
                      package = "MSstatsConvert")
evidence = read.csv(evidence_path)
pg = read.csv(pg_path)
imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
                         "MSstats", "MaxQuant")
cleaned_data = MSstatsClean(imported, protein_id_col = "Proteins")
annot_path = system.file("tinytest/raw_data/MaxQuant/annotation.csv", 
                         package = "MSstatsConvert")
mq_annot = MSstatsMakeAnnotation(cleaned_data, read.csv(annot_path),
                                 Run = "Rawfile")
                               
# To filter M-peptides and oxidatin peptides 
m_filter = list(col_name = "PeptideSequence", pattern = "M", 
                filter = TRUE, drop_column = FALSE)
oxidation_filter = list(col_name = "Modifications", pattern = "Oxidation", 
                        filter = TRUE, drop_column = TRUE)
msstats_format = MSstatsPreprocess(
cleaned_data, mq_annot, 
feature_columns = c("PeptideSequence", "PrecursorCharge"),
columns_to_fill = list(FragmentIon = NA, ProductCharge = NA),
pattern_filtering = list(oxidation = oxidation_filter, m = m_filter)
)
# Output in the standard MSstats format
head(msstats_format)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_DIANNtoMSstatsFormat.R
\name{DIANNtoMSstatsFormat}
\alias{DIANNtoMSstatsFormat}
\title{Import Diann files}
\usage{
DIANNtoMSstatsFormat(
  input,
  annotation = NULL,
  global_qvalue_cutoff = 0.01,
  qvalue_cutoff = 0.01,
  pg_qvalue_cutoff = 0.01,
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeOxidationMpeptides = TRUE,
  removeProtein_with1Feature = TRUE,
  MBR = TRUE,
  labeledAminoAcids = NULL,
  quantificationColumn = "FragmentQuantCorrected",
  calculateAnomalyScores = FALSE,
  anomalyModelFeatures = c(),
  anomalyModelFeatureTemporal = c(),
  removeMissingFeatures = 0.5,
  anomalyModelFeatureCount = 100,
  runOrder = NULL,
  n_trees = 100,
  max_depth = "auto",
  numberOfCores = 1,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{name of MSstats input report from Diann, which includes fragment-level data.
Output fragment data with --export-quant flag in DIA-NN 2.0}

\item{annotation}{name of 'annotation.txt' data which includes Condition, BioReplicate, Run.}

\item{global_qvalue_cutoff}{The qvalue cutoff for the Q.Value column, i.e.
the run-specific precursor q-value. Default is 0.01.}

\item{qvalue_cutoff}{If MBR is false, the qvalue cutoff for the Global.Q.Value
column, i.e. global precursor q-value.  If MBR is true, the qvalue cutoff for the
Lib.Q.Value column, i.e. the q-value for the library created after the first MBR pass.
Default is 0.01.}

\item{pg_qvalue_cutoff}{If MBR is false, the qvalue cutoff for the Global.PG.Q.Value
column, i.e. the global q-value for the protein group.  If MBR is true, the
qvalue cutoff for the Lib.PG.Q.Value column, i.e. the protein group q-value for
the library created after the first MBR pass. Default is 0.01.}

\item{useUniquePeptide}{should unique peptides be removed}

\item{removeFewMeasurements}{should proteins with few measurements be removed}

\item{removeOxidationMpeptides}{should peptides with oxidation be removed}

\item{removeProtein_with1Feature}{should proteins with a single feature be removed}

\item{MBR}{True if analysis was done with match between runs}

\item{labeledAminoAcids}{Character vector of single-letter amino acid codes
that carry the SILAC label in protein turnover experiments, e.g.
\code{c("K")} or \code{c("K", "R")}.  Supplying this vector opts in to
protein-turnover mode; the exact amino acids determine behaviour only in the
\code{ModifiedSequence}-parsing path described below.

\strong{Channel-based path} (DIA-NN 2.x exports that include a
\code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
\emph{and} the input contains a \code{Channel} column, \code{Channel} values
are mapped directly to \code{IsotopeLabelType} (\code{"H"} → \code{"H"},
\code{"L"} → \code{"L"}, anything else → \code{NA}).  The amino acid codes
in \code{labeledAminoAcids} are \strong{not} used to validate or filter
\code{ModifiedSequence} in this path.

\strong{ModifiedSequence-parsing path} (DIA-NN 1.x exports without a
\code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
and no \code{Channel} column is present, each \code{ModifiedSequence} is
inspected for SILAC suffixes of the form \code{(SILAC-<AA>-H)} or
\code{(SILAC-<AA>-L)}, where \code{<AA>} is one of the supplied amino acid
codes.  Matching sequences are classified as \code{"H"} or \code{"L"};
sequences carrying neither suffix receive \code{IsotopeLabelType = NA}.
The SILAC suffix is then stripped from \code{PeptideSequence}.

When \code{NULL} (default), protein-turnover mode is disabled and all
peptides receive \code{IsotopeLabelType = "Light"}.}

\item{quantificationColumn}{Use 'FragmentQuantCorrected'(default) column for quantified intensities for DIANN 1.8.x.
Use 'FragmentQuantRaw' for quantified intensities for DIANN 1.9.x.
Use 'auto' for quantified intensities for DIANN 2.x where each fragment intensity is a separate column, e.g. Fr0Quantity.}

\item{calculateAnomalyScores}{Default is FALSE. If TRUE, will run anomaly detection model and calculate anomaly scores for each feature. Used downstream to weigh measurements in differential analysis.}

\item{anomalyModelFeatures}{character vector of quality metric column names to be used as features in the anomaly detection model. List must not be empty if calculateAnomalyScores=TRUE.}

\item{anomalyModelFeatureTemporal}{character vector of temporal direction corresponding to columns passed to anomalyModelFeatures. Values must be one of: \code{mean_decrease}, \code{mean_increase}, \code{dispersion_increase}, or NULL (to perform no temporal feature engineering). Default is empty vector. If calculateAnomalyScores=TRUE, vector must have as many values as anomalyModelFeatures (even if all NULL).}

\item{removeMissingFeatures}{Remove features with missing values in more than this fraction of runs. Default is 0.5. Only used if calculateAnomalyScores=TRUE.}

\item{anomalyModelFeatureCount}{Feature selection for anomaly model. Anomaly detection works on the precursor-level and can be much slower if all features used. We will by default filter to the top-100 highest intensity features. This can be adjusted as necessary. To turn feature-selection off, set this value to a high number (e.g. 10000). Only used if calculateAnomalyScores=TRUE.}

\item{runOrder}{Temporal order of MS runs. Should be a two column data.table with columns \code{Run} and \code{Order}, where \code{Run} matches the run name output by DIA-NN and \code{Order} is an integer. Used to engineer the temporal features defined in anomalyModelFeatureTemporal.}

\item{n_trees}{Number of trees to use in isolation forest when calculateAnomalyScores=TRUE. Default is 100.}

\item{max_depth}{Max tree depth to use in isolation forest when calculateAnomalyScores=TRUE. Default is "auto" which calculates depth as log2(N) where N is the number of runs. Otherwise must be an integer.}

\item{numberOfCores}{Number of cores for parallel processing anomaly detection model. When > 1, a logfile named 'MSstats_anomaly_model_progress.log' is created to track progress. Only works for Linux & Mac OS. Default is 1.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame in the MSstats required format.
}
\description{
Import Diann files
}
\examples{
input_file_path = system.file("tinytest/raw_data/DIANN/diann_input.tsv", 
                                package="MSstatsConvert")
annotation_file_path = system.file("tinytest/raw_data/DIANN/annotation.csv", 
                                package = "MSstatsConvert")
input = data.table::fread(input_file_path)
annot = data.table::fread(annotation_file_path)
output = DIANNtoMSstatsFormat(input, annotation = annot, MBR = FALSE, 
                                use_log_file = FALSE)
head(output)

# For DIANN 2.0, set quantificationColumn = 'auto'
input_file_path_2_0 = system.file("tinytest/raw_data/DIANN/diann_2.0.parquet", 
                                package="MSstatsConvert")
annotation_file_path_2_0 = system.file("tinytest/raw_data/DIANN/annotation_diann_2.0.csv", 
                                package = "MSstatsConvert")
input_2_0 = arrow::read_parquet(input_file_path_2_0)
annot_2_0 = data.table::fread(annotation_file_path_2_0)
output_2_0 = DIANNtoMSstatsFormat(input_2_0, annotation = annot_2_0, MBR = FALSE, 
                                use_log_file = FALSE, quantificationColumn = 'auto')
head(output_2_0)
}
\author{
Elijah Willie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_Metamorpheus.R
\name{.cleanRawMetamorpheus}
\alias{.cleanRawMetamorpheus}
\title{Clean raw Metamorpheus files}
\usage{
.cleanRawMetamorpheus(msstats_object, MBR = TRUE, qvalue_cutoff = 0.05)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsMetamorpheusFiles}.}

\item{MBR}{If TRUE, the function will include peaks detected by MBR}

\item{qvalue_cutoff}{The q-value cutoff for filtering peaks detected by MBR}
}
\value{
data.table
}
\description{
Clean raw Metamorpheus files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_balanced_design.R
\name{.getFullDesign}
\alias{.getFullDesign}
\title{Create a data.frame of each combination of values for given variables}
\usage{
.getFullDesign(input, group_col, feature_col, measurement_col, is_tmt)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}

\item{group_col}{name of column in \code{input}. Combination of values of
\code{feature_col} and \code{measurement_col} will be created within each unique value
of this column}

\item{is_tmt}{if TRUE, data will be treated as coming from TMT experiment.}

\item{`feature_col`}{name of the column that labels features}

\item{`measurement_col`}{name of a column with measurement labels - Runs in
label-free case, Channels in TMT case.}
}
\value{
data.table
}
\description{
Create a data.frame of each combination of values for given variables
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/converters_ProteinProspectortoMSstatsTMTFormat.R
\name{ProteinProspectortoMSstatsTMTFormat}
\alias{ProteinProspectortoMSstatsTMTFormat}
\title{Generate MSstatsTMT required input format from Protein Prospector output}
\usage{
ProteinProspectortoMSstatsTMTFormat(
  input,
  annotation,
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeProtein_with1Feature = FALSE,
  summaryforMultipleRows = sum,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL
)
}
\arguments{
\item{input}{Input txt peptide report file from Protein Prospector with
"Keep Replicates", "Mods in Peptide", and "Protein Mods" options selected.}

\item{annotation}{data frame which contains column Run, Fraction,
TechRepMixture, Mixture, Channel, BioReplicate, Condition.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements across runs.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have only 1 feature, which is the combination of peptide, precursor charge, fragment and charge. FALSE is default.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}
}
\value{
data.frame of class "MSstatsTMT"
}
\description{
Generate MSstatsTMT required input format from Protein Prospector output
}
\examples{
input = system.file("tinytest/raw_data/ProteinProspector/Prospector_TotalTMT.txt",
    package = "MSstatsConvert")
input = data.table::fread(input)
annot = system.file("tinytest/raw_data/ProteinProspector/Annotation.csv",
                                package = "MSstatsConvert")
annot = data.table::fread(annot)
output <- ProteinProspectortoMSstatsTMTFormat(input, annot)
head(output)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.nullAppender}
\alias{.nullAppender}
\title{Appender used not to write messages}
\usage{
.nullAppender(level, ...)
}
\arguments{
\item{level}{log level}

\item{...}{messages - ignored}
}
\value{
NULL invisibly
}
\description{
A convenience function written to save time on checking if messages should
be printed or logs should be written to a file.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.fileAppender}
\alias{.fileAppender}
\title{File appender: returns a function(level, ...) that writes one line to a file}
\usage{
.fileAppender(log_file_path, append = TRUE)
}
\description{
File appender: returns a function(level, ...) that writes one line to a file
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_balanced_design.R
\name{.makeBalancedDesign}
\alias{.makeBalancedDesign}
\title{Fill missing rows to create balanced design}
\usage{
.makeBalancedDesign(input, fill_missing, anomaly_metrics = c())
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}

\item{fill_missing}{if TRUE, missing Intensities values will be added to data}

\item{anomaly_metrics}{character vector of quality metric column names to be
used as features in an anomaly detection model.
and marked as NA}
}
\value{
data.table
}
\description{
Fill missing rows to create balanced design
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.makeExactFilterMessage}
\alias{.makeExactFilterMessage}
\title{Make a message about filtering based on fixed values}
\usage{
.makeExactFilterMessage(col_name, filter_symbols, behavior, fill_value)
}
\arguments{
\item{col_name}{chr, name of the column that will be the base for filtering}

\item{filter_symbols}{character vector of symbols that will be removed}

\item{behavior}{chr, if "remove", values below/above the threshold will be
removed, if "replace", they will be set to \code{fill_value}.}

\item{fill_value}{if \code{behavior} = "replace", values below/above the threshold
will be replaced with \code{fill_value}. Defaults to \code{NA}.}
}
\value{
character - message
}
\description{
Make a message about filtering based on fixed values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.logSuccess}
\alias{.logSuccess}
\title{Make a message about successful data cleaning/importing}
\usage{
.logSuccess(tool, event)
}
\arguments{
\item{tool}{name of a signal processing tool}
}
\value{
TRUE invisibly if logging was sucessful
}
\description{
Make a message about successful data cleaning/importing
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_MSstatsConvert.R
\docType{package}
\name{MSstatsConvert}
\alias{MSstatsConvert-package}
\alias{MSstatsConvert}
\title{MSstatsConvert: An R Package to Convert Data from Mass Spectrometry Signal Processing Tools to MSstats Format}
\description{
MSstatsConvert helps convert data from different types of mass spectrometry experiments and
signal processing tools to a format suitable for statistical analysis with the MSstats and MSstatsTMT packages.
}
\section{Main functions}{

\code{\link{MSstatsLogsSettings}} for logs management,
\code{\link{MSstatsImport}} for importing files created by signal processing tools,
\code{\link{MSstatsClean}} for re-formatting imported files into a consistent format,
\code{\link{MSstatsPreprocess}} for preprocessing cleaned files,
\code{\link{MSstatsBalancedDesign}} for handling fractions and creating balanced data.
}

\author{
\strong{Maintainer}: Anthony Wu \email{wu.anthon@northeastern.edu}

Authors:
\itemize{
  \item Mateusz Staniak \email{mtst@mstaniak.pl}
  \item Devon Kohler \email{kohler.d@northeastern.edu}
  \item Meena Choi \email{mnchoi67@gmail.com}
  \item Ting Huang \email{thuang0703@gmail.com}
  \item Olga Vitek \email{o.vitek@northeastern.edu}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.countCommonFeatures}
\alias{.countCommonFeatures}
\title{Get common values from two vectors of features}
\usage{
.countCommonFeatures(features_1, features_2)
}
\arguments{
\item{features_1}{vector of feature names}

\item{features_2}{vector of feature_names}
}
\value{
character vector of common values of \code{features_1} and \code{features_2}
}
\description{
Get common values from two vectors of features
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_MaxQtoMSstatsTMTFormat.R
\name{MaxQtoMSstatsTMTFormat}
\alias{MaxQtoMSstatsTMTFormat}
\title{Generate MSstatsTMT required input format from MaxQuant output}
\usage{
MaxQtoMSstatsTMTFormat(
  evidence,
  proteinGroups,
  annotation,
  which.proteinid = "Proteins",
  rmProt_Only.identified.by.site = FALSE,
  useUniquePeptide = TRUE,
  rmPSM_withfewMea_withinRun = TRUE,
  rmProtein_with1Feature = FALSE,
  summaryforMultipleRows = sum,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{evidence}{name of 'evidence.txt' data, which includes feature-level data.}

\item{proteinGroups}{name of 'proteinGroups.txt' data.}

\item{annotation}{data frame which contains column Run, Fraction, TechRepMixture, Mixture, Channel, BioReplicate, Condition. Refer to the example 'annotation.mq' for the meaning of each column.}

\item{which.proteinid}{Use 'Proteins' (default) column for protein name. 'Leading.proteins' or 'Leading.razor.proteins' or 'Gene.names' can be used instead to get the protein ID with single protein. However, those can potentially have the shared peptides.}

\item{rmProt_Only.identified.by.site}{TRUE will remove proteins with '+' in 'Only.identified.by.site' column from proteinGroups.txt, which was identified only by a modification site. FALSE is the default.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{rmPSM_withfewMea_withinRun}{TRUE (default) will remove the features that have 1 or 2 measurements within each Run.}

\item{rmProtein_with1Feature}{TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame of class "MSstatsTMT"
}
\description{
Generate MSstatsTMT required input format from MaxQuant output
}
\examples{
evidence = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_ev.csv",
                                      package = "MSstatsConvert"))
proteinGroups = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_pg.csv",
                                      package = "MSstatsConvert"))
annotation.mq = data.table::fread(system.file("tinytest/raw_data/MaxQuantTMT/mq_annotation.csv",
                                      package = "MSstatsConvert"))
input.mq <- MaxQtoMSstatsTMTFormat(evidence, proteinGroups, annotation.mq)
head(input.mq)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_dt_operations.R
\name{.selectMSstatsColumns}
\alias{.selectMSstatsColumns}
\title{Select columns for MSstats format}
\usage{
.selectMSstatsColumns(input, anomaly_metrics)
}
\arguments{
\item{input}{data.table}
}
\value{
data.table
}
\description{
Select columns for MSstats format
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.handleFractionsTMT}
\alias{.handleFractionsTMT}
\title{Remove peptide ions overlapped among multiple fractions of the same biological mixture}
\usage{
.handleFractionsTMT(input)
}
\arguments{
\item{input}{data.table preprocessed by one of the .cleanRaw* functions and
merged with annotation.}
}
\value{
\code{data.table}
}
\description{
Remove peptide ions overlapped among multiple fractions of the same biological mixture
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{MSstatsLogsSettings}
\alias{MSstatsLogsSettings}
\title{Set how MSstats will log information from data processing}
\usage{
MSstatsLogsSettings(
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  base = "MSstats_log_",
  pkg_name = "MSstats"
)
}
\arguments{
\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing wil be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{base}{start of the file name.}

\item{pkg_name}{currently "MSstats", "MSstatsPTM" or "MSstatsTMT".
Each package can use its own separate log settings.}
}
\value{
TRUE invisibly in case of successful logging setup.
}
\description{
Set how MSstats will log information from data processing
}
\examples{
# No logging and no messages
MSstatsLogsSettings(FALSE, FALSE, FALSE)
# Log, but do not display messages
MSstatsLogsSettings(TRUE, FALSE, FALSE)
# Log to an existing file
file.create("new_log.log")
MSstatsLogsSettings(TRUE, TRUE, log_file_path = "new_log.log")
# Do not log, but display messages
MSstatsLogsSettings(FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{getDataType}
\alias{getDataType}
\alias{getDataType,MSstatsInputFiles-method}
\title{Get type of dataset from an MSstatsInputFiles object.}
\usage{
getDataType(msstats_object)

\S4method{getDataType}{MSstatsInputFiles}(msstats_object)
}
\arguments{
\item{msstats_object}{object that inherits from \code{MSstatsInputFiles} class.}
}
\value{
character - label of a data type. Currently, "MSstats" or "MSstatsTMT"

character "MSstats" or "MSstatsTMT".
}
\description{
Get type of dataset from an MSstatsInputFiles object.
}
\examples{
evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
                            package = "MSstatsConvert")
pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
                      package = "MSstatsConvert")
evidence = read.csv(evidence_path)
pg = read.csv(pg_path)
imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
                         "MSstats", "MaxQuant")
class(imported)
getDataType(imported) # "MSstats"

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_PhilosophertoMSstatsTMTFormat.R
\name{PhilosophertoMSstatsTMTFormat}
\alias{PhilosophertoMSstatsTMTFormat}
\title{Convert Philosopher (Fragpipe) output to MSstatsTMT format.}
\usage{
PhilosophertoMSstatsTMTFormat(
  input,
  annotation,
  protein_id_col = "Protein",
  peptide_id_col = "Peptide.Sequence",
  Purity_cutoff = 0.6,
  PeptideProphet_prob_cutoff = 0.7,
  useUniquePeptide = TRUE,
  rmPSM_withfewMea_withinRun = TRUE,
  rmPeptide_OxidationM = TRUE,
  rmProtein_with1Feature = FALSE,
  summaryforMultipleRows = sum,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{data.frame of \code{msstats.csv} file produced by Philosopher}

\item{annotation}{annotation with Run, Fraction, TechRepMixture, Mixture, Channel,
BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column. Channel column should be
consistent with the channel columns (Ignore the prefix "Channel ") in msstats.csv file. Run column should be consistent with the Spectrum.File columns in msstats.csv file.}

\item{protein_id_col}{Use 'Protein'(default) column for protein name.
'Master.Protein.Accessions' can be used instead to get the protein ID with single protein.}

\item{peptide_id_col}{Use 'Peptide.Sequence'(default) column for peptide sequence.
'Modified.Peptide.Sequence' can be used instead to get the modified peptide sequence.}

\item{Purity_cutoff}{Cutoff for purity. Default is 0.6}

\item{PeptideProphet_prob_cutoff}{Cutoff for the peptide identification probability. Default is 0.7.
The probability is confidence score determined by PeptideProphet and higher values indicate greater confidence.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{rmPSM_withfewMea_withinRun}{TRUE (default) will remove the features that have 1 or 2 measurements within each Run.}

\item{rmPeptide_OxidationM}{TRUE (default) will remove the peptides including oxidation (M) sequence.}

\item{rmProtein_with1Feature}{TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
\code{data.frame} of class \code{MSstatsTMT}
}
\description{
Convert Philosopher (Fragpipe) output to MSstatsTMT format.
}
\examples{
input_file_path = system.file("tinytest/raw_data/Philosopher/msstats.csv", 
                     package = "MSstatsConvert")
annotation_file_path = system.file("tinytest/raw_data/Philosopher/MSstatsTMT_annotation.csv", 
                    package = "MSstatsConvert")
input = data.table::fread(input_file_path)
annotation = data.table::fread(annotation_file_path)
msstats_format = PhilosophertoMSstatsTMTFormat(input, annotation)
head(msstats_format)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{MSstatsBalancedDesign}
\alias{MSstatsBalancedDesign}
\title{Creates balanced design by removing overlapping fractions and filling incomplete rows}
\usage{
MSstatsBalancedDesign(
  input,
  feature_columns,
  fill_incomplete = TRUE,
  handle_fractions = TRUE,
  fix_missing = NULL,
  remove_few = TRUE,
  anomaly_metrics = c()
)
}
\arguments{
\item{input}{\code{data.table} processed by the \code{MSstatsPreprocess} function}

\item{feature_columns}{str, names of columns that define spectral features}

\item{fill_incomplete}{if TRUE (default), ensures that rows with missing data
for specific features are added as NA. For example, if the y10 ion of
peptideA is measured in the "disease" samples but entirely missing for the
"healthy" samples, rows with NA values will be created for the y10 ion of
peptideA in the "healthy" group. This process increases the number of
rows to account for all possible feature-sample combinations.}

\item{handle_fractions}{if TRUE (default), overlapping fractions will be resolved}

\item{fix_missing}{str, optional. Defaults to NULL, which means no action.
If not NULL, must be one of the options: "zero_to_na" or "na_to_zero".
If "zero_to_na", Intensity values equal exactly to 0 will be converted to NA.
If "na_to_zero", missing values will be replaced by zeros.}

\item{remove_few}{lgl, if TRUE, features with one or two measurements
across runs will be removed.}

\item{anomaly_metrics}{character vector of names of columns with quality metrics}
}
\value{
data.frame of class \code{MSstatsValidated}
}
\description{
Creates balanced design by removing overlapping fractions and filling incomplete rows
}
\examples{
unbalanced_data = system.file("tinytest/raw_data/unbalanced_data.csv", 
                              package = "MSstatsConvert")
unbalanced_data = data.table::as.data.table(read.csv(unbalanced_data))
balanced = MSstatsBalancedDesign(unbalanced_data, 
                                 c("PeptideSequence", "PrecursorCharge",
                                   "FragmentIon", "ProductCharge"))
dim(balanced) # Now balanced has additional rows (with Intensity = NA)
# for runs that were not included in the unbalanced_data table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_ProgenesistoMSstatsFormat.R
\name{ProgenesistoMSstatsFormat}
\alias{ProgenesistoMSstatsFormat}
\title{Import Progenesis files}
\usage{
ProgenesistoMSstatsFormat(
  input,
  annotation,
  useUniquePeptide = TRUE,
  summaryforMultipleRows = max,
  removeFewMeasurements = TRUE,
  removeOxidationMpeptides = FALSE,
  removeProtein_with1Peptide = FALSE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{name of Progenesis output, which is wide-format. 'Accession', 'Sequence', 'Modification', 'Charge' and one column for each run are required.}

\item{annotation}{name of 'annotation.txt' or 'annotation.csv' data which includes Condition, BioReplicate, Run information. It will be matched with the column name of input for MS runs.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements across runs.}

\item{removeOxidationMpeptides}{TRUE will remove the peptides including 'oxidation (M)' in modification. FALSE is default.}

\item{removeProtein_with1Peptide}{TRUE will remove the proteins which have only 1 peptide and charge. FALSE is default.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame in the MSstats required format.
}
\description{
Import Progenesis files
}
\examples{
progenesis_raw = system.file("tinytest/raw_data/Progenesis/progenesis_input.csv", 
                             package = "MSstatsConvert")
annot = system.file("tinytest/raw_data/Progenesis/progenesis_annot.csv", 
                    package = "MSstatsConvert")
progenesis_raw = data.table::fread(progenesis_raw) 
annot = data.table::fread(annot)

progenesis_imported = ProgenesistoMSstatsFormat(progenesis_raw, annot,
                                                use_log_file = FALSE)
head(progenesis_imported)

}
\author{
Meena Choi, Olga Vitek, Ulrich Omasits
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_PDtoMSstatsTMTFormat.R
\name{PDtoMSstatsTMTFormat}
\alias{PDtoMSstatsTMTFormat}
\title{Convert Proteome Discoverer output to MSstatsTMT format.}
\usage{
PDtoMSstatsTMTFormat(
  input,
  annotation,
  which.proteinid = "Protein.Accessions",
  useNumProteinsColumn = TRUE,
  useUniquePeptide = TRUE,
  rmPSM_withfewMea_withinRun = TRUE,
  rmProtein_with1Feature = FALSE,
  summaryforMultipleRows = sum,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{PD report or a path to it.}

\item{annotation}{annotation with Run, Fraction, TechRepMixture, Mixture, Channel,
BioReplicate, Condition columns or a path to file. Refer to the example 'annotation' for the meaning of each column.}

\item{which.proteinid}{Use 'Protein.Accessions'(default) column for protein name. 'Master.Protein.Accessions' can be used instead to get the protein name with single protein.}

\item{useNumProteinsColumn}{logical, TRUE (default) removes shared peptides by information of # Proteins column in PSM sheet.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{rmPSM_withfewMea_withinRun}{TRUE (default) will remove the features that have 1 or 2 measurements within each Run.}

\item{rmProtein_with1Feature}{TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
\code{data.frame} of class \code{MSstatsTMT}
}
\description{
Convert Proteome Discoverer output to MSstatsTMT format.
}
\examples{
raw.pd = data.table::fread(system.file("tinytest/raw_data/PDTMT/pdtmt_input.csv",
                                      package = "MSstatsConvert"))
annotation.pd = data.table::fread(system.file("tinytest/raw_data/PDTMT/pd_annotation.csv",
                                      package = "MSstatsConvert"))
head(raw.pd)
head(annotation.pd)
input.pd <- PDtoMSstatsTMTFormat(raw.pd, annotation.pd)
head(input.pd)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.makeScoreFilterMessage}
\alias{.makeScoreFilterMessage}
\title{Make a message about filtering based on a score}
\usage{
.makeScoreFilterMessage(
  score_column,
  score_threshold,
  direction,
  behavior,
  fill_value
)
}
\arguments{
\item{score_column}{chr, name of the column that contains scores.}

\item{score_threshold}{num, values below or above this threshold will be
removed from the data.}

\item{direction}{chr, if "greater" only values above the threshold will be
retained, if "smaller" - below the threshold.}

\item{behavior}{chr, if "remove", values below/above the threshold will be
removed, if "replace", they will be set to \code{fill_value}.}

\item{fill_value}{if \code{behavior} = "replace", values below/above the threshold
will be replaced with \code{fill_value}. Defaults to \code{NA}.}
}
\value{
character - message
}
\description{
Make a message about filtering based on a score
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_OpenSWATH.R
\name{.cleanRawOpenSWATH}
\alias{.cleanRawOpenSWATH}
\title{Clean raw OpenSWATH files}
\usage{
.cleanRawOpenSWATH(msstats_object)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectroMineFiles}.}
}
\value{
data.table
}
\description{
Clean raw OpenSWATH files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_DIAUmpiretoMSstatsFormat.R
\name{DIAUmpiretoMSstatsFormat}
\alias{DIAUmpiretoMSstatsFormat}
\title{Import DIA-Umpire files}
\usage{
DIAUmpiretoMSstatsFormat(
  raw.frag,
  raw.pep,
  raw.pro,
  annotation,
  useSelectedFrag = TRUE,
  useSelectedPep = TRUE,
  removeFewMeasurements = TRUE,
  removeProtein_with1Feature = FALSE,
  summaryforMultipleRows = max,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{raw.frag}{name of FragSummary_date.xls data, which includes feature-level data.}

\item{raw.pep}{name of PeptideSummary_date.xls data, which includes selected fragments information.}

\item{raw.pro}{name of ProteinSummary_date.xls data, which includes selected peptides information.}

\item{annotation}{name of annotation data which includes Condition, BioReplicate, Run information.}

\item{useSelectedFrag}{TRUE will use the selected fragment for each peptide. 'Selected_fragments' column is required.}

\item{useSelectedPep}{TRUE will use the selected peptide for each protein. 'Selected_peptides' column is required.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements across runs.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have only 1 feature, which is the combination of peptide, precursor charge, fragment and charge. FALSE is default.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame in the MSstats required format.
}
\description{
Import DIA-Umpire files
}
\examples{
diau_frag = system.file("tinytest/raw_data/DIAUmpire/dia_frag.csv", 
                             package = "MSstatsConvert")
diau_pept = system.file("tinytest/raw_data/DIAUmpire/dia_pept.csv", 
                             package = "MSstatsConvert")
diau_prot = system.file("tinytest/raw_data/DIAUmpire/dia_prot.csv", 
                             package = "MSstatsConvert")
annot = system.file("tinytest/raw_data/DIAUmpire/annot_diau.csv", 
                    package = "MSstatsConvert")
diau_frag = data.table::fread(diau_frag) 
diau_pept = data.table::fread(diau_pept) 
diau_prot = data.table::fread(diau_prot) 
annot = data.table::fread(annot)
diau_frag = diau_frag[, lapply(.SD, function(x) if (is.integer(x)) as.numeric(x) else x)]
# In case numeric columns are not interpreted correctly

diau_imported = DIAUmpiretoMSstatsFormat(diau_frag, diau_pept, diau_prot, 
                                         annot, use_log_file = FALSE)
head(diau_imported)

}
\author{
Meena Choi, Olga Vitek
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.handleFractionsLF}
\alias{.handleFractionsLF}
\title{Handle overlapping features}
\usage{
.handleFractionsLF(input)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}
}
\value{
data.table
}
\description{
Handle overlapping features
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_SpectroMine.R
\name{.cleanRawSpectroMineTMT}
\alias{.cleanRawSpectroMineTMT}
\title{Clean raw SpectroMine TMT data}
\usage{
.cleanRawSpectroMineTMT(msstats_object)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectroMineFiles}.}
}
\value{
\code{data.table}
}
\description{
Clean raw SpectroMine TMT data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\docType{class}
\name{MSstatsInputFiles-class}
\alias{MSstatsInputFiles-class}
\alias{MSstatsDIAUmpireFiles-class}
\alias{MSstatsMaxQuantFiles-class}
\alias{MSstatsOpenMSFiles-class}
\alias{MSstatsOpenSWATHFiles-class}
\alias{MSstatsProgenesisFiles-class}
\alias{MSstatsProteomeDiscovererFiles-class}
\alias{MSstatsSkylineFiles-class}
\alias{MSstatsSpectroMineFiles-class}
\alias{MSstatsSpectronautFiles-class}
\alias{MSstatsPhilosopherFiles-class}
\alias{MSstatsDIANNFiles-class}
\alias{MSstatsFragPipeFiles-class}
\alias{MSstatsMetamorpheusFiles-class}
\alias{MSstatsProteinProspectorFiles-class}
\title{Class to model files that describe a single MS dataset.}
\description{
Class to model files that describe a single MS dataset.

MSstatsDIAUmpireFiles: class for DIAUmpire files.

MSstatsMaxQuantFiles: class for MaxQuant files.

MSstatsOpenMSFiles: class for OpenMS files.

MSstatsOpenSWATHFiles: class for OpenSWATH files.

MSstatsProgenesisFiles: class for Progenesis files.

MSstatsProteomeDiscovererFiles: class for ProteomeDiscoverer files.

MSstatsSkylineFiles: class for Skyline files.

MSstatsSkylineFiles: class for SpectroMine files.

MSstatsSpectronautFiles: class for Spectronaut files.

MSstatsPhilosopherFiles: class for Philosopher files.

MSstatsDIANNFiles: class for DIA-NN files.

MSstatsFragPipeFiles: class for FragPipe files.

MSstatsMetamorpheusFiles: class for Metamorpheus files.

MSstatsProteinProspectorFiles: class for ProteinProspector files.
}
\section{Slots}{

\describe{
\item{\code{files}}{named list of files generated by a signal processing tools.
In most cases, this will be a single file named \code{input}.
In some cases, multiple files are used, for example \code{MaxQuant} outputs
\code{evidence} and \code{proteinGroups} files.}

\item{\code{type}}{character: "MSstats" or "MSstatsTMT".}

\item{\code{tool}}{character: name of a signal processing tools that generated the
output. Possible values are: DIAUmpire, MaxQuant, OpenMS, OpenSWATH,
Progenesis, ProteomeDiscoverer, Skyline, SpectroMine, Spectronaut.}

\item{\code{version}}{description of a software version of the signal processing tool.
Not implemented yet.}
}}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_shared_peptides.R
\name{.handleSharedPeptides}
\alias{.handleSharedPeptides}
\title{Handle shared peptides.}
\usage{
.handleSharedPeptides(
  input,
  remove_shared = TRUE,
  protein_column = "ProteinName",
  peptide_column = "PeptideSequence"
)
}
\arguments{
\item{input}{data.table pre-processed by one of the .cleanRaw* functions.}

\item{remove_shared}{lgl, if TRUE, shared peptides will be removed}

\item{protein_column}{chr, name of the column with names of proteins.}

\item{peptide_column}{chr, name of the column with peptide sequences.}
}
\value{
data.table
}
\description{
Handle shared peptides.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_dt_operations.R
\name{.fillValues}
\alias{.fillValues}
\title{Set column to a single value}
\usage{
.fillValues(input, fill_list)
}
\arguments{
\item{input}{data.table preprocessed by one of the \verb{cleanRaw*} functions.}

\item{fill_list}{named list, names correspond to column names, elements
to values that will be used in the columns.}
}
\value{
data.table
}
\description{
Set column to a single value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clean_features.R
\name{.handleSingleFeaturePerProtein}
\alias{.handleSingleFeaturePerProtein}
\title{Remove proteins only identified by a single feature}
\usage{
.handleSingleFeaturePerProtein(input, remove_single_feature)
}
\arguments{
\item{input}{\code{data.table} pre-processed by one of the .cleanRaw* functions.}

\item{remove_single_feature}{lgl, if TRUE, proteins with a single feature
will be removed.}
}
\value{
\code{data.table}
}
\description{
Remove proteins only identified by a single feature
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_checks.R
\name{.checkMSstatsParams}
\alias{.checkMSstatsParams}
\title{Check validity of parameters to the \code{MSstatsImport} function.}
\usage{
.checkMSstatsParams(
  input,
  annotation,
  feature_columns,
  remove_shared_peptides,
  remove_single_feature_proteins,
  feature_cleaning
)
}
\value{
none, throws an error if any of the assertions fail
}
\description{
Check validity of parameters to the \code{MSstatsImport} function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_MaxQtoMSstatsFormat.R
\name{MaxQtoMSstatsFormat}
\alias{MaxQtoMSstatsFormat}
\title{Import MaxQuant files}
\usage{
MaxQtoMSstatsFormat(
  evidence,
  annotation,
  proteinGroups,
  proteinID = "Proteins",
  useUniquePeptide = TRUE,
  summaryforMultipleRows = max,
  removeFewMeasurements = TRUE,
  removeMpeptides = FALSE,
  removeOxidationMpeptides = FALSE,
  removeProtein_with1Peptide = FALSE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{evidence}{name of 'evidence.txt' data, which includes feature-level data.}

\item{annotation}{name of 'annotation.txt' data which includes Raw.file, Condition, BioReplicate, Run, IsotopeLabelType information.}

\item{proteinGroups}{name of 'proteinGroups.txt' data. It needs to matching protein group ID. If proteinGroups=NULL, use 'Proteins' column in 'evidence.txt'.}

\item{proteinID}{'Proteins'(default) or 'Leading.razor.protein' for Protein ID.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements across runs.}

\item{removeMpeptides}{TRUE will remove the peptides including 'M' sequence. FALSE is default.}

\item{removeOxidationMpeptides}{TRUE will remove the peptides including 'oxidation (M)' in modification. FALSE is default.}

\item{removeProtein_with1Peptide}{TRUE will remove the proteins which have only 1 peptide and charge. FALSE is default.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame in the MSstats required format.
}
\description{
Import MaxQuant files
}
\note{
Warning: MSstats does not support for metabolic labeling or iTRAQ experiments.
}
\examples{
mq_ev = data.table::fread(system.file("tinytest/raw_data/MaxQuant/mq_ev.csv",
                                      package = "MSstatsConvert"))
mq_pg = data.table::fread(system.file("tinytest/raw_data/MaxQuant/mq_pg.csv",
                                      package = "MSstatsConvert"))
annot = data.table::fread(system.file("tinytest/raw_data/MaxQuant/annotation.csv",
                                      package = "MSstatsConvert"))
maxq_imported = MaxQtoMSstatsFormat(mq_ev, annot, mq_pg, use_log_file = FALSE)
head(maxq_imported)

}
\author{
Meena Choi, Olga Vitek.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_Skyline.R
\name{.cleanRawSkyline}
\alias{.cleanRawSkyline}
\title{Clean raw data from Skyline}
\usage{
.cleanRawSkyline(msstats_object)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectroMineFiles}.}
}
\value{
data.table
}
\description{
Clean raw data from Skyline
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_Philosopher.R
\name{.cleanRawPhilosopher}
\alias{.cleanRawPhilosopher}
\title{Clean raw Philosopher files}
\usage{
.cleanRawPhilosopher(
  msstats_object,
  protein_id_col,
  peptide_id_col,
  channels,
  remove_shared_peptides
)
}
\arguments{
\item{msstats_object}{object of class MSstatsPhilosopherFiles}

\item{protein_id_col}{character name of a column that identifies proteins}

\item{peptide_id_col}{character name of a column that identifies peptides}

\item{channels}{character vector of channel labels}

\item{remove_shared_peptides}{logical, if TRUE, shared peptides will be
removed based on the IsUnique column from Philosopher output}
}
\value{
data.table
}
\description{
Clean raw Philosopher files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.logConverterOptions}
\alias{.logConverterOptions}
\title{Log information about converter options}
\usage{
.logConverterOptions(
  feature_columns,
  remove_shared_peptides,
  remove_single_feature_proteins,
  feature_cleaning,
  is_tmt = FALSE
)
}
\arguments{
\item{feature_columns}{character vector of names of columns that
define spectral features.}

\item{remove_shared_peptides}{logical, if TRUE shared peptides will be removed.}

\item{remove_single_feature_proteins}{logical, if TRUE, proteins that only have
one feature will be removed.}

\item{feature_cleaning}{named list with maximum two (for \code{MSstats} converters)
or three (for \code{MSstatsTMT} converter) elements. If \code{handle_few_measurements} is
set to "remove", feature with less than three measurements will be removed
(otherwise it should be equal to "keep"). \code{summarize_multiple_psms} is a function
that will be used to aggregate multiple feature measurements in a run. It should
return a scalar and accept an \code{na.rm} parameter. For \code{MSstatsTMT} converters,
setting \code{remove_psms_with_any_missing} will remove features which have missing
values in a run from that run.}

\item{is_tmt}{If TRUE, the dataset comes from a TMT experiment}
}
\value{
TRUE invisibly if message was logged
}
\description{
Log information about converter options
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_filtering.R
\name{.handleFiltering}
\alias{.handleFiltering}
\title{Handle PSM/proteins scores}
\usage{
.handleFiltering(input, score_filtering, exact_filtering, pattern_filtering)
}
\arguments{
\item{input}{\code{data.table} preprocessed by one of the .cleanRaw* functions.}

\item{score_filtering}{list of by-score filtering controls.}

\item{exact_filtering}{list of exact filtering controls.}

\item{pattern_filtering}{list of by-pattern filtering controls.}
}
\value{
data.table
}
\description{
Handle PSM/proteins scores
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_annotation.R
\name{.mergeAnnotation}
\alias{.mergeAnnotation}
\title{Merge annotation with feature data}
\usage{
.mergeAnnotation(input, annotation)
}
\arguments{
\item{annotation}{data.table with annotation}

\item{data.table}{preprocessed by one of the .cleanRaw functions.}
}
\value{
data.table
}
\description{
Merge annotation with feature data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_SkylinetoMSstatsFormat.R
\name{SkylinetoMSstatsFormat}
\alias{SkylinetoMSstatsFormat}
\title{Import Skyline files}
\usage{
SkylinetoMSstatsFormat(
  input,
  annotation = NULL,
  removeiRT = TRUE,
  filter_with_Qvalue = TRUE,
  qvalue_cutoff = 0.01,
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeOxidationMpeptides = FALSE,
  removeProtein_with1Feature = FALSE,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{name of MSstats input report from Skyline, which includes feature-level data.}

\item{annotation}{name of 'annotation.txt' data which includes Condition, BioReplicate, Run. If annotation is already complete in Skyline, use annotation=NULL (default). It will use the annotation information from input.}

\item{removeiRT}{TRUE (default) will remove the proteins or peptides which are labeled 'iRT' in 'StandardType' column. FALSE will keep them.}

\item{filter_with_Qvalue}{TRUE(default) will filter out the intensities that have greater than qvalue_cutoff in DetectionQValue column. Those intensities will be replaced with zero and will be considered as censored missing values for imputation purpose.}

\item{qvalue_cutoff}{Cutoff for DetectionQValue. default is 0.01.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements across runs.}

\item{removeOxidationMpeptides}{TRUE will remove the peptides including 'oxidation (M)' in modification. FALSE is default.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have only 1 feature, which is the combination of peptide, precursor charge, fragment and charge. FALSE is default.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame in the MSstats required format.
}
\description{
Import Skyline files
}
\examples{
skyline_raw = system.file("tinytest/raw_data/Skyline/skyline_input.csv",
                          package = "MSstatsConvert")
skyline_raw = data.table::fread(skyline_raw)
skyline_imported = SkylinetoMSstatsFormat(skyline_raw)
head(skyline_imported)

}
\author{
Meena Choi, Olga Vitek
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{MSstatsImport}
\alias{MSstatsImport}
\title{Import files from signal processing tools.}
\usage{
MSstatsImport(input_files, type, tool, tool_version = NULL, ...)
}
\arguments{
\item{input_files}{list of paths to input files or \code{data.frame} objects.
Interpretation of this parameter depends on values of parameters \code{type} and \code{tool}.}

\item{type}{chr, "MSstats" or "MSstatsTMT".}

\item{tool}{chr, name of a signal processing tool that generated input files.}

\item{tool_version}{not implemented yet. In the future, this parameter will allow
handling different versions of each signal processing tools.}

\item{...}{optional additional parameters to \code{data.table::fread}.}
}
\value{
an object of class \code{MSstatsInputFiles}.
}
\description{
Import files from signal processing tools.
}
\examples{
evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
                            package = "MSstatsConvert")
pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
                      package = "MSstatsConvert")
evidence = read.csv(evidence_path)
pg = read.csv(pg_path)
imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
                         "MSstats", "MaxQuant")
class(imported)
head(getInputFile(imported, "evidence"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_balanced_design.R
\name{.fixMissingValues}
\alias{.fixMissingValues}
\title{Change labels for missing values}
\usage{
.fixMissingValues(input, fix_missing = NULL)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}

\item{fix_missing}{missing values can be labeled by \code{NA}, \code{0} or both.
If \code{NULL}, data were processed by Skyline, so missing values will be denoted
by both \code{NA} and \code{0}. If "na_to_zero", \code{NA} values will be replaced by \code{0}.
If "zero_to_na", \code{0} values will be replaced by \code{NA}}
}
\value{
data.table
}
\description{
Change labels for missing values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_ProteomeDiscoverer.R
\name{.cleanRawPDMSstats}
\alias{.cleanRawPDMSstats}
\title{Clean raw PD output}
\usage{
.cleanRawPDMSstats(
  msstats_object,
  quantification_column,
  protein_id_column,
  sequence_column,
  remove_shared,
  run_column = "SpectrumFile"
)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectroMineFiles}.}

\item{quantification_column}{chr, name of a column used for quantification.}

\item{protein_id_column}{chr, name of a column with protein IDs.}

\item{sequence_column}{chr, name of a column with peptide sequences.}

\item{remove_shared}{lgl, if TRUE, shared peptides will be removed.}
}
\value{
data.table
}
\description{
Clean raw PD output
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{MSstatsMakeAnnotation}
\alias{MSstatsMakeAnnotation}
\title{Create annotation}
\usage{
MSstatsMakeAnnotation(input, annotation, ...)
}
\arguments{
\item{input}{data.table preprocessed by the MSstatsClean function}

\item{annotation}{data.table}

\item{...}{key-value pairs, where keys are names of columns of \code{annotation}}
}
\value{
data.table
}
\description{
Create annotation
}
\examples{
evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
                            package = "MSstatsConvert")
pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
                      package = "MSstatsConvert")
evidence = read.csv(evidence_path)
pg = read.csv(pg_path)
imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
                         "MSstats", "MaxQuant")
cleaned_data = MSstatsClean(imported, protein_id_col = "Proteins")
annot_path = system.file("tinytest/raw_data/MaxQuant/annotation.csv", 
                         package = "MSstatsConvert")
mq_annot = MSstatsMakeAnnotation(cleaned_data, read.csv(annot_path),
                                 Run = "Rawfile")
head(mq_annot)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_balanced_design.R
\name{.checkDuplicatedMeasurements}
\alias{.checkDuplicatedMeasurements}
\title{Check if there are duplicated measurements within run}
\usage{
.checkDuplicatedMeasurements(input)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}
}
\value{
character vector of feature labels
}
\description{
Check if there are duplicated measurements within run
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clean_features.R
\name{.adjustIntensities}
\alias{.adjustIntensities}
\title{Fix invalid intensities: infinite to NA, between 0 and 1 to 0}
\usage{
.adjustIntensities(input)
}
\arguments{
\item{input}{data.table}
}
\value{
data.table
}
\description{
Fix invalid intensities: infinite to NA, between 0 and 1 to 0
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_DIANN.R
\name{.cleanRawDIANN}
\alias{.cleanRawDIANN}
\title{Clean raw Diann files}
\usage{
.cleanRawDIANN(
  msstats_object,
  MBR = TRUE,
  quantificationColumn = "FragmentQuantCorrected",
  global_qvalue_cutoff = 0.01,
  qvalue_cutoff = 0.01,
  pg_qvalue_cutoff = 0.01,
  calculateAnomalyScores = FALSE,
  anomalyModelFeatures = c(),
  labeledAminoAcids = NULL
)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsDIANNFiles}.}

\item{MBR}{True if analysis was done with match between runs}

\item{quantificationColumn}{Use 'FragmentQuantCorrected'(default) column for quantified intensities for DIANN 1.8.x.
Use 'FragmentQuantRaw' for quantified intensities for DIANN 1.9.x.
Use 'auto' for quantified intensities for DIANN 2.x where each fragment intensity is a separate column, e.g. Fr0Quantity.}

\item{global_qvalue_cutoff}{The qvalue cutoff for the Q.Value column, i.e.
the run-specific precursor q-value. Default is 0.01.}

\item{qvalue_cutoff}{If MBR is false, the qvalue cutoff for the Global.Q.Value
column, i.e. global precursor q-value.  If MBR is true, the qvalue cutoff for the
Lib.Q.Value column, i.e. the q-value for the library created after the first MBR pass.
Default is 0.01.}

\item{pg_qvalue_cutoff}{If MBR is false, the qvalue cutoff for the Global.PG.Q.Value
column, i.e. the global q-value for the protein group.  If MBR is true, the
qvalue cutoff for the Lib.PG.Q.Value column, i.e. the protein group q-value for
the library created after the first MBR pass. Default is 0.01.}

\item{calculateAnomalyScores}{Default is FALSE. If TRUE, will run anomaly detection model and calculate anomaly scores for each feature. Used downstream to weigh measurements in differential analysis.}

\item{anomalyModelFeatures}{character vector of quality metric column names to be used as features in the anomaly detection model. List must not be empty if calculateAnomalyScores=TRUE.}

\item{labeledAminoAcids}{Character vector of single-letter amino acid codes
that carry the SILAC label in protein turnover experiments, e.g.
\code{c("K")} or \code{c("K", "R")}.  Supplying this vector opts in to
protein-turnover mode; the exact amino acids determine behaviour only in the
\code{ModifiedSequence}-parsing path described below.

\strong{Channel-based path} (DIA-NN 2.x exports that include a
\code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
\emph{and} the input contains a \code{Channel} column, \code{Channel} values
are mapped directly to \code{IsotopeLabelType} (\code{"H"} → \code{"H"},
\code{"L"} → \code{"L"}, anything else → \code{NA}).  The amino acid codes
in \code{labeledAminoAcids} are \strong{not} used to validate or filter
\code{ModifiedSequence} in this path.

\strong{ModifiedSequence-parsing path} (DIA-NN 1.x exports without a
\code{Channel} column): when \code{labeledAminoAcids} is non-\code{NULL}
and no \code{Channel} column is present, each \code{ModifiedSequence} is
inspected for SILAC suffixes of the form \code{(SILAC-<AA>-H)} or
\code{(SILAC-<AA>-L)}, where \code{<AA>} is one of the supplied amino acid
codes.  Matching sequences are classified as \code{"H"} or \code{"L"};
sequences carrying neither suffix receive \code{IsotopeLabelType = NA}.
The SILAC suffix is then stripped from \code{PeptideSequence}.

When \code{NULL} (default), protein-turnover mode is disabled and all
peptides receive \code{IsotopeLabelType = "Light"}.}
}
\value{
data.table
}
\description{
Clean raw Diann files
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clean_features.R
\name{.cleanByFeature}
\alias{.cleanByFeature}
\title{Perform by-feature operations.}
\usage{
.cleanByFeature(
  input,
  feature_columns,
  cleaning_control,
  anomaly_metrics = c()
)
}
\arguments{
\item{input}{\code{data.table} preprocessed by one of the cleanRaw* functions.}

\item{feature_columns}{character vector of names of columns that define features.}

\item{cleaning_control}{named list of two or three elements.
See the documentation for \code{MSstatsImport} for details.}

\item{anomaly_metrics}{character vector of quality metric column names to be
used as features in an anomaly detection model.}
}
\value{
\code{data.table}
}
\description{
Perform by-feature operations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_PhilosophertoMSstatsTMTFormat.R
\name{.getPhilosopherInput}
\alias{.getPhilosopherInput}
\title{Convert Philosopher parameters to consistent format}
\usage{
.getPhilosopherInput(input, path, folder)
}
\arguments{
\item{input}{data.frame of \code{msstats.csv} file produced by Philosopher}

\item{path}{character. Path to a file or directory containing msstats.csv output(s) from Philosopher. Used when \code{input} is NULL.}

\item{folder}{logical. If TRUE, \code{path} is treated as a directory and all msstats files within it are read. If FALSE, \code{path} is treated as a single file path.}
}
\description{
Convert Philosopher parameters to consistent format
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.getOverlappingFeatures}
\alias{.getOverlappingFeatures}
\title{Get features that are overlapped among multiple runs}
\usage{
.getOverlappingFeatures(input)
}
\arguments{
\item{input}{data.table preprocessed by one of the .cleanRaw* functions and
merged with annotation.}
}
\value{
\code{data.table}
}
\description{
Get features that are overlapped among multiple runs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.addFractions}
\alias{.addFractions}
\title{Add a Fraction column to the output of \code{MSstatsPreprocess}}
\usage{
.addFractions(input)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}
}
\value{
data.table
}
\description{
Add a Fraction column to the output of \code{MSstatsPreprocess}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_checks.R
\name{.findAvailable}
\alias{.findAvailable}
\title{Select an available options from a set of possibilities}
\usage{
.findAvailable(possibilities, option_set, fall_back = NULL)
}
\arguments{
\item{possibilities}{possible legal values of a variable}

\item{option_set}{set of values that includes one of the \code{possibilities}}

\item{fall_back}{if there is none of the \code{possibilities} in \code{option_set},
or there are multiple hits, default to \code{fall_back}}
}
\value{
same as option_set, usually character
}
\description{
Select an available options from a set of possibilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_ProteomeDiscoverer.R
\name{.cleanRawPDTMT}
\alias{.cleanRawPDTMT}
\title{Clean raw TMT data from Proteome Discoverer}
\usage{
.cleanRawPDTMT(
  msstats_object,
  remove_shared = TRUE,
  remove_protein_groups = TRUE,
  protein_id_column = "ProteinAccessions",
  intensity_columns_regexp = "Abundance",
  run_column = "SpectrumFile"
)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectroMineFiles}.}

\item{remove_shared}{lgl, if TRUE, shared peptides will be removed.}

\item{remove_protein_groups}{if TRUE, proteins with numProteins > 1 will be removed.}

\item{protein_id_column}{chr, name of a column with protein IDs.}

\item{intensity_columns_regexp}{regular expressions that defines intensity columns.
Defaults to "Abundance", which means that columns that contain the word "Abundance"
will be treated as corresponding to intensities for different channels.}
}
\value{
\code{data.table}
}
\description{
Clean raw TMT data from Proteome Discoverer
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_classes.R
\name{.MSstatsFormat}
\alias{.MSstatsFormat}
\title{Output format for further analysis by MSstats}
\usage{
.MSstatsFormat(input, anomaly_metrics = c())
}
\arguments{
\item{input}{data.table}

\item{anomaly_metrics}{character vector of quality metric column names to be used as features in an anomaly detection model}
}
\value{
object of class MSstatsValidated that inherits from data.frame
}
\description{
Output format for further analysis by MSstats
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_OpenMStoMSstatsTMTFormat.R
\name{OpenMStoMSstatsTMTFormat}
\alias{OpenMStoMSstatsTMTFormat}
\title{Generate MSstatsTMT required input format for OpenMS output}
\usage{
OpenMStoMSstatsTMTFormat(
  input,
  useUniquePeptide = TRUE,
  rmPSM_withfewMea_withinRun = TRUE,
  rmProtein_with1Feature = FALSE,
  summaryforMultiplePSMs = sum,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{MSstatsTMT report from OpenMS}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{rmPSM_withfewMea_withinRun}{TRUE (default) will remove the features that have 1 or 2 measurements within each Run.}

\item{rmProtein_with1Feature}{TRUE will remove the proteins which have only 1 peptide and charge. Default is FALSE.}

\item{summaryforMultiplePSMs}{sum(default) or max - when there are multiple measurements for certain feature in certain run, select the feature with the largest summation or maximal value.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
\code{data.frame} of class \code{MSstatsTMT}.
}
\description{
Generate MSstatsTMT required input format for OpenMS output
}
\examples{
raw.om = data.table::fread(system.file("tinytest/raw_data/OpenMSTMT/openmstmt_input.csv",
                                      package = "MSstatsConvert"))
input.om <- OpenMStoMSstatsTMTFormat(raw.om)
head(input.om)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clean_features.R
\name{.aggregatePSMstoPeptideIons}
\alias{.aggregatePSMstoPeptideIons}
\title{Aggregate multiple PSMs to a single peptide ion.}
\usage{
.aggregatePSMstoPeptideIons(input, feature_columns, summary_function = sum)
}
\arguments{
\item{input}{data.table preprocessed by one of the cleanRaw* functions.}

\item{feature_columns}{chr, names of columns that define features.}

\item{summary_function}{function that will be used to aggregate intensities
if needed.}
}
\value{
data.table
}
\description{
Aggregate multiple PSMs to a single peptide ion.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_Spectronaut.R
\name{.cleanRawSpectronaut}
\alias{.cleanRawSpectronaut}
\title{Clean raw Spectronaut output.}
\usage{
.cleanRawSpectronaut(
  msstats_object,
  intensity,
  calculateAnomalyScores,
  anomalyModelFeatures,
  peptideSequenceColumn = "EG.ModifiedSequence",
  heavyLabels = NULL
)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectronautFiles}.}

\item{intensity}{Intensity column to use. Accepts legacy enum values
\code{'PeakArea'} (default, uses F.PeakArea), \code{'NormalizedPeakArea'}
(uses F.NormalizedPeakArea).  Can also be any raw
Spectronaut column name passed as a string (e.g.
\code{"FG.MS1Quantity"}); the column name is standardized internally.
For protein turnover workflows the recommended default is
\code{"FG.MS1Quantity"}.}

\item{calculateAnomalyScores}{Default is FALSE. If TRUE, will run anomaly detection model and calculate anomaly scores for each feature. Used downstream to weigh measurements in differential analysis.}

\item{anomalyModelFeatures}{character vector of quality metric column names to be used as features in the anomaly detection model. List must not be empty if calculateAnomalyScores=TRUE.}

\item{peptideSequenceColumn}{Name of the Spectronaut column that contains the
peptide sequence.  Defaults to \code{"EG.ModifiedSequence"}. The value is
standardized internally (dots and spaces removed) before column lookup.}

\item{heavyLabels}{Character list identifying the heavy isotope labels as it
appears inside square brackets in the peptide sequence column, e.g.
\code{c("Lys6")} matches peptides containing \code{[Lys6]}.
\code{c("Lys6", "Arg10")} matches peptides containing either \code{[Lys6]} or \code{[Arg10]}.
Supports any novel label name reported by Spectronaut (e.g. \code{"Leu6"},
\code{"Phe10"}).  When provided, peptides are
classified as heavy (\code{IsotopeLabelType = "H"}), light
(\code{IsotopeLabelType = "L"}), or unlabeled
(\code{IsotopeLabelType = NA}) based on its labeled sequence.  When
\code{NULL} (default) all peptides receive \code{IsotopeLabelType = "L"}.
Useful for protein turnover experiments.}
}
\value{
\code{data.table}
}
\description{
Clean raw Spectronaut output.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{getInputFile}
\alias{getInputFile}
\alias{getInputFile,MSstatsInputFiles-method}
\alias{getInputFile,MSstatsPhilosopherFiles-method}
\title{Get one of files contained in an instance of \code{MSstatsInputFiles} class.}
\usage{
getInputFile(msstats_object, file_type)

\S4method{getInputFile}{MSstatsInputFiles}(msstats_object, file_type = "input")

\S4method{getInputFile}{MSstatsPhilosopherFiles}(msstats_object, file_type = "input")
}
\arguments{
\item{msstats_object}{object that inherits from \code{MSstatsPhilosopherFiles} class.}

\item{file_type}{character name of a type file. Usually equal to "input".}
}
\value{
data.table

data.table

data.table
}
\description{
Get one of files contained in an instance of \code{MSstatsInputFiles} class.
}
\examples{
evidence_path = system.file("tinytest/raw_data/MaxQuant/mq_ev.csv", 
                            package = "MSstatsConvert")
pg_path = system.file("tinytest/raw_data/MaxQuant/mq_pg.csv", 
                      package = "MSstatsConvert")
evidence = read.csv(evidence_path)
pg = read.csv(pg_path)
imported = MSstatsImport(list(evidence = evidence, protein_groups = pg),
                         "MSstats", "MaxQuant")
class(imported)
head(getInputFile(imported, "evidence"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_shared_peptides.R
\name{.removeSharedPeptides}
\alias{.removeSharedPeptides}
\title{Remove peptides assigned to more than one protein.}
\usage{
.removeSharedPeptides(input, protein_column, peptide_column)
}
\arguments{
\item{input}{data.table pre-processed by one of the .cleanRaw* functions.}

\item{protein_column}{chr, name of the column with names of proteins.}

\item{peptide_column}{chr, name of the column with peptide sequences.}
}
\value{
data.table
}
\description{
Remove peptides assigned to more than one protein.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{MSstatsSaveSessionInfo}
\alias{MSstatsSaveSessionInfo}
\title{Save session information}
\usage{
MSstatsSaveSessionInfo(
  path = NULL,
  append = TRUE,
  base = "MSstats_session_info_"
)
}
\arguments{
\item{path}{optional path to output file. If not provided, "MSstats_session_info"
and current timestamp will be used as a file name}

\item{append}{if TRUE and file given by the \code{path} parameter already exists,
session info will be appended to the file}

\item{base}{beginning of a file name}
}
\value{
TRUE invisibly after session info was saved
}
\description{
Save session information
}
\examples{
MSstatsSaveSessionInfo("session_info.txt")
MSstatsSaveSessionInfo("session_info.txt", base = "MSstatsTMT_session_info_")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_logging.R
\name{.consoleAppender}
\alias{.consoleAppender}
\title{Console appender: returns a function(level, ...) that writes one line to console}
\usage{
.consoleAppender()
}
\description{
Console appender: returns a function(level, ...) that writes one line to console
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MSstatsConvert_core_functions.R
\name{MSstatsAnomalyScores}
\alias{MSstatsAnomalyScores}
\title{Run Anomaly Model}
\usage{
MSstatsAnomalyScores(
  input,
  quality_metrics,
  temporal_direction,
  missing_run_count,
  n_feat,
  run_order,
  n_trees,
  max_depth,
  cores
)
}
\arguments{
\item{input}{data.table preprocessed by the MSstatsBalancedDesign function}

\item{quality_metrics}{character vector of quality metrics to use in the model}

\item{temporal_direction}{character vector of same length as quality_metrics indicating temporal feature to create.}

\item{missing_run_count}{numeric, maximum allowed fraction of missing runs per feature.}

\item{n_feat}{numeric, maximum number of features per protein to use in the model.}

\item{run_order}{data.frame with two columns: Run and Order. Order should be numeric and indicate the order of runs.}

\item{n_trees}{numeric, number of trees to use in the isolation forest model. Default is 100.}

\item{max_depth}{numeric or "auto", maximum depth of each tree. Default is "auto" which sets depth to log2(N) where N is the number of runs.}

\item{cores}{numeric, number of cores to use for parallel processing. Default is 1.}
}
\value{
data.table
}
\description{
Detects anomalous measurements in mass spectrometry data using an isolation forest algorithm.
This function identifies unusual precursor measurements based on quality metrics and their
temporal patterns. For features with insufficient quality metric data, it assigns anomaly
scores based on the median score of similar features (same peptide and charge combination).
The model supports parallel processing for improved performance on large datasets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_annotation.R
\name{.checkAnnotation}
\alias{.checkAnnotation}
\title{Check if the annotation is valid}
\usage{
.checkAnnotation(input, annotation)
}
\arguments{
\item{input}{data processed by the MSstatsClean}

\item{annotation}{annotation created by the MSstatsMakeAnnotation function}
}
\value{
TRUE invisibly if the annotation is correct, throws an error otherwise
}
\description{
Check if the annotation is valid
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_dt_operations.R
\name{.getChannelColumns}
\alias{.getChannelColumns}
\title{Get intensity columns from wide-format data}
\usage{
.getChannelColumns(col_names, ...)
}
\arguments{
\item{col_names}{names of columns, where some of the columns store intensity
value for different channels}

\item{...}{varying number of strings that define channel columns.}
}
\value{
character vector of column names that correspond to channel intensities
}
\description{
Get intensity columns from wide-format data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_MaxQuant.R
\name{.cleanRawMaxQuant}
\alias{.cleanRawMaxQuant}
\title{Clean raw output from MaxQuant}
\usage{
.cleanRawMaxQuant(
  msstats_object,
  protein_id_col,
  remove_by_site = FALSE,
  channel_columns = "Reporterintensitycorrected"
)
}
\arguments{
\item{msstats_object}{object that inherits from MSstatsInputFiles class.}

\item{protein_id_col}{character, name of a column with names of proteins.}

\item{remove_by_site}{logical, if TRUE, proteins only identified by site will
be removed.}

\item{channel_columns}{character, regular expression that identifies channel columns
in TMT data.}
}
\value{
data.table
}
\description{
Clean raw output from MaxQuant
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_ProteomeDiscoverer.R
\name{.cleanRawPD}
\alias{.cleanRawPD}
\title{Clean raw Proteome Discoverer data}
\usage{
.cleanRawPD(
  msstats_object,
  quantification_column,
  protein_id_column,
  sequence_column,
  remove_shared,
  remove_protein_groups = TRUE,
  intensity_columns_regexp = "Abundance"
)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectroMineFiles}.}

\item{quantification_column}{chr, name of a column used for quantification.}

\item{protein_id_column}{chr, name of a column with protein IDs.}

\item{sequence_column}{chr, name of a column with peptide sequences.}

\item{remove_shared}{lgl, if TRUE, shared peptides will be removed.}

\item{remove_protein_groups}{if TRUE, proteins with numProteins > 1 will be removed.}

\item{intensity_columns_regexp}{regular expressions that defines intensity columns.
Defaults to "Abundance", which means that columns that contain the word "Abundance"
will be treated as corresponding to intensities for different channels.}
}
\value{
data.table
}
\description{
Clean raw Proteome Discoverer data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clean_features.R
\name{.filterFewMeasurements}
\alias{.filterFewMeasurements}
\title{Remove features with a small number of (non-missing) measurements across runs}
\usage{
.filterFewMeasurements(
  input,
  min_intensity,
  remove_few,
  feature_columns = NULL
)
}
\arguments{
\item{input}{\code{data.table} pre-processed by one of the .cleanRaw* functions.}

\item{min_intensity}{minimum intensity that will be considered non-missing.}

\item{remove_few}{logical, if TRUE, features that have less than three
measurements will be removed. If FALSE, only features with all missing runs
will be removed.}

\item{feature_columns}{chr, vector of names of columns that define features.}
}
\value{
data.table
}
\description{
Remove features with a small number of (non-missing) measurements across runs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_filtering.R
\name{.filterByScore}
\alias{.filterByScore}
\title{Filter PSMs / proteins by a given score column.}
\usage{
.filterByScore(
  input,
  score_column,
  score_threshold,
  direction,
  behavior,
  handle_na = "keep",
  fill_value = NA,
  filter = TRUE,
  drop = TRUE
)
}
\arguments{
\item{input}{\code{data.table} preprocessed by one of the .cleanRaw* functions.}

\item{score_column}{chr, name of the column that contains scores.}

\item{score_threshold}{num, values below or above this threshold will be
removed from the data.}

\item{direction}{chr, if "greater" only values above the threshold will be
retained, if "smaller" - below the threshold.}

\item{behavior}{chr, if "remove", values below/above the threshold will be
removed, if "replace", they will be set to \code{fill_value}.}

\item{fill_value}{if \code{behavior} = "replace", values below/above the threshold
will be replaced with \code{fill_value}. Defaults to \code{NA}.}

\item{filter}{If TRUE, filtering will be performed.}

\item{drop}{if TRUE, \code{score_column} will be removed.}
}
\value{
data.table
}
\description{
Filter PSMs / proteins by a given score column.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_filtering.R
\name{.filterManyColumns}
\alias{.filterManyColumns}
\title{Filter rows that contain specifed symbols in multiple columns.}
\usage{
.filterManyColumns(input, filter_columns, filter_symbols)
}
\arguments{
\item{input}{data.table preprocessed by one of the \verb{cleanRaw*} functions.}

\item{filter_columns}{chr, names of columns in which elements will be matched
and removed.}

\item{filter_symbols}{chr, vector of strings. Rows with corresponding elements
in \code{filter_columns} will be removed.}
}
\value{
data.table
}
\description{
Filter rows that contain specifed symbols in multiple columns.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_fractions.R
\name{.checkMultiRun}
\alias{.checkMultiRun}
\title{Check if fractionation exists}
\usage{
.checkMultiRun(input)
}
\arguments{
\item{input}{output of \code{MSstatsPreprocess}}
}
\value{
list of two elements: \code{has_fractions} (logical) indicates if
fractions was detected in the \code{input} dataset, \code{is_risky} (logical)
indicates if there was a problem with detecting fractionation.
}
\description{
Check if fractionation exists
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_OpenMStoMSstatsFormat.R
\name{OpenMStoMSstatsFormat}
\alias{OpenMStoMSstatsFormat}
\title{Import OpenMS files}
\usage{
OpenMStoMSstatsFormat(
  input,
  annotation = NULL,
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeProtein_with1Feature = FALSE,
  summaryforMultipleRows = max,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{name of MSstats input report from OpenMS, which includes feature(peptide ion)-level data.}

\item{annotation}{name of 'annotation.txt' data which includes Condition, BioReplicate, Run.
Run should be the same as filename.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements across runs.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have only 1 feature, which is the combination of peptide, precursor charge, fragment and charge. FALSE is default.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame in the MSstats required format.
}
\description{
Import OpenMS files
}
\examples{
openms_raw = data.table::fread(system.file("tinytest/raw_data/OpenMS/openms_input.csv", 
                                           package = "MSstatsConvert"))
openms_imported = OpenMStoMSstatsFormat(openms_raw, use_log_file = FALSE)
head(openms_imported)

}
\author{
Meena Choi, Olga Vitek.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_filtering.R
\name{.filterByPattern}
\alias{.filterByPattern}
\title{Handle filtering by pattern}
\usage{
.filterByPattern(input, col_name, patterns, filter, drop)
}
\arguments{
\item{input}{\code{data.table} preprocessed by one of the .cleanRaw* functions.}

\item{col_name}{chr, name of the column with peptide sequences.}

\item{patterns}{chr, regular expression - matching peptides will be
removed from the data.}

\item{filter}{lgl, if TRUE, peptides will be actually filtered.}

\item{drop}{lgl, if TRUE, the \code{column} will be dropped.}
}
\value{
data.table
}
\description{
Handle filtering by pattern
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_Progenesis.R
\name{.cleanRawProgenesis}
\alias{.cleanRawProgenesis}
\title{Clean raw Progenesis output}
\usage{
.cleanRawProgenesis(msstats_object, runs, fix_colnames = TRUE)
}
\arguments{
\item{msstats_object}{an object of class \code{MSstatsSpectroMineFiles}.}

\item{runs}{chr, vector of Run labels.}

\item{fix_colnames}{lgl, if TRUE, one of the rows will be used as colnames.}
}
\value{
data.table
}
\description{
Clean raw Progenesis output
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_dt_operations.R
\name{.getDataTable}
\alias{.getDataTable}
\title{Read file from a provided path or convert given data.frame to data.table}
\usage{
.getDataTable(input, ...)
}
\arguments{
\item{input}{report from a signal processing tool or a path to it}

\item{...}{additional parameters for data.table::fread}
}
\value{
data.table
}
\description{
Read file from a provided path or convert given data.frame to data.table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_Skyline.R
\name{.checkDDA}
\alias{.checkDDA}
\title{Check validity of DDA data}
\usage{
.checkDDA(input)
}
\arguments{
\item{input}{data.table preprocessed by one of the \verb{cleanRaw*} functions.}
}
\value{
logical

logical, TRUE means that the \code{input} dataset comes from a DDA experiment
}
\description{
Check validity of DDA data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/converters_SpectronauttoMSstatsFormat.R
\name{SpectronauttoMSstatsFormat}
\alias{SpectronauttoMSstatsFormat}
\title{Import Spectronaut files}
\usage{
SpectronauttoMSstatsFormat(
  input,
  annotation = NULL,
  intensity = "PeakArea",
  peptideSequenceColumn = "EG.ModifiedSequence",
  heavyLabels = NULL,
  excludedFromQuantificationFilter = TRUE,
  filter_with_Qvalue = FALSE,
  qvalue_cutoff = 0.01,
  useUniquePeptide = TRUE,
  removeFewMeasurements = TRUE,
  removeProtein_with1Feature = FALSE,
  summaryforMultipleRows = max,
  calculateAnomalyScores = FALSE,
  anomalyModelFeatures = c(),
  anomalyModelFeatureTemporal = c(),
  removeMissingFeatures = 0.5,
  anomalyModelFeatureCount = 100,
  runOrder = NULL,
  n_trees = 100,
  max_depth = "auto",
  numberOfCores = 1,
  use_log_file = TRUE,
  append = FALSE,
  verbose = TRUE,
  log_file_path = NULL,
  ...
)
}
\arguments{
\item{input}{name of Spectronaut output, which is long-format. ProteinName, PeptideSequence, PrecursorCharge, FragmentIon, ProductCharge, IsotopeLabelType, Condition, BioReplicate, Run, Intensity, F.ExcludedFromQuantification are required. Rows with F.ExcludedFromQuantification=True will be removed.}

\item{annotation}{name of 'annotation.txt' data which includes Condition, BioReplicate, Run. If annotation is already complete in Spectronaut, use annotation=NULL (default). It will use the annotation information from input.}

\item{intensity}{Intensity column to use. Accepts legacy enum values
\code{'PeakArea'} (default, uses F.PeakArea), \code{'NormalizedPeakArea'}
(uses F.NormalizedPeakArea).  Can also be any raw
Spectronaut column name passed as a string (e.g.
\code{"FG.MS1Quantity"}); the column name is standardized internally.
For protein turnover workflows the recommended default is
\code{"FG.MS1Quantity"}.}

\item{peptideSequenceColumn}{Name of the Spectronaut column that contains the
peptide sequence.  Defaults to \code{"EG.ModifiedSequence"}. The value is
standardized internally (dots and spaces removed) before column lookup.}

\item{heavyLabels}{Character list identifying the heavy isotope labels as it
appears inside square brackets in the peptide sequence column, e.g.
\code{c("Lys6")} matches peptides containing \code{[Lys6]}.
\code{c("Lys6", "Arg10")} matches peptides containing either \code{[Lys6]} or \code{[Arg10]}.
Supports any novel label name reported by Spectronaut (e.g. \code{"Leu6"},
\code{"Phe10"}).  When provided, peptides are
classified as heavy (\code{IsotopeLabelType = "H"}), light
(\code{IsotopeLabelType = "L"}), or unlabeled
(\code{IsotopeLabelType = NA}) based on its labeled sequence.  When
\code{NULL} (default) all peptides receive \code{IsotopeLabelType = "L"}.
Useful for protein turnover experiments.}

\item{excludedFromQuantificationFilter}{Remove rows with F.ExcludedFromQuantification=TRUE Default is TRUE.}

\item{filter_with_Qvalue}{FALSE(default) will not perform any filtering. TRUE will filter out the intensities that have greater than qvalue_cutoff in EG.Qvalue column. Those intensities will be replaced with zero and will be considered as censored missing values for imputation purpose.}

\item{qvalue_cutoff}{Cutoff for EG.Qvalue. default is 0.01.}

\item{useUniquePeptide}{TRUE (default) removes peptides that are assigned for more than one proteins.
We assume to use unique peptide for each protein.}

\item{removeFewMeasurements}{TRUE (default) will remove the features that have 1 or 2 measurements across runs.}

\item{removeProtein_with1Feature}{TRUE will remove the proteins which have only 1 feature, which is the combination of peptide, precursor charge, fragment and charge. FALSE is default.}

\item{summaryforMultipleRows}{max or sum - when there are multiple measurements for certain feature and certain run, use highest or sum of multiple intensities. Default is max for label-free converters and sum for TMT converters.}

\item{calculateAnomalyScores}{Default is FALSE. If TRUE, will run anomaly detection model and calculate anomaly scores for each feature. Used downstream to weigh measurements in differential analysis.}

\item{anomalyModelFeatures}{character vector of quality metric column names to be used as features in the anomaly detection model. List must not be empty if calculateAnomalyScores=TRUE.}

\item{anomalyModelFeatureTemporal}{character vector of temporal direction corresponding to columns passed to anomalyModelFeatures. Values must be one of: \code{mean_decrease}, \code{mean_increase}, \code{dispersion_increase}, or NULL (to perform no temporal feature engineering). Default is empty vector. If calculateAnomalyScores=TRUE, vector must have as many values as anomalyModelFeatures (even if all NULL).}

\item{removeMissingFeatures}{Remove features with missing values in more than this fraction of runs. Default is 0.5. Only used if calculateAnomalyScores=TRUE.}

\item{anomalyModelFeatureCount}{Feature selection for anomaly model. Anomaly detection works on the precursor-level and can be much slower if all features used. We will by default filter to the top-100 highest intensity features. This can be adjusted as necessary. To turn feature-selection off, set this value to a high number (e.g. 10000). Only used if calculateAnomalyScores=TRUE.}

\item{runOrder}{Temporal order of MS runs. Should be a two column data.table with columns \code{Run} and \code{Order}, where \code{Run} matches the run name output by Spectronaut and \code{Order} is an integer. Used to engineer the temporal features defined in anomalyModelFeatureTemporal.}

\item{n_trees}{Number of trees to use in isolation forest when calculateAnomalyScores=TRUE. Default is 100.}

\item{max_depth}{Max tree depth to use in isolation forest when calculateAnomalyScores=TRUE. Default is "auto" which calculates depth as log2(N) where N is the number of runs. Otherwise must be an integer.}

\item{numberOfCores}{Number of cores for parallel processing anomaly detection model. When > 1, a logfile named 'MSstats_anomaly_model_progress.log' is created to track progress. Only works for Linux & Mac OS. Default is 1.}

\item{use_log_file}{logical. If TRUE, information about data processing
will be saved to a file.}

\item{append}{logical. If TRUE, information about data processing will be added
to an existing log file.}

\item{verbose}{logical. If TRUE, information about data processing will be printed
to the console.}

\item{log_file_path}{character. Path to a file to which information about
data processing will be saved.
If not provided, such a file will be created automatically.
If \code{append = TRUE}, has to be a valid path to a file.}

\item{...}{additional parameters to \code{data.table::fread}.}
}
\value{
data.frame in the MSstats required format.
}
\description{
Import Spectronaut files
}
\examples{
spectronaut_raw = system.file("tinytest/raw_data/Spectronaut/spectronaut_input.csv",
                              package = "MSstatsConvert")
spectronaut_raw = data.table::fread(spectronaut_raw)
spectronaut_imported = SpectronauttoMSstatsFormat(spectronaut_raw, use_log_file = FALSE)
head(spectronaut_imported)

}
\author{
Meena Choi, Olga Vitek
}
